import {
  browser,
  by,
  element,
  ElementArrayFinder,
  ElementFinder,
  Key,
  logging,
} from "protractor";
import { GlobalFunctions } from "../global-functions";
import { SNSE004_MPage } from "./online-sale.po";

describe(`ขาย Online - Normal case`, () => {
  let CustomerCode: ElementFinder,
    CustomerName: ElementFinder,
    BranchCode: ElementFinder,
    BranchName: ElementFinder,
    Address: ElementFinder,
    // Address2: ElementFinder,
    OrderNumber: ElementFinder,
    OrderClass: ElementFinder,
    PaymentTerm: ElementFinder,
    OrderDate: ElementFinder,
    OrderType: ElementFinder,
    BOFlag: ElementFinder,
    RouteCode: ElementFinder,
    TransportCode: ElementFinder,
    DeliveryCode: ElementFinder,
    TotalQuantity: ElementFinder,
    TotalAmount: ElementFinder,
    AvailableCredit: ElementFinder,
    PrintRemark: ElementFinder,
    PrintPickingCard: ElementFinder,
    PartsNo: ElementArrayFinder,
    SaveButton: ElementFinder;

  let setElements = (page: SNSE004_MPage) => {
    CustomerCode = page.getCustomerCodeElement();
    CustomerName = page.getCustomerNameElement();
    BranchCode = page.getBranchCodeElement();
    BranchName = page.getBranchNameElement();
    Address = page.getAddressElement();
    // Address2 = page.getAddress2Element();
    OrderNumber = page.getOrderNumberElement();
    OrderClass = page.getOrderClassElement();
    PaymentTerm = page.getPaymentTermElement();
    OrderDate = page.getOrderDateElement();
    OrderType = page.getOrderTypeElement();
    BOFlag = page.getBOFlagElement();
    RouteCode = page.getRouteCodeElement();
    TransportCode = page.getTransportCodeElement();
    DeliveryCode = page.getDeliveryCodeElement();
    TotalQuantity = page.getTotalQuantityElement();
    TotalAmount = page.getTotalAmountElement();
    AvailableCredit = page.getAvailableCreditElement();
    PrintRemark = page.getPrintRemarkElement();
    PrintPickingCard = page.getPrintPickingCardElement();
    PartsNo = page.getAllPartNoElement();
    SaveButton = page.getSaveButtonElement();
  };

  let fn = new GlobalFunctions();
  let page: SNSE004_MPage = new SNSE004_MPage();
  let timeFormat = Math.floor(new Date().getTime() / 10000).toString(); //10s

  let orderClassList: Array<any> = [
    // {
    //   OrderClass: "1",
    //   OrderClassRegEx: /^1/,
    // },
    // {
    //   OrderClass: "2",
    //   OrderClassRegEx: /^2/,
    // },
    {
      OrderClass: "4",
      OrderClassRegEx: /^4/,
    },
  ];
  let orderClass: string;
  let orderClassRegex: RegExp;
  let orderTypeList: Array<any> = [
    { ORDER_TYPE: "1", DESCRIPTION: "E/O", REF_ORDER_CLASS: ["1", "2"] },
    { ORDER_TYPE: "3", DESCRIPTION: "R/O", REF_ORDER_CLASS: ["1", "2", "4"] },
  ];
  let customerCode: string;
  let orderNo: string;
  let partList: Array<any>;

  orderClassList.forEach((o) => {
    beforeAll(() => {
      page.navigateTo();

      setElements(page);

      customerCode = "008114A";
      orderClass = o.OrderClass;
      orderClassRegex = o.OrderClassRegEx;
      orderNo = orderClass + "N" + timeFormat.substr(1, 8);

      /*
          select p.part_no, p.part_name_en, p.lock_sales, s.oh_qty
          from tpm_parts_master p, tpm_stock_master s
          where p.company_code = s.company_code
            and p.part_no = s.part_no
            and p.lock_sales = 0
            and s.oh_qty > 0;
        */
      partList = [
        {
          ORDER_PART_NO: "4972540UB5",
          ORDER_QTY: 1,
        },
        {
          ORDER_PART_NO: "4971752YB0",
          ORDER_QTY: 1,
        },
        {
          ORDER_PART_NO: "K9Y164JEE1",
          ORDER_QTY: 1,
        },
      ];
    });

    beforeEach(() => {
      browser.driver.sleep(300);
    });

    it(`Should able to key customer`, async () => {
      CustomerCode.sendKeys(customerCode);
      fn.selectEnterKey();

      browser.waitForAngular();

      var autoCompleteText = await page.getDynamicElementText(
        ".cdk-overlay-container auto-complete-query-dialog span.mat-form-field-label-wrapper>label>span"
      );

      if (autoCompleteText == "Select Customer") {
        fn.selectDownKey(); //Highlight 1st item
        fn.selectEnterKey(); //Select item
        fn.selectEnterKey(); //Confirm select item
      }

      browser.waitForAngular();
      expect(CustomerName.getAttribute("value")).toBeTruthy();
    });

    it(`Branch should not be null`, () => {
      expect(BranchCode.getAttribute("value")).toBeTruthy();
      expect(BranchName.getAttribute("value")).toBeTruthy();
    });

    it(`Should able to key order: ` + orderNo, () => {
      OrderNumber.sendKeys(orderNo);
      fn.selectEnterKey(); //Set order class ...
      expect(OrderNumber.getAttribute("value")).toBeTruthy();
    });

    it(`Order class should match with first digit of order number`, async () => {
      expect(OrderClass.getText()).toMatch(orderClassRegex);
    });

    it(`Payment term should not be null`, () => {
      expect(PaymentTerm.getAttribute("value")).toBeTruthy();
    });

    it(`Order date should not be null`, () => {
      expect(OrderDate.getAttribute("value")).toBeTruthy();
    });

    it(`Order type should match with relation of order class`, async () => {
      let orderType = (await OrderType.getText()).substr(0, 1);

      expect(
        orderTypeList.find((v) => {
          return (
            v.REF_ORDER_CLASS.indexOf(orderClass) >= 0 &&
            v.ORDER_TYPE == orderType
          );
        })
      ).toBeTruthy();
    });

    it(`B/O flag should not be null`, () => {
      expect(BOFlag.getText()).toBeTruthy();
    });

    it(`Route Code should not be null`, () => {
      expect(RouteCode.getAttribute("value")).toBeTruthy();
    });

    it(`Transport code should not be null`, () => {
      expect(TransportCode.getAttribute("value")).toBeTruthy();
    });

    it(`Deliver code should not be null`, () => {
      expect(DeliveryCode.getText()).toBeTruthy();
    });

    it(`Part no should able to enter`, async () => {
      var rows: ElementArrayFinder = page.getDetailRowsElement();

      await rows
        .each(async (elem, idx) => {
          var partItem: ElementFinder = await elem.element(
            by.css("td.mat-column-ORDER_PART_NO input[name=ORDER_PART_NO]")
          );
          var orderQtyItem: ElementFinder = await elem.element(
            by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
          );

          if (idx < partList.length) {
            await partItem.sendKeys(partList[idx].ORDER_PART_NO);
            fn.selectEnterKey();
            await browser.waitForAngular();

            expect(orderQtyItem.getAttribute("disabled")).not.toEqual("true");
          }
        })
        .catch((err) => {
          console.log(err);
        });
    });

    it(`Order qty should able to enter`, () => {
      var rows: ElementArrayFinder = page.getDetailRowsElement();

      partList.forEach(async (part, idx) => {
        var elem: ElementFinder = rows.get(idx);
        var orderQtyItem: ElementFinder = await elem.element(
          by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
        );

        expect(orderQtyItem.getAttribute("disabled")).not.toEqual("true");
        await orderQtyItem.sendKeys(part.ORDER_QTY);
        fn.selectEnterKey();
      });
    });

    it(`Amount should not be null`, () => {
      var rows: ElementArrayFinder = page.getDetailRowsElement();

      partList.forEach(async (part, idx) => {
        var elem: ElementFinder = rows.get(idx);
        var orderQtyItem: ElementFinder = await elem.element(
          by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
        );
        var unitPriceItem: ElementFinder = await elem.element(
          by.css("td.mat-column-UNIT_PRICE input[name=UNIT_PRICE]")
        );
        var amountItem: ElementFinder = await elem.element(
          by.css("td.mat-column-AMOUNT")
        );

        var amount: Number =
          fn.convertStringToNumber(await orderQtyItem.getAttribute("value")) *
          fn.convertStringToNumber(await unitPriceItem.getAttribute("value"));

        expect(
          fn.convertStringToNumber(await amountItem.getText())
        ).not.toEqual(0);
        expect(fn.convertStringToNumber(await amountItem.getText())).toEqual(
          amount
        );
      });
    });

    it(`Save button able to click`, () => {
      expect(SaveButton.getAttribute("disabled")).not.toEqual("true");
      SaveButton.click();
    });

    it(`Confirm modal appeared and confirmable`, () => {
      var confirmModal: ElementFinder = page.getMessageModalElement(
        ".modal-confirm"
      );
      var confirmButton: ElementFinder = confirmModal.element(
        by.css(".mat-dialog-actions button[name=Confirm]")
      );

      expect(
        confirmModal.element(by.css(".mat-dialog-title")).getText()
      ).toMatch(/CONFIRMATION/);
      expect(
        confirmModal.element(by.css(".mat-dialog-content")).getText()
      ).toMatch(/Do you want to save changed you have made\?/);

      confirmButton.click();
      browser.waitForAngular();
    });

    it(`Save completed`, () => {
      var infoModal: ElementFinder = page.getMessageModalElement(".modal-info");
      expect(
        infoModal.element(by.css(".mat-dialog-content")).getText()
      ).toMatch(/Do Number = DO-(\d){2}-(\d){7}/);

      browser.driver.sleep(1000);
    });
  });
});
