import {
  browser,
  by,
  element,
  ElementArrayFinder,
  ElementFinder,
  Key,
  logging,
} from "protractor";
import { isRegExp } from "util";
import { GlobalFunctions } from "../global-functions";
import { SNSE004_MPage } from "./online-sale.po";

describe(`ขาย Online - Lock sales case`, () => {
  let CustomerCode: ElementFinder,
    CustomerName: ElementFinder,
    BranchCode: ElementFinder,
    BranchName: ElementFinder,
    Address: ElementFinder,
    // Address2: ElementFinder,
    OrderNumber: ElementFinder,
    OrderClass: ElementFinder,
    PaymentTerm: ElementFinder,
    OrderDate: ElementFinder,
    OrderType: ElementFinder,
    BOFlag: ElementFinder,
    RouteCode: ElementFinder,
    TransportCode: ElementFinder,
    DeliveryCode: ElementFinder,
    TotalQuantity: ElementFinder,
    TotalAmount: ElementFinder,
    AvailableCredit: ElementFinder,
    PrintRemark: ElementFinder,
    PrintPickingCard: ElementFinder,
    PartsNo: ElementArrayFinder,
    SaveButton: ElementFinder;

  let setElements = (page: SNSE004_MPage) => {
    CustomerCode = page.getCustomerCodeElement();
    CustomerName = page.getCustomerNameElement();
    BranchCode = page.getBranchCodeElement();
    BranchName = page.getBranchNameElement();
    Address = page.getAddressElement();
    // Address2 = page.getAddress2Element();
    OrderNumber = page.getOrderNumberElement();
    OrderClass = page.getOrderClassElement();
    PaymentTerm = page.getPaymentTermElement();
    OrderDate = page.getOrderDateElement();
    OrderType = page.getOrderTypeElement();
    BOFlag = page.getBOFlagElement();
    RouteCode = page.getRouteCodeElement();
    TransportCode = page.getTransportCodeElement();
    DeliveryCode = page.getDeliveryCodeElement();
    TotalQuantity = page.getTotalQuantityElement();
    TotalAmount = page.getTotalAmountElement();
    AvailableCredit = page.getAvailableCreditElement();
    PrintRemark = page.getPrintRemarkElement();
    PrintPickingCard = page.getPrintPickingCardElement();
    PartsNo = page.getAllPartNoElement();
    SaveButton = page.getSaveButtonElement();
  };

  let fn = new GlobalFunctions();
  let page: SNSE004_MPage = new SNSE004_MPage();
  let timeFormat = Math.floor(new Date().getTime() / 10000).toString(); //10s

  let orderClass: string;
  let orderClassRegex: RegExp;
  let orderTypeList: Array<any> = [
    { ORDER_TYPE: "1", DESCRIPTION: "E/O", REF_ORDER_CLASS: ["1", "2"] },
    { ORDER_TYPE: "3", DESCRIPTION: "R/O", REF_ORDER_CLASS: ["1", "2", "4"] },
  ];
  let customerCode: string;
  let orderNo: string;
  let partNo117: string;
  let partNoLocked: string;
  // let partList: Array<any>;

  beforeAll(() => {
    page.navigateTo();

    setElements(page);

    customerCode = "008114A";
    orderClass = "4";
    orderClassRegex = /^4/;
    orderNo = orderClass + "L" + timeFormat.substr(1, 8);

    /*
        select p.part_no, p.part_name_en, p.lock_sales, s.oh_qty
        from tpm_parts_master p, tpm_stock_master s
        where p.company_code = s.company_code
          and p.part_no = s.part_no
          and p.lock_sales > 0;
      */

    partNoLocked = "933014JJ4B";
    partNo117 = "21430C991A";
  });

  beforeEach(() => {
    browser.driver.sleep(300);
  });

  it(`Should able to key customer`, async () => {
    CustomerCode.sendKeys(customerCode);
    fn.selectEnterKey();

    browser.waitForAngular();

    var autoCompleteText = await page.getDynamicElementText(
      ".cdk-overlay-container auto-complete-query-dialog span.mat-form-field-label-wrapper>label>span"
    );

    if (autoCompleteText == "Select Customer") {
      fn.selectDownKey(); //Highlight 1st item
      fn.selectEnterKey(); //Select item
      fn.selectEnterKey(); //Confirm select item
    }

    browser.waitForAngular();
    expect(CustomerName.getAttribute("value")).toBeTruthy();
  });

  it(`Branch should not be null`, () => {
    expect(BranchCode.getAttribute("value")).toBeTruthy();
    expect(BranchName.getAttribute("value")).toBeTruthy();
  });

  it(`Should able to key order: ` + orderNo, () => {
    OrderNumber.sendKeys(orderNo);
    fn.selectEnterKey(); //Set order class ...
    expect(OrderNumber.getAttribute("value")).toBeTruthy();
  });

  it(`Order class should match with first digit of order number`, async () => {
    expect(OrderClass.getText()).toMatch(orderClassRegex);
  });

  it(`Payment term should not be null`, () => {
    expect(PaymentTerm.getAttribute("value")).toBeTruthy();
  });

  it(`Order date should not be null`, () => {
    expect(OrderDate.getAttribute("value")).toBeTruthy();
  });

  it(`Order type should match with relation of order class`, async () => {
    let orderType = (await OrderType.getText()).substr(0, 1);

    expect(
      orderTypeList.find((v) => {
        return (
          v.REF_ORDER_CLASS.indexOf(orderClass) >= 0 &&
          v.ORDER_TYPE == orderType
        );
      })
    ).toBeTruthy();
  });

  it(`B/O flag should not be null`, () => {
    expect(BOFlag.getText()).toBeTruthy();
  });

  it(`Route Code should not be null`, () => {
    expect(RouteCode.getAttribute("value")).toBeTruthy();
  });

  it(`Transport code should not be null`, () => {
    expect(TransportCode.getAttribute("value")).toBeTruthy();
  });

  it(`Deliver code should not be null`, () => {
    expect(DeliveryCode.getText()).toBeTruthy();
  });

  it(`Part locked should not able to enter`, async () => {
    var rows: ElementArrayFinder = page.getDetailRowsElement();
    var row: ElementFinder = rows.get(0);

    var partItem: ElementFinder = await row.element(
      by.css("td.mat-column-ORDER_PART_NO input[name=ORDER_PART_NO]")
    );

    partItem.sendKeys(partNoLocked);
    fn.selectEnterKey();
    browser.waitForAngular();

    expect(
      page
        .getMessageModalElement(".modal-warning")
        .element(by.css(".mat-dialog-title"))
        .getText()
    ).toMatch(/WARNING!/);
    expect(
      page
        .getMessageModalElement(".modal-warning")
        .element(by.css(".mat-dialog-content"))
        .getText()
    ).toMatch(/This Part No. is lock!/);

    page
      .getMessageModalElement(".modal-warning")
      .element(by.css(".mat-dialog-actions button"))
      .click();

    browser.waitForAngular();

    expect(partItem.getAttribute("value")).toBeFalsy();
  });

  it(`Part locked 117 should popup confirm message and able to cancel`, async () => {
    var rows: ElementArrayFinder = page.getDetailRowsElement();
    var row: ElementFinder = rows.get(0);

    var partItem: ElementFinder = await row.element(
      by.css("td.mat-column-ORDER_PART_NO input[name=ORDER_PART_NO]")
    );

    partItem.sendKeys(partNo117);
    fn.selectEnterKey();
    browser.waitForAngular();

    expect(
      page
        .getMessageModalElement(".modal-confirm")
        .element(by.css(".mat-dialog-title"))
        .getText()
    ).toMatch(/CONFIRMATION/);
    expect(
      page
        .getMessageModalElement(".modal-confirm")
        .element(by.css(".mat-dialog-content"))
        .getText()
    ).toMatch(/Are you sure to continue\?/);

    page
      .getMessageModalElement(".modal-confirm")
      .element(by.css(".mat-dialog-actions button[name=Cancel]"))
      .click();

    browser.waitForAngular();

    expect(partItem.getAttribute("value")).toBeFalsy();
  });

  it(`Part locked 117 should popup confirm message and able to enter`, async () => {
    var rows: ElementArrayFinder = page.getDetailRowsElement();
    var row: ElementFinder = rows.get(0);

    var partItem: ElementFinder = await row.element(
      by.css("td.mat-column-ORDER_PART_NO input[name=ORDER_PART_NO]")
    );
    var orderQtyItem: ElementFinder = await row.element(
      by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
    );

    partItem.sendKeys(partNo117);
    fn.selectEnterKey();
    browser.waitForAngular();

    expect(
      page
        .getMessageModalElement(".modal-confirm")
        .element(by.css(".mat-dialog-title"))
        .getText()
    ).toMatch(/CONFIRMATION/);
    expect(
      page
        .getMessageModalElement(".modal-confirm")
        .element(by.css(".mat-dialog-content"))
        .getText()
    ).toMatch(/Are you sure to continue\?/);

    page
      .getMessageModalElement(".modal-confirm")
      .element(by.css(".mat-dialog-actions button[name=Confirm]"))
      .click();

    browser.waitForAngular();

    expect(partItem.getAttribute("value")).toBeTruthy();
    expect(orderQtyItem.getAttribute("disabled")).not.toEqual("true");
  });

  it(`Order qty should able to enter`, async () => {
    var rows: ElementArrayFinder = page.getDetailRowsElement();

    var elem: ElementFinder = rows.get(0);
    var orderQtyItem: ElementFinder = await elem.element(
      by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
    );

    expect(orderQtyItem.getAttribute("disabled")).not.toEqual("true");
    await orderQtyItem.sendKeys(1);
    fn.selectEnterKey();
  });

  it(`Amount should not be null`, async () => {
    var rows: ElementArrayFinder = page.getDetailRowsElement();

    var elem: ElementFinder = rows.get(0);
    var orderQtyItem: ElementFinder = await elem.element(
      by.css("td.mat-column-ORDER_QTY input[name=ORDER_QTY]")
    );
    var unitPriceItem: ElementFinder = await elem.element(
      by.css("td.mat-column-UNIT_PRICE input[name=UNIT_PRICE]")
    );
    var amountItem: ElementFinder = await elem.element(
      by.css("td.mat-column-AMOUNT")
    );

    var amount: Number =
      fn.convertStringToNumber(await orderQtyItem.getAttribute("value")) *
      fn.convertStringToNumber(await unitPriceItem.getAttribute("value"));

    expect(fn.convertStringToNumber(await amountItem.getText())).not.toEqual(0);
    expect(fn.convertStringToNumber(await amountItem.getText())).toEqual(
      amount
    );
  });

  it(`Save button able to click`, () => {
    expect(SaveButton.getAttribute("disabled")).not.toEqual("true");
    SaveButton.click();
  });

  it(`Confirm modal appeared and confirmable`, () => {
    var confirmModal: ElementFinder = page.getMessageModalElement(
      ".modal-confirm"
    );
    var confirmButton: ElementFinder = confirmModal.element(
      by.css(".mat-dialog-actions button[name=Confirm]")
    );

    expect(confirmModal.element(by.css(".mat-dialog-title")).getText()).toMatch(
      /CONFIRMATION/
    );
    expect(
      confirmModal.element(by.css(".mat-dialog-content")).getText()
    ).toMatch(/Do you want to save changed you have made\?/);

    confirmButton.click();
    browser.waitForAngular();
  });

  it(`Save completed`, () => {
    var infoModal: ElementFinder = page.getMessageModalElement(".modal-info");
    expect(infoModal.element(by.css(".mat-dialog-content")).getText()).toMatch(
      /Do Number = null/
    );

    browser.driver.sleep(1000);
  });
});
