import {
  browser,
  by,
  element,
  ElementArrayFinder,
  ElementFinder,
  Key,
  logging,
} from "protractor";
import { GlobalFunctions } from "../global-functions";
import { SNPE015P_Page } from "./pre-received-WH.po";
import { SNPE015R_Page } from "./received-WH.po";



describe(`Received Local Parts By Invoice`, () => {
  let invoiceNo: string,
      vendorCode: string;

  describe(`SNPE015_P - Received Local Parts By Invoice (Pre-receive)`, () => {
    let VendorCode: ElementFinder,
      VendorName: ElementFinder,
      InvoiceNo: ElementFinder,
      InvoiceDate: ElementFinder,
      TotalSticker: ElementFinder,
      TotalQty: ElementFinder,
      InvoiceStatus: ElementFinder,
      AcceptPartsNo: ElementArrayFinder,
      SelectPO: ElementFinder,
      ReportButton: ElementFinder,
      SaveButton: ElementFinder,
      AcceptButton: ElementFinder;

    let setElements = (page: SNPE015P_Page) => {
      VendorCode = page.getVendorCodeElement();
      VendorName = page.getVendorNameElement();
      InvoiceNo = page.getInvoiceNoElement();
      InvoiceDate = page.getInvoiceDateElement();
      TotalSticker = page.getTotalStickerElement();
      TotalQty = page.getTotalQuantityElement();
      InvoiceStatus = page.getInvoiceStatusElement();
      AcceptPartsNo = page.getAllPartNoElement();
      SelectPO = page.getDialogModalElement();
      SaveButton = page.getSaveButtonElement();
      AcceptButton = page.getAcceptButtonElement();
    };

    let fn = new GlobalFunctions();
    let page: SNPE015P_Page = new SNPE015P_Page();
    let timeFormat = Math.floor(new Date().getTime() / 10000).toString(); //10s
    let partList: Array<any>;

    beforeAll(() => {
      page.navigateTo();

      setElements(page);

      vendorCode = "383";
      invoiceNo = "2PO" + timeFormat.substr(1, 8);

      /*SELECT vendor_code,accept_part_no FROM TPM_INDENT_STATUS
      WHERE COMPANY_CODE = '01'
      AND status_code IN ('B','N','A')
      AND VENDOR_TYPE = 'L';*/

      partList = [
        {
          ACCEPT_PART_NO: "622576KT1B",
          QTY: 1,
        },
        {
          ACCEPT_PART_NO: "738226KT0A",
          QTY: 1,
        },
        {
          ACCEPT_PART_NO: "908106KT0C",
          QTY: 1,
        },
      ];

    });

    beforeEach(() => {
      browser.driver.sleep(300);
    });

    it(`Should able to key Vendor Code`, async () => {
      VendorCode.sendKeys(vendorCode);
      fn.selectEnterKey();

      browser.waitForAngular();
      expect(VendorCode.getAttribute("value")).toBeTruthy();
    });

    it(`Vendor Name should not be null`, () => {
      expect(VendorName.getAttribute("value")).toBeTruthy();
    });

    it(`Should able to key Invoice No.`, () => {
      InvoiceNo.sendKeys(invoiceNo);
      fn.selectEnterKey(); //Set Invoice No...
      expect(InvoiceNo.getAttribute("value")).toBeTruthy();
    });

    it(`Invoice Date should not be null`, async () => {
      expect(InvoiceDate.getAttribute("value")).toBeTruthy();
    });

    it(`Accept Part No should able to enter`, async () => {
      var rows: ElementArrayFinder = page.getDetailRowsElement();

      await rows.each(async (elem, idx) => {
        var partItem: ElementFinder = await elem.element(by.css("td.mat-column-ACCEPT_PART_NO input[name=acceptpartno]"));

        if (idx < partList.length) {
          await partItem.sendKeys(partList[idx].ACCEPT_PART_NO);
          fn.selectEnterKey();
          await browser.waitForAngular();

          var error: ElementFinder = page.getMessageModalElement(".modal-error");
          var selectPO: ElementFinder = page.getDialogModalElement();

          if (error) {
            fn.selectEnterKey();
            browser.waitForAngular();
          }
          else if (selectPO) {
            it(`Select PO`, async () => {
              var rows: ElementArrayFinder = page.getSelectPOElement();
              rows.each(async (elem, idx) => {
                var CheckboxItem: ElementFinder = await elem.element(by.css("td.mat-column-CHECK mat-checkbox[name=CHECK_BOX]"));
                await CheckboxItem.click();
                await browser.waitForAngular();
              });
            });

            it(`Accept button able to click`, async () => {
              expect(AcceptButton.getAttribute('disabled')).not.toEqual("true");
              AcceptButton.click();
              browser.waitForAngular();
            });
          }
        }
      });
    });

    it(`Save button able to click`, () => {
      expect(SaveButton.getAttribute("disabled")).not.toEqual("true");
      SaveButton.click();
    });

    it(`Save completed`, () => {
      var infoModal: ElementFinder = page.getMessageModalElement(".modal-info");
      expect(infoModal.element(by.css(".mat-dialog-content")).getText()).toEqual('Transaction complete: applied and saved.');
      fn.selectEnterKey();

      browser.driver.sleep(1000);
    });

  })

  describe(`SNPE015_R - Received Local Parts By Invoice (Receive)`, () => {
    let DocumentDateFrom: ElementFinder,
      DocumentDateTo: ElementFinder,
      PrereceivingNo: ElementFinder,
      VendorCode: ElementFinder,
      InvoiceNo: ElementFinder,
      SearchButton: ElementFinder,
      ConfirmButton: ElementFinder; 

    let setElements = (page: SNPE015R_Page) => {
      DocumentDateFrom = page.getDocumentDateFromElement();
      DocumentDateTo = page.getDocumentDateToElement();
      PrereceivingNo = page.getPrereceivingNoElement();
      VendorCode = page.getVendorCodeElement();
      InvoiceNo = page.getInvoiceNoElement();
      SearchButton = page.SearchButtonElement();
      ConfirmButton = page.ConfirmButtonElement();
    };

    let fn = new GlobalFunctions();
    let page: SNPE015R_Page = new SNPE015R_Page();
    let reciveVendor: string,
        reciveInVoice: string,
        prereciveNo: string;

    reciveVendor = vendorCode;
    reciveInVoice = invoiceNo;
    prereciveNo = "";

    beforeAll(() => {
      page.navigateTo();

      setElements(page);

    });

    beforeEach(() => {
      browser.driver.sleep(300);
    });

    it(`Pre-invoice Date From should not be null`, async () => {
      expect(DocumentDateFrom.getAttribute("value")).toBeTruthy();
    });

    it(`Pre-invoice To From should not be null`, async () => {
      expect(DocumentDateTo.getAttribute("value")).toBeTruthy();
    });

    it(`Should able to key Pre-receiving No.`, async () => {
      PrereceivingNo.sendKeys(prereciveNo);
      fn.selectEnterKey();

      browser.waitForAngular();
    });

    it(`Should able to key Vendor Code`, async () => {
      VendorCode.sendKeys(reciveVendor);
      fn.selectEnterKey();
      browser.waitForAngular();
    });

    it(`Should able to key Invoice No.`, async () => {
      InvoiceNo.sendKeys(reciveInVoice);
      browser.waitForAngular();
    });

    it(`Search button able to click`, () => {
      expect(SearchButton.getAttribute("disabled")).not.toEqual("true");
      SearchButton.click();

      browser.waitForAngular();
    });

    it(`Should able to click edit order`, async () => {
      var EditBtn: ElementFinder = await element(by.css("td.mat-column-EDIT>button"));
      await EditBtn.click();
      browser.waitForAngular();
    });

    it(`Confirm Button ablr to cilck`, async () => {
      expect(ConfirmButton.getAttribute("disabled")).not.toEqual("true");
      await ConfirmButton.click();
      var errorModal: ElementFinder = page.getMessageModalElement(".modal-error");

      if (errorModal) {
        fn.selectEnterKey();
        browser.waitForAngular();
      }

    });

    it(`Confirm modal appeared and confirmable`, () => {
      var confirmModal: ElementFinder = page.getMessageModalElement(".modal-confirm");
      var confirmButton: ElementFinder = confirmModal.element(by.css(".mat-dialog-actions button[name=Confirm]"));

      expect(confirmModal.element(by.css(".mat-dialog-title")).getText()).toMatch(/CONFIRMATION/);
      expect(confirmModal.element(by.css(".mat-dialog-content")).getText()).toMatch(/Are you sure to confirm\?/);

      confirmButton.click();
      browser.waitForAngular();
    });

    it(`Confirm completed`, () => {
      var infoModal: ElementFinder = page.getMessageModalElement(".modal-info");
      expect(infoModal.element(by.css(".mat-dialog-content")).getText()).toEqual('The document has been confirmed to receive');
      fn.selectEnterKey();
    });
  });

});





