import {
  browser,
  by,
  element,
  ElementArrayFinder,
  ElementFinder,
  Key,
  logging,
} from "protractor";
import { GlobalFunctions } from "../global-functions";
import { SNPE015R_Page } from "./received-WH.po";


describe(`SNPE015_R - Received Local Parts By Invoice (Receive)`, () => {
  let DocumentDateFrom: ElementFinder,
    DocumentDateTo: ElementFinder,
    PrereceivingNo: ElementFinder,
    VendorCode: ElementFinder,
    InvoiceNo: ElementFinder,
    SearchButton: ElementFinder,
    ConfirmButton: ElementFinder;

  let setElements = (page: SNPE015R_Page) => {
    DocumentDateFrom = page.getDocumentDateFromElement();
    DocumentDateTo = page.getDocumentDateToElement();
    PrereceivingNo = page.getPrereceivingNoElement();
    VendorCode = page.getVendorCodeElement();
    InvoiceNo = page.getInvoiceNoElement();
    SearchButton = page.SearchButtonElement();
    ConfirmButton = page.ConfirmButtonElement();
  };

  let fn = new GlobalFunctions();
  let page: SNPE015R_Page = new SNPE015R_Page();
  let vendorCode: string;
  let invoiceNo: string;
  let prereceivingNo: string
  let documentDateFrom: any;

  beforeAll(() => {
    page.navigateTo();

    setElements(page);

    vendorCode = ""; 
    prereceivingNo = ""; //"PRC-21-0000039"; 
    invoiceNo = ""; //"2PO61240706"
  });

  beforeEach(() => {
    browser.driver.sleep(300);
  });

  it(`Pre-invoice Date From should not be null`, async () => {
    expect(DocumentDateFrom.getAttribute("value")).toBeTruthy();
  });

  it(`Pre-invoice To From should not be null`, async () => {
    expect(DocumentDateTo.getAttribute("value")).toBeTruthy();
  });

  it(`Should able to key Pre-receiving No.`, async () => {
    PrereceivingNo.sendKeys(prereceivingNo);
    fn.selectEnterKey();

    browser.waitForAngular();
  });

  it(`Should able to key Vendor Code`, async () => {
    VendorCode.sendKeys(vendorCode);
    fn.selectEnterKey();
    browser.waitForAngular();
  });

  it(`Should able to key Invoice No.`, async () => {
    InvoiceNo.sendKeys(invoiceNo);
    browser.waitForAngular();
  });

  it(`Search button able to click`, () => {
    expect(SearchButton.getAttribute("disabled")).not.toEqual("true");
    SearchButton.click();

    browser.waitForAngular();
  });

  it(`Should able to click edit order`, async () => {
    var EditBtn: ElementFinder = await element(by.css("td.mat-column-EDIT>button"));
    await EditBtn.click();
    browser.waitForAngular();
  });

  it(`Confirm Button able to cilck`, async () => {
    expect(ConfirmButton.getAttribute("disabled")).not.toEqual("true");
    await ConfirmButton.click();
    var errorModal: ElementFinder = page.getMessageModalElement(".modal-error");

    if (errorModal) {
      fn.selectEnterKey();
      browser.waitForAngular();
    }
  });

  it(`Confirm modal appeared and confirmable`, () => {
    var confirmModal: ElementFinder = page.getMessageModalElement(".modal-confirm");
    var confirmButton: ElementFinder = confirmModal.element(by.css(".mat-dialog-actions button[name=Confirm]"));

    expect(confirmModal.element(by.css(".mat-dialog-title")).getText()).toMatch(/CONFIRMATION/);
    expect(confirmModal.element(by.css(".mat-dialog-content")).getText()).toMatch(/Are you sure to confirm\?/);

    confirmButton.click();
    browser.waitForAngular();
  });

  it(`Confirm completed`, () => {
    var infoModal: ElementFinder = page.getMessageModalElement(".modal-info");
    expect(infoModal.element(by.css(".mat-dialog-content")).getText()).toEqual('The document has been confirmed to receive');
    fn.selectEnterKey();
  });



});
