import {
  Component,
  ContentChild,
  TemplateRef,
  ViewChild,
  Inject,
  Input,
  Output,
  EventEmitter,
} from "@angular/core";
import {
  MatDialog,
  MatDialogRef,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { Overlay } from "@angular/cdk/overlay";

@Component({
  selector: "dialog-template",
  template:
    '<button mat-icon-button (click)="OpenDialog()"><i class="material-icons">{{iconName}}</i></button>',
})
export class DialogTemplateComponent {
  @Input() iconName: string = "edit";
  @Input() title: string = "Dialog";
  @Input() width: string = "75%";
  @Input() minWidth: string = "auto";
  @Input() maxWidth: string = "auto";
  @Input() height: string = "100%";
  @Input() minHeight: string = "auto";
  @Input() maxHeight: string = "auto";
  @Input() colors: string = "#000000";
  @Output() onClose: EventEmitter<any> = new EventEmitter();

  @ContentChild("content", { static: false }) templateRefContent;
  @ContentChild("action", { static: false }) templateRefAction;

  public isOpen: boolean = false;

  constructor(private dialog: MatDialog, private overlay: Overlay) {}
  private dialogRef: any;

  ngOnDestroy() {
    setTimeout(() => {
      this.dialogRef ? this.dialogRef.close() : null;
    }, 100);
  }

  OpenDialog() {
    this.dialogRef = this.dialog.open(DialogTemplateDialog, {
      panelClass: "dialog-panel",
      autoFocus: true,
      width: this.width,
      height: this.height,
      minHeight: this.minHeight,
      maxHeight: this.maxHeight,
      minWidth: this.minWidth,
      maxWidth: this.maxWidth,
      data: {
        title: this.title,
        colors: this.colors,
        templateRefContent: this.templateRefContent,
        templateRefAction: this.templateRefAction,
        onClose: this.onClose,
      },
    });

    this.isOpen = true;

    this.dialogRef.afterClosed().subscribe((result) => {
      this.isOpen = false;
    });
  }
  Close() {
    this.dialogRef.close();
  }
}

@Component({
  selector: "dialog-template-dialog",
  templateUrl: "./dialog-template.component.html",
  styleUrls: ["./dialog-template.component.css"],
})
export class DialogTemplateDialog {
  public templateRefContent: TemplateRef<any>;
  public templateRefAction: TemplateRef<any>;
  public title: string;
  public colors: string;
  public onClose: EventEmitter<any> = new EventEmitter();

  constructor(
    public dialogRef: MatDialogRef<DialogTemplateDialog>,
    @Inject(MAT_DIALOG_DATA) public dialogData: any
  ) {}

  ngOnInit() {
    this.colors = this.dialogData.colors;
    this.title = this.dialogData.title;
    this.templateRefContent = this.dialogData.templateRefContent;
    this.templateRefAction = this.dialogData.templateRefAction;
    this.onClose = this.dialogData.onClose;
  }

  OnCloseClick() {
    if (this.onClose.observers.length > 0) {
      this.onClose.emit("closed");
    } else {
      this.dialogRef.close();
    }
  }
}
