import {
  Component,
  EventEmitter,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
} from "@angular/core";
import { FormControl } from "@angular/forms";
import {
  MomentDateAdapter,
  MAT_MOMENT_DATE_ADAPTER_OPTIONS,
} from "@angular/material-moment-adapter";
import {
  DateAdapter,
  MAT_DATE_LOCALE,
  MAT_DATE_FORMATS,
} from "@angular/material/core";
import { MatDatepicker } from "@angular/material/datepicker";
import * as moment from "moment";

export const MY_FORMATS = {
  parse: {
    dateInput: ["MM/YYYY", "MMYYYY", "MMYY"],
  },
  display: {
    dateInput: "MMMM YYYY",
    monthYearLabel: "MMM YYYY",
    dateA11yLabel: "LL",
    monthYearA11yLabel: "MMMM YYYY",
  },
};

@Component({
  selector: "month-picker",
  templateUrl: "month-picker.component.html",
  providers: [
    {
      provide: DateAdapter,
      useClass: MomentDateAdapter,
      deps: [MAT_DATE_LOCALE, MAT_MOMENT_DATE_ADAPTER_OPTIONS],
    },

    { provide: MAT_DATE_FORMATS, useValue: MY_FORMATS },
  ],
})
export class MonthPickerComponent implements OnInit {
  constructor() {}
  _date = new FormControl(moment());
  @Input() date: moment.Moment;
  @Output() dataChange: EventEmitter<moment.Moment> =
    new EventEmitter<moment.Moment>();

  ngOnInit() {}

  chosenYearHandler(normalizedYear: moment.Moment) {
    const ctrlValue = this._date.value;
    ctrlValue.year(normalizedYear.year());
    this.setDataValue(ctrlValue);
  }

  chosenMonthHandler(
    normalizedMonth: moment.Moment,
    datepicker: MatDatepicker<moment.Moment>
  ) {
    const ctrlValue = this._date.value;
    ctrlValue.month(normalizedMonth.month());
    this.setDataValue(ctrlValue);
    datepicker.close();
  }

  private setDataValue(ctrlValue) {
    this._date.setValue(ctrlValue);
    this.dataChange.emit(ctrlValue);
  }
}
