import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve, Router } from '@angular/router';
import { HttpService } from './http.service';
import { catchError, map } from 'rxjs/operators';
import { of, Observable } from 'rxjs';
import { HttpErrorResponse } from '@angular/common/http';

export interface FormRole {
    INS: string;
    CHG: string;
    DEL: string;
    INQ: string;
}

@Injectable()
export class RoleResolver implements Resolve<FormRole> {
    constructor(
        private http: HttpService,
        private router: Router
    ) { }

    resolve(snapshot: ActivatedRouteSnapshot): Observable<FormRole> {
        let formId = snapshot.url[0].path;
        let roleDetail: FormRole = {
            INS: "N",
            CHG: "N",
            DEL: "N",
            INQ: "N"
        }

        return this.http.GetFormRole(formId)
            .pipe(
                map((response: FormRole) => {
                    return response
                }),
                catchError((error: HttpErrorResponse) => {
                    if(error.status === 400) {
                        this.router.navigate(['/404']);
                    }

                    return of(roleDetail);
                })
            )
    }
}
