import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';

import { SharedComponentModule } from '../shared/shared-component.module';
import { AuthGuard } from 'src/app/core/auth-guard.service';

import { MaterialImportModule } from '../shared/material.module';
import { RoleResolver } from 'src/app/core/role-resolver.service';
import { K9Component } from 'src/app/pages/k9/k9.component';
import { SNM117PComponent } from 'src/app/pages/k9/SNM117P/SNM117P.component';
import { SNM118PComponent } from 'src/app/pages/k9/SNM118P/SNM118P.component';
import { SNM119PComponent } from 'src/app/pages/k9/SNM119P/SNM119P.component';
import { SNM120PComponent } from 'src/app/pages/k9/SNM120P/SNM120P.component';

@NgModule({
    declarations: [
        K9Component,
        SNM117PComponent,
        SNM118PComponent,
        SNM119PComponent,
        SNM120PComponent
    ],
    imports: [
        CommonModule,
        HttpClientModule,
        FormsModule,
        ReactiveFormsModule,
        SharedComponentModule,

        //Material imports
        MaterialImportModule,


        RouterModule.forChild([
            {
                path: '',
                component: K9Component,
                children: [
                    {
                        path: 'SNM117P',
                        component: SNM117PComponent,
                        resolve: {
                            role: RoleResolver
                        },
                    },
                    {
                        path: 'SNM118P',
                        component: SNM118PComponent,
                        resolve: {
                            role: RoleResolver
                        },
                    },
                    {
                        path: 'SNM119P',
                        component: SNM119PComponent,
                        resolve: {
                            role: RoleResolver
                        }
                    },
                    {
                        path: 'SNM120P',
                        component: SNM120PComponent,
                        resolve: {
                            role: RoleResolver
                        }
                    }
                ],
                canActivate: [AuthGuard]
            }
        ])
    ],
    exports: [],
    providers: [
        AuthGuard,
        RoleResolver
    ],
    entryComponents: []
})
export class K9Module { }


