import { NgModule } from "@angular/core";
import { CommonModule } from "@angular/common";
import { HttpClientModule } from "@angular/common/http";
import { ReactiveFormsModule, FormsModule } from "@angular/forms";
import { RouterModule } from "@angular/router";

import { SharedComponentModule } from "../shared/shared-component.module";
import { AuthGuard } from "src/app/core/auth-guard.service";

import { PurChaseComponent } from "src/app/pages/purchase/purchase.component";
import { MaterialImportModule } from "../shared/material.module";
import { SNSE140Component } from "src/app/pages/purchase/SNSE140/SNSE140.component";
import { RoleResolver } from "src/app/core/role-resolver.service";
import { PurchaseReportComponent } from "src/app/pages/purchase/PurchaseReport/PurchaseReport.component";

@NgModule({
  declarations: [
    PurChaseComponent, 
    SNSE140Component,
    PurchaseReportComponent
  ],
  imports: [
    CommonModule,
    HttpClientModule,
    FormsModule,
    ReactiveFormsModule,
    SharedComponentModule,
    FormsModule,

    //Material imports
    MaterialImportModule,

    RouterModule.forChild([
      {
        path: "",
        component: PurChaseComponent,
        children: [
          {
            path: "SNSE140",
            component: SNSE140Component,
            resolve: {
              role: RoleResolver,
            },
          },
          {
            path: "PURCHASEREF",
            component: PurchaseReportComponent,
            resolve: {
              role: RoleResolver,
            },
          },
        ],
        canActivate: [AuthGuard],
      },
    ]),
  ],
  exports: [],
  providers: [AuthGuard, RoleResolver],
})
export class PurChaseModule {}
