import { Component, ElementRef, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { ModalService } from '../../../core/modal.service';
import { NgForm } from '@angular/forms';
import { Globals } from 'src/app/core/global';
import * as moment from 'moment';
import { SelectionModel } from '@angular/cdk/collections';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';

@Component({
  selector: 'SNIE003',
  templateUrl: './SNIE003.component.html',
  styleUrls: ['./SNIE003.component.css']
})

export class SNIE003Component implements OnInit {

  constructor(
    private http: HttpService,
    private auth: AuthDecode,
    private viewref: ViewContainerRef,
    private modal: ModalService,
    private router: Router,
    private global: Globals
  ) { }

  public ConfigDocS: any = {
    ColName: "DOCUMENT_NO",
    PlaceholderText: "Select Document No",
    SourceController: "AC",
    SourceMethodName: "SNIE003_GetDocNo",
    NoOfShowRecords: 30,
    MinLengthSearch: 5,
    UpperCase: false
  }
 

  public ConfigReason: any = {
    ColName: "LOOKUP_REASON",
    PlaceholderText: "Select Reason Code",
    SourceController: "AC",
    SourceMethodName: "SNIE003_GetReason",
    NoOfShowRecords: 30,
    MinLengthSearch: 0,
    UpperCase: false
  }

  public ConfigPart: any = {
    ColName: "LOOKUP_PART",
    PlaceholderText: "Select Part",
    SourceController: "AC",
    SourceMethodName: "SNIE003_GetPart",
    NoOfShowRecords: 30,
    MinLengthSearch: 3
  }

  public tblFooterWorkFile: any = [
    {
      fieldName: "PART_NAME",
      value: "Total",
      show: true
    },
    {
      fieldName: "QUANTITY",
      value: null,
      show: true
    },
    {
      fieldName: "AMOUNT",
      value: null,
      show: true
    }]

  public params: any = {
    DOCUMENT_DATE: moment()
  }
  public data: any = {};
  public ParamsWorkfile : any = {}

  public tableData: any = [];
  public MainDatatable: any;
  public sDocData: any;
  public sReasonData: any;
  public sBranchData: any;
  public sPartData: any;
  public IsSearch: any = true;
  public confirmHeader: boolean = false;
  public tableWorkFile = []
  public DisableSave : boolean = false
  public selection = new SelectionModel<any>(true, []);
  private tempDocSPTF = []
  public tableSPTF = []
  public tableKitReceiveHis = []
  public iSfirstSave : boolean = true;
  public ParamsDetailData : any = {}
  public tableWorkFileComplete = []

  @ViewChild("autoCompleteReasonAdd") autoCompleteReasonAdd: any;
  @ViewChild("Autoreason", { static: false }) Autoreason: AutoCompleteQueryComponent;
  @ViewChild("inputDocType") inputDocType: ElementRef;
  @ViewChild("dialogPart", { static: false }) dialogPart: any;
  @ViewChild("dialogSPTF", { static: false }) dialogSPTF: any; 
  @ViewChild("dialogKitHis", { static: false }) dialogKitHis: any; 

  ngOnInit() {
    
    
  }

  // ngAfterViewInit(): void {
  //   setTimeout(() => {
  //     this.inputDocType.nativeElement.select();
  //     this.inputDocType.nativeElement.focus();
  //   }, 100);
  // }


  public Search() {
    this.params._DOCUMENT_DATE = this.global.ConvertDateToFormatSTD(this.params.DOCUMENT_DATE);

    this.http.post("AC", "SNIE003_GetList", this.params)
      .subscribe(result => {
        this.MainDatatable = result.Table;
      });
  }


  public View(P_data) {
    this.confirmHeader = true;
    this.IsSearch = false

    this.http.post("AC","SNIE003_GetHeaderAndDetail",P_data)
    .subscribe(res =>{
      this.data = res.Table[0]
      this.tableData = res.Table1
    })
  }

  public AddNew() {

    this.data = {};
    this.tableSPTF = []
    this.tableWorkFile = []
    this.tableWorkFileComplete = []
    this.http.get("AC","SNPE033_GetDefaultValue")
    .subscribe(res =>{
      
      this.data = res.Table[0]
      //this.data.DOCUMENT_DATE = moment();
      this.data.DOCUMENT_NO = "AUTO";
      //this.data.VAT_RATE = "0.00";
      this.tableData = [];
      this.IsSearch = false;
      this.confirmHeader = false;
      this.tableData = [...this.tableData, { 'EDIT': 'DEL', 'ITEM_NO': 1,KIT_FLAG : 'Y'    }
                                         ,{ 'EDIT': 'DEL', 'ITEM_NO': 2 ,KIT_FLAG : 'Y' }
                                         ,{ 'EDIT': 'DEL', 'ITEM_NO': 3 ,KIT_FLAG : 'Y' }
                                         ,{ 'EDIT': 'DEL', 'ITEM_NO': 4 ,KIT_FLAG : 'Y' }
                                         ,{ 'EDIT': 'ADD', 'ITEM_NO': 5 ,KIT_FLAG : 'Y' }];
    })

  

    // setTimeout(() => {
    //   this.inputDocType.nativeElement.select();
    //   this.inputDocType.nativeElement.focus();
    // }, 100);
  }
  

  public GetDocType(Doc) {
    this.params.DOCUMENT_TYPE = Doc.DOCUMENT_TYPE;
  }

  public GetDocTypeAdd(Doc) {
    if (Doc) {
      this.data.DOCUMENT_TYPE = Doc.DOCUMENT_TYPE;
      this.data.DOC_TYPE_DESC = Doc.DOC_NAME;
      this.data.JOB_TYPE = Doc.JOB_TYPE;
      this.data.VAT_FLAG = Doc.VAT_FLAG;
      this.data.VAT_RATE = Doc.VAT_RATE;
    }
  }

  public GetReason(Rea) {
    this.params.REASON_CODE = Rea.REASON_CODE;
  }

  public GetReasonAdd(Rea) {
    this.data.REASON_CODE = Rea.REASON_CODE;
    this.data.DSP_REASON = Rea.REASON_DESC;
  }


  public GetPart(evt, data, index) {
    
    if (evt.PART_NO) {
      if (this.tableData.find((v, i) => i !== index && v.PART_NO === evt.PART_NO)) {
        this.modal.warning("Duplicate Part number!", this.viewref)
        data.PART_NO = null;
      }
      else {
        
        data.PART_NO = evt.PART_NO;
        data.PART_NAME = evt.PART_NAME_EN;
        data.UNIT_PRICE = evt.UNIT_PRICE;
        data.TOTAL_VAT = evt.TOTAL_VAT;
        data.OH_QTY = evt.OH_QTY;
        data.TRANSFER_QTY = evt.TRANSFER_QTY;
        data.SUSPEND_QTY = evt.SUSPEND_QTY;
        data.KIT = 'Click'
      }
    } else {
        
      data.PART_NO = evt.PART_NO;
      data.PART_NAME = evt.PART_NAME_EN;
      data.UNIT_PRICE = evt.UNIT_PRICE;
      data.TOTAL_VAT = evt.TOTAL_VAT;
      data.OH_QTY = evt.OH_QTY;
      data.TRANSFER_QTY = evt.TRANSFER_QTY;
      data.SUSPEND_QTY = evt.SUSPEND_QTY;
      data.KIT = null
    }
  }


  public SetKit(data){

    this.ParamsDetailData = data

    if(!this.confirmHeader){
      this.ParamsWorkfile.TMP_DOC_NO = "New MA";
      this.ParamsWorkfile.TMP_DOC_DATE = new Date();

      const filter = this.tableWorkFileComplete.filter(x => x.PA_PART_NO == this.ParamsDetailData.PART_NO)

      if(filter.length > 0){
        this.tableWorkFile = filter
        this.ParamsWorkfile.REASON_CODE = this.tableWorkFile[0].REASON_CODE
      }else{
        this.ParamsWorkfile.REASON_CODE = '409'
        setTimeout(() => {
          this.Autoreason.textInput = '409'
          this.Autoreason.query();
        }, 100);
      
        this.tableWorkFile = [];
        this.tableWorkFile = [...this.tableWorkFile, { 'EDIT': 'ADD', 'ITEM_NO': '' }];
      }

      this.AppendRowKit();
      this.dialogPart.OpenDialog();

    }else{
      //console.log('xxx',this.ParamsDetailData)
      this.http.post("AC", "SNIE003_GetKit", this.ParamsDetailData)
      .subscribe(res => {

    
        if (res.Table.length > 0) {
          this.tableWorkFile = res.Table;
          // for (var i = 0; i < this.tableWorkFile.length; i++) {
          //   this.tableWorkFile[i].EDIT = 'DEL';
          // }
          this.ParamsWorkfile.TMP_DOC_NO = this.ParamsDetailData.SUB_DOC_NO;
          this.ParamsWorkfile.TMP_DOC_DATE = this.tableWorkFile[0]?.DOCUMENT_DATE;
          this.ParamsWorkfile.REASON_CODE = this.tableWorkFile[0].REASON_CODE
          this.ParamsWorkfile.REASON_NAME_EN = this.tableWorkFile[0].REASON_NAME_EN
          this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
        }
        else {
          this.ParamsWorkfile.TMP_DOC_NO = this.ParamsDetailData.SUB_DOC_NO;
          this.ParamsWorkfile.TMP_DOC_DATE = this.tableWorkFile[0]?.DOCUMENT_DATE;
          this.tableWorkFile = [];
        }
      })

    
    this.dialogPart.title = "Part Kit";
    this.dialogPart.OpenDialog();
    }
    
  }



  public ShowReverseDetail(kitdata){

    let param = {
      PA_NO : this.data.DOCUMENT_NO,
      PA_ITEM_NO : this.ParamsDetailData.ITEM_NO,
      ACCEPT_PART_NO : kitdata.PART_NO
    }

    this.http.post("AC","SNIE003_GetPaKitHis",param)
    .subscribe(res => {
      this.tableKitReceiveHis = res.Table
      this.dialogKitHis.OpenDialog();
    })
  }

  public GetPartKit(evt, data, index) {

    if(evt.PART_NO) {
      var found = this.tableWorkFile.find((element, i) => index != i && element.PART_NO == evt.PART_NO);
      if (found !== undefined) {
        this.modal.warning("P# " + evt.PART_NO + " is duplicated!", this.viewref)
        this.tableWorkFile[index].PART_NO = null;
        this.tableWorkFile[index].PART_NAME = null;
        this.tableWorkFile[index].QUANTITY = null;
        this.tableWorkFile[index].OH_QTY = null;
        this.tableWorkFile[index].UNIT_PRICE = null;
        this.tableWorkFile[index].AMOUNT = null;
        return;
      }
  
      this.http.post("AC","SNIE003_Find_SP_TF",evt)
      .subscribe(res =>{
        
        this.tableSPTF = res.Table;
  
        if(this.tableSPTF.length > 0){
            this.dialogSPTF.OpenDialog();
        }
  
        //this.paramStock.SUSPEND_QTY = evt.SUSPEND_QTY
        //this.paramStock.TRANSFER_QTY = evt.TRANSFER_QTY
    
        this.tableWorkFile[index].PA_PART_NO = this.ParamsDetailData.PART_NO;
        this.tableWorkFile[index].PART_NO = evt.PART_NO;
        this.tableWorkFile[index].PART_NAME = evt.PART_NAME_EN;
        this.tableWorkFile[index].QUANTITY = evt.QUANTITY;
        this.tableWorkFile[index].TEMP_OH_QTY = evt.OH_QTY;
        this.tableWorkFile[index].OH_QTY = evt.OH_QTY;
        this.tableWorkFile[index].UNIT_PRICE = evt.UNIT_PRICE;
        this.tableWorkFile[index].AMOUNT = evt.AMOUNT;
        this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
        this.tableWorkFile = [...this.tableWorkFile];
  
        setTimeout(() => {
          this.AppendRow();
        }, 100);


      })
    }else{
        this.tableWorkFile[index].PART_NO = null;
        this.tableWorkFile[index].PART_NAME = null;
        this.tableWorkFile[index].QUANTITY = null;
        this.tableWorkFile[index].OH_QTY = null;
        this.tableWorkFile[index].UNIT_PRICE = null;
        this.tableWorkFile[index].AMOUNT = null;
        this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
        this.tableWorkFile = [...this.tableWorkFile];
    }
    
  }

  public keyupCalQ(evt, data) {
    if (data.QUANTITY > data.OH_QTY) {
      data.QUANTITY = data.OH_QTY;
      this.tableWorkFile = [...this.tableWorkFile];
      this.modal.warning('Item Qty cannot more than balance', this.viewref,
        () => {
          evt.target.focus();
        })
    }
    data.AMOUNT = (parseFloat(data.QUANTITY) * parseFloat(data.UNIT_PRICE)) || 0;
    this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
    this.tableWorkFile = [...this.tableWorkFile];
  }

  public ReverseQtyChange(event,data){
    
    if(event > data.NET_QTY){
      data.REVERSE_QTY = data.NET_QTY
    }
    
  }

  public AcceptSelectedSPTF(){


    if(this.selection.selected.length > 0){

      let find = this.selection.selected.some(x => x.REVERSE_QTY <= 0 || x.REVERSE_QTY == null)
      if(find){
        this.modal.warning("Revers qty must be more than 0",this.viewref)
        return;
      }

      this.selection.selected.map(x => x.PA_PART_NO = this.ParamsDetailData.PART_NO)

      let part_no = this.selection.selected[0].ACCEPT_PART_NO
      let total_revers = this.selection.selected.reduce((prev, next) => prev + (next.REVERSE_QTY || 0),0);
      //let new_unit_price = this.selection.selected.reduce((prev, next) => prev + ((next.REVERSE_QTY * next.UNIT_PRICE) || 0),0);
      let index = this.tableWorkFile.findIndex(x => x.PART_NO == part_no)

      this.tableWorkFile[index].OH_QTY = this.tableWorkFile[index].TEMP_OH_QTY + total_revers
      this.tableWorkFile[index].QUANTITY = total_revers
      //this.tableWorkFile[index].UNIT_PRICE = new_unit_price
      this.tableWorkFile[index].AMOUNT = this.global.Round(this.tableWorkFile[index].UNIT_PRICE * total_revers,2)
      this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
      this.tableWorkFile = [...this.tableWorkFile];

      this.tempDocSPTF = this.tempDocSPTF.filter(x => x.ACCEPT_PART_NO !== part_no)

      Array.prototype.push.apply(this.tempDocSPTF,this.selection.selected); 
      this.AppendRowKit()
      this.selection.clear()
      this.dialogSPTF.Close();
    }else{
      this.modal.warning("Please select data",this.viewref);
    }

  }

  public AcceptKitFlag(){

      let find = this.tableWorkFile.some(x => x.PART_NO && x.QUANTITY < 1)
      if(find){
        this.modal.warning("Quantity can not less than 1",this.viewref)
        return
      }
    this.tableWorkFile = this.tableWorkFile.filter(x => x.PART_NO);
    this.tableWorkFile.map(x => x.REASON_CODE = this.ParamsWorkfile.REASON_CODE)
    this.tableWorkFileComplete = this.tableWorkFileComplete.filter((x) =>  x.PA_PART_NO !== this.ParamsDetailData.PART_NO);
    Array.prototype.push.apply(this.tableWorkFileComplete,this.tableWorkFile); 
    

    // let new_unit_price = this.tableWorkFileComplete.filter((x) =>  x.PA_PART_NO == this.ParamsDetailData.PART_NO).reduce((prev, next) => prev + ((next.QUANTITY * next.UNIT_PRICE) || 0),0);
    
    // if(new_unit_price > 0){
     
    //   let index = this.tableData.findIndex(x => x.PART_NO == this.ParamsDetailData.PART_NO)
    //   this.tableData[index].UNIT_PRICE = this.global.Round(new_unit_price,2);
    //   this.tableData = this.tableData.slice();
    // }
    
    //this.AppendRowKit()
    this.dialogPart.Close();

  }

  public Save() {

    if(this.data.REASON_CODE == null){
      this.modal.warning("Please input reason code",this.viewref)
      return
    }

    this.modal.confirm("Do you want to save change you have made?",this.viewref,this.ConfirmSave)
  }

  ConfirmSave = () => {

    
    let DetailData = this.tableData.filter(x => x.PART_NO)
    let KitData = this.tableWorkFileComplete.filter(x => x.PART_NO && x.QUANTITY > 0)
    let find = DetailData.some(x => !x.QUANTITY || x.QUANTITY < 1)
    if(find){
      this.modal.warning("Quantity can not less than 1",this.viewref)
      return
    }

    DetailData.map((x) =>{
      let checkkit = KitData.some(v => v.PA_PART_NO == x.PART_NO)
      if(checkkit){
        x.KIT_FLAG = 'Y'
      }else{
        x.KIT_FLAG = 'N'
      }
    })

    this.data.DetailData = DetailData
    this.data.KitData = KitData
    this.data.SPTF_Data = this.tempDocSPTF

    this.http.post("AC","SNIE003_Confirm",this.data)
    .subscribe(res =>{
      this.data.DOCUMENT_NO = res.DOCUMENT_NO
      this.confirmHeader = true
      this.modal.info('Transaction complete: applied and saved.', this.viewref);
      console.log(res)
      this.View(this.data);
    },err => {
      this.modal.error(err.message, this.viewref,() =>{},{width :'700px',height : '320px'})
    })
   
  }

  public Back(){
    this.dialogPart.Close();
  }

  

  public AppendRow() {

    if(this.confirmHeader) return;

    for (var i = 0; i < this.tableData.length; i++) {
      if (this.tableData[i].EDIT === "ADD") {
        this.tableData[i].EDIT = 'DEL';
      }
    }
    this.tableData.push({ 'EDIT': 'ADD', 
        ITEM_NO : '' ,
        KIT_FLAG : 'Y'                 
    });

    this.tableData.map((x,i)=> {
      x.ITEM_NO = i + 1
    })

    this.tableData = this.tableData.slice();
  }

  public DeleteRow(index,data) {

    if(this.confirmHeader) return;

    let indexx = this.tableData.findIndex(x => x == data)
    this.tableData.splice(indexx, 1);
    this.tableData.map((x,i)=> {
      x.ITEM_NO = i + 1
    })
    this.tableData = this.tableData.slice();

  }

  public AppendRowKit() {
    if(this.confirmHeader) return;

    for (var i = 0; i < this.tableWorkFile.length; i++) {
      if (this.tableWorkFile[i].EDIT === "ADD") {
        this.tableWorkFile[i].EDIT = 'DEL';
        this.tableWorkFile[i].ITEM_NO = this.tableWorkFile.length;
      }
    }
    this.tableWorkFile = [...this.tableWorkFile, { 'EDIT': 'ADD', 'ITEM_NO': '' }];
  }

  public DeleteRowKit(Item) {

    if(this.confirmHeader) return;

    if(this.tableWorkFile[Item].PART_NO){
      this.tableWorkFile[Item].FLAG = 'DEL';
    }

    this.tableWorkFile.splice(Item, 1);
    this.tblFooterWorkFile[2].value = this.tableWorkFile.reduce((prev, next) => prev + (next.AMOUNT || 0),0);
    this.tableWorkFile = [...this.tableWorkFile];
  }

 

  public isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.tableSPTF.length;
    return numSelected === numRows;
  }

  public Toggle() {
    this.isAllSelected() ?
      this.selection.clear() :
      this.tableSPTF.forEach(row => this.selection.select(row));
  }



  getDateParam(dt) {
    if (dt !== undefined) {
      dt = new Date(dt)
      let year = dt.getFullYear();
      let month = (1 + dt.getMonth()).toString().padStart(2, '0');
      let day = dt.getDate().toString().padStart(2, '0');
      var p_Date = year + '-' + month + '-' + day;
      return p_Date;
    }
  }

  public Cancel() {
    this.IsSearch = true;
  }
  public BackToGrid() {
    //location.reload();
    this.IsSearch = true;
  }

  public CalTotalQTY() {
    return this.tableData.reduce((prev, currObj) => prev + parseFloat((currObj.QUANTITY || 0)), 0);
  }

  public CalTotalAmount() {
    return this.tableData.reduce((prev, currObj) => prev + (parseFloat((currObj.QUANTITY || 0)) * parseFloat((currObj.UNIT_PRICE || 0))), 0);
  }

  public CalTotalVat() {
    return this.tableData.reduce((prev, currObj) => prev + (parseFloat((currObj.QUANTITY || 0)) * (parseFloat((currObj.UNIT_PRICE || 0)) * this.data.VAT_RATE / 100)), 0);
  }

}

