import { Component, Inject, OnInit, ViewContainerRef } from "@angular/core";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";

interface DialogData {
  DOCUMENT_TYPE: string;
  DOCUMENT_NO: string;
  INVOICE_NO: string;
  INVOICE_DATE: Date;
  DETAIL: any[];
  TABLE_FOOTER: any[];
}

@Component({
  selector: "upload-modal",
  templateUrl: "upload-modal.component.html",
})
export class SNPE024UploadModalComponent implements OnInit {
  constructor(
    public dialogRef: MatDialogRef<SNPE024UploadModalComponent>,
    @Inject(MAT_DIALOG_DATA) public data: DialogData,
    private global: Globals,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private http: HttpService
  ) {}

  public fileUploadConfig = {
    PlaceholderText: "Choose file",
    AcceptFileType: ".xlsx",
    MultipleSelect: false,
  };

  private files: File[] = [];
  public PartListData: any[] = [];
  public tblFooterDetail35: any[] = [];

  ngOnInit(): void {
    this.PartListData = this.data.DETAIL;
    this.tblFooterDetail35 = this.data.TABLE_FOOTER;
  }

  public DownloadTemplate(): void {
    this.global.GoToExportName(
      "PMS_RPT_ACC_SNPE024_UPLOAD_PRICE",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_DOCUMENT_TYPE: this.data.DOCUMENT_TYPE,
        P_DOCUMENT_NO: this.data.DOCUMENT_NO,
      },
      "Upload_Template".concat("_", this.data.DOCUMENT_NO),
      "EXCELOPENXML"
    );
  }

  public UploadFile(event): void {
    // console.log(event);
    this.files = event;

    if (this.files.length == 0) {
      this.modal.warning("Please Open xlsx File First...", this.viewRef);

      return;
    }

    let formData = new FormData();
    formData.append("file", this.files[0]);
    this.PartListData = [];
    this.http.postFile("AC", "SNPE024_M_Upload", formData).subscribe((res) => {
      this.PartListData = this.MappingUploadList(this.data.DETAIL, res.data);
    });
  }

  public SaveUpload() {
    this.dialogRef.close(this.PartListData);
  }

  private MappingUploadList(originalData, uploadData) {
    let failedCheck = false;
    let originalDataCopy = JSON.parse(JSON.stringify(originalData));

    let mergedList = originalDataCopy.map((original) => {
      const data = uploadData.find(
        (upload) =>
          parseFloat(original["ITEM_NO"]) == parseFloat(upload["ITEM NO"]) &&
          original["ACCEPT_PART_NO"] == upload["ACCEPT PART NO"]
      );
      if (!data) failedCheck = true;
      if (!(parseFloat(data["UNIT PRICE B"]) > 0)) failedCheck = true;

      //Set data
      if(!failedCheck) {
        original.UNIT_PRICE_OLD = original.UNIT_PRICE;
        original.UNIT_PRICE = parseFloat(data["UNIT PRICE B"])
        original.AMOUNT = parseFloat(original.UNIT_PRICE) * parseFloat(original.QUANTITY);
      }

      return original;
    });

    if (failedCheck || originalData.length != uploadData.length) {
      this.modal.error(
        "Upload items are not matched with original.",
        this.viewRef
      );
      return originalData;
    }

    this.modal.info("Upload data completed.", this.viewRef);
    return mergedList;
  }

  public CloseDialog(): void {
    this.dialogRef.close(this.data.DETAIL);
  }
}
