import { Component, OnInit, ViewContainerRef, ElementRef, ViewChild } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { tap } from 'rxjs/operators';
import { Globals } from 'src/app/core/global';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import * as moment from 'moment';

@Component({
  selector: 'snpe045',
  templateUrl: './snpe045.component.html',
  styleUrls: ['./snpe045.component.css']
})
export class SNPE045Component implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private el: ElementRef,
    private global: Globals
  ) { }

  @ViewChild("dialog", { static: true }) dialog: any;
  @ViewChild("autoCompleteVendor", { static: true }) autoCompleteVendor: AutoCompleteQueryComponent;

  private currentDate: moment.Moment = moment();
  public tableHeader: any = {};
  public DocNo: string;
  public DocType: string;
  public DocDate: moment.Moment;
  public Vendor: string;
  public VendorCode: string;
  public InvoiceNo: string;
  public InvoiceDate: moment.Moment;
  public DeliveryDate: moment.Moment;
  public InvoiceStatus: string;
  public InvoiceProcess: string;
  public Invoice: any = {};
  public UpdateBy: string;
  public UpdateDate: any;
  public ConfirmDate : any;
  public TableData: Array<any> = [];
  public MatchedPO: Array<any> = [];
  public ParamsSearch: any = {};
  public ShowList: boolean = false;
  public DocList: Array<any> = [];
  public tempData: any = {};
  public Iscal: boolean = false;
  public oldinvoice_date : any;
  public old_bl_awb_date : any;
  public FileList: Array<any> = [];
  public FileName: string = "";
  public customInvoice : string = "";
  @ViewChild("fileInput") fileInput: ElementRef;

  public vendorLookupConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor",
    SourceController: "WH",
    SourceMethodName: "SNPE036_GetVendorCode",
    NoOfShowRecords: 50,
    UpperCase: true
  }

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 20
  }

  public ConfigDocNo: any = {
    ColName: "DOCUMENT_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "AC",
    SourceMethodName: "SNPE045_GetDocList",
    NoOfShowRecords: 50,
    MinLengthSearch: 2,
    UpperCase: true
  }

  public ConfigInvoiceNo: any = {
    ColName: "REF_INV_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE036_GetInvList",
    NoOfShowRecords: 50,
    MinLengthSearch: 2
  }

  public paramHead : any = {}
  ngOnInit() {
    this.global.GetTokenDetail("ssNameEN");
    
  }

  GetDate() {
    return this.http.GetSystemDate()
      .pipe(tap((result: any) => {
        if (result) this.currentDate = result;
      }))
  }

  // Search() {
  //   this.http.get("WH", "SNPE036_GetDocumentList",
  //     this.global.EscapeCharacter(this.ParamsSearch.DOCUMENT_NO) || 'NULL',
  //     this.global.ConvertDateToFormatSTD(this.ParamsSearch.DOCUMENT_DATE) || 'NULL',
  //     this.global.EscapeCharacter(this.ParamsSearch.INVOICE_NO) || 'NULL',
  //     this.global.ConvertDateToFormatSTD(this.ParamsSearch.INVOICE_DATE) || 'NULL')
  //     .subscribe(result => {
  //       this.DocList = result.Table;
  //     })
  // }

  GetDocument(event) {
    
    if (event.DOCUMENT_NO) {
      this.Iscal = false;
      this.http.get("AC", "SNPE045_GetDocument", event.DOCUMENT_NO, event.DOCUMENT_TYPE, event.REF_INV_NO)
        .subscribe(result => {
          if (result.Table.length > 0) {
            this.tableHeader = result.Table[0];
            this.VendorCode = this.tableHeader.VENDOR_CODE;
            this.Vendor = this.tableHeader.VENDOR_NAME;

            this.DocNo = this.tableHeader.DOCUMENT_NO;
            this.DocType = this.tableHeader.DOCUMENT_TYPE
            this.DocDate = this.tableHeader.DOCUMENT_DATE;
            this.InvoiceNo = this.tableHeader.INVOICE_NO;
            this.InvoiceDate = this.tableHeader.INVOICE_DATE;
            this.DeliveryDate = this.tableHeader.BL_AWB_DATE;
            this.InvoiceStatus = this.tableHeader.INVOICE_STATUS;
            this.InvoiceProcess = this.tableHeader.INVOICE_PROCESS;
            this.UpdateBy = this.tableHeader.LAST_UPD_BY;
            this.UpdateDate = this.tableHeader.LAST_UPD_DATE;
            this.ConfirmDate = this.tableHeader.CONFIRM_DATE
            this.customInvoice = this.tableHeader.ATTRIBUTE3
            this.TableData = result.Table1;
            //console.log(this.ConfirmDate,'ConfirmDate')
            //console.log(this.TableData,'dddddddddddddddddd')
            this.oldinvoice_date = this.tableHeader.INVOICE_DATE;
            this.old_bl_awb_date = this.tableHeader.BL_AWB_DATE ? this.tableHeader.BL_AWB_DATE : this.tableHeader.INVOICE_DATE;
          } else {
            this.modal.info("Document no: " + event.DOCUMENT_NO + " is not found.", this.viewref);
            this.tableHeader = {};
            this.VendorCode = null;
            this.Vendor = null;

            this.DocNo = null;
            this.DocType = null
            this.DocDate = null;
            this.InvoiceNo = null;
            this.InvoiceDate = null;
            this.DeliveryDate = null;
            this.InvoiceStatus = null
            this.InvoiceProcess = null
            this.UpdateBy = null
            this.UpdateDate = null;
            this.ConfirmDate = null
            this.customInvoice = null
            this.TableData = [];
          }
        })
    } else {
      this.tableHeader = {};
      this.VendorCode = null;
      this.Vendor = null;

      this.DocNo = null;
      this.DocType = null
      this.DocDate = null;
      this.InvoiceNo = null;
      this.InvoiceDate = null;
      this.DeliveryDate = null;
      this.InvoiceStatus = null
      this.InvoiceProcess = null
      this.UpdateBy = null
      this.UpdateDate = null;
      this.ConfirmDate = null
      this.TableData = [];
      this.modal.warning("No document for input expense",this.viewref)
    }
    
  }

  CalDuty(){

    if(!this.DeliveryDate){
      this.modal.warning("Please input delivery date",this.viewref)
      return;
    }
    //this.modal.confirm('')
    let params = {
      INVOICE_NO : this.tableHeader.INVOICE_NO,
      INVOICE_DATE : this.tableHeader.INVOICE_DATE,
      VENDOR_CODE : this.tableHeader.VENDOR_CODE,
      BL_AWB_DATE : this.DeliveryDate,
      DOCUMENT_TYPE : this.tableHeader.DOCUMENT_TYPE,
      DOCUMENT_NO : this.tableHeader.DOCUMENT_NO,
      DETAIL : this.TableData
    }
    this.http.post("AC","SNPE045_Calculate",params)
    .subscribe(res =>{
        this.modal.info("Calculate successfully", this.viewref);
        this.TableData = res.Table
        this.ExpenseChanged();
        this.Iscal = true;
    },(err)=>{
        this.modal.warning("Found an error, please contact support team", this.viewref);
    })
  }

  SaveDocument() {

    if(!this.DeliveryDate || !this.InvoiceDate){
      this.modal.warning("Please input invoice date or delivery date date",this.viewref)
      return;
    }

    let message = '';
    let result = 'Are you sure to confirm?'
    let xwidth = '550px',xheight = '250px'
    if(!this.customInvoice){
      message = '<h4>Customs Invoice No is not entered</h4><br>'
      result = message.fontcolor("red") + '<h4>Are you sure to confirm?</h4>';
      xwidth = '600px'
      xheight = '300px'
    }

    this.TableData.map((d, i) => {
      d.DOCUMENT_NO = this.DocNo;
      d.DOCUMENT_TYPE = this.DocType;
      d.VENDOR_CODE = this.VendorCode;
      d.INVOICE_NO = this.InvoiceNo;
      d.INVOICE_DATE = this.global.ConvertDateToFormatSTD(this.InvoiceDate);
      d.BL_AWB_DATE = this.global.ConvertDateToFormatSTD(this.DeliveryDate);
      d.TOTAL_EXP_CLEANRANCE_AMT_B = this.tableHeader.EXP_CLEANRANCE_AMT_B;
      d.TOTAL_EXP_OTHER_AMT_B = this.tableHeader.EXP_OTHER_AMT_B;
      d.TOTAL_INCOME_TAX = this.TotalDuty();
      d.OLD_INVOICE_DATE = this.global.ConvertDateToFormatSTD(this.oldinvoice_date)
      d.OLD_BL_AWB_DATAE = this.global.ConvertDateToFormatSTD(this.old_bl_awb_date)
      d.CUSTOM_INVOICE  = this.customInvoice
      //return this.CheckRequiredFields(i, false) && d.INVOICE_PROCESS === 'N';
    })

    //console.log('saverow',this.TableData)
    //console.log('tableHeader',this.tableHeader)

    if (this.TableData.length > 0) {
      this.modal.confirm(result, this.viewref
        , () => {
          this.http.post("AC", "SNPE045_SaveDocument", this.TableData)
            .subscribe(result => {
              this.modal.info("Transaction complete: applied and saved.", this.viewref);
              //this.DocNo = result.DOCUMENT_NO;
              this.UpdateBy = this.global.GetTokenDetail("ssNameEN");
              this.UpdateDate = new Date();
            }, err => {
              this.modal.error(err.message, this.viewref);
            })
        }, () => { },{width: xwidth,height: xheight})
    } else {
      this.modal.error("Unable to process.", this.viewref);
    }
  }



  Clear(){
    this.modal.confirm("Do you want to clear expense amount?",this.viewref,this.ConfirmClear)
  }

  ConfirmClear = () =>{

    let param = {
      DOCUMENT_TYPE : this.DocType,
      DOCUMENT_NO : this.DocNo,
      INVOICE_NO : this.InvoiceNo,
      INVOICE_DATE : this.global.ConvertDateToFormatSTD(this.InvoiceDate),
      VENDOR_CODE : this.VendorCode,
      REF_INV_NO : this.InvoiceNo
    }
    this.http.post("AC","SNPE045_Clear",param)
    .subscribe(res =>{
        if(res.STATUS_MESSAGE == 'DONE'){
          this.modal.info("Clear expense amount complete",this.viewref)
          this.GetDocument(param)
        }else{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        }
    },err =>{
      this.modal.warning("Found an error, please contact support team", this.viewref);
    })
  }

  TotalDutyChange(event){

    //console.log('parseInt',);
    //console.log('event',event);
    //console.log('TotalDuty',this.TotalDuty());

    event = parseFloat(event.replace(/,/g, ''))
    let diffduty = this.global.Round(event - this.TotalDuty(),2)
    let maxDuty = Math.max.apply(Math, this.TableData.map((x) => { return x.INCOME_TAX; }))
    let index = this.TableData.findIndex(x => x. INCOME_TAX == maxDuty);
    //console.log('xxxxxxxxxxx',this.global.Round(this.TableData[index].INCOME_TAX + diffduty,2))
    this.TableData[index].INCOME_TAX =  this.global.Round(this.TableData[index].INCOME_TAX + diffduty,2)
    this.TableData.slice();
    //console.log('index',index)

  }

  SetVendorCodeLookup(value) {
    if (value.VENDOR_CODE) {
      this.Vendor = value.LOOKUP_VENDOR_NAME;
      this.VendorCode = value.VENDOR_CODE;

    } else {
      this.Vendor = null;
      this.VendorCode = null;
      this.el.nativeElement.querySelector("#vendorcode").focus();
      this.modal.error("Unable to find vendor.", this.viewref);
    }
  }

 

  public TotalQty() {
    return this.TableData.reduce((a, b) => a + (b.ITEM_QTY || 0), 0);
  }

  TotalDuty() {
    return this.TableData.reduce((a, b) => a + (b.INCOME_TAX || 0), 0);
  }

  TotalPrice() {
    return this.TableData.reduce((a, b) => a + ((b.ITEM_QTY || 0) * (b.UNIT_PRICE_B || 0))
      , 0);
  }

  TotalExcise() {
    return this.TableData.reduce((a, b) => a + (b.EXCISE || 0)
      , 0);
  }

  TotalInterior() {
    return this.TableData.reduce((a, b) => a + (b.INTERIOR || 0)
      , 0);
  }

  TotalAmount() {
    return this.TableData.reduce((a, b) => a + ((b.ITEM_QTY || 0) * (b.UNIT_PRICE_B || 0)
      + (b.EXP_CLEANRANCE_AMT_B || 0)
      + (b.EXP_OTHER_AMT_B || 0)
      + (b.INCOME_TAX || 0)
      + (b.EXCISE || 0)
      + (b.INTERIOR || 0)
    )
      , 0);
  }

  public ExpenseChanged() {
    let SumEXP_CLEANRANCE_AMT_B = 0;
    let sumEXP_OTHER_AMT_B = 0;
    //this.TableData
    this.TableData.forEach((value) => {

      if (value.ACCEPT_PART_NO != '' && value.ACCEPT_PART_NO != undefined) {

        value.EXP_CLEANRANCE_AMT_B = this.global.Round((this.tableHeader.EXP_CLEANRANCE_AMT_B * (value.ITEM_QTY * value.UNIT_PRICE_B) / this.TotalPrice()),2);
        value.EXP_OTHER_AMT_B = this.global.Round((this.tableHeader.EXP_OTHER_AMT_B * (value.ITEM_QTY * value.UNIT_PRICE_B) / this.TotalPrice()),2);

        SumEXP_CLEANRANCE_AMT_B += value.EXP_CLEANRANCE_AMT_B;
        sumEXP_OTHER_AMT_B += value.sumEXP_OTHER_AMT_B;
      }
    });

    let diffCLEANRANCE = SumEXP_CLEANRANCE_AMT_B - this.tableHeader.EXP_CLEANRANCE_AMT_B;
    let diffOTHER = sumEXP_OTHER_AMT_B - this.tableHeader.EXP_OTHER_AMT_B;

    if(diffCLEANRANCE !== 0){
      this.TableData[0].EXP_CLEANRANCE_AMT_B + diffCLEANRANCE
    }

    if(diffOTHER !== 0){
      this.TableData[0].EXP_OTHER_AMT_B + diffCLEANRANCE
    }

  }

  QtyChange(data) {

    if (data.ITEM_QTY > data.ORDER_QTY) {
      data.ITEM_QTY = data.ORDER_QTY
    }

    if (data.CALCULATE_TYPE == 'Capacity') {
      data.EXCISE = data.CAPACITY_RATE * (data.CAPACITY * data.ITEM_QTY);
    } else {
      data.EXCISE = (data.EXCISE_RATE / 100) * data.RETAIL_PRICE * data.ITEM_QTY;
    }

    data.INCOME_TAX = this.global.Round((data.UNIT_PRICE_B * data.ITEM_QTY * data.DUTY_RATE) / 100,2);
    data.INTERIOR = data.EXCISE / 10;
    //data.ITEM_QTY > data.ORDER_QTY ? data.ORDER_QTY : data.ITEM_QTY;
  }

  CalNewUnitPrice(partNo) {
    let PartFilter = this.TableData.filter(v => v.ACCEPT_PART_NO === partNo);
    let totalPartNetAmount = PartFilter.reduce((v1, v2) => v1 + this.CalNetAmount(v2), 0)
    let totalPartQty = PartFilter.reduce((v1, v2) => v1 + v2.ITEM_QTY, 0)

    return totalPartQty > 0 ? (totalPartNetAmount / totalPartQty) : PartFilter[0].UNIT_PRICE_B;
  }

  CalNetAmount(data) {
    return (data.ITEM_QTY * data.UNIT_PRICE_B) + data.EXP_CLEANRANCE_AMT_B + data.EXP_OTHER_AMT_B + data.INCOME_TAX + data.EXCISE + data.INTERIOR;
  }

  

  CalInterior(data) {
    data.INTERIOR = this.global.Round(data.EXCISE / 10,2);
  }


  BackToGrid() {
    this.ShowList = true;
  }
  Download(){
      this.http.Download("AC","SNPE045_Download",this.TableData)
      .subscribe(res =>{
        this.writeContents(res);
      })
  }

  private writeContents(content) {
    var currentDate = moment().format('DD-MM-YYYY');
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = "SNPE045_" + currentDate + ".xlsx" ;
    a.click();
  }

  openFileBrowser() {
    this.FileName = "";
    this.fileInput.nativeElement.value = '';
    this.fileInput.nativeElement.click();
  }

  selectFile(evt) {
    
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
        let File: File = Files[0];
        this.FileName = File.name;
        
        let formData = new FormData();
        formData.append('file', File);

        this.http.postFile("AC", "SNPE045_UploadFile", formData)
            .subscribe(result => {     
              
               this.TableData.map((x,i) => {
                 let data = result.find(y => y.ACCEPT_PART_NO == x.ACCEPT_PART_NO)
                 
                 if(data){
                 x.UNIT_PRICE_B = data.UNIT_PRICE_B
                 x.DUTY_RATE = data.DUTY_RATE
                 }
               })

               this.TableData = this.TableData.slice()
               this.modal.info("Upload complete", this.viewref)
               
            }, err => {
              this.modal.error("Upload not complete", this.viewref);
            })

    }
  }
}
