import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewContainerRef } from '@angular/core';
import { MatDatepicker } from '@angular/material/datepicker';
import { FormControl, FormGroup } from '@angular/forms';
import { DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE } from '@angular/material/core';
import { MAT_MOMENT_DATE_ADAPTER_OPTIONS, MomentDateAdapter } from '@angular/material-moment-adapter';
import * as moment from 'moment';
import { HttpService } from 'src/app/core/http.service';

export const MY_FORMATS = {
    parse: {
        dateInput: ['MM/YYYY', 'MMYYYY', 'MMYY'],
    },
    display: {
        dateInput: 'MMMM YYYY',
        monthYearLabel: 'MMM YYYY',
        dateA11yLabel: 'LL',
        monthYearA11yLabel: 'MMMM YYYY',
    },
};

@Component({
    selector: 'SNPP104_NP-search',
    templateUrl: 'SNPP104_NP-search.component.html',
    styleUrls: ['./../SNPP104_NP.component.css'],
    providers: [
        {
            provide: DateAdapter,
            useClass: MomentDateAdapter,
            deps: [MAT_DATE_LOCALE, MAT_MOMENT_DATE_ADAPTER_OPTIONS]
        },

        { provide: MAT_DATE_FORMATS, useValue: MY_FORMATS },
    ],
})

export class SNPP104_NPSearchComponent implements OnInit, OnChanges {
    constructor(
        private http: HttpService
    ) {}

    @Input() showUpload;
    @Input() params: any;
    @Output() showUploadChange: EventEmitter<boolean> = new EventEmitter<boolean>();
    @Output() searchSubmit: EventEmitter<any> = new EventEmitter<any>();

    public paramSearch: FormGroup = new FormGroup({
        ProcessDate: new FormControl(null),
        Status: new FormControl("")
    })
    public statusList: Array<any>;

    ngOnInit() { 
        this.GetStatusCode();
    }

    ngOnChanges(changes: SimpleChanges) {
        this.paramSearch.patchValue({
            ProcessDate: this.params.ProcessDate,
            Status: this.params.Status || ""
        })
    }

    GetStatusCode() {
        this.http.get("wPMS", "GetSnapData", "SP_NP_MT_DEAD_STOCK_STATUS")
            .subscribe(result => {
                this.statusList = result.Table;
            })
    }

    chosenYearHandler(normalizedYear: moment.Moment) {
        const ctrlValue = this.paramSearch.get('ProcessDate').value || moment();
        ctrlValue.year(normalizedYear.year());
        this.paramSearch.get('ProcessDate').setValue(ctrlValue);
    }

    chosenMonthHandler(normalizedMonth: moment.Moment, datepicker: MatDatepicker<moment.Moment>) {
        const ctrlValue = this.paramSearch.get('ProcessDate').value || moment();
        ctrlValue.month(normalizedMonth.month());
        this.paramSearch.get('ProcessDate').setValue(ctrlValue);
        datepicker.close();
    }

    AddDeadStock() {
        this.showUpload = true;
        this.showUploadChange.emit(this.showUpload);
    }

    Search() {
        this.searchSubmit.emit(this.paramSearch.getRawValue());
    }
}