import { Component, OnInit, Input, Output, EventEmitter, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { FormGroup, FormControl, FormBuilder } from '@angular/forms'
import { Router } from '@angular/router';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';

@Component({
    selector: 'snsj007_m',
    templateUrl: './SNSJ007_M.component.html',
    styleUrls: ['./SNSJ007_M.component.css']
})



export class SNSJ007_MComponent implements OnInit {


    constructor(
        private http: HttpService,
        private router: Router,
        private modal : ModalService,
        private viewref : ViewContainerRef,
        private global : Globals
    ) { }



    public CustomerList: any;
    private loaded: boolean = false;
    private fileList : any =[];

    ngOnInit() {



    }


    StartProcess(step){

        this.modal.confirm("Do you want to start process",this.viewref,() => ConfirmProcess())

        let ConfirmProcess = () =>{
            this.http.get("AC","SNSJ007_M_STARTPROCESS",step)
            .subscribe(res => {
                this.fileList = []
                if(res.Table.length > 0){
                    let index = 0;
                    let reportName = '';
                    let filename = ''
    
                    for(let item of res.Table){
                        
                        let reportParams = {
                          
                        }
    
                        if(item["REPORT_NAME"] == 'SNSR031'){
                            reportName = 'PMS_RPT_ACC_SNSR031'
                            filename = "SNSR031" + '_' + item["COMPANY_CODE"] + '_' + this.FormatCurrentDate() + '.' + 'PDF'
                            reportParams = {
                                P_COMPANY_CODE : item["COMPANY_CODE"],
                                P_DOC_DATE : this.FormatDate2(item["C_DATE"]),
                                P_STEP  : step
                            }
        
                        }else if(item["REPORT_NAME"] == 'SNSR032'){
                            reportName = 'PMS_RPT_ACC_SNSR032'
                            filename = "SNSR032" + '_' + item["COMPANY_CODE"] + '_' + this.FormatCurrentDate() + '.' + 'PDF'
                            reportParams = {
                                P_COMPANY_CODE : item["COMPANY_CODE"],
                                P_DOC_DATE : this.FormatDate2(item["C_DATE"]),
                            }
        
                        }else{
                            reportName = 'PMS_RPT_ACC_SNSR033'
                            filename = "SNSR033" + '_' + item["COMPANY_CODE"] + '_' + this.FormatCurrentDate() + '.' + 'PDF'
                            reportParams = {
                                P_COMPANY_CODE : item["COMPANY_CODE"]
                            }
        
                        }
    
                        this.http.getParams("AC", ["SNSJ007_M_GenerateReport",reportName,filename], reportParams)
                        .subscribe(result => {
                            this.fileList.push({
                                file_path : result.substring(1),
                                file_name : filename
                            })
                                index++
                            if(index == res.Table.length){
                                this.modal.info("Process Complete !.......",this.viewref)
                                //this.DownLoad();
                            }
                        })
    
                    }
                }else{
                    
                    if(res.Table1[0].STATUS == 'Done'){
                        this.modal.info(res.Table1[0].MESSAGE,this.viewref)
                    }else{
                        this.modal.warning(res.Table1[0].MESSAGE,this.viewref)
                    }
                }
    
                
            },(err) =>{
                this.modal.warning(err.message,this.viewref)
            })
        }
        
     
    }


    
    DownLoad(){
        this.http.DownloadMultipleFilesAsZip("AC","SNSJ007_M_DowLoad",this.fileList)
        .subscribe(res =>{
            console.log('DownloadMultipleFilesAsZip',res)
            this.modal.info("Process Complete !.......",this.viewref)
            this.WriteFile(res);
        })
    }

    WriteFile(res){
        var blob = new Blob([res], { type: "application/octet-stream" });                
        var fileURL = URL.createObjectURL(blob);
        var a = document.createElement('a');
        a.href = fileURL;
        a.target = '_blank';
        a.download = "SNSJ007_M.zip";
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
    }


    FormatCurrentDate() {

        let d = new Date();
        let Day = d.getDate().toString();
        let Month = d.getMonth() + 1;
        let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
        let Year = d.getFullYear().toString();
        let hours = d.getHours();
        let minute = d.getMinutes();
        let second = d.getSeconds();
        let milli = d.getMilliseconds();
        return  Day + '-' + NewMonth + '-' + Year + '_' + hours + '-' + minute + '-' + second + '-' + milli;
      }


      FormatDate2(date){
      
        let monthText = ["JAN", "FEB", "MAR", "APR", "MAY", "JUN","JUL", "AUG", "SEP", "OCT", "NOV", "DEC"];
  
        let d = new Date(date);
        let Day = d.getDate().toString();
        let NewDay = Day.toString().length < 2 ? '0' + Day.toString() : Day.toString()
        let Month = d.getMonth();
        let MonthText = monthText[Month]
        let Year = d.getFullYear().toString().substr(-2);
        return NewDay + MonthText + Year;
    }




}
