import { Component, OnInit, ViewContainerRef } from '@angular/core';
import { FormGroup, FormControl } from '@angular/forms';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { HttpService } from 'src/app/core/http.service';
import { AuthService } from 'src/app/core/auth-service.service';
import { Router } from '@angular/router';
import { ModalService } from 'src/app/core/modal.service';

@Component({
    selector: 'changepwd',
    templateUrl: 'changepwd.component.html',
    styleUrls: ['changepwd.component.css']
})

export class ChangePasswordComponent implements OnInit {
    constructor(
        private http: HttpService,
        private auth: AuthDecode,
        private authService: AuthService,
        private router: Router,
        private modal: ModalService,
        private viewref: ViewContainerRef
    ) { }

    public errMsg: string;
    public changePwdForm: FormGroup = new FormGroup({
        username: new FormControl({
            value: '',
            disabled: true
        }),
        oldpassword: new FormControl(''),
        newpassword: new FormControl('', { validators: this.passwordValidator }),
        confirmnewpassword: new FormControl('', { validators: this.passwordValidator })
    }, { validators: this.checkPasswords })
    public showoldpwd: boolean = false;
    public shownewpwd: boolean = false;
    public showcfmpwd: boolean = false;


    ngOnInit() {
        if (this.authService.isAuthenticated()) {
            this.changePwdForm.get("username").setValue(this.auth.getDetail(false).ssUserName)
        } else {
            this.changePwdForm.get("username").enable();
        }
    }

    changePassword() {
        this.http.postChangePwd(
            this.changePwdForm.get("username").value,
            this.changePwdForm.get("oldpassword").value,
            this.changePwdForm.get("newpassword").value,
            this.changePwdForm.get("confirmnewpassword").value
        ).subscribe(result => {
            this.modal.info("Password has beed changed.", this.viewref, () => {
                this.router.navigate(["/"]);
            });

        }, (err) => {
            if (err.error && !(err.error instanceof Object)) {
                this.errMsg = err.error;
            } else {
                this.errMsg = err.message;
            }
        })
    }

    passwordValidator(control: FormControl) {
        let score: number = 0;
        let pass: string = control.value;

        if (pass.length >= 8) {
            if (pass.match(/[0-9]/)) score++;
            if (pass.match(/[A-Z]/)) score++;
            if (pass.match(/[a-z]/)) score++;
            if (pass.match(/[\!|\#|\$|\%|\&|\<|\>|\?|\@|\[|\]|_|\(|\)]/)) score++;
        }

        return score >= 3 ? null : { "notStrong": true };
    }

    checkPasswords(group: FormGroup) {

        let oldPass = group.get('oldpassword');
        let newPass = group.get('newpassword');
        let confirmPass = group.get('confirmnewpassword');

        if (oldPass.valid && newPass.valid && confirmPass.valid) {
            return oldPass.value !== newPass.value && newPass.value === confirmPass.value ? null : { "notSame": true };
        }

        return null;
    }

    Cancel() {
        this.router.navigate(['/'])
    }
}