import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { Router } from "@angular/router";
import { FormControl, Validators } from "@angular/forms";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "update",
  templateUrl: "./update.component.html",
  styleUrls: ["./update.component.css"],
})
export class UpdateComponent implements OnInit {
  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewRef: ViewContainerRef
  ) {}

  @ViewChild("dialog", { static: false }) dialog: any;

  public paramSearch: any = {};
  public params: any = {};
  public tableData: any = [];
  public isAdd: boolean = false;

  //for mat-select document modules
  public moduleList: any = [
    { ID: 1, MODULE_NAME: "Common" },
    { ID: 2, MODULE_NAME: "Inventory Control" },
    { ID: 3, MODULE_NAME: "Warehouse" },
    { ID: 4, MODULE_NAME: "Accounting" },
    { ID: 5, MODULE_NAME: "Sale domestric" },
    { ID: 6, MODULE_NAME: "Sale Export" },
    { ID: 7, MODULE_NAME: "After Sales" },
    { ID: 8, MODULE_NAME: "Technical information" },
    { ID: 9, MODULE_NAME: "Purchasing" },
    { ID: 10, MODULE_NAME: "Other" },
  ];

  ngOnInit() {
    this.Search();
  }

  public Search() {
    this.tableData = [];
    this.http
      .post("System", "GetManualDocumentByName", this.paramSearch)
      .subscribe(
        (res) => {
          this.tableData = res;
        },
        () => {
          this.modal.warning(
            "Found an error, please contract support team",
            this.viewRef
          );
        }
      );
  }

  public Add() {
    this.params = {};
    this.isAdd = true;
    this.dialog.title = "Add New Manual Document";
    this.dialog.OpenDialog();
  }

  public Edit(data) {
    this.params = { ...data };
    this.isAdd = false;
    this.dialog.title = "Edit Manual Document";
    this.dialog.OpenDialog();
  }

  public Delete(data) {
    this.modal.confirm(
      "Do you want to delete this record?",
      this.viewRef,
      () => {
        this.http
          .post("System", "ManualDocumentDelete", data)
          .subscribe((res) => {
            if (res === "Done") {
              this.modal.info(
                "Transaction complete: 1 record applied and deleted.",
                this.viewRef
              );
              this.Search();
              this.dialog.Close();
            } else {
              this.modal.warning(
                "Found an error, please contract support team.",
                this.viewRef
              );
            }
          });
      }
    );
  }

  public Save() {
    if (
      this.params.DOCUMENT_NAME.trim() &&
      this.params.DOCUMENT_MODULE.trim() &&
      this.params.DOCUMENT_PATH.trim()
    ) {
      this.modal.confirm(
        "Do you want to save change you have made?",
        this.viewRef,
        this.ConfirmSave
      );
    } else {
      this.modal.warning("Document field is required.", this.viewRef);
    }
  }

  private ConfirmSave = () => {
    this.http.post("System", "ManualDocumentSave", this.params).subscribe(
      (res) => {
        if (res === "Done") {
          this.modal.info(
            "Transaction complete: applied and saved.",
            this.viewRef
          );
          this.Search();
          this.dialog.Close();
        } else {
          this.modal.warning(
            "No change to save. Something went wrong.",
            this.viewRef
          );
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team.",
          this.viewRef
        );
      }
    );
  };

  public Cancel() {
    this.dialog.Close();
  }

}
