import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';

@Component({
  selector: 'snm013P',
  templateUrl: './SNM013P.component.html',
  styleUrls: ['./SNM013P.component.css']
})

export class SNM013PComponent implements OnInit {

  public params: any = {};
  public Datatable: any = [];
  public Data: any = {};

  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private auth: AuthDecode,
    private global: Globals
  ) { }

  @ViewChild("dialogCreate", { static: false }) dialogCreate: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;



  ngOnInit() {
    this.InitData();
    this.Search();
  }

  Search = () => {
    this.Datatable = [];
    this.http.post('SL', 'SNM013P_GetData', this.params)
      .subscribe(res => {
        this.Datatable = res.Table;
      })
  }

  InitData() {
    this.Data = {};
  }

  ConfirmDelete(row) {
    this.InitData();
    this.Data = row;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  Delete = () => {
    this.http.post('SL', 'SNM013P_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
        }
      });
  }

  ConfirmSave() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
  }

  Save = () => {
    this.http.post('SL', 'SNM013P_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialogCreate.Close();
          this.dialogEdit.Close();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  CreateClick() {
    this.InitData();
    this.dialogCreate.OpenDialog();
  }

  EditClick(row) {
    this.InitData();
    this.Data = row;
    this.dialogEdit.OpenDialog();
  }

  CloseCreate() {
    this.InitData();
    this.dialogCreate.Close();
  }

  CloseEdit() {
    this.InitData();
    this.dialogEdit.Close();
  }

  public lookupSalesChannelConfig: any = {
    ColName: "LOOKUP_SALES_CHANNEL",
    PlaceholderText: "Select Sales Channel",
    SourceController: "SL",
    SourceMethodName: "SNM013P_GetSalesChannel",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedSalesChannel(value: any) {
    this.params.SALES_CHANNEL = value.SALES_CHANNEL;
    this.params.CHANNEL_NAME = value.CHANNEL_NAME;
    this.Search();
  }
}
