import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';
import * as moment from 'moment';

@Component({
  selector: 'snm017p',
  templateUrl: './SNM017P.component.html',
  styleUrls: ['./SNM017P.component.css']
})

export class SNM017PComponent implements OnInit {

  public params: any = {};

  public Data: any = {};

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals,
    private viewRef: ViewContainerRef
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialogReport", { static: false }) dialogReport: any;

  public MainDatatable: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;


  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_NAME",
    PlaceholderText: "Select Customer Code",
    SourceController: "AC",
    SourceMethodName: "SNM017P_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: { DATA: this.params },
    UpperCase: true
  }

  public lookupPartNoConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part Code",
    SourceController: "AC",
    SourceMethodName: "SNM017P_GetPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: { DATA: this.params },
    UpperCase: true
  }

  public lookupCurrencyCodeConfig: any = {
    ColName: "CURRENCY_CODE",
    PlaceholderText: "Select Currency Code",
    SourceController: "AC",
    SourceMethodName: "GetCurrency",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public dialogLookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_NAME",
    PlaceholderText: "Select Customer Code",
    SourceController: "AC",
    SourceMethodName: "SNM017P_GetCustomerCodeAdd",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public dialogLookupPartNoConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }


  ngOnInit() {
    this.InitData();
    //this.params.EFFECTIVE_DATE = new Date("January 1, 2000 07:00:00");
    this.params.EFFECTIVE_DATE_S = new Date();
    this.params.EFFECTIVE_DATE_E = new Date();
  }

  public SearchExportPrice = () => {

    if (this.params.CUSTOMER_CODE != null
      || this.params.DSP_EFFECTIVE_DATE != null || this.params.PART_NO != null) {
      this.http.post('AC', 'SNM017P_GetExport_Price', this.params)
        .subscribe(res => {
          this.MainDatatable = res.Table;
        })
    } else {
      this.modal.warning('Please select the Customer code', this.viewref)
      return;
    }


  }

  //DateChange() {
  //  this.params.CUSTOMER_CODE = null;
  //  this.params.PART_NO = null;
  //  this.params.CURRENCY_CODE = null;
  //}

  InitData() {
    this.Data = {};
  }

  CreateClick() {
    this.FrmTitle = "New Export Price";
    this.IsEdit = false;
    this.InitData();
    this.Data.EFFECTIVE_DATE = new Date();
    this.dialogMain.OpenDialog();
  }

  EditClick(row) {
    this.FrmTitle = "Edit Export Price";
    this.IsEdit = true;
    this.InitData();
    this.Data = JSON.parse(JSON.stringify(row));
    this.Data.OLD_FORMER_NET = this.Data.FORMER_NET
    this.Data.OLD_NET_PRICE = this.Data.NET_PRICE
    this.dialogMain.OpenDialog();
  }

  ConfirmDelete(row) {
    this.InitData();
    this.Data = row;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  Delete = () => {
    this.http.post('AC', 'SNM017P_ExportPrice_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.SearchExportPrice();
        }
      });
  }

  ConfirmSave() {
    if (this.IsEdit == true) {
      if (this.Data.OLD_FORMER_NET != this.Data.FORMER_NET || this.Data.OLD_NET_PRICE != this.Data.NET_PRICE) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.EFFECTIVE_DATE && this.Data.CUSTOMER_CODE && this.Data.PART_NO && this.Data.CURRENCY_CODE && (this.Data.NET_PRICE || this.Data.FORMER_NET)) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  Save = () => {
    this.http.post('AC', 'SNM017P_ExportPrice_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {

          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.params.CUSTOMER_CODE = this.Data.CUSTOMER_CODE
          this.params.PART_NO = this.Data.PART_NO
          this.SearchExportPrice();
          this.dialogMain.Close();
          this.InitData();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  CloseDialog() {
    this.InitData();
    this.dialogMain.Close();
  }


  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    //this.params.PART_NO = null;
    //if (value.CUSTOMER_CODE) {
    //  this.SearchExportPrice();
    //}
  }

  public SetSelectedPartNo(value: any) {
    this.params.PART_NO = value.PART_NO;
    //if (value.PART_NO) {
    //  this.SearchExportPrice();
    //}
  }

  public SetSelectedCurrencyCode(value: any) {
    this.params.CURRENCY_CODE = value.CURRENCY_CODE;
    //if (value.CURRENCY_CODE) {
    //  this.SearchExportPrice();
    //}
  }




  public SetSelectedDialogCustomerCode(value: any) {
    this.Data.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.Data.CUSTOMER_NAME = value.CUST_NAME_TH;
    this.Data.CURRENCY_CODE = value.CURRENCY_CODE
    this.Data.CURRENCY_NAME = value.CURRENCY_NAME
    this.GetFormerPrice();
  }

  public SetSelectedDialogPartNo(value: any) {
    this.Data.PART_NO = value.PART_NO;
    this.Data.PART_NAME = value.PART_NAME;
    this.GetFormerPrice();
  }

  public SetSelectedDialogCurrencyCode(value: any) {
    this.Data.CURRENCY_CODE = value.CURRENCY_CODE;
    this.Data.CURRENCY_NAME = value.CURRENCY_NAME;
    this.GetFormerPrice();
  }


  public DialogLookupEffectiveDateConfig: any = {
    ColName: "DSP_EFFECTIVE_DATE",
    PlaceholderText: "Select Effective Date",
    SourceController: "AC",
    SourceMethodName: "SNM017_GetEffectiveDate",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedEffectiveDate(value: any) {
    this.params.EFFECTIVE_DATE = value.EFFECTIVE_DATE;
    this.params.DSP_EFFECTIVE_DATE = value.DSP_EFFECTIVE_DATE;
  }

  public SetSelectedDialogEffectiveDateStart(value: any) {
    this.params.EFFECTIVE_DATE_S = value.EFFECTIVE_DATE;
  }

  public SetSelectedDialogEffectiveDateEnd(value: any) {
    this.params.EFFECTIVE_DATE_E = value.EFFECTIVE_DATE;
  }


  public OpenReport() {
    this.dialogReport.OpenDialog();
  }

  public CloseReport() {
    this.dialogReport.Close();
  }

  public GoToReport() {
    if (this.params.CUSTOMER_CODE) {
      this.global.GoToReport("PMS_RPT_SALES_SNM017R", {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_EFFECTIVE_DATE: this.params.EFFECTIVE_DATE ? moment(this.params.EFFECTIVE_DATE).format('DD-MM-YYYY') : null,
        P_CUSTOMER_CODE: this.params.CUSTOMER_CODE,
        P_PART_NO: this.params.PART_NO,
        P_CURRENCY_CODE: this.params.CURRENCY_CODE,
      })
    }
    else {
      this.modal.warning("Please fill customer code ", this.viewRef);
    }
  }

  public GetFormerPrice() {
    if (this.Data.EFFECTIVE_DATE && this.Data.CUSTOMER_CODE && this.Data.CURRENCY_CODE && this.Data.PART_NO) {
      this.http.post('AC', 'SNM017P_GetFormer_Price', this.Data)
        .subscribe(res => {
          this.Data.FORMER_NET = res.Table[0].FORMER_NET;
        });
    }
    else
      this.Data.FORMER_NET = 0;
  }
}
