import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';

@Component({
  selector: 'snm100p',
  templateUrl: './SNM100P.component.html',
  styleUrls: ['./SNM100P.component.css']
})

export class SNM100PComponent implements OnInit {

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;
  @ViewChild("dialogReport", { static: false }) dialogReport: any;

  public params: any = {}
  public FactorList: any = [];
  public OrderClassList: any = [];
  public Data: any = {};
  public FrmTitle: any;
  public IsEdit: any = true;

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM100P_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  ngOnInit() {
    this.Search()
    this.getOrderClass()
  }

  private Search() {
    this.http.post('SL', 'SNM100P_Getdata', this.params)
      .subscribe(res => {
        this.FactorList = res.Table;
      })
  }

  private getOrderClass() {
    this.http.get('SL', 'SNM100P_GetOrderClass')
      .subscribe(res => {
        this.OrderClassList = res.Table;
      })
  }

  private InitData() {
    this.Data = {};
  }

  public OpenMainDialog() {
    this.FrmTitle = "New Export Factor";
    this.InitData();
    this.IsEdit = false;
    this.dialogMain.OpenDialog();
  }

  public Edit(P_data) {
    this.FrmTitle = "Edit Export Factor";
    this.InitData();
    this.Data = JSON.parse(JSON.stringify(P_data));
    this.Data.OLD_LEAD_TIME_ETE = this.Data.LEAD_TIME_ETE
    this.IsEdit = true;
    this.dialogEdit.OpenDialog();
  }

  public Delete(P_data) {
    this.Data = P_data;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.ConfirmDeleteRow);
  }

  private ConfirmDeleteRow = () => {
    this.http.post('SL', 'SNM100P_ETE_Factor_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
        }
      });
  }

  public SaveAdd() {
    if (this.IsEdit == true) {
      if (this.Data.OLD_LEAD_TIME_ETE != this.Data.LEAD_TIME_ETE) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.CUSTOMER_CODE && this.Data.ORDER_CLASS && this.Data.LEAD_TIME_ETE) {
        this.http.post('SL', 'SNM100P_CheckDuplicate', this.Data)
          .subscribe(res => {
            if (res.Table[0].STATUS == 'DONE') {
              this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
            }
            else if (res.Table[0].STATUS == 'FAIL') {
              this.modal.warning("Data is duplicated.", this.viewref);
            }
            else if (res.Table[0].STATUS == 'ERROR') {
              this.modal.error('Found an error, please contract support team', this.viewref);
            }
          });
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  private ConfirmSaveAdd = () => {
    this.http.post('SL', 'SNM100P_ETE_Factor_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.params.CUSTOMER_CODE = this.Data.CUSTOMER_CODE
          this.params.CUSTOMER_NAME = this.Data.CUSTOMER_NAME

          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          if (this.IsEdit == true) {
            this.dialogEdit.Close();
          } else {
            this.dialogMain.Close();
          }
          this.IsEdit = false;
          this.InitData();
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  public CancelMain() {
    this.InitData();
    this.dialogMain.Close();

  }

  public CancelEdit() {
    this.InitData();
    this.dialogEdit.Close();
  }

  public SetSelectedCustomerCode(value: any) {
    this.Data.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.Data.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  public SetParamsCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.Search()
  }

  public lookupMasterCodeConfig: any = {
    ColName: "MASTER_CODE",
    PlaceholderText: "Select Master Code",
    SourceController: "SL",
    SourceMethodName: "SNM100P_GetMasterCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0
  }

  public SetParamsMasterCodeStart(value: any) {
    this.params.MASTER_CODE_S = value.MASTER_CODE;
  }

  public SetParamsMasterCodeEnd(value: any) {
    this.params.MASTER_CODE_E = value.MASTER_CODE;
  }

  public OpenReport() {
    this.dialogReport.OpenDialog();
  }

  public CloseReport() {
    this.dialogReport.Close();
  }

  public GoToReport() {
    if (this.params.MASTER_CODE_S && this.params.MASTER_CODE_E) {
      this.global.GoToReport("PMS_RPT_SALES_SNM047R", {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_MASTER_CODE_S: this.params.MASTER_CODE_S,
        P_MASTER_CODE_E: this.params.MASTER_CODE_E,
      })
    }
    else {
      this.modal.warning("Please fill Master Code", this.viewref);
    }
  }
}


