import { Component, OnInit, ViewChild, ViewContainerRef, ElementRef } from '@angular/core'
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { DialogTemplateComponent } from 'src/app/components/shared/dialog-template/dialog-template.component';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';
import * as moment from 'moment';
import { NgForm } from '@angular/forms';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';

@Component({

  selector: 'snse003',
  templateUrl: './SNSE003.component.html',
  styleUrls: ['./SNSE003.component.css']

})

export class SNSE003Component implements OnInit {

  public param: any = {};
  public orderList: Array<any>;
  public orderHeader: any;
  public orderDetail: Array<any>;
  public customerData: any;
  public orderClassList: Array<any>;
  public orderTypeList: Array<any> = [
    { ORDER_TYPE: '1', DESCRIPTION: 'E/O', REF_ORDER_CLASS: ['1', '2'] },
    { ORDER_TYPE: '2', DESCRIPTION: 'Firm', REF_ORDER_CLASS: ['3', '6'] },
    { ORDER_TYPE: '3', DESCRIPTION: 'R/O', REF_ORDER_CLASS: ['1', '2', '4'] },
    { ORDER_TYPE: '4', DESCRIPTION: 'C/O', REF_ORDER_CLASS: ['5'] },
    { ORDER_TYPE: '6', DESCRIPTION: 'Oil', REF_ORDER_CLASS: ['7'] }
  ];
  public IsEdit: boolean = false;
  public IsSearch: boolean = true;
  public orderNo: string;

  @ViewChild("headerForm") headerForm: NgForm;
  @ViewChild("orderDialog") orderDialog: DialogTemplateComponent;
  @ViewChild("autoCompleteTransport") autoCompleteTransport: AutoCompleteQueryComponent;

  public CustomerLookupConfig: any = {
    ColName: "LOOKUP_CUST",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetCustomer",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };

  public CustSearchConfig: any = {
    ColName: "CUSTOMER",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetSearchCust",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };

  public BranchLookupConfig: any = {
    ColName: "LOOKUP_BRANCH_NAME",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetCustomerBranch",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: null
    },
    UpperCase: true
  };

  public BranchSearchConfig: any = {
    ColName: "BRANCH",
    PlaceholderText: "Select Branch",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetSearchBranch",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.param
    },
    UpperCase: true
  };

  public OrderSearchConfig: any = {
    ColName: "ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetSearchOrder",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.param
    },
    UpperCase: true
  };

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetParts",
    NoOfShowRecords: 20,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public RouteLookupConfig: any = {
    ColName: "LOOKUP_ROUTE",
    PlaceholderText: "Select Delivery Route",
    SourceController: "wPMS",
    SourceMethodName: "GetDeliveryRoute",
    NoOfShowRecords: 40,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public TransportLookupConfig: any = {
    ColName: "LOOKUP_TRANSPORT",
    PlaceholderText: "Select Transport Code",
    SourceController: "wPMS",
    SourceMethodName: "GetTransportCode",
    NoOfShowRecords: 40,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public InvSearchConfig: any = {
    ColName: "LOOKUP_INVOICE_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "SL",
    SourceMethodName: "SNSE003_GetSearchInv",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };
  SetInvoiceNo(value: any) {
    this.param.INVOICE_NO = value.INVOICE_NO;
    this.param.INVOICE_DATE = value.INVOICE_DATE;
  }
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private el: ElementRef,
    private global: Globals
  ) { }

  ngOnInit() {
    this.GetOrderClass();
    this.AddOrder();
  }

  public Search() {
    if (!this.param.CUSTOMER_CODE && !this.param.INVOICE_NO) {
      this.modal.warning("Please input customer code or invoice no.!!!", this.viewRef);
      return;
    }
    this.orderList = [];
    this.http.post("SL", "SNSE003_GetOrderList", this.param)
      .subscribe(res => {
        this.orderList = res.Table;
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewRef);

      })
  }

  ViewOrder(order) {
    this.orderDetail = [];

    this.http.post("SL", "SNSE003_GetOrderDetail", order)
      .subscribe(res => {
        this.orderHeader = res.Table[0];
        this.orderDetail = res.Table1;

        this.orderHeader._TRANSPORT = this.orderHeader.TRANSPORT_CODE;

        this.IsSearch = false;
        this.IsEdit = false;

        this.orderDetail.map(v => {
          v._PART_NO = v.ORDER_PART_NO;
        })

        setTimeout(() => {
          this.autoCompleteTransport.textInput = this.orderHeader._TRANSPORT.toString();
          this.autoCompleteTransport.query()
        }, 200)
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewRef);
      })
  }

  AddOrder() {
    this.IsSearch = false;
    this.IsEdit = true;
    this.orderDetail = [];
    this.orderHeader = {};
    this.customerData = {};
    this.orderHeader.ORDER_DATE = moment();
    this.orderHeader.BO_FLAG = 'Y';
    this.orderHeader.DELIVERY_CODE = '03';
    this.CreateDetailRow(0);

  }

  async Save() {

    if (this.headerForm.valid) {
      if (this.orderHeader.ORDER_CLASS == "7") {
        if (this.orderHeader.CREDIT_LIMIT - this.CalTotalCreditAmount() < 0) {
          this.modal.error("THIS CUSTOMER OVER CREDIT LIMIT.", this.viewRef);
          return;
        }
      }
      let detailRows = this.orderDetail.filter(v => v.ORDER_PART_NO)

      if (detailRows.length <= 0) {
        this.modal.warning("Please select at least 1 Parts.", this.viewRef)
        return;
      }
      if (detailRows.find(v => ((!v.ORDER_QTY || v.ORDER_QTY === 0) || (!v.UNIT_PRICE || v.UNIT_PRICE === 0)))) {
        this.modal.warning("Some record in transaction detail have unfinished detail.", this.viewRef);
        return;
      }


      this.modal.confirm('Do you want to save changed you have made?', this.viewRef,
        () => {
          this.orderHeader.PART_LIST = this.orderDetail.filter(v => v.ORDER_PART_NO);
          this.orderHeader._ORDER_DATE = this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE);

          this.http.post("SL", "SNSE003_SaveDocument", this.orderHeader)
            .subscribe(res => {
              if (res == true) { //Error Incorrect date or Batch running
                return;
              }
              this.modal.info('Transaction complete: applied and saved.', this.viewRef);
              this.ViewOrder(this.orderHeader);
            })
        }, () => { });

    } else {
      this.modal.warning("Please fill all required(*) fields.", this.viewRef);
    }

  }


  CreateDetailRow(tblIndex) {

    let currentMaxRow: number = this.orderDetail.length || 0;
    let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

    if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.orderDetail.push({
          STATUS: 'N'
        });
      }

      this.orderDetail = this.orderDetail.slice();
    }

  }

  private GetOrderClass() {
    this.http.post("wPMS", ["GetOrderClass", "null", "10"], { JOB_TYPE: 1 })
      .subscribe(result => {
        this.orderClassList = result.Table;
      })
  }

  CheckActiveRow(index, data) {
    if (this.IsEdit && data.ORDER_PART_NO) {
      this.CreateDetailRow(index);
      return true;
    }
    return false;
  }

  CheckOrder(ref) {
    if (this.orderHeader.ORDER_NO && this.orderHeader.CUSTOMER_CODE) {
    this.orderHeader.ORDER_NO = this.orderHeader.ORDER_NO?.toUpperCase();
      this.http.post("SL", "SNSE003_CheckOrder", this.orderHeader)
      .subscribe(res => {
        if (res?.Result === "Fail") {
          this.modal.error(res.Message, this.viewRef);
          if (ref === 'customercode') {
            this.orderHeader.CUSTOMER_CODE = null;
            this.orderHeader.CUSTOMER_NAME = null;
            this.orderHeader.CREDIT_LIMIT = null;
          } else if (ref === "orderno") {
            this.orderHeader.ORDER_NO = null;
          }

          return;
        }

        if (ref === "orderno") {
          this.orderHeader.ORDER_CLASS = this.orderHeader.ORDER_NO.substr(0, 1)
          this.OrderClassChange(this.orderHeader.ORDER_CLASS);
        }

      }, (err) => {
        this.modal.error(err.Message, this.viewRef);
      })
    }
  }

  OrderQtyChange(evt, data) {

    data.STATUS = 'N';
    data.REASON_CODE = '0';

    if (parseFloat(data.ORDER_QTY) % parseFloat(data.SALES_PACKAGE) > 0) {
      this.modal.error("ไม่ได้สั่งซื้อตาม Sales package!! (" + data.SALES_PACKAGE + ")", this.viewRef);
      data.ORDER_QTY = null;

      if (data.STATUS === 'N') {
        data.STATUS = 'W';
        data.REASON_CODE = '103';
      }
    }
    if (data.ORDER_QTY) {
      this.CheckCampaign(data);
    }
  }

  private CheckCampaign(data) {
      this.http.post("SL", "SNSE003_GetCampaignSpecialDiscount",
        {
          ORDER_NO: this.orderHeader.ORDER_NO,
          ORDER_CLASS: this.orderHeader.ORDER_CLASS,
          ORDER_DATE: this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE),
          SALES_CHANNEL: this.orderHeader.SALES_CHANNEL,
          CUSTOMER_CODE: this.orderHeader.CUSTOMER_CODE,
          BRANCH_CODE: this.orderHeader.BRANCH_CODE,
          PART_LIST: this.orderDetail.filter(v => v.ORDER_PART_NO === data._PART_NO)
        }).subscribe(res => {
          if(res.Table.length > 0) {
            res.Table.forEach(v => {
              let item = this.orderDetail.find(d => d.ORDER_PART_NO === v.PART_NO);
              item.DISCOUNT_GROUP = v.DISCOUNT_GROUP;
              item.DISCOUNT_CODE = v.DISCOUNT_CODE;
              item.CAMPAIGN_ID = v.CAMPAIGN_ID;
              item.CAMPAIGN_TYPE = v.CAMPAIGN_TYPE;
              item.CAMPAIGN_REF = v.CAMPAIGN_REF;
              item.DSP_CAMPAIGN_NAME = v.DSP_CAMPAIGN_NAME;
              item.DISCOUNT_RATE1 = v.DISCOUNT_RATE1;
              item.DISCOUNT_RATE2 = v.DISCOUNT_RATE2;
              item.DISCOUNT_RATE3 = v.DISCOUNT_RATE3;
              item.DISCOUNT_AMT = v.DISCOUNT_AMT;
              item.RETAIL_PRICE_B = v.RETAIL_PRICE_B;
              item.RETAIL_PRICE_C = v.RETAIL_PRICE_C;
              item.AVERAGE_COST = v.AVERAGE_COST;
              item.STANDARD_COST = v.STANDARD_COST;
              item.NET_AMT_B = v.NET_AMT_B;
              item.UNIT_PRICE_B = v.UNIT_PRICE_B;
              //item.UNIT_PRICE = v.UNIT_PRICE;
            })
          }

        }, (err) => {
          this.modal.warning("Found an error, please contact support team", this.viewRef);
          return;
        })
  }

  SetCustCode(value: any) {
    this.param.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.param.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.param.BRANCH_CODE = value.BRANCH_CODE;
    this.param.BRANCH_NAME = value.BRANCH_NAME;

  }

  SetOrderNo(value: any) {

    this.param.ORDER_NO = value.ORDER_NO;
  }

  SetCustomerLookup(cust: any) {

    this.orderHeader.CUSTOMER_CODE = cust?.CUSTOMER_CODE;
    this.orderHeader.CUSTOMER_NAME = cust?.CUST_NAME_TH;
    this.orderHeader.CREDIT_LIMIT = cust?.CREDIT_LIMIT;
    this.customerData.CREDIT_LIMIT = cust?.CREDIT_LIMIT;
    this.orderHeader.BRANCH_CODE = null;
    this.orderHeader.BRANCH_NAME = null;
    this.BranchLookupConfig.SourceParams.CUSTOMER_CODE = cust?.CUSTOMER_CODE;

    this.orderHeader.BRANCH_CODE = cust?.BRANCH_CODE;
    this.orderHeader.BRANCH_NAME = cust?.BRANCH_NAME_TH;
    this.customerData.ROUTE_CODE_RO = cust?.ROUTE_CODE_RO;
    this.customerData.TRANSPORT_RO = cust?.TRANSPORT_RO;
    this.customerData.ROUTE_CODE_EO = cust?.ROUTE_CODE_EO;
    this.customerData.TRANSPORT_EO = cust?.TRANSPORT_EO;

    this.OrderClassChange(this.orderHeader.ORDER_CLASS);
  }

  SetBranchLookup(branch) {
    this.orderHeader.BRANCH_CODE = branch.BRANCH_CODE;
    this.orderHeader.BRANCH_NAME = branch.BRANCH_NAME_TH;
    this.customerData.ROUTE_CODE_RO = branch.ROUTE_CODE_RO;
    this.customerData.TRANSPORT_RO = branch.TRANSPORT_RO;
    this.customerData.ROUTE_CODE_EO = branch.ROUTE_CODE_EO;
    this.customerData.TRANSPORT_EO = branch.TRANSPORT_EO;

    this.OrderClassChange(this.orderHeader.ORDER_CLASS);
  }

  public SetNewPartNo(part: any, data: any, index: number) {

    if (this.orderDetail.find((v, i) => index !== i && part.PART_NO && part.PART_NO == v.ORDER_PART_NO)) {
      this.modal.warning("Duplicate part number!", this.viewRef);
      data._PART_NO = null;
      return;
    }

    data.DISCOUNT_GROUP = null;
    data.RETAIL_PRICE_B = null;
    data.UNIT_PRICE_B = null;
    data.OVERSEA_FLAG_C = null;
    data.DISCOUNT_RATE1 = null;
    data.DISCOUNT_RATE2 = null;
    data.DISCOUNT_RATE3 = null;
    data.AVERAGE_COST = null;
    data.STANDARD_COST = null;
    data.RETAIL_PRICE = null;
    data.UNIT_PRICE = null;
    data.UNIT_EDP = null;
    data.RETAIL_PRICE_C = null;
    data.UNIT_PRICE_C = null;
    data.ORDER_PART_NO = null;
    data.ORDER_PART_NAME = null;
    data.UNIT_PRICE_EDITABLE = null;
    data.SALES_PACKAGE = null;
    data.ORDER_QTY = null;

    if (part.PART_NO) {

      //Check customer oil
      this.http.post("SL", "SNSE003_CheckOilPart", {
        PartNo: part.PART_NO,
        CustomerCode: this.orderHeader.CUSTOMER_CODE,
        BranchCode: this.orderHeader.BRANCH_CODE,
        OrderClass: this.orderHeader.ORDER_CLASS
      }).subscribe(res => {
        if (res.result === "Done") {
          this.http.get("SL", "SNSE003_SNSC003"
            , this.orderHeader.CUSTOMER_CODE
            , this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE)
            , part.PART_NO
            , this.orderHeader.ORDER_CLASS
            , part.VENDOR_CODE)
            .subscribe(result => {
              data.DISCOUNT_GROUP = result.DISC_GROUP;
              data.RETAIL_PRICE_B = result.RETAIL_D;
              data.UNIT_PRICE_B = result.UNIT_D;
              data.OVERSEA_FLAG_C = result.OVERSEAFLAG;
              data.DISCOUNT_RATE1 = result.DISC_1;
              data.DISCOUNT_RATE2 = result.DISC_2;
              data.DISCOUNT_RATE3 = result.DISC_3;
              data.AVERAGE_COST = result.AVG_COST;
              data.STANDARD_COST = result.STD_COST;

              this.orderHeader.DISCOUNT_GROUP = data.DISCOUNT_GROUP;
              data.RETAIL_PRICE = data.RETAIL_PRICE_B;
              data.UNIT_PRICE = data.UNIT_PRICE_B;
              data.UNIT_EDP = data.UNIT_PRICE
              data.RETAIL_PRICE_C = 0;
              data.UNIT_PRICE_C = 0;
              data.ORDER_PART_NO = part.PART_NO;
              if (data.RETAIL_PRICE == 1) {
                this.modal.warning("This parts not ready to sell!", this.viewRef);
                data.DISCOUNT_GROUP = null;
                data.RETAIL_PRICE_B = null;
                data.UNIT_PRICE_B = null;
                data.OVERSEA_FLAG_C = null;
                data.DISCOUNT_RATE1 = null;
                data.DISCOUNT_RATE2 = null;
                data.DISCOUNT_RATE3 = null;
                data.AVERAGE_COST = null;
                data.STANDARD_COST = null;
                data.RETAIL_PRICE = null;
                data.UNIT_PRICE = null;
                data.UNIT_EDP = null;
                data.RETAIL_PRICE_C = null;
                data.UNIT_PRICE_C = null;
                data.ORDER_PART_NO = null;
                data.ORDER_PART_NAME = null;
                data.UNIT_PRICE_EDITABLE = null;
                data.SALES_PACKAGE = null;
                data.ORDER_QTY = null;
                data._PART_NO = null;
                data.ORDER_PART_NAME = null;
                data.UNIT_PRICE_EDITABLE = null;
                data.SALES_PACKAGE = null;
                return;
              }
            }, (err) => {
              console.log(err, 'err');
              this.modal.warning("Found an error, please contract support team", this.viewRef);
            });

          data._PART_NO = part.PART_NO;
          data.ORDER_PART_NAME = part.PART_NAME;
          data.UNIT_PRICE_EDITABLE = part.EDITABLE;
          data.SALES_PACKAGE = part.SALES_PACKAGE;
        } else {
          this.modal.warning(res.result, this.viewRef);
        }
      })
    }

  }

  SetRouteCode(route) {
    this.orderHeader.ROUTE_CODE = route.ROUTE_CODE;
  }

  SetTransCode(trans) {
    this.orderHeader.TRANSPORT_CODE = trans.TRANSPORT_CODE;
    this.orderHeader._TRANSPORT = trans.LOOKUP_TRANSPORT;
  }

  OrderClassChange(orderClass) {
    this.orderHeader.ORDER_TYPE = null
    if (orderClass) {
      this.orderHeader.ROUTE_CODE = (orderClass == '1' || orderClass == '2') ? this.customerData.ROUTE_CODE_EO : this.customerData.ROUTE_CODE_RO;
      this.orderHeader.TRANSPORT_CODE = (orderClass == '1' || orderClass == '2') ? this.customerData.TRANSPORT_EO : this.customerData.TRANSPORT_RO;
      this.orderHeader.ORDER_TYPE = this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(orderClass) >= 0)[0]?.ORDER_TYPE;

      this.autoCompleteTransport.textInput = this.orderHeader.TRANSPORT_CODE.toString();
      this.autoCompleteTransport.query()
    } else {
      this.orderHeader.ROUTE_CODE = null;
      this.orderHeader.TRANSPORT_CODE = null;
    }

    if (orderClass === '7') {
      this.orderHeader.BO_FLAG = 'Y';
    }
  }


  CalTotalCreditAmount() {
    return this.orderDetail.reduce((prev, currObj) => prev + (parseFloat(currObj.ORDER_QTY) * parseFloat(currObj.UNIT_PRICE)), 0);
  }

  CalTotalAmt() {
    return this.orderDetail.reduce((o, n) => o + (parseFloat(n.ORDER_QTY || 0) * parseFloat(n.UNIT_PRICE || 0)), 0)
  }
  CalTotalQty() {
    return this.orderDetail.reduce((o, n) => o + parseFloat(n.ORDER_QTY || 0), 0)
  }

  BackToGrid() {
    this.IsSearch = true;
  }

  public DeleteRow(index) {
    this.modal.confirm("Are you sure to remove this row?", this.viewRef
      , () => {
        this.orderDetail.splice(index, 1);
        this.orderDetail = [...this.orderDetail];
        //this.ExpenseChanged()
        //this.CheckCampaign()
      }, () => { })
  }

  filterOrderTypeList() {
    return this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(this.orderHeader.ORDER_CLASS) >= 0)
  }

  public CancelAutoCompletePart(data: any) {
    data._PART_NO = data.ORDER_PART_NO
  }
}
