import { Component, ElementRef, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { NgForm } from '@angular/forms';
import * as moment from 'moment';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import { DialogTemplateCallerComponent } from 'src/app/components/shared/dialog-template-caller/dialog-template-caller.component';
import { Globals } from 'src/app/core/global';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'SNSE004_A',
  templateUrl: 'SNSE004_A.component.html',
  styleUrls: ['SNSE004_A.component.css']
})

export class SNSE004_AComponent implements OnInit {
  public param: any = {};
  public orderList: Array<any>;
  public orderHeader: any = {};
  public orderDetail: Array<any>;
  public customerData: any;
  public orderClassList: Array<any>;
  public orderTypeList: Array<any> = [
    { ORDER_TYPE: '1', DESCRIPTION: 'E/O', REF_ORDER_CLASS: ['1', '2'] },
    // { ORDER_TYPE: '2', DESCRIPTION: 'Firm', REF_ORDER_CLASS: ['3', '6'] },
    { ORDER_TYPE: '3', DESCRIPTION: 'R/O', REF_ORDER_CLASS: ['1', '2', '4'] },
    { ORDER_TYPE: '4', DESCRIPTION: 'C/O', REF_ORDER_CLASS: ['5'] },
    // { ORDER_TYPE: '6', DESCRIPTION: 'Oil', REF_ORDER_CLASS: ['7'] }
  ];
  public IsEdit: boolean = false;
  public IsSearch: boolean = true;
  public hideCols: any = [];
  public orderNoParam: string;

  @ViewChild("headerForm") headerForm: NgForm;
  @ViewChild("dialogPartsInfo") dialogPartsInfo: DialogTemplateCallerComponent;
  @ViewChild("dialogBOList") dialogBOList: DialogTemplateCallerComponent;
  @ViewChild("autoCompleteTransport") autoCompleteTransport: AutoCompleteQueryComponent;

  public CustomerSearchConfig: any = {
    ColName: "LOOKUP_CUST",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetSearchCust",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };

  public CustomerLookupConfig: any = {
    ColName: "LOOKUP_CUST",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetCustomer",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };

  public BranchLookupConfig: any = {
    ColName: "LOOKUP_BRANCH_NAME",
    PlaceholderText: "Select Branch",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetBranchCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: null
    },
    UpperCase: true
  };

  public BranchSearchConfig: any = {
    ColName: "LOOKUP_BRANCH",
    PlaceholderText: "Select Branch",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetSearchBrach",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.param
    },
    UpperCase: true
  };

  public OrderNoLookupConfig: any = {
    ColName: "LOOKUP_ORDER",
    PlaceholderText: "Select Order No",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetCustCodeOrderNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.param
    },
    UpperCase: true
  };

  public DocNoLookupConfig: any = {
    ColName: "LOOKUP_DOCUMENT_NO",
    PlaceholderText: "Select Document No",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetDocumentNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  };

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetParts",
    SourceParams: {
      CUSTOMER_CODE: null,
      BRANCH_CODE: null,
      ORDER_CLASS: null,
    },
    NoOfShowRecords: 20
  }

  public RouteLookupConfig: any = {
    ColName: "LOOKUP_ROUTE",
    PlaceholderText: "Select Delivery Route",
    SourceController: "wPMS",
    SourceMethodName: "GetDeliveryRoute",
    NoOfShowRecords: 40,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public TransportLookupConfig: any = {
    ColName: "LOOKUP_TRANSPORT",
    PlaceholderText: "Select Transport Code",
    SourceController: "wPMS",
    SourceMethodName: "GetTransportCode",
    NoOfShowRecords: 40,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public ChangeRouteCustLookupConfig: any = {
    ColName: "LOOKUP_CUST",
    PlaceholderText: "Select Route No",
    SourceController: "SL",
    SourceMethodName: "SNSE004_GetChangeRouteCust",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      ORDER_CLASS: null,
      ORDER_TYPE: null
    },
    UpperCase: true
  }

  public SetChangeRouteCustLookup(cust: any) {
    if (cust) {
      this.orderHeader.CHANGE_ROUTE_CUSTOMER = cust.CUSTOMER_CODE;
      this.orderHeader.DSP_CHANGE_ROUTE_CUST_NAME = cust.DSP_CUST_NAME;
      this.orderHeader.CHANGE_ROUTE_BRANCH = cust.BRANCH_CODE;
      this.orderHeader.ROUTE_CODE = cust.ROUTE_CODE;
      this.orderHeader._TRANSPORT = cust.TRANSPORT_CODE;

      if (this.orderHeader.DELIVERY_CODE = '01') {
        this.orderHeader.ROUTE_CODE = 'A9999';
        this.orderHeader._TRANSPORT = '99999';
        this.orderHeader.TRANSPORT_CODE = '99999';
      }
    }
    else {
      this.orderHeader.CHANGE_ROUTE_BRANCH = null;
      this.orderHeader.DSP_CHANGE_ROUTE_CUST_NAME = '';
      this.orderHeader.ROUTE_CODE = this.orderHeader.CUR_ROUTE_CODE;
      this.orderHeader._TRANSPORT = this.orderHeader.CUR_TRANSPORT_CODE;
      this.orderHeader.TRANSPORT_CODE = this.orderHeader.CUR_TRANSPORT_CODE;
    }

    if (!this.orderHeader.CHANGE_ROUTE_CUSTOMER) {
      this.orderHeader.ROUTE_CODE = this.orderHeader.CUR_ROUTE_CODE;
      this.orderHeader._TRANSPORT = this.orderHeader.CUR_TRANSPORT_CODE;
      this.orderHeader.TRANSPORT_CODE = this.orderHeader.CUR_TRANSPORT_CODE;
    }
  }

  constructor(
    protected http: HttpService,
    protected modal: ModalService,
    protected viewRef: ViewContainerRef,
    protected el: ElementRef,
    protected global: Globals
  ) { }

  ngOnInit() {
    this.GetOrderClass();
    this.AddOrder();

  }

  CheckDateControl() {
    return this.http.GetSystemDate()
      .subscribe((curDate: any) => {
        if (curDate && moment(curDate).format('YYYYMMDD') !== moment().format('YYYYMMDD')) {
          this.modal.error("The current system date is not allow to create the order",this.viewRef)
          this.IsEdit = false;
          this.IsSearch = true;
        }
      })
  }

  public Search() {

    if (!this.param.CUSTOMER_CODE && !this.param.DOCUMENT_NO) {
      this.modal.warning("Please input customer code or invoice no.!!!", this.viewRef);
      return;
    }
    this.orderList = [];
    this.http.post("SL", "SNSE004_GetOrderList", this.param)
      .subscribe(res => {
        this.orderList = res.Table;
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewRef);
      });

  }

  public ViewOrder(order) {
    this.orderDetail = [];
    this.hideCols = ["CHK_STATUS"]

    this.http.post("SL", "SNSE004_GetOrderDetail", order)
      .subscribe(res => {
        this.orderHeader = res.Table[0];
        this.orderDetail = res.Table1;

        this.orderHeader._TRANSPORT = this.orderHeader.TRANSPORT_CODE;

        this.IsSearch = false;
        this.IsEdit = false;

        this.orderDetail.map(v => {
          v._PART_NO = v.ORDER_PART_NO;
        })

        setTimeout(() => {
          this.autoCompleteTransport.textInput = this.orderHeader._TRANSPORT.toString();
          this.autoCompleteTransport.query()
        }, 200)
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      })
  }

  public AddOrder() {
    this.IsSearch = false;
    this.IsEdit = true;
    this.orderDetail = [];
    this.orderHeader = {};
    this.customerData = {};
    this.orderHeader.ORDER_DATE = moment();
    this.orderHeader.BO_FLAG = 'Y';
    this.orderHeader.DELIVERY_CODE = '03';
    this.orderHeader.PRINT_PICKING_CARD = true;
    this.CreateDetailRow(0);
    this.hideCols = []
    // this.CheckDateControl();


    this.orderDetail = this.orderDetail.slice();
  }

  public Save() {
    if (this.headerForm.valid) {
      //Check Credit 
      if (this.orderHeader.PAYMENT_TERM != 0 && (this.orderHeader.BRANCH_CODE_SAP == 'TH2' || !this.orderHeader.BRANCH_CODE_SAP)) {
        if ((this.orderHeader.CURRENT_CREDIT_BALANCE - (this.CalTotalAmt() + (this.CalTotalAmt() * (this.orderHeader.VAT_RATE / 100)))) < 0) {
          this.modal.error("Not Enough Credit for this order. <br/> Available Credit : " + (this.orderHeader.CURRENT_CREDIT_BALANCE).toLocaleString() + "<br/>" + ' Amount(Inc. Vat) : ' + (this.CalTotalAmt() + (this.CalTotalAmt() * (this.orderHeader.VAT_RATE / 100))).toLocaleString(), this.viewRef, () => { }, { height: "300px" });
          return;
        }
      }

      if (this.orderHeader.ORDER_CLASS == "7") {
        if (this.orderHeader.CREDIT_LIMIT - this.CalTotalCreditAmount() < 0) {
          this.modal.error("THIS CUSTOMER OVER CREDIT LIMIT.", this.viewRef);
          return;
        }
      }
      let detailRows = this.orderDetail.filter(v => v.ORDER_PART_NO)

      if (detailRows.length <= 0) {
        this.modal.warning("Please select at least 1 Parts.", this.viewRef)
        return;
      }
      if (detailRows.find(v => ((!v.ORDER_QTY || v.ORDER_QTY === 0) || (!v.UNIT_PRICE || v.UNIT_PRICE === 0)))) {
        this.modal.warning("Some record in transaction detail have unfinished detail.", this.viewRef);
        return;
      }
      this.modal.confirm('Do you want to save changed you have made?', this.viewRef,
        () => {
          this.orderHeader.PART_LIST = this.orderDetail.filter(v => v.ORDER_PART_NO);
          this.orderHeader._ORDER_DATE = this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE);

          this.http.post("SL", "SNSE004_A_SaveDocument", this.orderHeader)
            .subscribe(res => {

              if (res.STATUS_TEXT =='DONE') {               
                this.modal.info('Transaction complete: applied and saved.', this.viewRef);
              }
              else if (res == true) { //Error Incorrect date or Batch running
                return;
              }
              else {
                this.modal.info('Transaction complete: saved. But there is no O/H qty.', this.viewRef);
              }

              this.ViewOrder(this.orderHeader);

            }, (err) => {
              console.log(err, 'err')
              this.modal.warning("Found an error, please contact support team", this.viewRef);
            });
        }, () => { });

    } else {
      this.modal.warning("Please fill all required(*) fields.", this.viewRef);
    }
  }

  public CreateDetailRow(tblIndex) {
    if (this.IsEdit) {
      let currentMaxRow: number = this.orderDetail.length || 0;
      let nextMaxRow: number = 12; //Limit to 12  //(Math.floor(currentMaxRow / 10) + 1) * 10;

      // if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.orderDetail.push({
          STATUS: 'N'
        });
      }
      this.orderDetail = this.orderDetail.slice();
      // }
    }
  }

  private GetOrderClass() {
    this.http.post("wPMS", ["GetOrderClass", "null", "10"], { JOB_TYPE: 1 })
      .subscribe(result => {
        this.orderClassList = result.Table.filter(v => ['1','2','3','4','5'].indexOf(v.ORDER_CLASS) >= 0);
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      })
  }

  public CheckActiveRow(index, data) {
    if (this.IsEdit && data.ORDER_PART_NO) {
      this.CreateDetailRow(index);
      return true;
    }
    return false;
  }

  public CheckOrder(ref) {
    if (this.orderHeader.ORDER_NO && this.orderHeader.CUSTOMER_CODE) {
      this.orderHeader.ORDER_NO = this.orderHeader.ORDER_NO?.toUpperCase();
      this.http.post("SL", "SNSE004_CheckOrder", this.orderHeader)
        .subscribe(res => {
          if (res?.Result === "Fail") {
            this.modal.error(res.Message, this.viewRef);
            if (ref === 'customercode') {
              this.orderHeader.CUSTOMER_CODE = null;
              this.orderHeader.CUSTOMER_NAME = null;
              this.orderHeader.CREDIT_LIMIT = null;
            } else if (ref === "orderno") {
              this.orderHeader.ORDER_NO = null;
            }

            return;
          }

          if (ref === "orderno") {
            let ChkOrderClass5 = this.orderHeader.ORDER_NO.substr(0, 2);
            if (ChkOrderClass5 == '5X' || ChkOrderClass5 == '5W' || ChkOrderClass5 == '5A') {
              this.modal.warning("Order Class 5X 5W 5A discount are not support in this form!", this.viewRef);
              this.orderHeader.ORDER_NO = null;
              this.orderHeader.ORDER_CLASS = null;
              this.OrderClassChange(this.orderHeader.ORDER_CLASS);
              return
            }
            let orderClass = this.orderHeader.ORDER_NO.substr(0, 1);

            if (this.orderClassList.find(v => v.ORDER_CLASS.toString() === orderClass.toString())) {
              this.orderHeader.ORDER_CLASS = this.orderHeader.ORDER_NO.substr(0, 1)
              this.OrderClassChange(this.orderHeader.ORDER_CLASS);
            } else {
              this.modal.warning("Order class: " + orderClass + " Not found!", this.viewRef);
              this.orderHeader.ORDER_NO = null;
              this.orderHeader.ORDER_CLASS = null;
              this.OrderClassChange(this.orderHeader.ORDER_CLASS);
            }
            this.PartNoLookupConfig.SourceParams.ORDER_CLASS = this.orderHeader.ORDER_CLASS;
          }

        }, (err) => {
          this.modal.error(err.Message, this.viewRef);
        })
    }
  }

  public ValidateHeader() {
    this.http.get("SL", "SNSE004_Validate"
      , this.orderHeader.CUSTOMER_CODE
      , this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE))
      .subscribe(result => {
        console.log(result);
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      })
  }

  public OrderQtyChange(evt, data) {

    data.STATUS = 'N';
    data.REASON_CODE = '0';

    if (parseFloat(data.ORDER_QTY) % parseFloat(data.SALES_PACKAGE) > 0) {
      this.modal.error("ไม่ได้สั่งซื้อตาม Sales package!! (" + data.SALES_PACKAGE + ")", this.viewRef);
      data.ORDER_QTY = null;

      if (data.STATUS === 'N') {
        data.STATUS = 'W';
        data.REASON_CODE = '103';
      }
    }
    if (data.ORDER_QTY > 0) {
      this.CheckCampaign(data);
    }
  }

  public CheckCampaign(data) {
    this.http.post("SL", "SNSE003_GetCampaignSpecialDiscount",
      {
        ORDER_NO: this.orderHeader.ORDER_NO,
        ORDER_CLASS: this.orderHeader.ORDER_CLASS,
        ORDER_DATE: this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE),
        SALES_CHANNEL: this.orderHeader.SALES_CHANNEL,
        CUSTOMER_CODE: this.orderHeader.CUSTOMER_CODE,
        BRANCH_CODE: this.orderHeader.BRANCH_CODE,
        PART_LIST: this.orderDetail.filter(v => v.ORDER_PART_NO === data._PART_NO)
      }).subscribe(res => {
        if (res.Table.length > 0) {
          res.Table.forEach(v => {
            let item = this.orderDetail.find(d => d.ORDER_PART_NO === v.PART_NO);
            item.DISCOUNT_GROUP = v.DISCOUNT_GROUP;
            item.DISCOUNT_CODE = v.DISCOUNT_CODE;
            item.CAMPAIGN_ID = v.CAMPAIGN_ID;
            item.CAMPAIGN_TYPE = v.CAMPAIGN_TYPE;
            item.CAMPAIGN_REF = v.CAMPAIGN_REF;
            item.DSP_CAMPAIGN_NAME = v.DSP_CAMPAIGN_NAME;
            item.DISCOUNT_RATE1 = v.DISCOUNT_RATE1;
            item.DISCOUNT_RATE2 = v.DISCOUNT_RATE2;
            item.DISCOUNT_RATE3 = v.DISCOUNT_RATE3;
            item.DISCOUNT_AMT = v.DISCOUNT_AMT;
            item.RETAIL_PRICE_B = v.RETAIL_PRICE_B;
            item.RETAIL_PRICE_C = v.RETAIL_PRICE_C;
            item.AVERAGE_COST = v.AVERAGE_COST;
            item.STANDARD_COST = v.STANDARD_COST;
            item.NET_AMT_B = v.NET_AMT_B;
            item.UNIT_PRICE_B = v.UNIT_PRICE_B;
            item.UNIT_PRICE = v.UNIT_PRICE;
          })
        }

      }, (err) => {
        this.modal.warning("Found an error, please contact support team", this.viewRef);
        return;
      })
  }

  public OpenPartsInfo() {
    this.dialogPartsInfo.OpenDialog();
  }

  public OpenBOETPList() {
    this.dialogBOList.OpenDialog();
  }

  public SetCustCode(value: any) {
    this.param.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.param.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.param.BRANCH_CODE = value.BRANCH_CODE;
    this.param.BRANCH_NAME = value.BRANCH_NAME;
  }

  public SetOrderNo(value: any) {
    this.param.ORDER_NO = value.LOOKUP_ORDER;
  }

  public SetDocNo(value: any) {
    this.param.DOCUMENT_NO = value.DOCUMENT_NO;
    this.param.DOCUMENT_DATE = value.DOCUMENT_DATE;
  }

  public SetCustomerLookup(cust: any) {

    this.orderHeader.CUSTOMER_CODE = cust?.CUSTOMER_CODE;
    this.orderHeader.CUSTOMER_NAME = cust?.CUST_NAME_TH;
    this.orderHeader.CREDIT_LIMIT = cust?.CREDIT_LIMIT;
    this.customerData.CREDIT_LIMIT = cust?.CREDIT_LIMIT;
    this.orderHeader.PAYMENT_TERM = cust?.PAYMENT_TERM;
    this.orderHeader.PAYMENT_TYPE = cust?.PAYMENT_TYPE;
    this.orderHeader.CUST_ADDR = cust?.CUST_ADDR;
    this.orderHeader.BRANCH_CODE_SAP = cust?.BRANCH_CODE_SAP;
    this.orderHeader.CURRENT_CREDIT_BALANCE = cust?.CURRENT_CREDIT_BALANCE;
    this.orderHeader.VAT_RATE = cust?.VAT_RATE;

    this.BranchLookupConfig.SourceParams.CUSTOMER_CODE = cust?.CUSTOMER_CODE;

    this.orderHeader.BRANCH_CODE = cust?.BRANCH_CODE;
    this.orderHeader.BRANCH_NAME = cust?.BRANCH_NAME_TH;
    this.orderHeader.BRANCH_DSP = cust?.BRANCH_DSP;
    this.customerData.ROUTE_CODE_RO = cust?.ROUTE_CODE_RO;
    this.customerData.TRANSPORT_RO = cust?.TRANSPORT_RO;
    this.customerData.ROUTE_CODE_EO = cust?.ROUTE_CODE_EO;
    this.customerData.TRANSPORT_EO = cust?.TRANSPORT_EO;

    this.OrderClassChange(this.orderHeader.ORDER_CLASS);
    this.PartNoLookupConfig.SourceParams.CUSTOMER_CODE = cust?.CUSTOMER_CODE;
    this.PartNoLookupConfig.SourceParams.BRANCH_CODE = cust?.BRANCH_CODE;
  }

  public SetBranchLookup(branch) {
    this.orderHeader.BRANCH_CODE = branch.BRANCH_CODE;
    this.orderHeader.BRANCH_NAME = branch.BRANCH_NAME_TH;
    this.customerData.ROUTE_CODE_RO = branch.ROUTE_CODE_RO;
    this.customerData.TRANSPORT_RO = branch.TRANSPORT_RO;
    this.customerData.ROUTE_CODE_EO = branch.ROUTE_CODE_EO;
    this.customerData.TRANSPORT_EO = branch.TRANSPORT_EO;
    this.orderHeader.BRANCH_CODE_SAP = branch.BRANCH_CODE_SAP;
    this.orderHeader.CURRENT_CREDIT_BALANCE = branch.CURRENT_CREDIT_BALANCE;
    this.orderHeader.VAT_RATE = branch.VAT_RATE;

    this.OrderClassChange(this.orderHeader.ORDER_CLASS);
    this.PartNoLookupConfig.SourceParams.BRANCH_CODE = branch.BRANCH_CODE;
  }

  public SetNewPartNo(part: any, data: any, index: number) {
    if (this.orderDetail.find((v, i) => index !== i && part.PART_NO && part.PART_NO == v.ORDER_PART_NO)) {
      this.modal.warning("Duplicate part number!", this.viewRef);
      data._PART_NO = null;
      return;
    }

    if (part.LOCK_SALES === 117) {
      this.modal.confirm(part.REASON_NAME + "! <br/>Are you sure to continue?", this.viewRef
        , () => this.GetCampaignDiscount(part, data)
        , () => {
          data._PART_NO = null;
        })
    } else if (part.LOCK_SALES > 0) {
      this.modal.warning("This Part No. is lock!", this.viewRef)
      data._PART_NO = null;
    } else {
      this.GetCampaignDiscount(part, data)
    }
  }

  public GetCampaignDiscount(part: any, data: any) {

    data.ORDER_PART_NO = null;
    data.ORDER_QTY = null;
    data.DISCOUNT_GROUP = null;
    data.RETAIL_PRICE_B = null;
    data.UNIT_PRICE_B = null;
    data.OVERSEA_FLAG_C = null;
    data.DISCOUNT_RATE1 = null;
    data.DISCOUNT_RATE2 = null;
    data.DISCOUNT_RATE3 = null;
    data.REF_DISCOUNT_RATE1 = null;
    data.REF_DISCOUNT_RATE2 = null;
    data.REF_DISCOUNT_RATE3 = null;
    data.AVERAGE_COST = null;
    data.STANDARD_COST = null;
    data.RETAIL_PRICE = null;
    data.UNIT_PRICE = null;
    data.UNIT_EDP = null;
    data.RETAIL_PRICE_C = null;
    data.UNIT_PRICE_C = null;
    data.ORDER_PART_NAME = null;
    data.UNIT_PRICE_EDITABLE = null;
    data.SALES_PACKAGE = null;
    data.OH_QTY = null;
    data.QUOTA_CHECK = null;
    data.REMAIN_UNIT = null;

    if(part.PART_NO) {
      this.http.get("SL", "SNSE004_SNSC003"
        , this.orderHeader.CUSTOMER_CODE
        , this.global.ConvertDateToFormatSTD(this.orderHeader.ORDER_DATE)
        , part.PART_NO
        , this.orderHeader.ORDER_CLASS
        , part.VENDOR_CODE)
        .subscribe(result => {
          data.DISCOUNT_GROUP = result.DISC_GROUP;
          data.RETAIL_PRICE_B = result.RETAIL_D;
          data.UNIT_PRICE_B = result.UNIT_D;
          data.OVERSEA_FLAG_C = result.OVERSEAFLAG;
          data.DISCOUNT_RATE1 = result.DISC_1;
          data.DISCOUNT_RATE2 = result.DISC_2;
          data.DISCOUNT_RATE3 = result.DISC_3;
          data.REF_DISCOUNT_RATE1 = result.DISC_1;
          data.REF_DISCOUNT_RATE2 = result.DISC_2;
          data.REF_DISCOUNT_RATE3 = result.DISC_3;
          data.AVERAGE_COST = result.AVG_COST;
          data.STANDARD_COST = result.STD_COST;

          this.orderHeader.DISCOUNT_GROUP = data.DISCOUNT_GROUP;
          data.RETAIL_PRICE = data.RETAIL_PRICE_B;
          data.UNIT_PRICE = data.UNIT_PRICE_B;
          data.UNIT_EDP = data.UNIT_PRICE
          data.RETAIL_PRICE_C = 0;
          data.UNIT_PRICE_C = 0;
          data.ORDER_PART_NO = part.PART_NO;
        }, (err) => {
          console.log(err, 'err');
          this.modal.warning("Found an error, please contact support team", this.viewRef);
        })
    }

    data._PART_NO = part.PART_NO;
    data.ORDER_PART_NAME = part.PART_NAME;
    data.UNIT_PRICE_EDITABLE = part.EDITABLE;
    data.SALES_PACKAGE = part.SALES_PACKAGE;
    data.OH_QTY = part.OH_QTY;
    data.QUOTA_CHECK = part.QUOTA_CHECK;
    data.REMAIN_UNIT = part.REMAIN_UNIT;

    console.log(data)
  }

  public SetRouteCode(route) {
    this.orderHeader.ROUTE_CODE = route.ROUTE_CODE;
  }

  public SetTransCode(trans) {
    this.orderHeader.TRANSPORT_CODE = trans.TRANSPORT_CODE;
    this.orderHeader.ROUTE_NAME = trans.TRANS_NAME_EN;
    this.orderHeader._TRANSPORT = trans.LOOKUP_TRANSPORT;

    // this.GetTransportName(this.orderHeader.TRANSPORT_CODE);
  }

  public OrderClassChange(orderClass) {
    this.orderHeader.ORDER_TYPE = null
    if (this.orderHeader.PAYMENT_TYPE == '1') {
      this.orderHeader.DELIVERY_CODE = '01';
      this.orderHeader.ROUTE_CODE = 'A9999';
      this.orderHeader.TRANSPORT_CODE = '99999';
      this.orderHeader._TRANSPORT = '99999';
      this.orderHeader.ROUTE_NAME = 'รอรับทันที'
      this.orderHeader.ORDER_TYPE = this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(orderClass) >= 0)[0]?.ORDER_TYPE;

      this.orderHeader.CUR_TRANSPORT_CODE = this.orderHeader.TRANSPORT_CODE;
      this.orderHeader.CUR_ROUTE_CODE = this.orderHeader.ROUTE_CODE;

    } else {
      this.orderHeader.DELIVERY_CODE = '03';
      if (orderClass) {
        this.orderHeader.ROUTE_CODE = (orderClass == '1' || orderClass == '2') ? this.customerData.ROUTE_CODE_EO : this.customerData.ROUTE_CODE_RO;
        this.orderHeader.TRANSPORT_CODE = (orderClass == '1' || orderClass == '2') ? this.customerData.TRANSPORT_EO : this.customerData.TRANSPORT_RO;
        this.orderHeader.ORDER_TYPE = this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(orderClass) >= 0)[0]?.ORDER_TYPE;

        this.orderHeader.CUR_TRANSPORT_CODE = this.orderHeader.TRANSPORT_CODE;
        this.orderHeader.CUR_ROUTE_CODE = this.orderHeader.ROUTE_CODE;

        this.autoCompleteTransport.textInput = this.orderHeader.TRANSPORT_CODE.toString();
        this.autoCompleteTransport.query()
        // this.GetTransportName(this.orderHeader.TRANSPORT_CODE);
      } else {
        this.orderHeader.ROUTE_CODE = null;
        this.orderHeader.TRANSPORT_CODE = null;

        this.orderHeader.CUR_TRANSPORT_CODE = this.orderHeader.TRANSPORT_CODE;
        this.orderHeader.CUR_ROUTE_CODE = this.orderHeader.ROUTE_CODE;
      }
    }

    this.ChangeRouteCustLookupConfig.SourceParams.ORDER_CLASS = orderClass;
    this.ChangeRouteCustLookupConfig.SourceParams.ORDER_TYPE = this.orderHeader.ORDER_TYPE;
  }

  public DeliveryCodeChange(evt) {
    if (evt.value === '01') {
      this.orderHeader.ROUTE_CODE = 'A9999';
      this.orderHeader.TRANSPORT_CODE = '99999';
      this.orderHeader._TRANSPORT = '99999';
      this.orderHeader.ROUTE_NAME = 'รอรับทันที';
    } else if (evt.value === '02') {
      this.orderHeader.ROUTE_CODE = (this.orderHeader.ORDER_CLASS == '1' || this.orderHeader.ORDER_CLASS == '2') ? this.customerData.ROUTE_CODE_EO : this.customerData.ROUTE_CODE_RO;
      this.orderHeader.TRANSPORT_CODE = (this.orderHeader.ORDER_CLASS == '1' || this.orderHeader.ORDER_CLASS == '2') ? this.customerData.TRANSPORT_EO : this.customerData.TRANSPORT_RO;
      this.orderHeader.ORDER_TYPE = this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(this.orderHeader.ORDER_CLASS) >= 0)[0]?.ORDER_TYPE;
      this.orderHeader.ROUTE_NAME = 'รอรับ';
    } else {

      this.orderHeader.ROUTE_CODE = (this.orderHeader.ORDER_CLASS == '1' || this.orderHeader.ORDER_CLASS == '2') ? this.customerData.ROUTE_CODE_EO : this.customerData.ROUTE_CODE_RO;
      this.orderHeader.TRANSPORT_CODE = (this.orderHeader.ORDER_CLASS == '1' || this.orderHeader.ORDER_CLASS == '2') ? this.customerData.TRANSPORT_EO : this.customerData.TRANSPORT_RO;
      this.orderHeader.ORDER_TYPE = this.orderTypeList.filter(v => v.REF_ORDER_CLASS.indexOf(this.orderHeader.ORDER_CLASS) >= 0)[0]?.ORDER_TYPE;

      this.autoCompleteTransport.textInput = this.orderHeader.TRANSPORT_CODE?.toString();
      this.autoCompleteTransport.query()
    }
  }

  public DeleteRow(index) {
    this.modal.confirm("Are you sure to remove this row?", this.viewRef
      , () => {
        this.orderDetail.splice(index, 1);
        this.CreateDetailRow(index)
        // this.orderDetail = [...this.orderDetail];
        //this.ExpenseChanged()
        //this.CheckCampaign();
      }, () => { })
  }

  public CalTotalCreditAmount() {
    return this.orderDetail.reduce((prev, currObj) => prev + (parseFloat(currObj.ORDER_QTY) * parseFloat(currObj.UNIT_PRICE)), 0);
  }

  public CalTotalAmt() {

    if (this.IsEdit) {
      return this.orderDetail.reduce((o, n) => o + (parseFloat(n.ORDER_QTY || 0) * parseFloat(n.UNIT_PRICE || 0)), 0);
    } else {
      return this.orderHeader.TOTAL_NET_B;
    }
  }

  public CalTotalQty() {
    if (this.IsEdit) {
      return this.orderDetail.reduce((o, n) => o + parseFloat(n.ORDER_QTY || 0), 0);
    } else {
      return this.orderHeader.TOTAL_ORDER_QTY;
    }
  
  }

  public BackToGrid() {
    this.IsSearch = true;
    this.param = {};
  }

  public filterOrderTypeList() {
    return this.orderTypeList.filter(v => !this.orderHeader.ORDER_CLASS || v.REF_ORDER_CLASS.indexOf(this.orderHeader.ORDER_CLASS) >= 0)
  }

  public CancelAutoCompletePart(data: any) {
    data._PART_NO = data.ORDER_PART_NO
  }

  public RecalUnitPrice(data, fieldName) {
    if (fieldName == 'DISCOUNT_RATE1' || 'DISCOUNT_RATE2' || 'DISCOUNT_RATE3') {
      if (data.REF_DISCOUNT_RATE1 != data.DISCOUNT_RATE1 || data.REF_DISCOUNT_RATE2 != data.DISCOUNT_RATE2 || data.REF_DISCOUNT_RATE3 != data.DISCOUNT_RATE3) {
        data.REF_DISCOUNT_RATE1 = data.DISCOUNT_RATE1;
        data.REF_DISCOUNT_RATE2 = data.DISCOUNT_RATE2;
        data.REF_DISCOUNT_RATE3 = data.DISCOUNT_RATE3;
        data.UNIT_PRICE = Math.round(data.RETAIL_PRICE_B *
          ((100 - data.DISCOUNT_RATE1) / 100) *
          ((100 - data.DISCOUNT_RATE2) / 100) *
          ((100 - data.DISCOUNT_RATE3) / 100));
        data.UNIT_PRICE_B = data.UNIT_PRICE;
      }
    }

    if (fieldName == 'UNIT_PRICE') {
      if (data.UNIT_PRICE != data.UNIT_PRICE_B) {
        data.REF_DISCOUNT_RATE1 = 0;
        data.REF_DISCOUNT_RATE2 = 0;
        data.REF_DISCOUNT_RATE3 = 0;
        data.DISCOUNT_RATE1 = 0;;
        data.DISCOUNT_RATE2 = 0;
        data.DISCOUNT_RATE3 = 0;
        data.UNIT_PRICE_B = data.UNIT_PRICE;
      }
    }

    let N_RETAIL_AMT_B = data.RETAIL_PRICE_B * data.ORDER_QTY;
    data.NET_AMT_B = data.UNIT_PRICE_B * data.ORDER_QTY;
    if ((N_RETAIL_AMT_B - data.NET_AMT_B) > 0) { data.DISCOUNT_AMT = (N_RETAIL_AMT_B - data.NET_AMT_B); }
    else { data.DISCOUNT_AMT = 0; }
  }
}
