import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { NgForm } from '@angular/forms';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';
import * as moment from 'moment';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';


@Component({
  selector: 'snse063a4',
  templateUrl: './SNSE063_A4.component.html',
  styleUrls: ['./SNSE063_A4.component.css']
})

export class SNSE063_A4Component implements OnInit {

  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }


  public tableDetail: any = [];
  public tableList: any = [];
  public params: any = {};
  public paramSearch: any = {};
  public data: any = {};
  public vatFlagList: Array<any> = [{ VAT_FLAG: 'Y' }, { VAT_FLAG: 'N' }];
  public count = 0;

  public checkRecord: boolean = true;
  public IsSearch: boolean = true;
  public DiscountOldValue: any;
  public dataFilter: any = []
  public tempDate: any;
  public isView: boolean = false;
  public isSave: boolean = false;
  public isAdd: boolean = false;

  @ViewChild("autoCompleteBranch", { static: false }) autoCompleteBranch: AutoCompleteQueryComponent;
  @ViewChild("autoCompleteBranchST", { static: false }) autoCompleteBranchST: AutoCompleteQueryComponent;
  @ViewChild("autoCompleteTransportCode") autoCompleteTransportCode: AutoCompleteQueryComponent;

  ngOnInit() {
    this.paramSearch.DOCUMENT_DATE = moment();
    this.ConfigDocNo.SourceParams.DOCUMENT_DATE = this.paramSearch.DOCUMENT_DATE;
  }

  // -------------------------------- Search Page --------------------------------

  public ConfigDocNo: any = {
    ColName: "DOCUMENT_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetDocList",
    NoOfShowRecords: 50,
    MinLengthSearch: 5,
    SourceParams: {
      DOCUMENT_DATE: null
    },
    UpperCase: true
  }

  public ConfigCustomerCodeList: any = {
    ColName: "LOOKUP_CUS_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetCustomerCodeList",
    NoOfShowRecords: 60,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public ConfigOrderNoList: any = {
    ColName: "ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetOrderNoList",
    NoOfShowRecords: 60,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetCustomerCodeList(values: any) {
    this.paramSearch.CUSTOMER_CODE = values.CUSTOMER_CODE;
    this.paramSearch.CUST_NAME_TH = values.CUST_NAME_TH;
    this.paramSearch.BRANCH_CODE = values.BRANCH_CODE;
    this.paramSearch.BRANCH_NAME_TH = values.BRANCH_NAME_TH;
  }

  public Search() {
    let param = {
      DOCUMENT_NO: this.paramSearch.DOCUMENT_NO,
      DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.paramSearch.DOCUMENT_DATE) || 'NULL',
      CUSTOMER_CODE: this.paramSearch.CUSTOMER_CODE,
      ORDER_NO: this.paramSearch.ORDER_NO
    }
    this.tableList = [];
    this.http.post("SL", "SNSE063_A4_SearchData", param)
      .subscribe(res => {
        this.tableList = res.Table;
      }, err => {
        this.modal.error("Found an error, please contract support team", this.viewref);
      })
  }

  public ViewClick(event) {
    //console.log(event)
    this.isView = true
    this.IsSearch = false;
    this.isAdd = false;
    this.params = event;
    this.tableDetail = []
    this.http.post("SL", "SNSE063_A4_GetDetail", { DOCUMENT_NO: this.params.DOCUMENT_NO })
      .subscribe(res => {
        /*       console.log(res)*/
        this.tableDetail = res.Table
        this.params.DELIVERY_CODE = this.tableDetail[0].DELIVERY_CODE
      })
  }
  public ChangeDocDate() {
    this.ConfigDocNo.SourceParams.DOCUMENT_DATE = this.paramSearch.DOCUMENT_DATE;
  }

  // -------------------------------- End Search Page --------------------------------



  // -------------------------------- Add Page --------------------------------

  public ConfigCustomerCode: any = {
    ColName: "LOOKUP_CUS_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetCuscode",
    NoOfShowRecords: 60,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetCustomerCode(values: any) {
    //Bill To
    this.params.CUSTOMER_CODE = values.CUSTOMER_CODE;
    this.params.CUST_NAME_TH = values.CUST_NAME_TH;
    this.params.BRANCH_CODE = values.BRANCH_CODE;
    this.params.BRANCH_NAME_TH = values.BRANCH_NAME_TH;

    this.params.DSP_CUST_ADDR = values.DSP_CUST_ADDR;
    this.params.CUST_ADDR1 = values.CUST_ADDR1;
    this.params.CUST_ADDR2 = values.CUST_ADDR2;

    //Ship To
    this.params.CUSTOMER_CODE_ST = values.CUSTOMER_CODE;
    this.params.CUST_NAME_TH_ST = values.CUST_NAME_TH;
    this.params.BRANCH_CODE_ST = values.BRANCH_CODE;
    this.params.BRANCH_NAME_TH_ST = values.BRANCH_NAME_TH;

    this.params.DSP_CUST_ADDR_ST = values.DSP_CUST_ADDR_ST;
    this.params.CUST_ADDR1_ST = values.BRANCH_ADDR1;
    this.params.CUST_ADDR2_ST = values.BRANCH_ADDR2;

    this.BranchLookupConfig.SourceParams.DATA = this.params.CUSTOMER_CODE;
    this.BranchLookupST.SourceParams.DATA = this.params.CUSTOMER_CODE_ST;

    this.params.ROUTE_CODE_EO = values.ROUTE_CODE_EO
    this.params.TRANSPORT_EO = values.TRANSPORT_EO
    this.params.ROUTE_CODE_RO = values.ROUTE_CODE_RO
    this.params.TRANSPORT_RO = values.TRANSPORT_RO
    this.params.PAYMENT_TYPE = values.PAYMENT_TYPE

    if (this.params.PAYMENT_TYPE == '1') {
      this.params.DELIVERY_CODE = '01';
      this.params.ROUTE_CODE = 'A9999';
      this.params.TRANSPORT_CODE = '99999';
    } else {
      this.params.DELIVERY_CODE = '03';
    }

    if (this.params.ORDER_CLASS == '1' || this.params.ORDER_CLASS == '2') {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_EO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_EO;
    } else {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_RO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_RO;
    }

    this.BranchLookupST.SourceParams.DATA = this.params.CUSTOMER_CODE_ST;
    this.autoCompleteTransportCode.textInput = this.params.TRANSPORT_CODE.toString();
    this.autoCompleteTransportCode.query()
  }

  public SetCustomerCodeST(values: any) {
    this.params.CUSTOMER_CODE_ST = values.CUSTOMER_CODE;
    this.params.CUST_NAME_TH_ST = values.CUST_NAME_TH;
    this.params.BRANCH_CODE_ST = values.BRANCH_CODE;
    this.params.BRANCH_NAME_TH_ST = values.BRANCH_NAME_TH;

    this.params.DSP_CUST_ADDR_ST = values.DSP_CUST_ADDR_ST;
    this.params.CUST_ADDR1_ST = values.BRANCH_ADDR1;
    this.params.CUST_ADDR2_ST = values.BRANCH_ADDR2;

    this.params.ROUTE_CODE_EO = values.ROUTE_CODE_EO
    this.params.TRANSPORT_EO = values.TRANSPORT_EO
    this.params.ROUTE_CODE_RO = values.ROUTE_CODE_RO
    this.params.TRANSPORT_RO = values.TRANSPORT_RO
    this.params.PAYMENT_TYPE = values.PAYMENT_TYPE

    if (this.params.CUSTOMER_CODE_ST != this.params.CUSTOMER_CODE) {
      this.params.TEMP_ORDER_TYPE = '5N';
      this.params.ORDER_CLASS = '5';
    } else {
      this.params.TEMP_ORDER_TYPE = '';
      this.params.ORDER_CLASS = '2';
    }

    if (this.params.PAYMENT_TYPE == '1') {
      this.params.DELIVERY_CODE = '01';
      this.params.ROUTE_CODE = 'A9999';
      this.params.TRANSPORT_CODE = '99999';
    } else {
      this.params.DELIVERY_CODE = '03';
    }

    if (this.params.ORDER_CLASS == '1' || this.params.ORDER_CLASS == '2') {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_EO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_EO;
    } else {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_RO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_RO;
    }

    this.BranchLookupST.SourceParams.DATA = this.params.CUSTOMER_CODE_ST;

    if (this.params.TRANSPORT_CODE) {
      this.autoCompleteTransportCode.textInput = this.params.TRANSPORT_CODE.toString();
      this.autoCompleteTransportCode.query()
    }

  }

  public BranchLookupConfig: any = {
    ColName: "LOOKUP_BRANCH_CODE",
    PlaceholderText: "Select Branch Code",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetBranchCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params.CUSTOMER_CODE
    },
    UpperCase: true
  }

  public SetBranchLookup(values: any) {
    this.params.BRANCH_CODE = values.BRANCH_CODE;
    this.params.BRANCH_NAME_TH = values.BRANCH_NAME_TH;

    this.params.DSP_CUST_ADDR = values.DSP_CUST_ADDR;
    this.params.CUST_ADDR1 = values.CUST_ADDR1;
    this.params.CUST_ADDR2 = values.CUST_ADDR2;
  }

  public BranchLookupST: any = {
    ColName: "LOOKUP_BRANCH_CODE",
    PlaceholderText: "Select Branch Code",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetBranchCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params.CUSTOMER_CODE_ST
    },
    UpperCase: true
  }

  public SetBranchLookupST(values: any) {
    this.params.BRANCH_CODE_ST = values.BRANCH_CODE;
    this.params.BRANCH_NAME_TH_ST = values.BRANCH_NAME_TH;

    this.params.DSP_CUST_ADDR_ST = values.DSP_CUST_ADDR_ST;
    this.params.CUST_ADDR1_ST = values.BRANCH_ADDR1;
    this.params.CUST_ADDR2_ST = values.BRANCH_ADDR2;

    this.params.ROUTE_CODE_EO = values.ROUTE_CODE_EO
    this.params.TRANSPORT_EO = values.TRANSPORT_EO
    this.params.ROUTE_CODE_RO = values.ROUTE_CODE_RO
    this.params.TRANSPORT_RO = values.TRANSPORT_RO
    this.params.PAYMENT_TYPE = values.PAYMENT_TYPE

    if (this.params.ORDER_CLASS == '1' || this.params.ORDER_CLASS == '2') {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_EO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_EO;
    } else {
      this.params.ROUTE_CODE = this.params.ROUTE_CODE_RO;
      this.params.TRANSPORT_CODE = this.params.TRANSPORT_RO;
    }

    if (this.params.PAYMENT_TYPE == '1') {

      this.params.DELIVERY_CODE = '01';
      this.params.ROUTE_CODE = 'A9999';
      this.params.TRANSPORT_CODE = '99999';
    } else {
      this.params.DELIVERY_CODE = '03';
    }

    if (this.params.TRANSPORT_CODE) {
      this.autoCompleteTransportCode.textInput = this.params.TRANSPORT_CODE.toString();
      this.autoCompleteTransportCode.query()
    }
  }

  public ConfigRouteCode: any = {
    ColName: "LOOKUP_ROUTE_CODE",
    PlaceholderText: "Select Route Code",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GetRouteCode",
    NoOfShowRecords: 60,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetRouteCode(values: any) {
    this.params.ROUTE_CODE = values.ROUTE_CODE;
  }

  public ConfigTransportCode: any = {
    ColName: "LOOKUP_TRANSPORT",
    PlaceholderText: "Select Transport Code",
    SourceController: "wPMS",
    SourceMethodName: "GetTransportCode",
    NoOfShowRecords: 40,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetTransCode(trans) {
    this.params.TRANSPORT_CODE = trans.TRANSPORT_CODE;
    this.params._TRANSPORT = trans.LOOKUP_TRANSPORT;
  }

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "SL",
    SourceMethodName: "SNSE063A4_GETPART",
    NoOfShowRecords: 30
  }

  public SetNewPartNo(part: any, data: any, index: number) {
    let findIndex = this.tableDetail.findIndex((x, i) => x.PART_NO == part.PART_NO && i != index)
    if (findIndex > -1) {
      this.modal.warning('Part No. : ' + part.PART_NO + ' duplicate!', this.viewref)
      return;
    }

    let param = {
      CUSTOMER_CODE: this.params.CUSTOMER_CODE,
      ORDER_DATE: this.params.ORDER_DATE,
      PART_NO: part.PART_NO,
      ORDER_CLASS: this.params.ORDER_CLASS,
      VENDOR_CODE: part.VENDOR_CODE
    }

    this.http.post("SL", "SNSE063_A4_FindUnitPrice", param)
      .subscribe(result => {

        if (result.Table[0].CHECK_RETAIL_PRICE == 'N') {
          this.modal.warning('Part No. : ' + part.PART_NO + ' Retail price <= 1!', this.viewref)
          return;
        }


        data.ORDER_QTY = 0;
        data.DISCOUNT_GROUP = result.Table[0].DISCOUNT_GROUP;
        data.RETAIL_PRICE_B = result.Table[0].RETAIL_PRICE_B;
        data.UNIT_PRICE_B = result.Table[0].UNIT_PRICE_B;
        data.OVERSEA_FLAG_C = result.Table[0].OVERSEA_FLAG;
        data.DISCOUNT_RATE1 = result.Table[0].DISCOUNT_RATE1;
        data.DISCOUNT_RATE2 = result.Table[0].DISCOUNT_RATE2;
        data.DISCOUNT_RATE3 = result.Table[0].DISCOUNT_RATE3;
        data.AVERAGE_COST = result.Table[0].AVERAGE_COST;
        data.STANDARD_COST = result.Table[0].STANDARD_COST;

        this.params.DISCOUNT_GROUP = data.DISCOUNT_GROUP;
        data.RETAIL_PRICE = data.RETAIL_PRICE_B;
        data.DISCOUNT_CODE = result.Table[0].DISCOUNT_CODE
        //data.UNIT_EDP = data.UNIT_PRICE_B;
        // data.RETAIL_PRICE_C = 0;
        // data.UNIT_PRICE_C = 0;

        data.PART_NO = part.PART_NO;
        data.PART_NAME = part.PART_NAME;
        data.PART_NAME_T = part.PART_NAME;
        data.SALES_PACKAGE = part.SALES_PACKAGE;
        data.B_OH_QTY = part.B_OH_QTY;
        data.A_QTY = 0;
        data.AMOUNT = 0;
        data.LOCK_SALES = part.LOCK_SALES
        data.VENDOR_CODE = part.VENDOR_CODE
      })
  }

  public SetVat(VatFlag) {
    this.params.VAT_FLAG = VatFlag;
    this.http.post("SL", "SNSE063A4_GetVatRate", this.params)
      .subscribe(res => {
        let vatrate = res.Table[0].VAT_RATE
        this.params.VAT_RATE_SH = vatrate.toFixed(2) + '%';
        this.params.VAT_RATE = vatrate;
      })
  }

  public ChkOrderNo() {
    if (this.params.TEMP_ORDER_TYPE) {
      if ((this.params.TEMP_ORDER_TYPE.length + this.params.ORDER_NO.length) > 10) {
        this.modal.warning("Order Number " + this.params.TEMP_ORDER_TYPE + this.params.ORDER_NO + " has more than 10 digit! Please check!", this.viewref)
        this.params.ORDER_NO = null;
        return;
      }
    }

    if (this.params.TEMP_ORDER_TYPE) {
      this.params.REF_DOC_NO = this.params.TEMP_ORDER_TYPE + this.params.ORDER_NO;
    } else {
      this.params.REF_DOC_NO = this.params.ORDER_NO;
    }
    this.http.post("SL", "SNSE063A4_CheckOrderNo", this.params)
      .subscribe(res => {
        let CNT_DN_ORDER = res.Table[0].CNT_DN_ORDER
        if (CNT_DN_ORDER > 0) {
          this.modal.warning("Order Number : " + this.params.ORDER_NO || " is duplicate!", this.viewref)
          this.params.ORDER_NO = null
        }
      })
  }

  public CheckActiveRow(index, data) {
    if (this.ValidHeader() && data.PART_NO && this.isAdd) {
      this.CreateDetailRow(index);
      return true;
    }
    return false;
  }

  public CreateDetailRow(tblIndex) {
    let currentMaxRow: number = this.tableDetail.length || 0;
    let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

    if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.tableDetail.push({
          SEQ: i
        });
      }
      this.tableDetail = this.tableDetail.slice();
    }
  }

  public Save() {
    this.dataFilter = []
    this.dataFilter = this.tableDetail.filter(x => x.PART_NO && x.ORDER_QTY)
    if (this.dataFilter.length > 0) {
      let find = this.dataFilter.some(x => x.A_QTY == 0 || x.ORDER_QTY > x.A_QTY)

      if (find) {
        this.modal.warning("Invalid value number, Please check.", this.viewref);
      } else {
        this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave)
      }
    } else {
      this.modal.warning("No detail for save, please select part no", this.viewref)
    }
  }

  public ConfirmSave = () => {
    let param = {
      CUSTOMER_CODE: this.params.CUSTOMER_CODE,
      CUSTOMER_NAME: this.params.CUST_NAME_TH,
      BRANCH_CODE: this.params.BRANCH_CODE,
      CUST_ADDR1: this.params.CUST_ADDR1,
      CUST_ADDR2: this.params.CUST_ADDR2,

      CUSTOMER_CODE_ST: this.params.CUSTOMER_CODE_ST,
      CUSTOMER_NAME_ST: this.params.CUST_NAME_TH_ST,
      BRANCH_CODE_ST: this.params.BRANCH_CODE_ST,
      CUST_ADDR1_ST: this.params.CUST_ADDR1_ST,
      CUST_ADDR2_ST: this.params.CUST_ADDR2_ST,

      ORDER_NO: this.params.ORDER_NO,
      ORDER_CLASS: this.params.ORDER_CLASS,
      ORDER_DATE: this.params.ORDER_DATE,
      VAT_FLAG: this.params.VAT_FLAG,
      VAT_RATE: this.params.VAT_RATE,
      DELIVERY_CODE: this.params.DELIVERY_CODE,
      ROUTE_CODE: this.params.ROUTE_CODE,
      TRANSPORT_CODE: this.params.TRANSPORT_CODE,
      REF_OTHER_DOC: this.params.REF_OTHER_DOC,
      TEMP_ORDER_TYPE: this.params.TEMP_ORDER_TYPE,
      Detail: this.dataFilter
    }

    this.http.post("SL", "SNSE063_A4_SaveDocument", param)
      .subscribe(res => {
        if (res.DOCUMENT_NO) {
          this.modal.info("Transaction complete.", this.viewref)
          this.params.DOCUMENT_NO = res.DOCUMENT_NO
          this.isSave = true;
          if (this.checkRecord) {
            this.OpenReport();
          }

        } else {
          this.modal.error("Transaction failed, please contact administrator to resolve.", this.viewref);
          console.log(res)
        }
      }, err => {
        this.modal.error("Transaction failed, please contact administrator to resolve.", this.viewref);
      })
  }

  public OpenReport() {
    this.global.GoToReport("PMS_RPT_SALES_SNSR063R_A4", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_S_CUSTOMER_CODE: this.params.CUSTOMER_CODE,
      P_E_CUSTOMER_CODE: this.params.CUSTOMER_CODE_ST,
      P_S_DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.params.ORDER_DATE),
      P_E_DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.params.ORDER_DATE),
      P_S_DOCUMENT_NO: this.params.DOCUMENT_NO,
      P_E_DOCUMENT_NO: this.params.DOCUMENT_NO
    });
  }

  public BackToGrid() {
    this.IsSearch = true;
  }

  public SetOrderQty(data) {

    if (data.B_OH_QTY == 0 && !data.FLAG_CHAIN) {
      let param = {
        PART_NO: data.PART_NO,
        DOCUMENT_DATE: this.params.ORDER_DATE,
        CUSTOMER_CODE: this.params.CUSTOMER_CODE,
        VENDOR_CODE: data.VENDOR_CODE
      }
      this.http.post("SL", "SNSE063_A4_FIndChainPart", param)
        .subscribe(res => {
          if (res.FLAG_CHAIN == 'C') {
            if (data.ORDER_QTY > res.B_OH_QTY_CHAIN) {
              data.A_QTY = res.B_OH_QTY_CHAIN
            } else {
              data.A_QTY = data.ORDER_QTY
            }

            data.FLAG_CHAIN = res.FLAG_CHAIN;
            data.PART_NAME = res.NEW_PART
            data.UNIT_PRICE_B = res.AVERAGE_COST;
            data.AMOUNT = this.global.Round(data.ORDER_QTY * data.UNIT_PRICE_B, 2);
            data.B_OH_QTY = res.B_OH_QTY_CHAIN;

          } else if (res.FLAG_CHAIN == 'N') {
            this.modal.warning("Invalid value number, Please check.", this.viewref);
            data.A_QTY = data.B_OH_QTY;
            data.AMOUNT = this.global.Round(data.ORDER_QTY * data.UNIT_PRICE_B, 2);
            data.FLAG_CHAIN = res.FLAG_CHAIN;
          } else {
            this.modal.error(res.FLAG_CHAIN, this.viewref)
            //data.PART_NO = ''
            //data.PART_NAME = ''
            data.DISCOUNT_RATE1 = 0;
            data.DISCOUNT_RATE2 = 0;
            data.DISCOUNT_RATE3 = 0;
            data.UNIT_PRICE_B = 0;
            data.AMOUNT = 0;
            data.ORDER_QTY = 0;
            data.RETAIL_PRICE = 0;
            data.FLAG_CHAIN = res.FLAG_CHAIN;
            //return;
          }

          data.LOCK_SALES = res.LOCK_SALES
          //console.log('ress',res)
        })
    } else {

      if (data.LOCK_SALES == 0) {

        if (data.ORDER_QTY > data.B_OH_QTY) {

          this.modal.warning("Invalid value number, Please check.", this.viewref);
          data.A_QTY = data.B_OH_QTY;
          data.ORDER_QTY = data.A_QTY;
          data.UNIT_PRICE_B = data.AVERAGE_COST;
          data.AMOUNT = data.ORDER_QTY * data.UNIT_PRICE_B;
          //data.FLAG_CHAIN = 'N';

        } else if (data.ORDER_QTY == 0) {

          this.count = 0;
          data.A_QTY = 0;
          data.DISCOUNT_RATE1 = 0;
          data.DISCOUNT_RATE2 = 0;
          data.DISCOUNT_RATE3 = 0;
          //data.PART_NAME = data.PART_NAME_T;
          data.UNIT_PRICE_B = data.AVERAGE_COST;
          data.AMOUNT = data.ORDER_QTY * data.UNIT_PRICE_B;
          //data.FLAG_CHAIN = 'N';
        } else {
          //this.count = 0;
          //data.PART_NAME = data.PART_NAME_T;
          data.A_QTY = data.ORDER_QTY;
          data.UNIT_PRICE_B = data.AVERAGE_COST;
          data.AMOUNT = data.ORDER_QTY * data.UNIT_PRICE_B;
        }
      }
      else {

        if (data.LOCK_SALES != 0) {
          this.modal.error("THIS ORDER_PART LOCK SALES ERROR CODE = " + data.LOCK_SALES, this.viewref)
          //data.PART_NO = ''
          //data.PART_NAME = ''
          data.A_QTY = 0;
          data.DISCOUNT_RATE1 = 0;
          data.DISCOUNT_RATE2 = 0;
          data.DISCOUNT_RATE3 = 0;
          data.UNIT_PRICE_B = 0;
          data.AMOUNT = 0;
          data.ORDER_QTY = 0;
          data.RETAIL_PRICE = 0;
          //data.FLAG_CHAIN = "THIS ORDER_PART LOCK SALES ERROR CODE = " + data.LOCK_SALES;
          return;
        }
      }
    }

  }

  public DiscountChange(data, event, rate, oldvalue) {

    if (event > 990.00) {
      this.modal.error("Field must be of from 990.00", this.viewref)
      switch (rate) {
        case 1:
          data.DISCOUNT_RATE1 = oldvalue
          break;
        case 2:
          data.DISCOUNT_RATE2 = oldvalue
          break;
        case 3:
          data.DISCOUNT_RATE3 = oldvalue
          break;
        default:
          console.log("rate not macth")
      }
      return;
    }


    if (!event) {
      switch (rate) {
        case 1:
          data.DISCOUNT_RATE1 = 0
          break;
        case 2:
          data.DISCOUNT_RATE2 = 0
          break;
        case 3:
          data.DISCOUNT_RATE3 = 0
          break;
        default:
          console.log("rate not macth")
      }

    }

    //console.log(data);
    if (data.DISCOUNT_RATE1 == 0 && data.DISCOUNT_RATE2 == 0 && data.DISCOUNT_RATE3 == 0) {
      data.UNIT_PRICE_B = data.AVERAGE_COST;
      data.AMOUNT = data.ORDER_QTY * data.UNIT_PRICE_B;
    } else {
      let NewUnitPrice = Math.round(data.RETAIL_PRICE * (((100 - (data.DISCOUNT_RATE1 || 0)) / 100) * ((100 - (data.DISCOUNT_RATE2 || 0)) / 100) * ((100 - (data.DISCOUNT_RATE3 || 0)) / 100)));
      data.UNIT_PRICE_B = Math.round(NewUnitPrice);
      data.AMOUNT = data.ORDER_QTY * data.UNIT_PRICE_B;
    }
  }

  public RePrintPicking() {
  }

  public CalTotalAmount() {
    return this.tableDetail.reduce((prev, currObj) => prev + (parseFloat((currObj.ORDER_QTY || 0)) * parseFloat((currObj.UNIT_PRICE_B || 0))), 0);
  }

  public CalTotalVat() {
    return this.tableDetail.reduce((prev, currObj) => prev + (parseFloat((currObj.ORDER_QTY || 0)) * (parseFloat((currObj.UNIT_PRICE_B || 0)) * this.params.VAT_RATE / 100)), 0);
  }

  public DeleteRow(index) {
    this.modal.confirm("Are you sure to remove this row?", this.viewref
      , () => {
        this.tableDetail.splice(index, 1);
        this.tableDetail = [...this.tableDetail];
        //this.ExpenseChanged()
      }, () => { })
  }

  public AddOrder() {
    this.isAdd = true;
    this.IsSearch = false;
    this.isView = false;
    this.isSave = false;
    this.tableDetail = [];
    this.params = {};
    this.params.DELIVERY_CODE = '03';
    this.params.ORDER_CLASS = '2';
    this.params.ORDER_DATE = moment();
    this.params.UPDATE_DATE = new Date();
    this.params.VAT_FLAG = 'Y';
    this.SetVat('Y');
    this.params.DOCUMENT_NO = 'AUTO'
    this.CreateDetailRow(0);
  }

  public ValidHeader() {
    if (this.params.CUSTOMER_CODE &&
      this.params.BRANCH_CODE &&
      this.params.CUSTOMER_CODE_ST &&
      this.params.BRANCH_CODE_ST &&
      this.params.ORDER_NO &&
      this.params.ORDER_CLASS &&
      this.params.ORDER_DATE &&
      this.params.VAT_FLAG &&
      this.params.DELIVERY_CODE &&
      this.params.ROUTE_CODE &&
      this.params._TRANSPORT) {
      return true;
    } else {
      return false;
    }

  }

  public CheckRefDoc() {
    if (this.params.REF_OTHER_DOC != null) {
      this.params.REF_OTHER_DOC = this.params.REF_OTHER_DOC.toUpperCase();
    }
    this.http.post("SL", "SNSE063_A4_CheckRefOtherDoc", this.params)
      .subscribe(res => {
        //console.log(res);

        if (res.Table[0].MESSAGE) {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
          this.params.REF_OTHER_DOC = '';
        }
      });
  }
  //-------------------------------- End Add Page --------------------------------
}
