import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';


@Component({
  selector: 'SNSE152',
  templateUrl: 'SNSE152.component.html',
  styleUrls: ['SNSE152.component.css']
})

export class SNSE152Component implements OnInit {

  public params: any = {}
  public selectedrow: any;
  public PROCESS_LOG: any = [];
  public formData: any = [];

  constructor(
    private Http: HttpService,
    private router: Router,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogProcess", { static: false }) dialogProcess: any;
  @ViewChild("dialogEffective", { static: false }) dialogEffective: any;

  public tableData: any = [];
  public DataIsSelected: any = [];

  public selection = new SelectionModel<any>(true, []);

  public selection2 = new SelectionModel<any>(true, []);

  ngOnInit() {
    this.params.OPP_GR_RATE = '1';
    this.params.COMPLETE_STATUS = '';
    this.params.SETTING_TYPE = '';
    this.params.CAL_DIS_RATE = 'D';
    this.params.CONFIRM_STATUS = 'N';
    this.params.EFFECTIVE_DATE = new Date();
    this.params.ITP_FLAG = ''
  }

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE152_getCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupPartNoConfig: any = {
    ColName: "LOOKUP_PART_NO",
    PlaceholderText: "Select Part No",
    SourceController: "SL",
    SourceMethodName: "SNSE152_getPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupModelCodeConfig: any = {
    ColName: "LOOKUP_MODEL_CODE",
    PlaceholderText: "Select Mode Code",
    SourceController: "SL",
    SourceMethodName: "SNSE152_getModelCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupPricingRoleConfig: any = {
    ColName: "LOOKUP_PRICING_ROLE",
    PlaceholderText: "Select Pricing Role",
    SourceController: "SL",
    SourceMethodName: "SNSE152_getPricingRole",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCode(values: any) {
    this.params.CUSTOMER_CODE = values.CUSTOMER_CODE;
  }

  public SetSelectedPartNo(values: any) {
    this.params.PART_NO = values.PART_NO;
  }

  public SetSelectedModelCode(values: any) {
    this.params.MODEL_CODE = values.MODEL_CODE;
  }

  public SetSelectedPricingRole(values: any) {
    this.params.PRICING_ROLE = values.PRICING_ROLE;
  }

  Search() {
    this.Http.post('SL', 'SNSE152_getData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;
        this.PROCESS_LOG.push('Loading TPM_FOB_PRICE_MASTER... ' + this.tableData.length + ' records loaded...');
        this.selection.clear()
        this.DataIsSelected = [];
        this.formData = [];
      });
  }

  rowClick(row) {
    this.params.MESSAGE_ERROR = row.MESSAGE_ERROR
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.tableData.length;
    return numSelected === numRows;
  }

  Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
      this.tableData.map(x => {
        this.DataIsSelected = [];
      })
    } else {
      this.tableData.forEach(row => this.selection.select(row));
      this.DataIsSelected = [];
      this.tableData.map(x => {
        this.DataIsSelected.push(x);
      })
    }
    this.params.CNT_SEL = this.DataIsSelected.length;
  }

  CheckboxChange(event, data) {
    if (event.checked) {
      this.DataIsSelected.push(data);
    }
    else {
      this.DataIsSelected.splice(this.DataIsSelected.indexOf(data), 1);
    }
    this.params.CNT_SEL = this.DataIsSelected.length;
  }

  radioChange(data, type) {
    if (type === 'G') {
      data.FOB_PRICE_FLAG = 'G'
    }
    else if (type === 'A') {
      data.FOB_PRICE_FLAG = 'A'
    }

    if (data.FOB_PRICE_FLAG == 'G') {
      data.FOB_PRICE_FC = ((data.GLOBAL_FOB_PRICE) / (data.EXCHANGE_RATE));
    }
    else if (data.FOB_PRICE_FLAG == 'A') {
      data.FOB_PRICE_FC = ((data.ADJ_FOB_PRICE) / (data.EXCHANGE_RATE));
    }
  }

  radioChangeAll(type) {
    this.tableData.forEach((e) => {
      if (type === 'G') {
        e.FOB_PRICE_FLAG = 'G'
      }
      else if (type === 'A') {
        e.FOB_PRICE_FLAG = 'A'
      }

      if (e.FOB_PRICE_FLAG == 'G') {
        e.FOB_PRICE_FC = ((e.GLOBAL_FOB_PRICE) / (e.EXCHANGE_RATE));
      }
      else if (e.FOB_PRICE_FLAG == 'A') {
        e.FOB_PRICE_FC = ((e.ADJ_FOB_PRICE) / (e.EXCHANGE_RATE));
      }
    });
  }

  clickProcessLog() {
    this.dialogProcess.OpenDialog();
  }

  ExportExcel() {
    this.Http.Download('SL', "SNSE152_ExportExcel", JSON.stringify(this.tableData))
      .subscribe(res => {
        if (res) {
          this.WriteContents(res)
          this.PROCESS_LOG.push('Export complete ' + this.tableData.length + ' records.');
        } else {
          this.modal.error('Download not complete', this.viewref)
        }
      })
  }

  WriteContents(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = "SNSE152" + ".xlsx";
    a.click();
  }


  OpenConfirmModal() {
    this.formData = [];
    if (this.DataIsSelected.length > 0) {
      this.dialogEffective.OpenDialog();
    } else {
      this.modal.warning("Please select data for confirm registration.", this.viewref);
    }
  }

  ConfirmRegis() {
    this.dialogEffective.Close();
    this.formData = [];
    this.DataIsSelected.forEach((e) => {
      e.EFFECTIVE_DATE = this.params.EFFECTIVE_DATE
      this.formData.push(e)
    });
    this.Http.post('SL', 'SNSE152_RegisConfirm', this.formData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info("Comfirm registration completed.", this.viewref);
          this.PROCESS_LOG.push('Update data complete : ' + this.formData.length);
          this.Search();
          this.formData = [];
        }
        else {
          this.modal.warning("Confirm Registration Fail!", this.viewref);
        }
      });
  }

  closeModal() {
    this.dialogEffective.Close();
  }


  public CalGPRate(type, data) {
    var V_ISC_COST = data.STANDARD_COST;
    if ((data.STANDARD_COST <= 0.01) && (data.TEMP_COST > 0)) {
      V_ISC_COST = data.TEMP_COST;
    }
    if ((type == 'A') && (data.ADJ_FOB_PRICE > 0)) {
      data.ADJ_GP_RATE = ((((data.ADJ_FOB_PRICE - V_ISC_COST) / data.ADJ_FOB_PRICE)) * 100);
    }
    var V_COST = data.STANDARD_COST;
    if ((data.STANDARD_COST <= 0.01) && (data.TEMP_COST != 0)) {
      V_COST = data.TEMP_COST;
    }
    data.DIF_ADJ_COST = data.ADJ_FOB_PRICE - V_COST;

    if (data.FOB_PRICE_FLAG == 'G') {
      data.FOB_PRICE_FC = ((data.GLOBAL_FOB_PRICE) / (data.EXCHANGE_RATE));
    }
    else if (data.FOB_PRICE_FLAG == 'A') {
      data.FOB_PRICE_FC = ((data.ADJ_FOB_PRICE) / (data.EXCHANGE_RATE));
    }
  }


}
