import { Component, OnInit, ViewContainerRef, ViewChild } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';


@Component({
  selector: 'snse208',
  templateUrl: './SNSE208.component.html',
  styleUrls: ['./SNSE208.component.css']
})

export class SNSE208Component implements OnInit {

  public params: any = {};
  public DataUploadList: any = [];
  public files: any = [];
  public FileName: string = "";
  public ListLog: any = [];

  public DataIsSelected: any = [];
  public selection = new SelectionModel<any>(true, []);

  @ViewChild("dialogLog", { static: false }) dialogLog: any;

  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  ngOnInit() {
    this.params.OPT_LAYOUT = 'L1';
    this.params.TOTAL_RECORD = 0;
    this.params.TOTAL_SELECTED = 0;
    this.params.TOTAL_ERROR = 0;
  }

  public OptionChange(value) {
    this.params.OPT_LAYOUT = value;
  }

  public ConfirmSave() {
    if (this.DataIsSelected.length > 0) {

      this.modal.confirm('Do you want to upload selected approved quantity?', this.viewref, this.Save)
    }
    else {
      this.modal.info('No record selected!', this.viewref)
    }
  }

  public Save = () => {
    this.Http.post('SL', 'SNSE208_Save', this.DataIsSelected)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE' && res.Table[0].ERR_MESSAGE == 'DONE') {
          this.modal.info("Upload approved quantity allocation completed!", this.viewref);
          this.DataUploadList = [];
          this.DataIsSelected = [];
          this.selection.clear();
          this.params.TOTAL_RECORD = 0;
          this.params.TOTAL_SELECTED = 0;
          this.params.TOTAL_ERROR = 0;
        } else {
          this.modal.warning(res.Table[0].ERR_MESSAGE, this.viewref);
        }
      }, (err) => {
        this.modal.error("Upload error! Please check file or data for upload.", this.viewref);
      });
  }

  public isAllSelected() {
    let countRow = this.DataUploadList.filter(e => e.ERROR_FLAG === '0');
    const numSelected = this.selection.selected.length;
    const numRows = countRow.length;
    return numSelected === numRows;
  }

  public Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
      this.DataUploadList.map(x => {
        this.DataIsSelected = [];
        x.SEL_UPLOAD = 0;
      })
    } else {
      this.DataUploadList.forEach((e) => {
        if (e.ERROR_FLAG != 1 && e.SEQ != null) {
          this.selection.select(e)
          this.DataIsSelected.push(e);
          e.SEL_UPLOAD = 1;
        }
      });
    }
    this.params.TOTAL_SELECTED = this.DataIsSelected.length;
  }

  public CheckboxChange(event, data) {
    if (event.checked) {
      this.DataIsSelected.push(data);
      data.SEL_UPLOAD = 1;
    }
    else {
      this.DataIsSelected.splice(this.DataIsSelected.indexOf(data), 1);
      data.SEL_UPLOAD = 0;
    }
    this.params.TOTAL_SELECTED = this.DataIsSelected.length;
  }

  public OnFileSelect(fileInput) {
    this.files = [];
    this.FileName = null;
    this.DataIsSelected = []
    this.DataUploadList = []
    this.selection.clear()
    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;
  }

  public UpLoad() {
    this.DataUploadList = [];
    this.DataIsSelected = [];
    this.selection.clear();
    this.params.TOTAL_RECORD = 0;
    this.params.TOTAL_SELECTED = 0;
    this.params.TOTAL_ERROR = 0;
    if (this.files.length == 0) {
      this.modal.error('Please Open .xlsx File First...', this.viewref)
    }
    else {
      let formData = new FormData();
      formData.append('file', this.files[0]);
      formData.append('value', this.params.OPT_LAYOUT);
      this.Http.postFile('SL', 'SNSE208_Upload', formData)
        .subscribe(res => {
          this.DataUploadList = [];
          this.Http.get('SL', 'SNSE208_GetTempData')
            .subscribe(res => {
              this.DataUploadList = res.Table;
              if (this.DataUploadList.length > 0) {
                this.params.TOTAL_RECORD = this.DataUploadList.length
                this.DataUploadList.forEach((e, index) => {
                  e.SEQ = index + 1;
                  if (e.ERROR_MSG) {
                    this.params.TOTAL_ERROR++;
                  }
                });
              }
            });
        }, (err) => {
          this.modal.error("Upload error! Please check file or data for upload.", this.viewref);
        });
    }
  }

  public Export() {
    if (this.DataUploadList.length > 0) {
      this.Http.Download("SL", "SNSE208_getDataForExport", JSON.stringify(this.params))
        .subscribe(result => {
          if (result) {
            this.download(result)
            this.modal.info('Export completed!', this.viewref)
          }
        }, (err) => {
          console.log(err, 'err')
        })
    }
    else {
      this.modal.warning('There is no record to export!', this.viewref)
    }
  }

  private download(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type })
    a.href = URL.createObjectURL(file);
    a.download = "SNSE208.csv"
    a.click()
  }
  public DownloadTemplate() {
    let link = "assets/templates/SNSE208_Upload_Template.xlsx"
    window.open(link, '_blank')
  }
}
