import { Component, OnInit, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';


@Component({
  selector: 'snse210',
  templateUrl: 'SNSE210.component.html',
  styleUrls: ['../domsale.component.css']
})

export class SNSE210Component implements OnInit {

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  public params: any = {};
  public tableData: any = [];
  public formData: any = [];

  public DataUploadList: any = [];
  public files: any = [];
  public FileName: string = "";

  public ConfigCustomerCode: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE210_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      GROUP_PART_NO: null
    }
  }

  public SetCustomerCode(values: any) {
    this.params.CUSTOMER_CODE = values.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = values.CUSTOMER_NAME;
    this.ConfigBranchCode.SourceParams.CUSTOMER_CODE = values.CUSTOMER_CODE;
    this.ConfigGroupPartNo.SourceParams.CUSTOMER_CODE = values.CUSTOMER_CODE;

  }

  public ConfigBranchCode: any = {
    ColName: "LOOKUP_BRANCH_CODE",
    PlaceholderText: "Select Branch Code",
    SourceController: "SL",
    SourceMethodName: "SNSE210_GetBranch",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      CUSTOMER_CODE: null,
      GROUP_PART_NO: null
    }
  }

  public SetBranchCode(values: any) {
    this.params.BRANCH_CODE = values.BRANCH_CODE;
    this.params.BRANCH_NAME = values.BRANCH_NAME
    this.ConfigGroupPartNo.SourceParams.BRANCH_CODE = values.BRANCH_CODE;
  }

  public ConfigGroupPartNo: any = {
    ColName: "LOOKUP_GROUP_PART_NO",
    PlaceholderText: "Select Group Part No",
    SourceController: "SL",
    SourceMethodName: "SNSE210_GetGroupPart",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: false,
    SourceParams: {
      CUSTOMER_CODE: null,
      BRANCH_CODE: null
    }
  }

  public SetGroupPartNo(values: any) {
    this.params.GROUP_PART_NO = values.GROUP_PART_NO;
    this.params.GROUP_PART_NAME = values.GROUP_PART_NAME;
    this.ConfigCustomerCode.SourceParams.GROUP_PART_NO = values.GROUP_PART_NO;
    this.ConfigBranchCode.SourceParams.GROUP_PART_NO = values.GROUP_PART_NO;
  }

  ngOnInit() {
  }

  public Search() {
    if (!this.params.CUSTOMER_CODE && !this.params.GROUP_PART_NO) {
      this.modal.warning("Please fill Customer Code or Group Part No.", this.viewref);
      return;
    }
    this.tableData = [];
    this.formData = [];
    this.http.post('SL', 'SNSE210_GetData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;
        if (this.tableData.length > 0) {
          this.tableData.forEach((e) => {
            e.OLD_ADJUST_UNIT = e.ADJUST_UNIT
            e.REF_REMAIN_UNIT = e.REMAIN_UNIT - e.OLD_ADJUST_UNIT
            e.OLD_REMAIN_UNIT = e.REMAIN_UNIT
          });
        }

      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public Save() {
    this.formData = this.tableData.filter(e => e.ADJUST_UNIT != e.OLD_ADJUST_UNIT)
    if (this.formData.length <= 0) {
      this.modal.warning('There is no data change.', this.viewref)
      return;
    }
    this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave)
  }

  private ConfirmSave = () => {
    this.http.post("SL", "SNSE210_Update", this.formData)
      .subscribe(res => {
        console.log(res)
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          this.modal.warning("Found an error, please contract support team", this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public AdjustQtyChange(data) {
    if (data.ADJUST_UNIT == null) {
      data.REMAIN_UNIT = null
    } else {
      data.REMAIN_UNIT = data.ADJUST_UNIT + data.REF_REMAIN_UNIT;
    }

    if (data.REMAIN_UNIT < 0) {
      data.REMAIN_UNIT = 0;
    }
    if (data.OLD_REMAIN_UNIT <= 0 && data.REMAIN_UNIT <= 0) {
      data.ADJUST_UNIT = data.OLD_ADJUST_UNIT
    }
    else if (data.OLD_REMAIN_UNIT > 0 && data.REMAIN_UNIT <= 0) {
      data.ADJUST_UNIT = data.OLD_ADJUST_UNIT - data.OLD_REMAIN_UNIT
    }
  }


  //UPLOAD
  public OnFileSelect(fileInput) {
    this.FileName = null;
    this.files = [];

    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;
  }

  public DownloadTemplate() {
    let link = "assets/templates/PMS_Upload_Quota_Forecast.xlsx"
    window.open(link, '_blank')
  }

  public UpLoad() {
    if (this.files.length == 0) {
      this.modal.error('Please Select .xlsx File.', this.viewref)
    } else {
      let formData = new FormData();
      formData.append('file', this.files[0]);

      this.http.postFile('SL', 'SNSE210_Upload', formData)
        .subscribe(res => {
          this.DataUploadList = [];
          this.http.get('SL', 'SNSE210_GetTempData')
            .subscribe(res => {
              this.DataUploadList = res.Table;
              if (this.DataUploadList.length > 0) {
                this.DataUploadList.forEach((e, index) => {
                  e.SEQ = index + 1;
                });
              }
            });
        }, (err) => {
          this.modal.error("Upload error! Please check file or data for upload.", this.viewref);

        });
    }
  }

  public SaveUpload() {
    let foundErr = this.DataUploadList.some(x => x.REMARK)
    if (foundErr) {
      this.modal.warning("Some data has error message! Please check data.", this.viewref);
      return;
    }
    this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSaveUpload)
  }

  private ConfirmSaveUpload = () => {
    this.http.get("SL", "SNSE210_Save_Forecast")
      .subscribe(res => {
        console.log(res)
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.DataUploadList = [];
        }
        else {
          this.modal.warning("Found an error, please contract support team", this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }
}
