import { Component, OnInit, Input, Output, EventEmitter, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import * as moment from 'moment';

@Component({
  selector: 'SNM123S',
  templateUrl: './SNM123S.component.html',
  styleUrls: ['../exportbiz.component.css']
})


export class SNM123SComponent implements OnInit {

  public params: any = {}
  public dataEdit: any = {};
  public datatable: any = [];
  public dataPrice: any = {};
  public currentDate: moment.Moment = moment();

  public iShow: number = 1;
  public IsComfirmed: boolean = false;

  @ViewChild("dialogLog", { static: false }) dialogLog: any;
  @Input("pPartNo") partNo: string;
  @Input("pCustomerCode") customerCode: string;
  @Input("pOrderNo") orderNo: string;


  @Output() datacallback = new EventEmitter<string>();

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM123_GetCustomer",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.params.CURRENCY_CODE = value.CURRENCY_CODE;
  }


  ngOnInit() {
    //this.Search();

    if (this.customerCode != undefined) {
      this.params.CUSTOMER_CODE = this.customerCode;
      this.params.PART_NO = this.partNo;
      this.params.ORDER_NO = this.orderNo;
      this.params.FILTER_NO = 2;

      this.GetData();

    }
  }

  private GetData() {
    this.http.post('SL', 'SNM123_GetData', this.params)
      .subscribe(res => {
        this.datatable = res.Table[0];
        this.EditData(res.Table[0]);
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewRef);
      })

  }

  EditData(data) {
    data.RECAL_FLAG = 'N';
    this.dataEdit = data;
    this.dataEdit.EFFECTIVE_DATE = this.currentDate;

    this.http.post('SL', 'SNM123_GetExportPriceInfo', data)
      .subscribe(res => {
        this.dataPrice = res.Table[0];
        if (this.dataPrice.CUSTOMER_TYPE == 'Consolidated NSC') {
          //-- Consolidated
          this.iShow = 1;
        } else if (this.dataPrice.PART_TYPE == '' || this.dataPrice.PART_TYPE == null) {
          //-- Non Consolidated use GLP
          this.iShow = 2;
        } else if (this.dataPrice.PART_TYPE.substring(0, 3) == 'Fix') {
          //-- Non Consolidated use Fix , FOB < ISC
          this.iShow = 3;
        } else {
          //-- Non Consolidated use Part type
          this.iShow = 4;
        }

      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewRef);
      })
  }

  public ConfirmSave() {
    this.modal.confirm('Do you want to confirm this price?', this.viewRef, this.Save)
  }

  private Save = () => {
    this.dataEdit.FILTER_NO = 2;
    this.dataEdit.NEW_PRICE = this.dataPrice.NEW_PRICE;
    this.http.post('SL', 'SNM123_Save', this.dataEdit)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          // this.GetData();
          this.IsComfirmed = true;
          //this.dataPrice.FORMER_PRICE = this.dataPrice.NEW_PRICE;
          this.dataPrice.FORMER_PRICE = this.dataEdit.UNIT_PRICE_C;
          this.dataEdit.UNIT_PRICE_C = this.dataPrice.NEW_PRICE;
          this.modal.info('Successfully to save new price', this.viewRef);
          this.datacallback.emit('DONE');

        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewRef);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewRef);
      })
  }


}
