import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';


@Component({
  selector: 'snse170',
  templateUrl: './SNSE170.component.html',
  styleUrls: ['../exportbiz.component.css']
})

export class SNSE170Component implements OnInit {

  public params: any = {};
  public Data: any = {};
  public OldData: any = {};

  public MainDatatable: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;
  public canCreate: any = true;


  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialog", { static: false }) dialog: any;


  public DayList: any = [{ 'NO': 1, 'DAY': 'MONDAY' },
  { 'NO': 2, 'DAY': 'TUESDAY' },
  { 'NO': 3, 'DAY': 'WEDNESDAY' },
  { 'NO': 4, 'DAY': 'THURSDAY' },
  { 'NO': 5, 'DAY': 'FRIDAY' },
  { 'NO': 6, 'DAY': 'SATURDAY' },
  { 'NO': 7, 'DAY': 'SUNDAY' }];

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) { }

  ngOnInit() {
  }

  InitData() {
    this.Data = {};
    this.FrmTitle = '';
  }

  public lookupCustomerCodeConfig: any = {
    ColName: "CODE_NAME",
    PlaceholderText: "Select Customer Code",
    SourceController: "wPMS",
    SourceMethodName: "GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupCustomerCodeSTDConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "WH",
    SourceMethodName: "GetCustomerCodeSTD",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.Search();
  }

  public SetSelectedCustomerCodeDialog(value: any) {
    this.Data.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.Data.CUSTOMER_NAME = value.CUST_NAME_TH;
  }

  Search = () => {
    this.http.post('WH', 'SNSE170_GetData', this.params)
      .subscribe(res => {
        this.MainDatatable = res.Table;
        if (this.MainDatatable.length > 0) {
          this.canCreate = false;
        }
        else {
          this.canCreate = true;
        }
      })
  }

  OpenMainDialog() {
    this.InitData();
    if (this.params.CUSTOMER_CODE) {
      this.Data.CUSTOMER_CODE = this.params.CUSTOMER_CODE
      this.Data.CUSTOMER_NAME = this.params.CUSTOMER_NAME
    }
    this.FrmTitle = "New Standard Working Process";
    this.IsEdit = false;
    this.dialogMain.OpenDialog();
  }

  Edit(P_data) {
    this.InitData();
    this.FrmTitle = "Edit Standard Working Process";
    this.Data = JSON.parse(JSON.stringify(P_data));
    this.OldData = JSON.parse(JSON.stringify(P_data));
    this.IsEdit = true;
    this.dialogMain.OpenDialog();
  }

  Delete(P_data) {
    this.Data = P_data;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.ConfirmDeleteRow);
  }

  ConfirmDeleteRow = () => {
    this.Data.LAST_IP_ADDR = this.global.GetTokenDetail("ssIPAddress");
    this.http.post('WH', 'SNSE170_StandardWorking_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
        }
      });
  }

  SaveAdd() {
    if (this.IsEdit == true) {
      if (this.OldData.PICKING_CARD != this.Data.PICKING_CARD
        || this.OldData.DEPARTURE_SEA != this.Data.DEPARTURE_SEA
        || this.OldData.LEADTIME_AIR != this.Data.LEADTIME_AIR
        || this.OldData.LEADTIME_SEA != this.Data.LEADTIME_SEA
        || this.OldData.SHIPMENT_AIR != this.Data.SHIPMENT_AIR
        || this.OldData.SHIPMENT_SEA != this.Data.SHIPMENT_SEA) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.CUSTOMER_CODE
        && this.Data.PICKING_CARD
        && this.Data.DEPARTURE_SEA
        && this.Data.LEADTIME_AIR
        && this.Data.LEADTIME_SEA
        && this.Data.SHIPMENT_AIR
        && this.Data.SHIPMENT_SEA) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  ConfirmSaveAdd = () => {
    this.http.post('WH', 'SNSE170_StandardWorking_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.params.CUSTOMER_CODE = this.Data.CUSTOMER_CODE
          this.params.CUSTOMER_NAME = this.Data.CUSTOMER_NAME
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  CancelAdd() {
    this.InitData();
    this.dialogMain.Close();
  }

  genDepartureDay() {
    var d = new Date();
    var currentYear = d.getFullYear();
    if (!this.params.TO_YEAR) {
      this.modal.warning('Please Enter Year Number.', this.viewref);
    }
    else if (!this.params.CUSTOMER_CODE) {
      this.modal.warning('Please Enter Customer Code.', this.viewref);
    }
    else if (this.params.TO_YEAR < currentYear) {
      this.modal.warning('Your Data Entry Less Than Current Year.', this.viewref);
    }
    else {
      this.http.post('WH', 'SNSE170_StandardWorking_GenDepartureDay', this.params)
        .subscribe(res => {
          this.modal.info('Create Departure Day Complete.', this.viewref);
          this.Search();
        });
    }
  }
}

  
