import { Component, OnInit, Renderer2, AfterViewInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from 'src/app/core/http.service';
import { MatCalendarCellCssClasses, MatCalendar, } from '@angular/material/datepicker';
import { ModalService } from '../../../core/modal.service';
import * as moment from 'moment';
import { NgForm } from '@angular/forms';
import { UniquePipe } from 'src/app/core/app.pipe';

enum PageAction {
  Search = 0,
  Detail,
  Retrive
}

@Component({
  selector: 'snse171',
  templateUrl: './SNSE171.component.html',
  styleUrls: ['./SNSE171.component.css'],
  providers: [UniquePipe]
})

export class SNSE171Component implements OnInit, AfterViewInit {

  public params: any = {};
  public Data: any = {};
  public CalendarData: any = {};

  public DateEventList: any = [];
  public DepartureDateList: any = []
  public Day: any;
  public DataList: any = [];
  public DataParamList: any = [];
  public selectedDate: any;
  public arrayDate: any = [];
  public departureArrDate: any = [];
  public pickingArrDate: any = [];

  public DayList: any = [{ 'NO': 0, 'DAY': '', 'DAY_NAME': 'All' },
  { 'NO': 1, 'DAY': 'MONDAY', 'DAY_NAME': 'Monday' },
  { 'NO': 2, 'DAY': 'TUESDAY', 'DAY_NAME': 'Tuesday' },
  { 'NO': 3, 'DAY': 'WEDNESDAY', 'DAY_NAME': 'Wednesday' },
  { 'NO': 4, 'DAY': 'THURSDAY', 'DAY_NAME': 'Thursday' },
  { 'NO': 5, 'DAY': 'FRIDAY', 'DAY_NAME': 'Friday' },
  { 'NO': 6, 'DAY': 'SATURDAY', 'DAY_NAME': 'Saturday' },
  { 'NO': 7, 'DAY': 'SUNDAY', 'DAY_NAME': 'Sunday' }];

  public YearForButton: any;
  public MonthForButton: any;
  public startAt: any;

  public rowData: any;
  public addData: any = {};
  public ProcessYearList: Array<number> = [];

  public lookupCustomerCodeSTDConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "WH",
    SourceMethodName: "GetCustomerCodeSTD",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public PageAction: number;

  @ViewChild(MatCalendar) calendar: MatCalendar<Date>;
  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("addDialog") addDialog: any;
  @ViewChild("addForm") addForm: NgForm;

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private renderer: Renderer2,
    private uniquePipe: UniquePipe
  ) { }

  ngOnInit() {
    this.params.ISSUE_PICKING_CARD = "";
    this.CurrentDate();
    this.GetEventCalendar();
  }

  //for click Next/Previous button on MatCalendar 
  ngAfterViewInit() {
    let Nextbutton = document.querySelectorAll('mat-calendar .mat-calendar-next-button');
    let Previousbutton = document.querySelectorAll('mat-calendar .mat-calendar-previous-button');
    if (Nextbutton) {
      Array.from(Nextbutton).forEach(button => {
        this.renderer.listen(button, "click", () => {
          let Month = (parseInt(this.CalendarData.MONTH) + 1)
          if (Month > 11) { Month = 0; this.CalendarData.YEAR = (parseInt(this.CalendarData.YEAR) + 1).toString(); }
          this.CalendarData.MONTH = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
          this.GetEventCalendar();
        });
      })
    }
    if (Previousbutton) {
      Array.from(Previousbutton).forEach(button => {
        this.renderer.listen(button, "click", () => {
          let Month = (parseInt(this.CalendarData.MONTH) - 1)
          if (Month < 0) { Month = 11; this.CalendarData.YEAR = (parseInt(this.CalendarData.YEAR) - 1).toString(); }
          this.CalendarData.MONTH = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
          this.GetEventCalendar();
        });
      })
    }
  }

  public OnClickRow(data) {
    if (!data.EDIT) {

      this.rowData = data
      let Year = data.PROCESS_YEAR;
      let Month = data.PROCESS_MONTH - 1;
      this.CalendarData.YEAR = Year.toString()
      this.CalendarData.MONTH = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()

      this.GetEventCalendar();
      this.startAt = moment(new Date(Year, Month));
      this.calendar._goToDateInView(this.startAt, 'month')
      this.Data.CUSTOMER_CODE = data.CUSTOMER_CODE
      this.Data.CUSTOMER_NAME = data.CUSTOMER_NAME
      this.GetCalendarWKP(data);
      this.calendar.updateTodaysDate();
    }
  }

  public YearSelected(date) {
    let Year = date._i.year;
    this.CalendarData.YEAR = Year.toString()
  }

  public MonthSelected(date) {
    let Year = date._i.year;
    let Month = date._i.month;
    this.CalendarData.YEAR = Year.toString()
    this.CalendarData.MONTH = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    this.GetEventCalendar();
  }

  private CurrentDate() {
    let d = new Date();
    let Month = d.getMonth();
    this.CalendarData.MONTH = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    this.CalendarData.YEAR = d.getFullYear().toString();
  }

  private GetEventCalendar() {
    var Year = this.CalendarData.YEAR
    var Month = (parseInt(this.CalendarData.MONTH) + 1).toString()
    this.http.get('WH', 'SNSE171_GetCalendar', Year, Month)
      .subscribe(res => {
        this.DateEventList = res.Table[0]
        this.AddDateVariables()
        this.CheckHolidayEvent()
      })
  }

  public AddDateVariables() {
    this.Day = [];
    this.YearForButton = this.CalendarData.YEAR
    this.MonthForButton = (parseInt(this.CalendarData.MONTH) - 1).toString()
    if (this.DateEventList) {
      for (var i = 1; i < 32; i++) {
        let dateString = this.YearForButton + '-' + this.MonthForButton + '-' + i;
        this.Day[i] = {
          "TYPE": this.DateEventList["DAY" + i],
          "DATE_d": new Date(dateString)
        };
      }
      return this.Day;
    }
  }

  //for change date text color to red show holiday on calendar
  private CheckHolidayEvent() {
    if (this.Day[1]) {
      var dateNumber = document.getElementsByClassName('mat-calendar-body-cell-content') as HTMLCollectionOf<HTMLElement>;
      for (var i = 1; i < dateNumber.length; i++) {
        if (this.Day[i].TYPE == 'H') {
          dateNumber[i - 1].style.color = 'red';
        }
      }
    }
  }

  public DateClass() {
    return (date: Date): MatCalendarCellCssClasses => {
      const highlightPickingDate = this.arrayDate
        .map(strDate => new Date(strDate))
        .some(d => d.getDate() === date["_d"].getDate() &&
          d.getMonth() === date["_d"].getMonth() &&
          d.getFullYear() === date["_d"].getFullYear());

      const highlightDepartureDate = this.departureArrDate
        .map(strDate2 => new Date(strDate2))
        .some(d => d.getDate() === date["_d"].getDate() &&
          d.getMonth() === date["_d"].getMonth() &&
          d.getFullYear() === date["_d"].getFullYear());

      if (highlightPickingDate) {
        return 'issue-date';
      }
      else if (highlightDepartureDate) {
        return 'departure-date';
      }
      else {
        return
      }
    }
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  public Search = () => {
    this.DataList = [];
    if (this.params.PROCESS_MONTH) {
      this.params.PROCESS_MONTH = this.params.PROCESS_MONTH.toString().length < 2 ? '0' + this.params.PROCESS_MONTH.toString() : this.params.PROCESS_MONTH.toString()
    }
    if (!this.params.CUSTOMER_CODE) {
      this.modal.warning('Please select Customer Code.', this.viewref);
    }
    else {
      this.http.post('WH', 'SNSE171_GetData', this.params)
        .subscribe(res => {
          this.DataList = res.Table;
          this.DataParamList = JSON.parse(JSON.stringify(res.Table))
          this.arrayDate = [];

          if (this.DataList.length > 0) {
            this.DataList.forEach((e) => {
              var dayName = e.ISSUE_PICKING_CARD
              var month = e.PROCESS_MONTH
              var year = e.PROCESS_YEAR
              var week1 = e.WEEK1
              var week2 = e.WEEK2
              var week3 = e.WEEK3
              var week4 = e.WEEK4
              var week5 = e.WEEK5
              this.CheckIssuePickingCard(dayName, month, year, week1, week2, week3, week4, week5)
            });


            this.OnClickRow(this.DataList[0])
          }
          //this.GetCalendarWKP(this.DataList[0]);

          this.DataList.push({
            'EDIT': 'ADD',
            'SEQ': null,
            'CUSTOMER_CODE': null,
            'PROCESS_YEAR': null,
            'PROCESS_MONTH': null,
            'ISSUE_PICKING_CARD': null,
            'CIPCD': null,
            'WEEK1': null,
            'WEEK2': null,
            'WEEK3': null,
            'WEEK4': null,
            'WEEK5': null,

          });
          this.DataList = this.DataList.slice();
          this.PageAction = PageAction.Retrive;
        })
    }
  }

  private GetCalendarWKP(data) {
    this.http.post('WH', 'SNSE171_GetCalendarWKP', data)
      .subscribe(res => {
        this.DepartureDateList = res.Table;
        this.departureArrDate = [];
        this.pickingArrDate = [];

        this.DepartureDateList.forEach((e, index) => {
          if (e.PROCESS_TYPE == "D") {
            this.departureArrDate[index] = new Date(e.PROCESS_DATE)
          }
          else if (e.PROCESS_TYPE == "P") {
            this.pickingArrDate[index] = new Date(e.PROCESS_DATE)
          }
        });
      })
  }

  private CheckIssuePickingCard(day, month, year, week1, week2, week3, week4, week5) {
    let dayNumber

    switch (day) {
      case "MONDAY":
        dayNumber = 1;
        break;
      case "TUESDAY":
        dayNumber = 2;
        break;
      case "WEDNESDAY":
        dayNumber = 3;
        break;
      case "THURSDAY":
        dayNumber = 4;
        break;
      case "FRIDAY":
        dayNumber = 5;
        break;
      case "SATURDAY":
        dayNumber = 6;
        break;
      case "SUNDAY":
        dayNumber = 0;
    }
    var days = new Date(year, month, 0).getDate();
    var ary = [];
    for (var i = 1; i <= days; i++) {
      var date = new Date(year, month - 1, i);
      if (date.getDay() == dayNumber) {
        let newDate = date.getDate().toString().length < 2 ? '0' + date.getDate().toString() : date.getDate().toString();
        month = month.toString().length < 2 ? '0' + month.toString() : month.toString();
        ary.push(new Date(year + "-" + month + "-" + newDate));
      }
    };
    if (week1 != "Y") {
      ary[0] = ''
    }
    if (week2 != "Y") {
      ary[1] = ''
    }
    if (week3 != "Y") {
      ary[2] = ''
    }
    if (week4 != "Y") {
      ary[3] = ''
    }
    if (week5 != "Y") {
      ary[4] = ''
    }
    for (var j = 0; j < ary.length; j++) {
      this.arrayDate.push(ary[j])
    }
  }

  public checkCIPCD(data, week, event) {
    var index = (parseInt(data.SEQ) - 1)
    if (event.checked == false) {
      data[week] = 'N'
    }
    else if (event.checked == true) {
      data[week] = 'Y'
    }
    if (data.EDIT == null) {
      if (data.WEEK1 == this.DataParamList[index].WEEK1 &&
        data.WEEK2 == this.DataParamList[index].WEEK2 &&
        data.WEEK3 == this.DataParamList[index].WEEK3 &&
        data.WEEK4 == this.DataParamList[index].WEEK4 &&
        data.WEEK5 == this.DataParamList[index].WEEK5) {
        return data.CIPCD = 'Y'
      }
      else {
        return data.CIPCD = 'N'
      }
    }
  }

  public AddBtnClick() {
    this.SearchDataAdd();
    this.addData.CUSTOMER_CODE = this.params.CUSTOMER_CODE;
    this.addData.PROCESS_YEAR = moment().format("YYYY")
    this.addDialog.OpenDialog();
  }

  private SearchDataAdd() {
    this.ProcessYearList = [];
    this.http.post('WH', 'SNSE171_GetData', {
      CUSTOMER_CODE: this.params.CUSTOMER_CODE
    })
      .subscribe(res => {
        this.ProcessYearList = this.uniquePipe.transform(res.Table, "PROCESS_YEAR");
        this.addData.OLD_PROCESS_YEAR = Math.max(...this.ProcessYearList);
      })
  }

  public SaveAdd() {
    if (!this.addForm.controls["processyear"].valid) {
      this.modal.warning("Incorrect Process Year.", this.viewref);
      return;
    }
    if (this.ProcessYearList.indexOf(this.addData.PROCESS_YEAR) >= 0) {
      this.modal.warning("Process year is already exists.", this.viewref);
      return;
    }
    if (this.addData.PROCESS_YEAR < moment().format("YYYY")) {
      this.modal.warning("Cannot create past year.", this.viewref);
      return;
    }
    this.modal.confirm('Do you want to add process year from ' + this.addData.OLD_PROCESS_YEAR + '?', this.viewref,
      () => {
        this.http.post("WH", "SNSE171_AddProcessYear", this.addData)
          .subscribe(result => {
            if (result == "Done") {
              this.modal.info('Transaction complete: applied and saved.', this.viewref);
              this.Search();
              this.CloseDialog();
            } else {
              this.modal.error(result, this.viewref);
            }
          }, err => {
            this.modal.error(err.message, this.viewref)
          })
      })

  }

  public SaveEdit() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveEdit)
  }

  private ConfirmSaveEdit = () => {
    var formData = this.DataList.filter(e => e.CUSTOMER_CODE && e.PROCESS_YEAR && e.PROCESS_MONTH);

    this.http.post('WH', 'SNSE171_PickingCard_Save', formData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  public CreateIssue() {

    var formData = this.DataList.filter(e => e.CIPCD === 'N' && e.CUSTOMER_CODE && e.PROCESS_YEAR && e.PROCESS_MONTH);

    this.http.post('WH', 'SNSE171_PickingCard_Save', formData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.http.post('WH', 'SNSE171_CreateIssuePicking', formData)
            .subscribe(res => {
              if (res.Table[0].STATUS == 'DONE') {
                this.Search();
                this.modal.info('Create Issue Picking Card Complete.', this.viewref);
              }
            });
        }
      })
  }

  public CloseDialog() {
    this.addDialog.Close();
  }


  public AppendRow(Item) {
    for (var i = 0; i < this.DataList.length; i++) {
      if (this.DataList[i].EDIT === "ADD") {
        this.DataList[i].EDIT = 'DEL';
        this.DataList[i].SEQ = this.DataList.length;
        this.DataList[i].CUSTOMER_CODE = this.params.CUSTOMER_CODE
        this.DataList[i].CIPCD = 'N'
        this.DataList[i].WEEK1 = 'N'
        this.DataList[i].WEEK2 = 'N'
        this.DataList[i].WEEK3 = 'N'
        this.DataList[i].WEEK4 = 'N'
        this.DataList[i].WEEK5 = 'N'
      }
    }
    this.DataList.push({
      'EDIT': 'ADD',
      'SEQ': null,
      'CUSTOMER_CODE': null,
      'PROCESS_YEAR': null,
      'PROCESS_MONTH': null,
      'ISSUE_PICKING_CARD': null,
      'CIPCD': null,
      'WEEK1': null,
      'WEEK2': null,
      'WEEK3': null,
      'WEEK4': null,
      'WEEK5': null,
    });
    this.DataList = this.DataList.slice();

  }

  public DeleteRow(Item) {
    if (this.PageAction == PageAction.Retrive) {
      let index = this.DataList.findIndex(x => x == Item)
      this.DataList.splice(index, 1);
      if (this.DataList.find(e => e.EDIT === 'ADD') == undefined) {
        this.DataList.push({
          'EDIT': 'ADD',
          'SEQ': null,
          'CUSTOMER_CODE': null,
          'PROCESS_YEAR': null,
          'PROCESS_MONTH': null,
          'ISSUE_PICKING_CARD': null,
          'CIPCD': null,
          'WEEK1': null,
          'WEEK2': null,
          'WEEK3': null,
          'WEEK4': null,
          'WEEK5': null,
        });
      }
      this.DataList = this.DataList.slice();
    }
  }

  public CheckProcessYear(data) {
    if (data.CUSTOMER_CODE) {
      if (data.PROCESS_YEAR.length < 4) {
        this.modal.warning("Format process Year is ''20XX''", this.viewref);
        data.PROCESS_YEAR = null;
      }
      else if (data.PROCESS_YEAR < new Date().getFullYear()) {
        this.modal.warning("Process Year < Current Year is " + new Date().getFullYear(), this.viewref);
        data.PROCESS_YEAR = null;
      }
    }
  }

  public CheckProcessMonth(data) {
    if (data.CUSTOMER_CODE) {
      if (data.PROCESS_YEAR) {
        if (data.PROCESS_MONTH) {
          let duplicate = this.DataParamList.some(e => e.PROCESS_YEAR === data.PROCESS_YEAR && e.PROCESS_MONTH === data.PROCESS_MONTH)
          if (duplicate) {
            this.modal.warning("Process month is duplicate.", this.viewref);
            data.PROCESS_MONTH = null;
          } else {
            this.GetPickingCard(data)
          }

        }
      }
    }
  }

  private GetPickingCard(data) {
    this.http.post('WH', 'SNSE171_GetPickingCard', data)
      .subscribe(res => {
        console.log(res)
        data.ISSUE_PICKING_CARD = res.Table[0].PICKING_CARD;
      });
  }//
}
