import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';

enum PageAction {
  Search = 0,
  Detail,
  Retrive
}

@Component({
  selector: 'SNSE214',
  templateUrl: './SNSE214.component.html',
  styleUrls: ['./SNSE214.component.css']
})

export class SNSE214Component implements OnInit {

  constructor(
    private Http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewRef: ViewContainerRef
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;

  public ListLog: any = [];
  public params: any = {};
  public dataHead: any = {};
  public MsgLog: any = [];

  public selection = new SelectionModel<any>(true, []);
  public DataIsSelected: any = [];
  public PageAction: number;

  public tableAllData: any = [];
  public tableData: any = [];

  public IsSearch: boolean = true;
  public IsDiff: number = 0;
  public uploadState: number = 0;

  public files: any = [];
  public FileName: string = ""
  public FileAttachedName: string = ""


  public ConfigCustomer: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Customer",
    SourceController: "SL",
    SourceMethodName: "SNSE005_N_GetCustomer",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  ngOnInit() {
    this.params.FROM_DATE = new Date("October 1, 2020 07:00:00");
    this.params.TO_DATE = new Date();
    this.params.CHK_BOX = 'N';
  }

  public delay(ms: number) {
    return new Promise(resolve => setTimeout(resolve, ms));
  }

  //--------------------------------------

  public Search() {
    this.tableAllData = [];
    this.tableData = [];
    this.Http.post('SL', 'SNSE214_GetData', this.params)
      .subscribe(res => {
        this.tableAllData = res.Table;
        if (this.params.CHK_BOX == 'N') {
          this.tableData = res.Table;
        } else {
          this.tableData = this.tableAllData.filter(v => v.FILE_NAME === 'No file');
        }

      }, (err) => {
        console.log('err' + err);
        this.modal.error("Found an error, please contract support team", this.viewRef);

      })
  }

  public ShowUploadDialog(data) {
    //this.params = {};
    this.params.CUSTOMER_CODE = data.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = data.CUSTOMER_NAME;
    this.params.ORDER_NO = data.ORDER_NO;
    this.params.ORDER_DATE = data.ORDER_DATE;
    this.FileAttachedName = '';
    this.dialogMain.OpenDialog();

  }

  public OnFileAttachedSelect(fileInput) {
    this.files = fileInput.target.files;
    this.FileAttachedName = this.files[0].name;
    this.uploadState = 1;
  }


  public AttachedFile() {

    if (this.files.length == 0) {
      this.modal.error('Please select the file...', this.viewRef)
    } else {

      let formData = new FormData();
      formData.append('file', this.files[0]);
      formData.append('CUSTOMER_CODE', this.params.CUSTOMER_CODE);
      formData.append('ORDER_NO', this.params.ORDER_NO);
      formData.append('ORDER_DATE', this.params.ORDER_DATE);

      this.Http.postFile('SL', 'SNSE214_Attachedfile', formData)
        .subscribe(async res => {
          if (res.substring(0, 4) == "DONE") {
            this.Search();
            this.modal.info("Upload attached file succesfully.", this.viewRef);

          } else {
            this.modal.warning("Found an error, please contract support team", this.viewRef);
          }

        }, (err) => {
          console.log('err' + err);
          this.modal.error("Found an error, please contract support team", this.viewRef);

        })

      this.uploadState = 0;
    }
  }


  public OnFileSelect(fileInput) {
    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;
  }


  public CheckChange(value) {
    if (value.checked == true) {
      this.params.CHK_BOX = 'Y';
      this.tableData = this.tableAllData.filter(v => v.FILE_NAME === 'No file');
    }
    else {
      this.params.CHK_BOX = 'N';
      this.tableData = this.tableAllData;
    }
    this.tableData.slice();
  }

  private isNumeric(number) {
    if (typeof number == "number") {
      return true;
    }
    else {
      return false;
    }
  }
}
