import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snse064',
  templateUrl: './snse064.component.html',
  styleUrls: ['./snse064.component.css']
})



export class SNSE064Component implements OnInit {


  constructor(
    private http: HttpService,
    private router: Router,
    private modal : ModalService,
    private viewref : ViewContainerRef
  ) { }

  public ConfigDoc: any = {
    ColName: "DOCUMENT_NO",
    ColValue: "",
    PlaceholderText: "Select Document No.",
    SourceController: "IC",
    SourceMethodName: "SNSE064_GetDocList",
    NoOfShowRecords: 30,
    UpperCase: true,
    MinLengthSearch : 5,
    SourceParams: {
      CUSTOMER_CODE : null,
      BRANCH_CODE : null
    },
  }


  public ConfigPart: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Part No.",
    SourceController: "IC",
    SourceMethodName: "SNSE064_GetPart",
    NoOfShowRecords: 30,
    UpperCase: true,
    MinLengthSearch : 0,
    SourceParams: {
      DOCUMENT_TYPE : null,
      DOCUMENT_NO : null
    },
  }

  public ConfigCustomer: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Customer.",
    SourceController: "IC",
    SourceMethodName: "SNSE064_Customer",
    NoOfShowRecords: 30,
    UpperCase: true,
    MinLengthSearch : 3,
  }


  public ConfigBranch: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Branch.",
    SourceController: "IC",
    SourceMethodName: "SNSE064_GetBranch",
    NoOfShowRecords: 30,
    UpperCase: false,
    MinLengthSearch : 0,
    SourceParams: {
      CUSTOMER_CODE : null
    },
  }

  public ConfigRuunig: any = {
    ColName: "RUNNING_NO",
    ColValue: "",
    PlaceholderText: "Select Running.",
    SourceController: "IC",
    SourceMethodName: "SNSE064_RunningNo",
    NoOfShowRecords: 30,
    UpperCase: true,
    MinLengthSearch : 1,
  }

  public paramSearch : any = {}
  public tableHeader = [];
  public showDetail : boolean = false
  public params : any = {};
  public iSAdd : boolean = false;
  public tableDetail = [];
  public defectiveList = [];
  public hideCols = []
  public iSSave : boolean = false;

  public tblFooter: any = [
    {
        fieldName: "LOCATION_NO",
        value: "Total Qty.",
        show: true
    },
    {
        fieldName: "RT_QTY",
        show: true
    }
]

  ngOnInit() {
    //this.paramSearch.DOC_DATE_SEARCH = new Date();
    this.GetDefective()
  }

  private GetDefective(){
    this.http.get("IC","SNSE064_GetDefactive")
    .subscribe(res =>{
      this.defectiveList = res.Table
    })
  }

  public Search(){

    //if(!this.paramSearch.RUNNING_NO && !this.paramSearch.DOC_DATE_SEARCH && !this.paramSearch.CUSTOMER_CODE){
    //  this.modal.warning("Please input 1 criteria",this.viewref);
    //  return
    //}

    if(this.paramSearch.DOC_DATE_SEARCH){
      this.paramSearch.DOC_DATE = this.FormatDate(this.paramSearch.DOC_DATE_SEARCH)
    }else{
      this.paramSearch.DOC_DATE = null
    }

    this.http.post("IC","SNSE064_GetData",this.paramSearch)
    .subscribe(res => {
      this.tableHeader = res.Table
    })
  }

  public ViewClick(event){
    
    this.showDetail = true
    this.hideCols = ["EDIT"];
    this.iSAdd = false;
    this.params = event

    this.http.get("IC","SNSE064_GetDetail",event.RUNNING_NO)
    .subscribe(res => {
      this.tableDetail = res.Table
    })
  }

  public SelectedCust(event){
    this.params.CUSTOMER_CODE = event.CUSTOMER_CODE
    this.params.CUST_NAME_TH = event.CUST_NAME_TH
    this.ConfigDoc.SourceParams.CUSTOMER_CODE = event.CUSTOMER_CODE
    this.ConfigBranch.SourceParams.CUSTOMER_CODE = event.CUSTOMER_CODE
  }

  public SelectedBranch(event){
    this.params.BRANCH_CODE = event.BRANCH_CODE
    this.params.BRANCH_NAME_TH = event.BRANCH_NAME_TH
    this.ConfigDoc.SourceParams.BRANCH_CODE = event.BRANCH_CODE
    
  }

  public SelectDocument(event,data){
    
    if(event == undefined){
      data.DO_NO = null
      data.DO_DATE = null
      data.DOCUMENT_TYPE = null
      return
    }

    
    data.DO_NO = event.DOCUMENT_NO
    data.DO_DATE = event.DOCUMENT_DATE
    data.DOCUMENT_TYPE = event.DOCUMENT_TYPE
  }


  public SetSourcePart(data){
    this.ConfigPart.SourceParams.DOCUMENT_NO = data.DO_NO
    this.ConfigPart.SourceParams.DOCUMENT_TYPE = data.DOCUMENT_TYPE
  }

  public SelectPart(event,data){

    if(event == undefined){
      data.PART_NO = null
      data.PART_NAME = null
      data.LOCATION_NO = null
      data.RT_QTY = null
      return
    }

    data.PART_NO = event.ACCEPT_PART_NO

    let check = this.tableDetail.filter(x => x.PART_NO == event.ACCEPT_PART_NO && x.DO_NO == data.DO_NO)

    if(check.length > 1){
      this.modal.warning("Duplicate Part No",this.viewref)
      data.PART_NO = null
    }else{
      data.PART_NO = event.ACCEPT_PART_NO
      data.PART_NAME = event.PART_NAME
      data.LOCATION_NO = event.LOCATION_NO
      data.RT_QTY = 1
      this.tableDetail = this.tableDetail.slice();
    }

  }

  public CreateRunning(){
    this.showDetail = true
    this.hideCols = [];
    this.iSAdd = true
    this.iSSave = false
    this.params = {}
    this.tableDetail = []
    this.params.DOC_DATE = new Date()
    this.params.RUNNING_NO = 'Auto'

    //this.ConfigDoc.SourceParams.CUSTOMER_CODE 

    this.tableDetail.push({ 'EDIT': 'DEL',ITEM_NO : 1},
    { 'EDIT': 'DEL',ITEM_NO : 2},
    { 'EDIT': 'DEL',ITEM_NO : 3},
    { 'EDIT': 'DEL',ITEM_NO : 4},
    { 'EDIT': 'ADD',ITEM_NO : 5});
  }

  public QtyChange(data){
    this.tableDetail = this.tableDetail.slice();
  }

  public Back(){
    this.showDetail = false
  }

  public Save(){

    let CompleteData = this.tableDetail.filter(x => x.DO_NO && x.PART_NO && x.RT_QTY > 0 && x.ATTRIBUTE2)

    if(CompleteData.length == 0){
      this.modal.warning("No detail data for save",this.viewref)
      return
    }else{
      let count = 0;
      this.tableDetail.forEach(x =>{
        if (x.DO_NO && !x.PART_NO) {
          this.modal.warning("Please insert part No at Item No. : " + x.ITEM_NO, this.viewref);
          count++;
          return
        }

        if (x.DO_NO && x.RT_QTY < 0) {
          this.modal.warning("Please insert Qty at Item No. : " + x.ITEM_NO, this.viewref);
          count++;
          return
        }

        if (x.DO_NO && !x.ATTRIBUTE2) {
          this.modal.warning("Please insert ชำรุด at Item No. : " + x.ITEM_NO, this.viewref);
          count++;
          return
        }
      })

      if(count != 0){
        return
      }
    }

    this.modal.confirm("Do want to save change you have made?",this.viewref,()=> ConfirmSave())

    let ConfirmSave = () => {

      let Data = {
        CUSTOMER_CODE : this.params.CUSTOMER_CODE,
        BRANCH_CODE : this.params.BRANCH_CODE,
        DETAIL : CompleteData
      }

      this.http.post("IC","SNSE064_Save",Data)
      .subscribe(res => {
        //console.log('reeee',res)
        if(res.STATUS == 'DONE'){
          this.iSSave = true;
          this.params.RUNNING_NO = res.RUNNING_NO;
          this.modal.info("Transaction complete record apply and save",this.viewref)
        }else{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        }
      },err =>{
        this.modal.warning("Found an error, please contact support team", this.viewref);
      })
    }

  }

  public AppendRow() {

    if(this.iSSave){
      return
    }

    for (var i = 0; i < this.tableDetail.length; i++) {
      if (this.tableDetail[i].EDIT === "ADD") {
        this.tableDetail[i].EDIT = 'DEL';
      }
    }
    this.tableDetail.push({ 'EDIT': 'ADD'});
    this.tableDetail.map((x,i)=> {
      x.ITEM_NO = i + 1
    })
    this.tableDetail = this.tableDetail.slice();
  }

  public DeleteRow(index,data) {

    if(this.iSSave){
      return
    }

    this.tableDetail.splice(index, 1);
    this.tableDetail.map((x,i)=> {
      x.ITEM_NO = i + 1
    })
    this.tableDetail = this.tableDetail.slice();
 
  }

  FormatDate(date) {

    let d = new Date(date);
    let Day = d.getDate().toString();
    let NewDay = Day.toString().length < 2 ? '0' + Day.toString() : Day.toString()
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    return Year + NewMonth + NewDay;
}

}
