import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "../../../core/modal.service";
import { SelectionModel } from "@angular/cdk/collections";

@Component({
  selector: "SNM141P",
  templateUrl: "./SNM141P.component.html",
})
export class SNM141PComponent implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
  ) {}

  public tableData: any = [];
  public tempData: any = [];
  public files: any = [];
  public error_list: any = [];
  public fileName: string = "";
  public duplicateParts: any;
  public total_error: number;
  public checkLockSales: boolean = true;
  public checkLockPO: boolean = true;
  public selection = new SelectionModel<any>(true, []);
  
  @ViewChild("dialog",{static: false}) dialog : any;

  ngOnInit() {}

  public DownLoadTemplate() {
    let link = "assets/templates/SNM141P_Upload_Template.xlsx";
    window.open(link, "_blank");
  }

  public FileChange(fileInput) {
    this.selection.clear();
    this.files = fileInput.target.files;
    this.fileName = this.files[0].name;

    this.UpLoad();
  }

  public ClearFile() {
    this.files = [];
    this.fileName = "";
  }

  private UpLoad() {
    let formData = new FormData();
    formData.append("file", this.files[0]);

    this.http.postFile("IC", "SNM141P_UploadExcel", formData).subscribe(
      (res) => {
        this.total_error = 0;
        this.tableData = res.filter((x) => x.PART_NO);
        let countrow = 0;
        this.total_error = this.tableData.filter((x, i) => {
          if (x.ERR_FLAG == "Y") {
            countrow += 1;
            this.error_list.push({
              Records: i + 1,
            });
          }
        });
        this.total_error = countrow;
           
        this.LSChange({checked : this.checkLockSales})
        this.LPChange({checked : this.checkLockPO})
      },
      (err) => {
        this.modal.error("Upload error", this.viewRef);
      }
    );
  }


  public LSChange(event){
    if(this.tableData.length == 0)return;

    // let msg_err_reason = 'Does not exist Lock PO on @TPM_REASON_CODE!'
    let msg_err_part = 'Does not exist on PARTS_MASTER!'

    if(event.checked){        
        this.tableData.map(x =>{
            if(x.ERR_FLAG_PART == 'Y'){
                x.MSG = msg_err_part
            }else if (x.ERR_FLAG_LS == 'Y'){
                x.MSG = x.ERR_MSG_LS
            }
        })
    }else{
        if(this.checkLockPO){   
          this.tableData.map(x =>{
              if(x.ERR_FLAG_PART == 'Y'){
                  x.MSG = msg_err_part
              }else if(x.ERR_FLAG_LP == 'Y'){
                  x.MSG = x.ERR_MSG_LP 
              }else if(x.ERR_FLAG_PART != 'Y' && x.ERR_FLAG_LP != 'Y'){
                  x.MSG = null
              }
          })
        }else{
          this.tableData.map(x =>{
                  x.MSG = null
          })
        }
    }

    this.tableData = this.tableData.slice();
    this.CheckSelect();
}

public LPChange(event){
  if(this.tableData.length == 0)return;
  
  // let msg_err_reason = 'Does not exist Lock Sales on @TPM_REASON_CODE!'
  let msg_err_part = 'Does not exist on PARTS_MASTER!'
    
  if(event.checked){
      this.tableData.map(x =>{
          if(x.ERR_FLAG_PART == 'Y'){
              x.MSG = msg_err_part
          }else if (x.ERR_FLAG_LP == 'Y'){
              x.MSG = x.ERR_MSG_LP
          }
      })
  }else{
      if(this.checkLockSales){        
        this.tableData.map(x =>{
          if(x.ERR_FLAG_PART == 'Y'){
              x.MSG = msg_err_part
          }else if(x.ERR_FLAG_LS == 'Y'){
              x.MSG = x.ERR_MSG_LS
          }else if(x.ERR_FLAG_PART != 'Y' && x.ERR_FLAG_LS != 'Y'){
              x.MSG = null
          }
      })
      }else{
        
        this.tableData.map(x =>{
                x.MSG = null
        })
      }
  }

  this.tableData = this.tableData.slice();
  this.CheckSelect();
}

public CheckErr(){
  return this.tableData.filter(x => x.ERR_FLAG == 'N').length
}

public CheckSelect(){
  if(!this.checkLockSales && !this.checkLockPO){
      this.selection.clear()
  }
}

isAllSelected() {
  const numSelected = this.selection.selected.length;
  const numRows = this.tableData.filter(x => !x.MSG).length;
  return numSelected === numRows;
}


  Toggle() {
    
    this.isAllSelected()
      ? this.selection.clear()
      : this.tableData.forEach((row) => {
        if (!row.MSG) {
          this.selection.select(row)
        }
        });


        console.log(this.selection.selected)
  }

  public UpdateData() {
    this.modal.confirm(
      "Do you want to save change you have made?",
      this.viewRef,
      this.ConfirmUpdateData
    );
  }

  private ConfirmUpdateData = () => {
    let isLockSales = this.checkLockSales == true ? "Y" : "N";
    let isLockPO = this.checkLockPO == true ? "Y" : "N";
    
    let datafilter = []
    if(this.checkLockSales && this.checkLockPO){
        datafilter = this.selection.selected.filter(x => x.ERR_FLAG_LS == 'N' && x.ERR_FLAG_LP == 'N')
    }else{
        if(this.checkLockSales){
            datafilter = this.selection.selected.filter(x => x.ERR_FLAG_LS == 'N')
        }else{
            datafilter = this.selection.selected.filter(x => x.ERR_FLAG_LP == 'N')
        }
    }

    let data = {
      IS_LOCK_SALES: isLockSales,
      IS_LOCK_PO: isLockPO,
      DETAIL: datafilter,
    };

    this.http.post("IC", "SNM141P_UpdateData", data).subscribe(
      (res) => {
        if (res.STATUS == "DONE") {
          this.modal.info(
            'Transaction complete: ' +  datafilter.length  +' record applied and saved.',
            this.viewRef
          );
          this.selection.clear();
        } else {
          this.modal.error(
            "No change to save. Something went wrong.",
            this.viewRef
          );
        }
      },
      (err) => {
        this.modal.error(
          "No change to save. Something went wrong.",
          this.viewRef
        );
      }
    );
  };
}
