import { Component, OnInit, ViewContainerRef, ViewChild, ElementRef } from "@angular/core";
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from 'src/app/core/global';
import * as moment from 'moment';


@Component({
  selector: 'snpe050',
  templateUrl: './SNPE050.component.html',
  styleUrls: ['./SNPE050.component.css']
})

export class SNPE050Component implements OnInit {

  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) { }

  public params: any = {};
  public tableData: any = [];
  public poRange: any = {};
  public poDate: moment.Moment;
  public file: any;
  public fileName: any;
  public isProcessed: boolean;

  @ViewChild("fileUpload") fileUpload: ElementRef;

  ngOnInit() {
    this.params.ORDER_TYPE = null;
    this.params.CNT_REC = 0;
    this.params.CNT_ERR = 0;
    this.isProcessed = false;
  }

  public DownloadTemplate() {
    let link = "assets/templates/PMS_UploadGen_PO_Template.xlsx"
    window.open(link, '_blank')
  }

  public OpenFileBrowser() {
    this.isProcessed = false;
    this.params.ORDER_TYPE = null;
    this.params.CNT_REC = 0;
    this.params.CNT_ERR = 0;
    this.tableData = [];
    this.fileName = "";
    this.fileUpload.nativeElement.value = '';
    this.fileUpload.nativeElement.click();
  }

  public SelectFile(evt) {
    this.file = null;
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
      this.file = Files[0];
      this.fileName = this.file.name;
    }
  }

  public PreviewExcel() {
    if (!this.fileName) {
      this.modal.warning("Please open the file...", this.viewref)
      return;
    }

    let formData = new FormData();
    formData.append('file', this.file);
    formData.append('type', this.params.TYPE);
    this.Http.postFile("IC", "SNPE050_Upload", formData)
      .subscribe(result => {
        this.Http.get("IC", "SNPE050_GetTempData")
          .subscribe(result => {
            this.tableData = result.Table;
            this.params.CNT_REC = this.tableData.length;
            this.params.CNT_ERR = this.tableData.filter(v => v.ERROR_MSG).length;

          })
      }, err => {
        console.log(err);
        this.modal.error("Found error, please contact support team.", this.viewref);
      })
  }

  public Generate() {
    let foundErr = null;
    foundErr = this.tableData.some(x => x.ERROR_MSG)
    if (this.tableData.length <= 0) {
      this.modal.warning('There is no data for generate PO!!', this.viewref)
      return;
    }
    if (foundErr) {
      this.modal.warning('Found ' + this.params.CNT_ERR + ' parts error, please check error message!!', this.viewref)
      return;
    }
    this.modal.confirm('Do you want to generate PO?', this.viewref, this.ConfirmGenerate)
  }

  private ConfirmGenerate = () => {
    this.Http.post("IC", "SNPE050_BeginProcess", this.params)
      .subscribe(result => {
        this.isProcessed = true;
        this.modal.info("Generate PO Completed.", this.viewref);
        this.poDate = moment();
        this.poRange = result;
      }, err => {
        console.log(err);
        this.modal.error("Found error, please contact support team.", this.viewref);
      })
  }

  public PrintReport() {
    this.global.GoToReport("PMS_RPT_IC_SNPP021", {
      P_PO_DATE: this.global.ConvertDateToFormatSTD(this.poDate),
      P_PO_NO_FROM: this.poRange.START_PO,
      P_PO_NO_TO: this.poRange.END_PO,
    })
  }
}
