import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router, ActivatedRoute } from '@angular/router';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';
import * as moment from 'moment';


@Component({
  selector: 'snpp021lo',
  templateUrl: './snpp021lo.component.html',
  styleUrls: ['./snpp021lo.component.css']
})



export class SNPP021LOComponent implements OnInit {

  @ContentChild(TemplateRef, { static: false }) templateRef;
  @ViewChild(MatPaginator, { static: true }) paginator: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort: MatSort;

  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private route: ActivatedRoute,
    private global: Globals
  ) {

  }

  public data: any = [];
  public vendorList: any = [];
  public poRange: any = {};
  public poDate: moment.Moment;

  ngOnInit() {
    this.GetData();
    this.GetVendor();
  }

  public GetData() {
    this.data = [];
    this.http.get("IC", "SNPP021LO_GetData")
      .subscribe(result => {
        this.data = result.Table;
      })
  }

  public RunProcess() {
    this.modal.confirm("Do you want to run process?", this.viewref,
      () => {
        this.http.post("IC", "SNPP021LO_BeginProcess", {})
          .subscribe(result => {
            this.modal.info("Process Completed!", this.viewref)
            this.poDate = moment();
            this.poRange = result;
          }, (err) => {
            this.modal.error(err.error, this.viewref);
          })
      }, () => { })
  }

  public PrintReport() {
    this.global.GoToReport("PMS_RPT_IC_SNPP021", {
      P_PO_DATE: this.global.ConvertDateToFormatSTD(this.poDate),
      P_PO_NO_FROM: this.poRange.START_PO,
      P_PO_NO_TO: this.poRange.END_PO,
    })
  }

  public GetVendor() {
    this.vendorList = [];
    this.http.get("IC", "SNPP021LO_GetVendor")
      .subscribe(result => {
        console.log(result)
        this.vendorList = result.Table;

      })
  }
}

