import { Component, OnInit, ViewContainerRef } from "@angular/core";
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import * as moment from 'moment';
import { ActivatedRoute } from '@angular/router';
import { Globals } from 'src/app/core/global';

@Component({
    selector: 'snpq002',
    templateUrl: './SNPQ002.component.html',
    styleUrls: ['./SNPQ002.component.css']
})

export class SNPQ002Component implements OnInit {
    constructor(
        private http: HttpService,
        private modal: ModalService,
        private viewref: ViewContainerRef,
        private route: ActivatedRoute,
        private global: Globals
    ) {
    }

    public Vendor: string;
    public VendorCode: string;
    public PONo: string;
    public PODate: moment.Moment;
    public DataList: Array<any> = [];

    public VendorLookupConfig: any = {
        ColName: "LOOKUP_VENDOR_NAME",
        PlaceholderText: "Select Vendor",
        SourceController: "wPMS",
        SourceMethodName: "GetVendorCode",
        NoOfShowRecords: 50,
        UpperCase: true
    }

    public POLookupConfig: any = {
        ColName: "PO_NO",
        ColValue: "PO_NO",
        PlaceholderText: "Select PO No.",
        SourceController: "wPMS",
        SourceMethodName: "GetPONo",
        NoOfShowRecords: 50
    }

    ngOnInit() { }

    GetData() {
        this.DataList = [];
        this.http.get("IC", "SNPQ002_QUERY", this.VendorCode || "null", this.PONo || "null", this.PODate && this.PODate.format("LL") || "null")
            .subscribe(result => {
                this.DataList = result.Table;
            }, err => {
                this.modal.error(err.message, this.viewref);
            })
    }

    SetVendor(data: any) {
        this.Vendor = data.LOOKUP_VENDOR_NAME;
        this.VendorCode = data.VENDOR_CODE;
    }

    Clear() {
        this.Vendor = null;
        this.VendorCode = null;
        this.PONo = null;
        this.PODate = null;
        this.DataList = undefined;
    }


}
