import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router, ActivatedRoute } from '@angular/router';
import { ModalService } from '../../../core/modal.service';
import { Globals } from 'src/app/core/global';
import * as cloneDeep from 'lodash/cloneDeep';

@Component({
    selector: 'snse148',
    templateUrl: './snse148.component.html',
    styleUrls: ['./snse148.component.css']
})



export class SNSE148Component implements OnInit {


    constructor(
        private http: HttpService,
        private router: Router,
        private modal: ModalService,
        private viewref: ViewContainerRef,
        private route: ActivatedRoute,
        private global: Globals
    ) {

    }

    private data: Array<any> = [];
    public filteredData: Array<any> = [];
    private Success: boolean = true;
    public CounterItem: number = 0;
    private origin: string = window.location.protocol + "//" + window.location.hostname + ":8090";
    public files: any = [];
    public FileName: string = ""
    public filters = {
        NAME: '',
    }
    public RecordFilter: string;

    public fileConfig: any = {
        PlaceholderText: 'Current Text Data',
        AcceptFileType: '.TSV',
        MultipleSelect: true
    }

    ngOnInit() {

    }

    Export() {
        let exportData = cloneDeep(this.data).map(v => {
            v.PART_NUMBER = '="' + v.PART_NUMBER + '"';
            v.VENDOR_CODE = '="' + (v.VENDOR_CODE || "") + '"';
            v.GROUP_PART_NO = '="' + (v.GROUP_PART_NO || "") + '"';

            return v;
        })

        this.http.Download('IC', 'SNSE148_Export', exportData)
            .subscribe(res => {
                if (res) {
                    this.writeContents(res)
                } else {
                    this.modal.error('EXPORT_NOT_COMPLETE', this.viewref)
                }

            })

    }

    Export_MPS() {
        let exportData = cloneDeep(this.data).map(v => {
            v.MODIFIED_QTY = v.MODIFIED_QTY || 0;

            return v;
        })

        this.http.Download('IC', 'SNSE148_Export_MPS', exportData)
            .subscribe(res => {
                if (res) {
                    this.writeContents(res)
                } else {
                    this.modal.error('EXPORT_NOT_COMPLETE', this.viewref)
                }

            })

    }

    writeContents(content) {
        const a = document.createElement('a');
        const file = new Blob([content], { type: content.type });
        a.href = URL.createObjectURL(file);
        a.download = "SNSE148" + this.global.CurrentDate() + ".tsv";
        a.click();
    }

    check() {
        this.data.forEach(v => { v._ERROR_FLAG = null });
        this.RecordFilter = null;

        let foundReason: any;
        foundReason = this.data.filter(x => x.MODIFICATION_REASON != 'D1' && x.MODIFICATION_REASON != 'D2' && x.MODIFICATION_REASON != null);
        if (foundReason.length > 0) {
            this.modal.warning('User can not enter another word except D1,D2,NULL', this.viewref)
            foundReason.forEach(v => { v._ERROR_FLAG = 'E' })
            this.RecordFilter = 'E';
            this.filterData();
        } else {
            let f1: any;
            f1 = this.data.filter(x => (x.MODIFICATION_REASON == 'D1' && x.MODIFIED_QTY > 0) || (x.MODIFICATION_REASON == 'D2' && x.MODIFIED_QTY > 0));
            if (f1.length > 0) {
                this.modal.warning("Please decide Modify Recommend Q'ty or Self Point Reason", this.viewref)
                f1.forEach(v => { v._ERROR_FLAG = 'E' })
                this.RecordFilter = 'E'
                this.filterData();
            } else {
                this.Export_MPS();
            }
        }
    }

    OnFileSelect(fileInput) {
        this.files = fileInput.target.files;
        this.FileName = this.files[0].name;
    }

    UpLoad() {

        if (this.files.length == 0) {
            this.modal.error('Please Open TSV File First...', this.viewref)
        } else {
            this.data = []
            let formData = new FormData();
            formData.append('file', this.files[0]);

            this.http.postFile('IC', 'SNSE148_UploadandGet', formData)
                .subscribe(res => {
                    this.data = res.Table;
                    this.CounterItem = this.data.length;
                    this.Success = false;
                    this.files = []
                    this.filteredData = this.data;
                    this.RecordFilter = null;
                })
        }
    }

    filterData() {
        this.filteredData = this.data.filter(v => !this.RecordFilter || v._ERROR_FLAG === this.RecordFilter);
    }

    findErrorRecord() {
        return this.data.find(v => v._ERROR_FLAG === 'E');
    }

}
