import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'snse158',
  templateUrl: './snse158.component.html',
  styleUrls: ['./snse158.component.css']
})

export class SNSE158Component implements OnInit {

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) {
  }

  public params: any = {};
  public tableData: any = []
  public processLog: any = []
  public DataIsSelected: any = [];
  public selection = new SelectionModel<any>(true, []);
  public disableCheckAll: boolean = true;

  public countN: any;

  @ViewChild("dialogProcess", { static: false }) dialogProcess: any;

  ngOnInit() {
    var date = new Date();
    this.params.FROM_DATE = new Date(date.getFullYear(), date.getMonth(), 2);
    this.params.TO_DATE = date;
    this.params.CREATE_PGM = "";
    this.params.UPD_STATUS = "";
    this.params.DSP_UPD_STATUS = 'Y'
    this.params.CNT_REC = 0;
    this.params.CNT_SEL = 0;
  }

  public ConfigVendorCode: any = {
    ColName: "LOOKUP_VENDOR_CODE",
    PlaceholderText: "Select Vendor Code",
    SourceController: "IC",
    SourceMethodName: "SNSE158_getVendorCode",
    NoOfShowRecords: 100,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public ConfigPartNo: any = {
    ColName: "LOOKUP_PART_NO",
    PlaceholderText: "Select Part No",
    SourceController: "IC",
    SourceMethodName: "SNSE158_getPartNo",
    NoOfShowRecords: 100,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetVendorCode(values: any) {
    this.params.DSP_VENDOR_CODE = values.LOOKUP_VENDOR_CODE
    this.params.VENDOR_CODE = values.VENDOR_CODE
    this.params.VENDOR_NAME = values.VENDOR_NAME

    this.Search();
  }

  public SetPartNo(values: any) {
    this.params.DSP_PART_NO = values.LOOKUP_PART_NO
    this.params.PART_NO = values.PART_NO
    this.params.PART_NAME = values.PART_NAME
    this.Search();
  }

  public Search() {
    this.tableData = [];
    this.selection.clear();
    this.DataIsSelected = [];
    this.disableCheckAll = this.params.UPD_STATUS !== 'N';
    if (this.params.UPD_STATUS == 'N') {
      this.params.DSP_UPD_STATUS = 'N'
    } else {
      this.params.DSP_UPD_STATUS = 'Y'
    }
    this.http.post("IC", "SNSE158_getData", this.params)
      .subscribe(res => {
        this.tableData = res.Table;
        this.processLog.push('Loading TPM_CURRENT_PRICE_PO... ');
        this.processLog.push(this.tableData.length + ' records loaded...');
        this.params.CNT_REC = this.tableData.length
        this.countN = 0;
        if (this.tableData.length > 0) {
          this.tableData.forEach((e) => {
            if (e.UPD_STATUS == 'N') {
              this.countN++
            }
          });
        }


      })
  }

  public UpdateRetailPrice() {
    if (this.DataIsSelected.length <= 0) {
      this.modal.error("There is no data select!", this.viewref)
      return;
    }

    this.processLog.push('Update price start on ' + this.FormatDate());
    this.modal.confirm("Do you want to save your change?", this.viewref,
      () => {
        this.http.post('IC', 'SNSE158_Update_Status_Y', this.DataIsSelected)
          .subscribe(res => {
            if (res.Table[0].STATUS === "DONE") {
              this.processLog.push(this.DataIsSelected.length + ' records committed...');
              this.modal.info('Save successfully : ' + this.DataIsSelected.length + ' data', this.viewref)
              this.DataIsSelected = []
              this.selection.clear();
              this.Search();
              this.processLog.push('Update price finished on ' + this.FormatDate());
            }
          }, (err) => {
            console.log(err, 'err')
          })
      }, () => { })
  }

  public CancelPartPrice() {
    if (this.DataIsSelected.length == 0) {
      this.modal.error("There is no data select!", this.viewref)
      return;
    }
    this.processLog.push('Cancel Update price start on ' + this.FormatDate());
    this.modal.confirm("Do you want to save your change?", this.viewref,
      () => {
        this.http.post('IC', 'SNSE158_Update_Status_C', this.DataIsSelected)
          .subscribe(res => {
            if (res.Table[0].STATUS === "DONE") {
              this.processLog.push(this.DataIsSelected.length + ' records committed...');
              this.modal.info('Save successfully : ' + this.DataIsSelected.length + ' data', this.viewref)
              this.DataIsSelected = []
              this.selection.clear();
              this.Search();
              this.processLog.push('Cancel Update price finished on ' + this.FormatDate());
            }
          }, (err) => {
            console.log(err, 'err')
          })
      }, () => { })
  }

  public ExportExcel() {
    this.http.Download("IC", "SNSE158_getData_Export", JSON.stringify(this.params))
      .subscribe(res => {
        if (res) {
          this.WriteContents(res)
          this.processLog.push('Export complete ' + this.tableData.length + ' records.');
        } else {
          this.modal.error('Download not complete', this.viewref)
        }
      })
  }

  public WriteContents(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    var date_export = new Date();
    a.href = URL.createObjectURL(file);
    a.download = "SNSE158" + "_" + date_export.getFullYear() + "/" + (date_export.getMonth() + 1) + "/" + date_export.getDate() + ".csv";
    a.click();
  }

  public isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.countN;
    return numSelected === numRows;
  }

  public Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
      this.tableData.map(x => {
        this.DataIsSelected = [];
      })
    } else {
      this.selection.clear();
      this.DataIsSelected = [];
      this.tableData.forEach((row) => {
        if (row.UPD_STATUS == 'N') {
          this.selection.select(row)
          this.DataIsSelected.push(row);
        }
      });
    }
    this.params.CNT_SEL = this.DataIsSelected.length
  }

  public CheckboxChange(event, data) {
    if (event.checked) {
      this.DataIsSelected.push(data);
    }
    else {
      this.DataIsSelected.splice(this.DataIsSelected.indexOf(data), 1);
    }
    this.params.CNT_SEL = this.DataIsSelected.length
  }

  public OpenProcessLog() {
    this.dialogProcess.OpenDialog();
  }



  public FormatDate() {

    let d = new Date();
    let Day = d.getDate().toString();
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    let hours = d.getHours();
    let minute = d.getMinutes();
    let second = d.getSeconds();

    return Day + '-' + NewMonth + '-' + Year + '_' + hours + '-' + minute + '-' + second;
  }
}


