import { Component, OnInit, ViewContainerRef, ViewChild } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';
import { DialogTemplateComponent } from 'src/app/components/shared/dialog-template/dialog-template.component';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
    selector: 'SNM117P',
    templateUrl: 'SNM117P.component.html',
    styleUrls: ['SNM117P.component.css', '../k9.component.css']
})

export class SNM117PComponent implements OnInit {
    constructor(
        private http: HttpService,
        private modal: ModalService,
        private viewref: ViewContainerRef,
        private global: Globals
    ) { }

    public locationCode: string;
    public locationList: Array<any>;
    public mode: string;
    public vendorName: string;
    public reasonDescription: string;
    @ViewChild("dialog") dialog: DialogTemplateComponent;


    public pdiForm: FormGroup = new FormGroup({
        PDI_LOCATION_CODE: new FormControl('', Validators.required),
        PDI_LOCATION_DESC: new FormControl('', Validators.required),
        VENDOR_CODE_MAPPING: new FormControl('', Validators.required),
        REASON_CODE_MAPPING: new FormControl('', Validators.required),
        VLM_SUB_INVENTORY_CODE: new FormControl({
            value: '',
            disabled: true
        }),
        VLM_LOCATION_CODE: new FormControl({
            value: '',
            disabled: true
        }),
        LAST_UPD_BY: new FormControl(null),
        LAST_UPD_DATE: new FormControl(null),
    })

    public LocationLookupConfig: any = {
        ColName: "LOOKUP_LOCATION",
        PlaceholderText: "Select PDI Location",
        SourceController: "wPMS",
        SourceMethodName: "GetPDILocationCode",
        NoOfShowRecords: 50,
        UpperCase: true
    }

    public VendorLookupConfig: any = {
        ColName: "LOOKUP_VENDOR_NAME",
        PlaceholderText: "Select Vendor",
        SourceController: "wPMS",
        SourceMethodName: "GetVendorCode",
        NoOfShowRecords: 50,
        UpperCase: true
    }

    public ReasonLookupConfig: any = {
        ColName: "LOOKUP_REASON",
        PlaceholderText: "Select Reason",
        SourceController: "wPMS",
        SourceMethodName: "GetReasonCodeByJobType",
        NoOfShowRecords: 50,
        MinLengthSearch: 0,
        SourceParams: {
            JOB_TYPE: 5
        }
    }

    ngOnInit() {
        this.GetLocationList();
    }

    GetLocationList() {
        this.locationList = [];
        this.http.get("K9", "SNM117P_GetPDILocation", this.locationCode)
            .subscribe(result => {
                this.locationList = result.Table;
            })
    }

    AddLocation() {
        this.mode = "Add";
        this.pdiForm.reset();
        this.pdiForm.get("PDI_LOCATION_CODE").enable();
        this.vendorName = null;
        this.reasonDescription = null;

        this.dialog.title = "Add PDI Location";
        this.dialog.OpenDialog();
    }

    EditLocation(row) {
        this.mode = "Edit";
        this.pdiForm.reset();
        this.pdiForm.get("PDI_LOCATION_CODE").disable();
        this.pdiForm.get("PDI_LOCATION_CODE").setValue(row.PDI_LOCATION_CODE);
        this.pdiForm.get("PDI_LOCATION_DESC").setValue(row.PDI_LOCATION_DESC);
        this.pdiForm.get("VENDOR_CODE_MAPPING").setValue(row.VENDOR_CODE_MAPPING);
        this.pdiForm.get("REASON_CODE_MAPPING").setValue(row.REASON_CODE_MAPPING);
        this.pdiForm.get("VLM_SUB_INVENTORY_CODE").setValue(row.VLM_SUB_INVENTORY_CODE);
        this.pdiForm.get("VLM_LOCATION_CODE").setValue(row.VLM_LOCATION_CODE);
        this.pdiForm.get("LAST_UPD_BY").setValue(row.LAST_UPD_BY);
        this.pdiForm.get("LAST_UPD_DATE").setValue(row.LAST_UPD_DATE);

        this.vendorName = row.VENDOR_NAME_EN;
        this.reasonDescription = row.REASON_NAME_EN;

        this.dialog.title = "Edit PDI Location";
        this.dialog.OpenDialog();
    }

    AddPDILocation() {
        this.modal.confirm("Are you sure to add to PDI location master?", this.viewref, () => {
            let data = this.pdiForm.getRawValue();
            this.http.post("K9", "SNM117P_SavePDILocation", data)
                .subscribe(result => {
                    this.modal.info("Add PDI Location complete.", this.viewref);
                    this.GetLocationList();
                    this.dialog.Close();
                }, (err) => {
                    this.modal.error(err.error, this.viewref);
                })
        }, () => { })
    }

    SavePDILocation() {
        this.modal.confirm("Are you sure to save this change?", this.viewref, () => {
            let data = this.pdiForm.getRawValue();
            this.http.post("K9", "SNM117P_SavePDILocation".concat("/", data.PDI_LOCATION_CODE), data)
                .subscribe(result => {
                    this.modal.info("Save PDI Location complete.", this.viewref);
                    this.GetLocationList();
                    this.dialog.Close();
                }, (err) => {
                    this.modal.error(err.error, this.viewref);
                })
        }, () => { })
    }

    DeletePDILocation(row) {
        this.modal.confirm("Are you sure to delete this record?", this.viewref, () => {
            this.http.post("K9", "SNM117P_DeletePDILocation".concat("/", row.PDI_LOCATION_CODE), {})
                .subscribe(result => {
                    this.modal.info("Delete from PDI Location complete.", this.viewref);
                    this.GetLocationList();
                    this.dialog.Close();
                }, (err) => {
                    this.modal.error(err.error, this.viewref);
                })
        }, () => { })
    }

    SetLocationCode(value) {
        this.locationCode = value.PDI_LOCATION_CODE;
    }

    SetVendorCode(value) {
        this.pdiForm.get("VENDOR_CODE_MAPPING").setValue(value.VENDOR_CODE);
        this.vendorName = value.VENDOR_NAME_EN;
    }

    SetReasonCode(value) {
        this.pdiForm.get("REASON_CODE_MAPPING").setValue(value.REASON_CODE);
        this.reasonDescription = value.REASON_NAME_EN;
    }
}