import { Component, OnInit, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { FormGroup, FormControl } from "@angular/forms";
import { Router } from "@angular/router";
import { AuthService } from "src/app/core/auth-service.service";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "login",
  templateUrl: "./login.component.html",
  styleUrls: ["./login.component.css"],
})
export class LoginComponent implements OnInit {
  public errMsg: string = "";
  public loginForm: FormGroup = new FormGroup({
    username: new FormControl(""),
    password: new FormControl(""),
    company: new FormControl(""),
  });
  public showpwd: boolean = false;

  constructor(
    private http: HttpService,
    private router: Router,
    private authService: AuthService,
    private modal: ModalService,
    private viewRef: ViewContainerRef
  ) {}

  ngOnInit(): void {
    this.loginForm.get("company").setValue("01");
  }

  async login() {
    this.errMsg = "";
    localStorage.clear();

    const batchResult = await this.CheckBatchProcess();

    if (!batchResult) {
      this.modal.confirm(
        "Batch process is running!<br/>Some function might not be usable.",
        this.viewRef,
        () => {
          this.CallProcessLogin();
        },
        () => {}
      );
    } else {
      this.CallProcessLogin();
    }
  }

  private CallProcessLogin() {
    this.http
      .postLogin(
        this.loginForm.get("username").value,
        this.loginForm.get("password").value,
        this.loginForm.get("company").value
      )
      .subscribe(
        (result) => {
          this.authService.storeTokens(result);
          this.router.navigate(["/main"]);
        },
        (err) => {
          if (err.error && !(err.error instanceof Object)) {
            this.errMsg = err.error;
          } else {
            this.errMsg = err.message;
          }
        }
      );
  }

  private async CheckBatchProcess(): Promise<boolean> {
    let data = {
      username: this.loginForm.get("username").value,
      password: this.loginForm.get("password").value,
      company: this.loginForm.get("company").value,
    };

    let batchResult: boolean = true;

    await this.http
      .post("System", "PreLogin", data)
      .toPromise()
      .catch((err) => {
        if (err.status === 503) {
          batchResult = false;
        }
      })
      .then();

    return batchResult;
  }

  ChangePassword() {
    this.router.navigate(["/", "changepwd"]);
  }
}
