import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';

@Component({
  selector: 'SKCUMTPRG',
  templateUrl: './SKCUMTPRG.component.html',
  styleUrls: ['./SKCUMTPRG.component.css']
})

export class SKCUMTPRGComponent implements OnInit {

  public params: any = {};
  public tableData: any = [];

  public Data: any = {};
  public DataRef: any = {};
  public showpwd: boolean = false;
  public dialog: any = null;

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public LookupPRGIDConfig: any = {
    ColName: "LOOKUP_PRG_ID",
    PlaceholderText: "Select Menu ID",
    SourceController: "System",
    SourceMethodName: "SKCUMTPRG_GetPRGID",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: false
  }

  public SetSelectedPRGID(value: any) {
    this.params.SK_PRG_ID = value.SK_PRG_ID;
    this.params.ENAME = value.ENAME;
    this.params.TNAME = value.TNAME;
    this.Search();
  }

  ngOnInit() {
    this.Search()
  }

  public Search() {
    this.http.post('System', 'SKCUMTPRG_GetData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;
      })
  }

  public EditClick(row) {
    this.Data = JSON.parse(JSON.stringify(row));
    this.DataRef = JSON.parse(JSON.stringify(row));
    this.dialogEdit.OpenDialog();
    this.dialog = 'edit'
  }

  public CloseDialogEdit() {
    this.dialogEdit.Close();
  }

  public CreateClick() {
    this.Data = {};
    this.DataRef = {};

    this.Data.PRG_TYPE = 'FRM';
    this.Data.ICON = 'B_FORM';

    this.dialogAdd.OpenDialog();
    this.dialog = 'add'
  }

  public CloseDialogAdd() {
    this.dialogAdd.Close();
  }

  public CheckDupPRG() {
    this.http.post('System', 'SKCUMTPRG_CheckUser', this.Data)
      .subscribe(res => {
        console.log(res)
        if (res.Table.length > 0) {
          this.modal.warning("This user is duplicated!", this.viewref);
          this.Data.SK_PRG_ID = null;
        }
      })
  }

  public ConfirmSaveEdit() {
    let notchange = (JSON.stringify(this.Data) == JSON.stringify(this.DataRef))
    if (notchange) {
      this.modal.warning('There is no data change.', this.viewref)
    }
    else {
      if (!this.Data.SK_PRG_ID || !this.Data.TNAME || !this.Data.ENAME) {
        this.modal.warning("Please fill all required(*) fields.", this.viewref);
        return
      }
      else
      {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
        //this.modal.warning('Do you want to save changed you have made?', this.viewref)
      }
    }
  }

  Save = () => {
    var nameEN = this.Data.ENAME.split(" ", 2);
    var nameTH = this.Data.TNAME.split(" ", 2);
    this.Data.FIRST_NAME_EN = nameEN[0];
    this.Data.LAST_NAME_EN = nameEN[1];
    this.Data.FIRST_NAME_TH = nameTH[0];
    this.Data.LAST_NAME_TH = nameTH[1];
    this.http.post('System', 'SKCUMTPRG_Save', this.Data)
      .subscribe(res => {
        if (res.STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          if (this.dialog == 'edit') {
            this.dialogEdit.Close();
          } else {
            this.dialogAdd.Close();
          }
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }


}
