
import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';

@Component({
    selector: 'SNM011P',
    templateUrl: 'SNM011P.component.html',
    styleUrls: ['SNM011P.component.css']
})

export class SNM011PComponent implements OnInit {
    constructor(
        private http: HttpService,
        private router: Router,
        private global: Globals,
        private modal: ModalService,
        private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

    public PartCategory: string;
    public PartCategoryCode: string;
    public PartCategoryList: Array<any> = [];
    public FluctuationList: Array<any> = [];
    public ParamEdit: any = {};
    public ParamAdd: any = {};
    public TempBefore: any;
    public TempAfter: any;

    public PartCategoryLookup: any = {
        ColName: "LOOKUP_PART_CAT_NAME",
        PlaceholderText: "Select Part Category",
        MinLengthSearch: 0,
  }

  public lookupConfigPart: any = {
    ColName: "LOOKUP_PART_CAT_NAME",
    PlaceholderText: "Select Part Category",
    SourceController: "IC",
    SourceMethodName: "SNM011P_GetPartCategory",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
  }

    ngOnInit() {
        this.GetPartCategory();
    }

    GetPartCategory() {
        this.http.get("wPMS", "GetPartCategory")
            .subscribe(result => {
                this.PartCategoryList = result.Table;
            })
    }

    SetPartCategoryLookup(val) {
        this.PartCategory = val.LOOKUP_PART_CAT_NAME;
        this.PartCategoryCode = val.PART_CATEGORY;
    }

    SearchFluctuation() {
        this.FluctuationList = []
        this.http.get("IC", "SNM011P_GetDetail", this.PartCategoryCode)
            .subscribe(result => {
                this.FluctuationList = result.Table;
            })
  }

  public SetSelectedPartCategoryAdd(value: any, data, action) {
    if (action == "select") {
      if (value) {
        this.ParamAdd.PART_CATEGORY = value.PART_CATEGORY;
        this.ParamAdd.CATEGORY_DESC = value.CATEGORY_DESC;
        this.ParamAdd.SEQ_NO = value.SEQ_NO + 1;
        this.ParamAdd.STD_DEVIATION_F_BEFORE = value.STD_DEVIATION_T;
        this.ParamAdd.STD_DEVIATION_F = value.STD_DEVIATION_T + 0.01;
        //this.ParamAdd.STD_DEVIATION_T = value.STD_DEVIATION_T;
        this.ParamAdd.FLUCTUATE_CODE = null;
      } else {
        this.modal.warning("Part Category not found in data base", this.viewref)
        this.ParamAdd.PART_CATEGORY = null;
        this.ParamAdd.CATEGORY_DESC = null;
        this.ParamAdd.SEQ_NO = null;
        this.ParamAdd.STD_DEVIATION_F_BEFORE = null;
        this.ParamAdd.STD_DEVIATION_F = null;
        //this.ParamAdd.STD_DEVIATION_T = null;
        //this.ParamAdd.FLUCTUATE_CODE = null;
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamAdd.PART_CATEGORY = null;
        return;
      }

      this.http.get("IC", "SNM011P_GetPartCategory", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamAdd.PART_CATEGORY = res.Table[0].PART_CATEGORY;
            this.ParamAdd.CATEGORY_DESC = res.Table[0].CATEGORY_DESC;
            this.ParamAdd.SEQ_NO = res.Table[0].SEQ_NO + 1;
            this.ParamAdd.STD_DEVIATION_F_BEFORE = res.Table[0].STD_DEVIATION_T;
            this.ParamAdd.STD_DEVIATION_F = res.Table[0].STD_DEVIATION_T + 0.01;
            //this.ParamAdd.STD_DEVIATION_T = res.Table[0].STD_DEVIATION_T;
            this.ParamAdd.FLUCTUATE_CODE = null;
          } else {
            this.ParamAdd.PART_CATEGORY = null;
            this.ParamAdd.CATEGORY_DESC = null;
            this.ParamAdd.SEQ_NO = null;
            this.ParamAdd.STD_DEVIATION_F_BEFORE = null;
            this.ParamAdd.STD_DEVIATION_F = null;
            //this.ParamAdd.STD_DEVIATION_T = null;
            this.ParamAdd.FLUCTUATE_CODE = null;
          }
        })
    }
  }

  public Add() {
    this.ParamAdd = {}
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    if (this.ParamAdd.PART_CATEGORY && this.ParamAdd.STD_DEVIATION_F > this.ParamAdd.STD_DEVIATION_F_BEFORE && this.ParamAdd.STD_DEVIATION_T >= this.ParamAdd.STD_DEVIATION_F && this.ParamAdd.FLUCTUATE_CODE) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
    } else if (this.ParamAdd.STD_DEVIATION_F_BEFORE >= this.ParamAdd.STD_DEVIATION_F) {
      this.modal.warning("Please Input Std Fm more than the previous Seq's Std Fm.", this.viewref);
    } else if (this.ParamAdd.STD_DEVIATION_T < this.ParamAdd.STD_DEVIATION_F) {
      this.modal.warning("Please Input Std To more than Std Fm.", this.viewref);
    } else {
      this.modal.warning("Please Input fill Round No.", this.viewref);
    }
  }

  public ConfirmSave = () => {
    this.http.post('IC', 'SNM011P_INSERT', this.ParamAdd)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.SearchFluctuation();
          this.ParamAdd = {};
          this.dialogAdd.Close();
        } else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        } else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  public EditRow(data) {
    this.ParamEdit = { ...data };
    this.TempBefore = this.FluctuationList.filter((item) => item['PART_CATEGORY'] === this.ParamEdit.PART_CATEGORY && this.ParamEdit.SEQ_NO > 1 && item['SEQ_NO'] === (this.ParamEdit.SEQ_NO - 1))[0];
    this.TempAfter = this.FluctuationList.filter((item) => item['PART_CATEGORY'] === this.ParamEdit.PART_CATEGORY && item['SEQ_NO'] === (this.ParamEdit.SEQ_NO + 1))[0];
    this.dialogEdit.OpenDialog();
  }

  public SaveEdit() {
    if (this.ParamEdit.STD_DEVIATION_T > this.ParamEdit.STD_DEVIATION_F && this.ParamEdit.FLUCTUATE_CODE &&
      ((this.ParamEdit.SEQ_NO === 1 && !this.TempAfter) ||
      (this.ParamEdit.SEQ_NO === 1 && this.TempAfter && this.ParamEdit.STD_DEVIATION_T < this.TempAfter.STD_DEVIATION_F) ||
      (this.ParamEdit.SEQ_NO > 1 && this.ParamEdit.STD_DEVIATION_F > this.TempBefore.STD_DEVIATION_T && !this.TempAfter) ||
      (this.ParamEdit.SEQ_NO > 1 && this.ParamEdit.STD_DEVIATION_F > this.TempBefore.STD_DEVIATION_T && this.TempAfter && this.ParamEdit.STD_DEVIATION_T < this.TempAfter.STD_DEVIATION_F))) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmEdit);
    } else if (this.ParamEdit.SEQ_NO === 1 && this.TempAfter && this.ParamEdit.STD_DEVIATION_T > this.TempAfter.STD_DEVIATION_F) {
      this.modal.warning("Please Input Std To less than the next Seq's Std Fm.", this.viewref);
    } else if (this.ParamEdit.SEQ_NO > 1 && this.TempAfter && this.ParamEdit.STD_DEVIATION_T >= this.TempAfter.STD_DEVIATION_F) {
      this.modal.warning("Please Input Std to less than the next Seq's Std To.", this.viewref);
    }  else if (this.ParamEdit.SEQ_NO > 1 && this.TempBefore && this.ParamEdit.STD_DEVIATION_F <= this.TempBefore.STD_DEVIATION_T) {
      this.modal.warning("Please Input Std Fm more than the previous Seq's Std To.", this.viewref);
    }else if (this.ParamEdit.STD_DEVIATION_F >= this.ParamEdit.STD_DEVIATION_T) {
        this.modal.warning("Please Input Std To more than Std Fm.", this.viewref);
    } else {
      this.modal.warning("Please Input fill Round No.", this.viewref);
    }    
  }

  public ConfirmEdit = () => {
    this.http.post('IC', 'SNM011P_UPDATE', this.ParamEdit)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.SearchFluctuation();
          this.dialogEdit.Close();
        } else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        } else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public CancleEdit() {
    this.dialogEdit.Close();
  }
}
