import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snm020p',
  templateUrl: './snm020p.component.html',
  styleUrls: ['./snm020p.component.css']
})


export class SNM020PComponent implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public paramSearch: any = {};
  public params: any = {};
  public paramEdit: any = {};
  public paramAdd: any = {};
  public categoryList: any = [];
  public tabledata: any = [];
  public isAdd: boolean = false;
  public process_log: any = [];
  

  public ConfigCategory: any = {
    ColName: "DESCRIPT_CODE",
    ColValue: "",
    PlaceholderText: "Select Description Code",
    SourceController: "TI",
    SourceMethodName: "SNM020P_GetDESCRIPT",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public ConfigSalesClass: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Sales Class",
    SourceController: "TI",
    SourceMethodName: "SNM020P_Salesclass", 
    MinLengthSearch: 0,
    NoOfShowRecords: 50
  }

  public ConfigProduct: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Product Code",
    SourceController: "TI",
    SourceMethodName: "SNM020P_Product",
    MinLengthSearch: 0,
    NoOfShowRecords: 50
  }

  public SetSelectedDescriptCode(value: any) {
    this.paramSearch.DESCRIPT_CODE = value.DESCRIPT_CODE;
    this.paramSearch.DESCRIPT_NAME = value.DESCRIPT_NAME;
  }

  public hideCols: Array<string> = [];
  DescriptCodeControl = new FormControl('', [Validators.required])

  public ngOnInit() {
    this.global.GetRoleDetail().CHG === "N" ? this.hideCols.push("EDIT") : null;
    this.global.GetRoleDetail().DEL === "N" ? this.hideCols.push("DELETE") : null;
  }

  public Search() {
    this.tabledata = [];
    this.http.post("TI", "SNM020P_GetDescriptionCode", this.paramSearch)
      .subscribe(res => {
        this.tabledata = res.Table;
      })
  }

  public Add() {
    this.DescriptCodeControl.reset();
    //this.paramAdd = {};
    //this.isAdd = true
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
  }


  public ConfirmSave = () => {
    this.http.post('TI', 'SNM020P_Insert', this.paramAdd)
      .subscribe(res => {
 
          if (res.Table[0].STATUS == 'Done') {
            this.modal.info('Transaction complete: applied and saved.', this.viewref);
            this.Search();
            this.paramAdd = this.global.ClearObject(this.paramAdd);
            this.dialogAdd.Close();

          } else if (res.Table[0].STATUS == 'Duplicate') {
            this.modal.warning('No change to save. Found duplicate record.', this.viewref);
          }
          else {
            this.modal.error('No change to save. Something went wrong.', this.viewref);
          }
        
      },err =>{
        this.modal.error('No change to save. Something went wrong.', this.viewref);
      })
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  public EditRow(data) {


    this.http.post("TI","SNM020P_GetDescriptionCode",data)
    .subscribe(res =>{
      if(res.Table.length > 0){
        this.DescriptCodeControl.reset();
        this.isAdd = false;
        this.paramEdit = res.Table[0];
        this.dialogEdit.OpenDialog();
      }else{
        this.modal.warning("Description Code not found",this.viewref)
      }
    })
  }

  public SaveEdit() {
    this.modal.confirm("Do you want to save changed you have made?", this.viewref, this.ConfirmEdit)
  }

  public ConfirmEdit = () => {

    this.http.post('TI', 'SNM020P_Update', this.paramEdit)
      .subscribe(res => {
        //console.log(res)
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialogEdit.Close();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      },err => {
        this.modal.error('No change to save. Something went wrong.', this.viewref);
      })
  }

  public CancleEdit() {
    this.dialogEdit.Close();
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete record?", this.viewref, () => this.ConfirmDelete(data))
  }

  public ConfirmDelete = data => {

    this.http.post('TI', 'SNM020P_Delete', data)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          //console.log(res)
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      },err =>{
        this.modal.error('No change to save. Something went wrong.', this.viewref);
      })

  }

  public ExportExcel() {
    this.http.Download("TI","SNM020P_Download", this.tabledata)
      .subscribe(res => {
        if (res) {
          this.WriteContents(res)
          this.process_log.push('Export complete ' + this.tabledata.length + ' records.');
        } else {
          this.modal.error('Download not complete', this.viewref)
        }
      },err =>{
        this.modal.error('Download not complete', this.viewref)
      })
  }

  public WriteContents(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = "SNM020P" + '_' + this.global.CurrentDate() + '' + ".xlsx";
    a.click();
  }

  public SetSalseClassAdd(event,data,action){
  
    if(action == "select"){
      if(event){
        this.paramAdd.SALES_CLASS = event.SALES_CLASS
      }else{
        //this.modal.warning("Sales Class not found in data base",this.viewref)
        this.paramAdd.SALES_CLASS = null
      }
    }
    else if(action == "cancel"){
     
        if (!data){
          this.paramAdd.SALES_CLASS = null;
          return;
        }

      this.http.get("TI","SNM020P_Salesclass",this.global.EscapeCharacter(data),'20')
      .subscribe(res =>{
        if(res.Table.length > 0){
          this.paramAdd.SALES_CLASS = res.Table[0].SALES_CLASS
        }else{
          this.paramAdd.SALES_CLASS = null;
        }
      })

    }
  }

  public SetProductCodeAdd(event,data,action){

    if(action == "select"){
      if(event){
        this.paramAdd.PRODUCT_CODE = event.PRODUCT_CODE
      }else{
        //this.modal.warning("Product Code not found in data base",this.viewref)
        this.paramAdd.PRODUCT_CODE = null
      }
    }
    else if(action == "cancel"){
     
        if (!data){
          //this.modal.warning("Main Model is not null",this.viewRef)
          this.paramAdd.PRODUCT_CODE = null;
          return;
        }

      this.http.get("TI","SNM020P_Product",this.global.EscapeCharacter(data),'20')
      .subscribe(res =>{
        if(res.Table.length > 0){
          this.paramAdd.PRODUCT_CODE = res.Table[0].PRODUCT_CODE
        }else{
          // this.modal.warning("Product Code not found in data base",this.viewRef)
          this.paramAdd.PRODUCT_CODE = null;
        }
      })

    }
    }

    public SetSalseClassEdit(event,data,action){
  
      if(action == "select"){
        if(event){
          this.paramEdit.SALES_CLASS = event.SALES_CLASS
        }else{
          //this.modal.warning("Sales Class not found in data base",this.viewref)
          this.paramEdit.SALES_CLASS = null
        }
      }
      else if(action == "cancel"){
       
          if (!data){
            this.paramEdit.SALES_CLASS = null;
            return;
          }
  
        this.http.get("TI","SNM020P_Salesclass",this.global.EscapeCharacter(data),'20')
        .subscribe(res =>{
          if(res.Table.length > 0){
            this.paramEdit.SALES_CLASS = res.Table[0].SALES_CLASS
          }else{
            this.paramEdit.SALES_CLASS = null;
          }
        })
  
      }
    }


    public SetProductCodeEdit(event,data,action){

      if(action == "select"){
        if(event){
          this.paramEdit.PRODUCT_CODE = event.PRODUCT_CODE
        }else{
          //this.modal.warning("Product Code not found in data base",this.viewref)
          this.paramEdit.PRODUCT_CODE = null
        }
      }
      else if(action == "cancel"){
       
          if (!data){
            //this.modal.warning("Main Model is not null",this.viewRef)
            this.paramEdit.PRODUCT_CODE = null;
            return;
          }
  
        this.http.get("TI","SNM020P_Product",this.global.EscapeCharacter(data),'20')
        .subscribe(res =>{
          if(res.Table.length > 0){
            this.paramEdit.PRODUCT_CODE = res.Table[0].PRODUCT_CODE
          }else{
            // this.modal.warning("Product Code not found in data base",this.viewRef)
            this.paramEdit.PRODUCT_CODE = null;
          }
        })
  
      }
    }



}

