import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';


@Component({
  selector: 'snm023p',
  templateUrl: './snm023p.component.html',
  styleUrls: ['./snm023p.component.css']
})



export class SNM023PComponent implements OnInit {

  public params: any = {};

  public Data: any = {};
  public OldData: any = {};

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialogReport", { static: false }) dialogReport: any;

  public MainDatatable: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;

  public lookupPartCodeConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part Code",
    SourceController: "wPMS",
    SourceMethodName: "GetPartCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 2,
    UpperCase: true
  }

  public lookupDescriptionCodeConfig: any = {
    ColName: "LOOKUP_DESCRIPT_CODE",
    PlaceholderText: "Select Description Code",
    SourceController: "TI",
    SourceMethodName: "GetDescriptCode",
    NoOfShowRecords: 100,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupTariffCodeConfig: any = {
    ColName: "TARIFF_CODE",
    PlaceholderText: "Select Tariff Code",
    SourceController: "TI",
    SourceMethodName: "GetTariffCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0
  }

  public lookupExpensiveCodeConfig: any = {
    ColName: "EXPENSIVE_CODE",
    PlaceholderText: "Select Expensive Code",
    SourceController: "TI",
    SourceMethodName: "GetExpensiveCode",
    NoOfShowRecords: 10,
    MinLengthSearch: 0
  }




  ngOnInit() {
  }

  public Search() {
    this.MainDatatable = [];
    this.http.post('TI', 'SNM023P_GetData', this.params)
      .subscribe(res => {
        this.MainDatatable = res.Table;
      })
  }


  InitData() {
    this.Data = {};
  }

  CreateClick() {
    this.InitData();
    this.FrmTitle = "New Part Code";
    this.IsEdit = false;
    this.dialogMain.title = 'Add Part Code'
    this.dialogMain.OpenDialog();
  }

  EditClick(row) {
   

    this.http.post("TI","SNM023P_GetData",row)
    .subscribe(res =>{
      if(res.Table.length > 0){
        this.InitData();
        this.IsEdit = true;
        this.FrmTitle = "Edit Part Code";
        this.Data = res.Table[0];
        this.OldData = JSON.parse(JSON.stringify(row));
        this.dialogMain.title = 'Edit Part Code'
        this.dialogMain.OpenDialog();
      }else{
        this.modal.warning("Part Code not found",this.viewref)
      }
    })
  }

  ConfirmDelete(row) {
    this.Data = row;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  Delete = () => {
    this.http.post('TI', 'SNM023P_PartCode_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
          this.params.PART_CODE = null;
          this.params.PART_NAME = null;
        }
      });
  }

  ConfirmSave() {
    if (this.IsEdit) {
      if (this.OldData.PART_NAME_TH != this.Data.PART_NAME_TH
        || this.OldData.DIMENSION_MAX != this.Data.DIMENSION_MAX
        || this.OldData.DESCRIPTION_CODE != this.Data.DESCRIPTION_CODE
        || this.OldData.DIMENSION_MID != this.Data.DIMENSION_MID
        || this.OldData.TARIFF_CODE != this.Data.TARIFF_CODE
        || this.OldData.DIMENSION_MIN != this.Data.DIMENSION_MIN
        || this.OldData.PACKAGING_CODE != this.Data.PACKAGING_CODE
        || this.OldData.EXPENSIVE_CODE != this.Data.EXPENSIVE_CODE) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.PART_CODE) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  Save = () => {

    if(this.IsEdit){
      this.http.post('TI', 'SNM023P_PartCode_Update', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.Search();
        }
        else {
          this.modal.error('Found an error, please contact support team', this.viewref);
        }
      },err =>{
        this.modal.error('Found an error, please contact support team', this.viewref);
      })
    }else{
      this.http.post('TI', 'SNM023P_PartCode_Insert', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.Search();
        }else if(res.Table[0].STATUS == 'DUPLICATE'){
          this.modal.info('No change to save found duplicate record', this.viewref);
        }
        else {
          this.modal.error('Found an error, please contact support team', this.viewref);
        }
      },err =>{
        this.modal.error('Found an error, please contact support team', this.viewref);
      })
    }

 
  }

  Cancel() {
    this.InitData();
    this.dialogMain.Close();
  }

  public SetSelectedPartCode(value: any) {
    this.params.PART_CODE = value.PART_CODE;
    this.params.PART_NAME = value.PART_NAME_TH
    this.params.PART_CODE_S = value.PART_CODE;
    this.params.PART_CODE_E = value.PART_CODE;
    //this.Search()
  }

  public SetSelectedPartCodeStart(value: any) {
    this.params.PART_CODE_S = value.PART_CODE;
  }

  public SetSelectedPartCodeEnd(value: any) {
    this.params.PART_CODE_E = value.PART_CODE;
  }

  public SetSelectedDescriptionCode(value: any) {
    if(value){
      this.Data.DESCRIPTION_CODE = value.DESCRIPT_CODE;
    }else{
      this.Data.DESCRIPTION_CODE = null;
    }
  }

  public CancelDescriptionCode(){
    if(this.Data.DESCRIPTION_CODE){

      this.http.get("TI","GetDescriptCode",this.Data.DESCRIPTION_CODE,'20')
      .subscribe(res => {
        if(res.Table.length > 0){
          this.Data.DESCRIPTION_CODE = res.Table[0].DESCRIPT_CODE
                  
        }else{
          this.Data.DESCRIPTION_CODE = null;
         
        }       
      })
    }else{
      this.Data.DESCRIPTION_CODE = null;
      
    }
  }

  public SetSelectedTariffCode(value: any) {
    if(value){
      this.Data.TARIFF_CODE = value.TARIFF_CODE;
    }else{
      this.Data.TARIFF_CODE = null;
    }
    
  }

  public CancelTariffCode(){
    if(this.Data.TARIFF_CODE){

      this.http.get("TI","GetTariffCode",this.Data.TARIFF_CODE,'20')
      .subscribe(res => {
        if(res.Table.length > 0){
          this.Data.TARIFF_CODE = res.Table[0].TARIFF_CODE
                  
        }else{
          this.Data.TARIFF_CODE = null;
         
        }       
      })
    }else{
      this.Data.TARIFF_CODE = null;
      
    }
  }

  public SetSelectedExpensiveCode(value: any) {
    this.Data.EXPENSIVE_CODE = value.EXPENSIVE_CODE;
  }

  ExportExcel() {
    this.http.Download('TI', 'SNM023P_Export', JSON.stringify(this.MainDatatable))
      .subscribe(res => {
        if (res) {
          this.WriteContents(res)
        } else {
          this.modal.error('Download not complete', this.viewref)
        }
      })
  }

  WriteContents(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = "SNM023P.xlsx";
    a.click();
  }

  public OpenReport() {
    this.dialogReport.OpenDialog();
  }

  public CloseReport() {
    this.dialogReport.Close();
  }

  public GoToReport() {
    if (this.params.PART_CODE_S && this.params.PART_CODE_E) {
      this.global.GoToReport("PMS_RPT_SALES_SNM023R", {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_PART_CODE_S: this.params.PART_CODE_S,
        P_PART_CODE_E: this.params.PART_CODE_E,

      })
    }
    else {
      this.modal.warning("Please fill Part Code", this.viewref);
    }
  }


  public GoToReporAll() {
    this.global.GoToReport("PMS_RPT_TI_SNM023P_All",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode")
      }
    );
  }

  public ChangeUpperCase() {
    this.Data.PART_CODE = this.Data.PART_CODE.toUpperCase()
  }
}

