import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';


@Component({
  selector: 'snm049p',
  templateUrl: './snm049p.component.html',
  styleUrls: ['./snm049p.component.css']
})



export class SNM049PComponent implements OnInit {

  public params: any = {};

  public Data: any = {
    COMPANY_CODE: null,
    COUNTRY_CODE: null,
    COUNTRY_NAME_EN: null,
    COUNTRY_NAME_TH: null,
    CREATE_BY: null,
    CREATE_DATE: null,
    CREATE_BY_NAME: null,
    LAST_UPD_PGM: null,
    LAST_IP_ADDR: null,
    LAST_UPD_BY: null,
    LAST_UPD_DATE: null,
    UPDATE_BY_NAME: null,
  };

  public modalEdit: boolean = false;

  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private auth: AuthDecode,
    private global: Globals
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public tableData: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;

  public lookupCountryCodeConfig: any = {
    ColName: "LOOKUP_COUNTRY_CODE",
    PlaceholderText: "Select Country Code",
    SourceController: "SL",
    SourceMethodName: "SNM049P_GetCountryCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  SetSelectedCountryCode(value: any) {
    this.params.COUNTRY_CODE = value.COUNTRY_CODE;
    this.params.COUNTRY_NAME = value.COUNTRY_NAME;
    this.Search()
  };

  ngOnInit() {
    this.Search()
  }

  Search = () => {
    this.http.post('SL', 'SNM049P_GetData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;
      })
  }

  InitData() {
    this.Data = {};
  }

  CreateClick() {
    this.FrmTitle = "New Country Code";
    this.InitData();
    this.IsEdit = false;
    this.modalEdit = false;
    this.dialogMain.OpenDialog();
  }

  EditClick(row) {
    this.FrmTitle = "Edit Country Code";
    this.InitData();
    this.Data = JSON.parse(JSON.stringify(row));
    this.Data.OLD_COUNTRY_NAME_EN = this.Data.COUNTRY_NAME_EN
    this.Data.OLD_COUNTRY_NAME_TH = this.Data.COUNTRY_NAME_TH
    this.IsEdit = true;
    this.modalEdit = true;
    this.dialogEdit.OpenDialog();
  }

  ConfirmDelete(row) {
    this.Data = row;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  Delete = () => {
    this.http.post('SL', 'SNM049P_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
          this.params.COUNTRY_CODE = null;
          this.params.COUNTRY_NAME = null;
        }
      });
  }

  CheckDuplicate = () => { 
  
  }

  ConfirmSave() {
    if (this.IsEdit == true) {
      if (this.Data.OLD_COUNTRY_NAME_EN != this.Data.COUNTRY_NAME_EN || this.Data.OLD_COUNTRY_NAME_TH != this.Data.COUNTRY_NAME_TH) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.COUNTRY_CODE && this.Data.COUNTRY_NAME_EN && this.Data.COUNTRY_NAME_TH) {
        this.http.post('SL', 'SNM049P_CheckDuplicate', this.Data)
          .subscribe(res => {
            if (res.Table[0].STATUS == 'DONE') {
              this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
            }
            else if (res.Table[0].STATUS == 'FAIL') {
              this.modal.warning("Data is duplicated.", this.viewref);
            }
            else if (res.Table[0].STATUS == 'ERROR') {
              this.modal.error('Found an error, please contract support team', this.viewref);
            }
          });
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  Save = () => {
    this.http.post('SL', 'SNM049P_Update', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          if (this.modalEdit == true) {
            this.CancelEdit();
          } else {
            this.CancelMain();
          }
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  CancelMain() {
    this.InitData();
    this.dialogMain.Close();

  }

  CancelEdit() {
    this.InitData();
    this.dialogEdit.Close();
  }
}

