import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';


@Component({
  selector: 'snm081p',
  templateUrl: './snm081p.component.html',
  styleUrls: ['./snm081p.component.css']
})



export class SNM081PComponent implements OnInit {

  public params: any = {};
  public modalEdit: boolean = false;
  public Data: any = {};

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public tableData: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;

  public lookupOriginCodeConfig: any = {
    ColName: "LOOKUP_ORIGIN_CODE",
    PlaceholderText: "Select Origin Code",
    SourceController: "IC",
    SourceMethodName: "SNM081P_GetOriginCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  SetSelectedOriginCode(value: any) {
    this.params.ORIGIN_CODE = value.ORIGIN_CODE;
    this.params.ORIGIN_NAME = value.ORIGIN_NAME;
    this.Search();
  };

  ngOnInit() {
    this.Search();
  }

  Search = () => {
    this.http.post('IC', 'SNM081P_GetData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;
      })
  }

  InitData() {
    this.Data = {};
  }

  CreateClick() {
    this.FrmTitle = "New Origin Code";
    this.InitData();
    this.IsEdit = false;
    this.modalEdit = false;
    this.dialogAdd.OpenDialog();
  }

  EditClick(row) {
    this.FrmTitle = "Edit Origin Code";
    this.InitData();
    this.Data = JSON.parse(JSON.stringify(row));
    this.Data.OLD_ORIGIN_NAME = this.Data.ORIGIN_NAME
    this.Data.OLD_ORIGIN_NAME2 = this.Data.ORIGIN_NAME2
    this.IsEdit = true;
    this.modalEdit = true;
    this.dialogEdit.OpenDialog();
  }

  ConfirmDelete(row) {
    this.Data = row;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  Delete = () => {
    this.http.post('IC', 'SNM081P_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
          this.params.ORIGIN_CODE = null;
          this.params.ORIGIN_NAME = null;
        }
      });
  }

  ConfirmSave() {
    if (this.IsEdit == true) {
      if (this.Data.OLD_ORIGIN_NAME != this.Data.ORIGIN_NAME || this.Data.OLD_ORIGIN_NAME2 != this.Data.ORIGIN_NAME2) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('There is no data change.', this.viewref)
      }
    }
    else {
      if (this.Data.ORIGIN_NAME && this.Data.ORIGIN_NAME2 && this.Data.ORIGIN_CODE) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
    }
  }

  Save = () => {
    this.http.post('IC', 'SNM081P_Update', this.Data)
      .subscribe(res => {
        if (res == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          if (this.modalEdit == true) {
            this.CancelEdit();
          } else {
            this.CancelMain();
          }
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  CancelMain() {
    this.InitData();
    this.dialogAdd.Close();
  }

  CancelEdit() {
    this.InitData();
    this.dialogEdit.Close();
  }
}

