import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'SNM126P',
  templateUrl: './SNM126P.component.html',
  styleUrls: ['./SNM126P.component.css']
})

export class SNM126PComponent implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef

  ) { }

  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public paramSearch: any = {};
  public paramAdd: any = {};
  public paramEdit: any = {};
  public tabledata: any = [];
  public isAdd: boolean = false;

  public hideCols: Array<string> = [];
  CustExportPriceGroupControl = new FormControl('', [Validators.required])

  ngOnInit() {
    this.global.GetRoleDetail().CHG === "N" ? this.hideCols.push("EDIT") : null;
    this.global.GetRoleDetail().DEL === "N" ? this.hideCols.push("DELETE") : null;
    this.Search();
  }

  public ConfigCurrencyCode: any = {
    ColName: "LOOKUP_CURRENCY",
    ColValue: "",
    PlaceholderText: "Select Currency Code",
    SourceController: "SL",
    SourceMethodName: "SNM126P_GetCurrencyCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public ConfigMainCustomer: any = {
    ColName: "LOOKUP_CUSTOMER",
    ColValue: "",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM126P_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public Search() {
    this.tabledata = [];
    this.http.post("SL", "SNM126P_GetData", this.paramSearch)
      .subscribe(res => {
        this.tabledata = res.Table;
      }, (err) => {
          console.log(err, 'err');
          this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public Add() {
    this.paramAdd = {};
    //this.CustExportPriceGroupControl.reset();
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
  }

  public ConfirmSave = () => {
    this.http.post('SL', 'SNM126P_Insert', this.paramAdd)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.paramAdd = this.global.ClearObject(this.paramAdd);
          this.dialogAdd.Close();

        } else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }

      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }


  public EditRow(data) {
    this.CustExportPriceGroupControl.reset();
    this.isAdd = false;
    this.paramEdit = data;
    this.dialogEdit.OpenDialog();
  }

  public SaveEdit() {
    this.modal.confirm("Do you want to save the changed?", this.viewref, this.ConfirmEdit)
  }

  public ConfirmEdit = () => {

    this.http.post('SL', 'SNM126P_Update', this.paramEdit)
      .subscribe(res => {
        console.log(res)
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialogEdit.Close();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public CancelEdit() {
    this.dialogEdit.Close();
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete this record?", this.viewref, () => this.ConfirmDelete(data));
  }

  public ConfirmDelete = data => {

    this.http.post('SL', 'SNM126P_Delete', data)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('The record has been deleted.', this.viewref);
          this.Search();
        }
        else {
          console.log(res)
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
          console.log(err, 'err');
          this.modal.warning("Found an error, please contact support team", this.viewref);
      }); 

  }

}

