import { Component, OnInit, ViewChild, ViewContainerRef, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'snm133p',
  templateUrl: './snm133p.component.html',
  styleUrls: ['./snm133p.component.css']
})

export class SNM133PComponent implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef

  ) { }

  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  @ViewChild("dialogExceptAdd", { static: false }) dialogExceptAdd: any;
  @ViewChild("dialogExceptEdit", { static: false }) dialogExceptEdit: any;

  @ViewChild("fileUpload") fileUpload: ElementRef;
  @ViewChild("fileUploadExcept") fileUploadExcept: ElementRef;


  public dataIsSelected: any = [];
  public selection = new SelectionModel<any>(true, []);



  public params: any = {};
  public paramAdd: any = {};
  public tableData: any = [];
  public tableDataAllClass: any = [];
  public tableDataExcept: any = [];
  public tableDataAllClassExcept: any = [];
  public tableTempData: any = [];
  public tableTempDataExcept: any = [];

  public IsPages: any = 0;
  public FileName: any;
  public file: any;
  public type: any;
  public orderClassList: Array<any>;

  public FileNameExcept: any;
  public fileExcept: any;

  CustExportPriceGroupControl = new FormControl('', [Validators.required])

  ngOnInit() {
    this.Search();
    this.GetOrderClass();
  }

  private GetOrderClass() {
    this.http.post("wPMS", ["GetOrderClass", "null", "10"], { JOB_TYPE: 1 })
      .subscribe(result => {
        this.orderClassList = result.Table;
      })
  }

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupBranchCodeConfig: any = {
    ColName: "LOOKUP_BRANCH_CODE",
    PlaceholderText: "Select Branch Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetBranchCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: ''
    },
    UpperCase: true
  }

  public lookupPartNoConfig: any = {
    ColName: "LOOKUP_PART_NO",
    PlaceholderText: "Select Part Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.params.BRANCH_CODE = value.BRANCH_CODE;
    this.lookupBranchCodeConfig.SourceParams.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.Search();

  }

  public SetSelectedBranchCode(value: any) {
    this.params.BRANCH_CODE = value.BRANCH_CODE;
    this.Search();
  }

  public SetSelectedPartNo(value: any) {
    this.params.PART_NO = value.PART_NO;
    this.Search();
  }

  public lookupCustomerCodeAddConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetCustomerCodeAdd",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupBranchCodeAddConfig: any = {
    ColName: "LOOKUP_BRANCH_CODE",
    PlaceholderText: "Select Branch Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetBranchCodeAdd",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: ''
    },
    UpperCase: true
  }

  public lookupPartNoAddConfig: any = {
    ColName: "LOOKUP_PART_NO",
    PlaceholderText: "Select Part Code",
    SourceController: "SL",
    SourceMethodName: "SNM133P_GetPartNoAdd",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCodeAdd(value: any) {
    this.paramAdd.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.paramAdd.CUSTOMER_NAME = value.CUSTOMER_NAME;
    this.lookupBranchCodeAddConfig.SourceParams.CUSTOMER_CODE = value.CUSTOMER_CODE;
  }

  public SetSelectedBranchCodeAdd(value: any) {
    this.paramAdd.BRANCH_CODE = value.BRANCH_CODE;
    this.paramAdd.BRANCH_NAME = value.BRANCH_NAME;
  }

  public SetSelectedPartNoAdd(value: any) {
    this.paramAdd.PART_NO = value.PART_NO;
    this.paramAdd.PART_NAME = value.PART_NAME;
  }

  public Search() {
    this.tableData = [];
    this.tableDataAllClass = [];
    this.http.post("SL", "SNM133P_GetData", this.params)
      .subscribe(res => {
        this.tableData = res.Table;
        this.tableDataAllClass = res.Table1;
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public Add() {
    this.dataIsSelected = [];
    this.selection.clear();
    this.paramAdd = {};
    this.type = 0;
    this.dialogAdd.OpenDialog();
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  public CancelEdit() {
    this.dialogEdit.Close();
  }

  public SaveAdd() {
    if (this.paramAdd.EFFECTIVE_DATE && this.paramAdd.DISABLE_DATE && this.paramAdd.CUSTOMER_CODE && this.paramAdd.BRANCH_CODE && this.paramAdd.PART_NO && this.dataIsSelected.length > 0) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
    }
    else {
      this.modal.warning('Please complete all request field!!', this.viewref)
    }
  }

  public ConfirmSave = () => {

    let arrSave = []
    this.dataIsSelected.forEach((row) => {
      arrSave.push({
        CUSTOMER_CODE: this.paramAdd.CUSTOMER_CODE,
        BRANCH_CODE: this.paramAdd.BRANCH_CODE,
        ORDER_CLASS: row,
        PART_NO: this.paramAdd.PART_NO,
        EFFECTIVE_DATE: this.paramAdd.EFFECTIVE_DATE,
        DISABLE_DATE: this.paramAdd.DISABLE_DATE,
      });
    });



    this.http.post('SL', 'SNM133P_Save', arrSave)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.paramAdd = {};
          if (this.type == 0) {
            this.dialogAdd.Close();
          } else {
            this.dialogEdit.Close();
          }
        } else if (res.Table[0].STATUS == 'DUPLICATE') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public EditRow(data) {
    this.dataIsSelected = [];
    this.selection.clear();
    this.paramAdd = data;
    let filterData = this.tableDataAllClass.filter(v => v.CUSTOMER_CODE === data.CUSTOMER_CODE &&
                                                         v.BRANCH_CODE === data.BRANCH_CODE &&
                                                         v.PART_NO === data.PART_NO &&
                                                         v.EFFECTIVE_DATE === data.EFFECTIVE_DATE &&
                                                         v.DISABLE_DATE === data.DISABLE_DATE)

    filterData.forEach((row) => {
      this.dataIsSelected.push(row.ORDER_CLASS);
      this.selection.select(row.ORDER_CLASS)
    });

    this.type = 1;
    this.dialogEdit.OpenDialog();
  }

  public SaveUpload() {
    let foundErr = this.tableTempData.some(x => x.MESSAGE)
    if (!foundErr) {
      this.modal.confirm("Do you want to save changed you have made?", this.viewref, this.ConfirmUpload)
    } else {
      this.modal.warning("Please check error message!!", this.viewref)
    }
  }

  public ConfirmUpload = () => {
    this.http.post('SL', 'SNM133P_SaveUpload', this.params)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.tableTempData = [];
          this.IsPages = 0;
          this.Search();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete record?", this.viewref, () => this.ConfirmDelete(data));
  }

  public ConfirmDelete = data => {
    this.http.post('SL', 'SNM133P_Delete', data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          console.log(res)
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  SelectFile(evt) {
    this.file = null;
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
      //let File: File = Files[0];
      this.file = Files[0];
      this.FileName = this.file.name;
    }
  }

  public OpenFileBrowser() {
    this.FileName = "";
    this.fileUpload.nativeElement.value = '';
    this.fileUpload.nativeElement.click();
  }

  DownloadTemplate() {
    let link = "assets/templates/PMS_Quota_Allocation.xlsx"
    window.open(link, '_blank')
  }

  Preview() {
    if (!this.FileName) {
      this.modal.error('Please Open xlsx File First...', this.viewref)
    } else {
      let formData = new FormData();
      formData.append('file', this.file);
      this.http.postFile("SL", "SNM133P_UploadExcel", formData)
        .subscribe(result => {
          this.http.get("SL", "SNM133P_GetTempData")
            .subscribe(result => {
              this.tableTempData = result.Table;
            })
        }, err => {
          console.log(err);
          this.modal.error("Found eror, please contact support team.", this.viewref);
        })
    }
  }

  ToUploadPage() {
    this.tableTempData = [];
    this.FileName = null;
    this.IsPages = 1;
  }

  ToExceptPage() {
    this.IsPages = 2;
    this.SearchExcept();
  }

  ToViewPage() {
    this.IsPages = 0;
  }

  OrderClassChange(orderClass) {
    let filterClass = this.orderClassList.filter(v => v.ORDER_PART_NO === orderClass)
  }

  //Except Quota ------------------------------------------------------------------------------------
  public AddExpect() {
    this.dataIsSelected = [];
    this.selection.clear();
    this.paramAdd = {};
    this.type = 0;
    this.dialogExceptAdd.OpenDialog();
  }

  public CancelAddExcept() {
    this.dialogExceptAdd.Close();
  }

  public EditExceptRow(data) {
    this.dataIsSelected = [];
    this.selection.clear();

    this.paramAdd = data;

    let filterData = this.tableDataAllClassExcept.filter(v => v.CUSTOMER_CODE === data.CUSTOMER_CODE &&
      v.BRANCH_CODE === data.BRANCH_CODE &&
      v.PART_NO === data.PART_NO &&
      v.EFFECTIVE_DATE === data.EFFECTIVE_DATE &&
      v.DISABLE_DATE === data.DISABLE_DATE)

    filterData.forEach((row) => {
      this.dataIsSelected.push(row.ORDER_CLASS);
      this.selection.select(row.ORDER_CLASS)
    });


    this.type = 1;
    this.dialogExceptEdit.OpenDialog();
  }

  public CancelEditExcept() {
    this.dialogExceptEdit.Close();
  }

  public SearchExcept() {
    this.tableDataExcept = [];
    this.http.get("SL", "SNM133P_GetDataExcept")
      .subscribe(res => {
        this.tableDataExcept = res.Table;
        this.tableDataAllClassExcept = res.Table1;
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public DeleteExcept(data) {
    this.modal.confirm("Do you want to delete except record?", this.viewref, () => this.ConfirmDeleteExcept(data));
  }

  public ConfirmDeleteExcept = data => {
    this.http.post('SL', 'SNM133P_DeleteExcept', data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.SearchExcept();
        }
        else {
          console.log(res)
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public SaveAddExcept() {
    if (this.paramAdd.EFFECTIVE_DATE && this.paramAdd.DISABLE_DATE && this.paramAdd.CUSTOMER_CODE && this.paramAdd.BRANCH_CODE && this.paramAdd.PART_NO && this.dataIsSelected.length > 0) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSaveExcept);
    }
    else {
      this.modal.warning('Please complete all request field!!', this.viewref)
    }
  }

  public ConfirmSaveExcept = () => {

    let arrSave = []
    this.dataIsSelected.forEach((row) => {
      arrSave.push({
        CUSTOMER_CODE: this.paramAdd.CUSTOMER_CODE,
        BRANCH_CODE: this.paramAdd.BRANCH_CODE,
        ORDER_CLASS: row,
        PART_NO: this.paramAdd.PART_NO,
        EFFECTIVE_DATE: this.paramAdd.EFFECTIVE_DATE,
        DISABLE_DATE: this.paramAdd.DISABLE_DATE,
      });
    });
    this.http.post('SL', 'SNM133P_SaveExcept', arrSave)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.SearchExcept();
          this.paramAdd = {};
          if (this.type == 0) {
            this.dialogExceptAdd.Close();
          } else {
            this.dialogExceptEdit.Close();
          }
        } else if (res.Table[0].STATUS == 'DUPLICATE') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public ToExceptUploadPage() {
    this.tableTempDataExcept = [];
    this.FileNameExcept = null;
    this.IsPages = 3;
  }


  public SelectFileExcept(evt) {
    this.fileExcept = null;
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
      //let File: File = Files[0];
      this.fileExcept = Files[0];
      this.FileNameExcept = this.fileExcept.name;
    }
  }

  public OpenFileBrowserExcept() {
    this.FileNameExcept = "";
    this.fileUploadExcept.nativeElement.value = '';
    this.fileUploadExcept.nativeElement.click();
  }

  DownloadTemplateExcept() {
    let link = "assets/templates/PMS_Quota_Allocation.xlsx"
    window.open(link, '_blank')
  }

  PreviewExcept() {
    if (!this.FileNameExcept) {
      this.modal.error('Please Open xlsx File First...', this.viewref)
    } else {
      let formData = new FormData();
      formData.append('file', this.fileExcept);
      this.http.postFile("SL", "SNM133P_UploadExcelExcept", formData)
        .subscribe(result => {
          this.http.get("SL", "SNM133P_GetTempDataExcept")
            .subscribe(result => {
              this.tableTempDataExcept = result.Table;
            })
        }, err => {
          console.log(err);
          this.modal.error("Found eror, please contact support team.", this.viewref);
        })
    }
  }

  public SaveUploadExcept() {
    let foundErr = this.tableTempDataExcept.some(x => x.MESSAGE)
    if (!foundErr) {
      this.modal.confirm("Do you want to save changed you have made?", this.viewref, this.ConfirmUploadExcept)
    } else {
      this.modal.warning("Please check error message!!", this.viewref)
    }
  }

  public ConfirmUploadExcept = () => {
    this.http.post('SL', 'SNM133P_SaveUploadExcept', this.params)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.tableTempDataExcept = [];
          this.IsPages = 2;
          this.SearchExcept();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err');
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }



  public CheckboxChange(event, orderClass) {
    if (event.checked) {
      this.dataIsSelected.push(orderClass);
    }
    else {
      this.dataIsSelected.splice(this.dataIsSelected.indexOf(orderClass), 1);
    }
  }
}
