import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';
import * as moment from 'moment';

@Component({
  selector: 'SNM137P',
  templateUrl: './SNM137P.component.html',
  styleUrls: ['./SNM137P.component.css']
})


export class SNM137PComponent implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public ParamSearch: any = {};
  public ParamEdit: any = {};
  public ParamAdd: any = {};
  public tabledata: any = [];
  public temptable: any = [];
  public currentDate: moment.Moment = moment();
  public formData: any = {};
  public partNumber: string;

  public lookupConfig: any = {
    ColName: "LOOKUP_PART_NO_VALUE",
    PlaceholderText: "Select Part",
    SourceController: "wPMS",
    SourceMethodName: "SNM137P_GET_PART_NO",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupConfigPart: any = {
    ColName: "LOOKUP_PART_NO_VALUE",
    PlaceholderText: "Select Part",
    SourceController: "wPMS",
    SourceMethodName: "SNM137P_GET_PART_NO_NON_DUP",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupConfigPackage: any = {
    ColName: "LOOKUP_PACKAGE_NAME_VALUE",
    PlaceholderText: "Select Package Name",
    SourceController: "wPMS",
    SourceMethodName: "SNM137P_GET_PACKAGE",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: false
  }

  public ngOnInit() {
    this.partNumber = '';
    this.Search();
  }

 

  public SetSelectedPART(value: any) {
    if (value) {
      this.partNumber = value.PART_NO;
      this.ParamSearch.PART_NO = value.PART_NO;
    } else {
      this.ParamSearch.PART_NO = '';
    }
  }

  public SetSelectedPARTAdd(value: any, data, action) {

    if (action == "select") {
      if (value) {
        this.ParamAdd.PART_NO = value.PART_NO
        this.ParamAdd.PART_NAME_EN = value.PART_NAME_EN
      } else {
        this.modal.warning("part no not found in data base", this.viewref)
        this.ParamAdd.PART_NO = null
        this.ParamAdd.PART_NAME_EN = null
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamAdd.PART_NO = null;
        this.ParamAdd.PART_NAME_EN = null
        return;
      }

      this.http.get("wPMS", "SNM137P_GET_PART_NO_NON_DUP", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamAdd.PART_NO = res.Table[0].PART_NO
            this.ParamAdd.PART_NAME_EN = res.Table[0].PART_NAME_EN
          } else {
            //this.modal.warning("part no not found in data base", this.viewref)
            this.ParamAdd.PART_NO = null;
            this.ParamAdd.PART_NAME_EN =  null;
          }
        })

    }
  }

  public SetSelectedPARTEdit(value: any) {
    if (value) {
      this.ParamEdit.PART_NO = value.PART_NO;
    } else {
      this.ParamEdit.PART_NO = '';
    }
  }

  public SetSelectedPACKAGEUNITAdd(value: any,data,action) {

    if (action == "select") {
      if (value) {
        this.ParamAdd.PACKAGE_UNIT = value.PACKAGE_UNIT
      } else {
        this.modal.warning("package not found in data base", this.viewref)
        this.ParamAdd.PACKAGE_UNIT = null
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamAdd.PACKAGE_UNIT = null;
        return;
      }

      this.http.get("wPMS", "SNM137P_GET_PACKAGE", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamAdd.PACKAGE_UNIT = res.Table[0].PACKAGE_UNIT
          } else {
            //this.modal.warning("package not found in data base", this.viewref)
            this.ParamAdd.PACKAGE_UNIT = null;
          }
        })

    }
  }

  public SetSelectedPACKAGEUNITEdit(value: any,data,action) {

    if (action == "select") {
      if (value) {
        this.ParamEdit.PACKAGE_UNIT = value.PACKAGE_UNIT
      } else {
        this.modal.warning("package not found in data base", this.viewref)
        this.ParamEdit.PACKAGE_UNIT = null
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamEdit.PACKAGE_UNIT = null;
        return;
      }

      this.http.get("wPMS", "SNM137P_GET_PACKAGE", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamEdit.PACKAGE_UNIT = res.Table[0].PACKAGE_UNIT
          } else {
            //this.modal.warning("package not found in data base", this.viewref)
            this.ParamEdit.PACKAGE_UNIT = null;
          }
        })

    }
    // if (value) {
    //   this.ParamEdit.PACKAGE_UNIT = value.PACKAGE_UNIT;
    // } else {
    //   this.ParamEdit.PACKAGE_UNIT = '';
    // }
  }

  public SetSelectedPURCHASEPACKAGEAdd(value: any,data,action) {

    if (action == "select") {
      if (value) {
        this.ParamAdd.DMS_PUR_PACKAGE_NAME = value.DMS_PUR_PACKAGE_NAME
      } else {
        this.modal.warning("package not found in data base", this.viewref)
        this.ParamAdd.DMS_PUR_PACKAGE_NAME = null
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamAdd.DMS_PUR_PACKAGE_NAME = null;
        return;
      }

      this.http.get("wPMS", "SNM137P_GET_PACKAGE", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamAdd.DMS_PUR_PACKAGE_NAME = res.Table[0].DMS_PUR_PACKAGE_NAME
          } else {
            //this.modal.warning("package not found in data base", this.viewref)
            this.ParamAdd.DMS_PUR_PACKAGE_NAME = null;
          }
        })

    }

    // if (value) {
    //   this.ParamAdd.DMS_PUR_PACKAGE_NAME = value.DMS_PUR_PACKAGE_NAME;
    // } else {
    //   this.ParamAdd.DMS_PUR_PACKAGE_NAME = '';
    // }
  }

  public SetSelectedPURCHASEPACKAGEEdit(value: any,data,action) {
    if (action == "select") {
      if (value) {
        this.ParamEdit.DMS_PUR_PACKAGE_NAME = value.DMS_PUR_PACKAGE_NAME
      } else {
        this.modal.warning("package not found in data base", this.viewref)
        this.ParamEdit.DMS_PUR_PACKAGE_NAME = null
      }
    }
    else if (action == "cancel") {

      if (!data) {
        this.ParamEdit.DMS_PUR_PACKAGE_NAME = null;
        return;
      }

      this.http.get("wPMS", "SNM137P_GET_PACKAGE", this.global.EscapeCharacter(data), '50')
        .subscribe(res => {

          if (res.Table.length > 0) {
            this.ParamEdit.DMS_PUR_PACKAGE_NAME = res.Table[0].DMS_PUR_PACKAGE_NAME
          } else {
            //this.modal.warning("package not found in data base", this.viewref)
            this.ParamEdit.DMS_PUR_PACKAGE_NAME = null;
          }
        })

    }

    // if (value) {
    //   this.ParamEdit.DMS_PUR_PACKAGE_NAME = value.DMS_PUR_PACKAGE_NAME;
    // } else {
    //   this.ParamEdit.DMS_PUR_PACKAGE_NAME = '';
    // }
  }

  public Search() {
    this.tabledata = [];
    this.http.post("wPMS", "SNM137P_GET_DATA", this.ParamSearch)
      .subscribe(res => {
        this.tabledata = res.Table;
        this.temptable = res.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public onkeypress(event, name) {
    if (event.key == "Enter") {
      const value = event.target.value;
      var check = this.tabledata.filter(key => {
        if (key[name] !== null) {
          const tempKey = key[name].toString().toLowerCase();
          const tempSearch = value.toLowerCase();
          return tempKey.includes(tempSearch);
        }
      });
      if (check.length > 0)
        this.tabledata = check;
      else
        this.tabledata = [];
    } else if (!event.target.value) {
      this.tabledata = this.temptable;
    }
  }

  public Add() {
    this.ParamAdd = {}
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    if (this.ParamAdd.PART_NO && this.ParamAdd.PACKAGE_UNIT && this.ParamAdd.DMS_RCV_SALE_UNIT && this.ParamAdd.DMS_PUR_PACKAGE_NAME) {
      if (this.ParamAdd.DMS_RCV_SALE_UNIT < 0) {
        this.modal.warning("Sale Package Unit more than 0.", this.viewref);
      } else {
        this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
      }
    } else {
      this.modal.warning("Please Input fill Round No.", this.viewref);
    }
  }

  public ConfirmSave = () => {

    this.http.post('wPMS', 'SNM137P_INSERT', this.ParamAdd)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.ParamAdd = {};
          this.dialogAdd.Close();
        }
        else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  public EditRow(data) {
    this.ParamEdit = { ...data };
    this.dialogEdit.OpenDialog();
  }

  public SaveEdit() {
    if (this.ParamEdit.PART_NO && this.ParamEdit.PACKAGE_UNIT && this.ParamEdit.DMS_RCV_SALE_UNIT && this.ParamEdit.DMS_PUR_PACKAGE_NAME) {
      if (this.ParamEdit.DMS_RCV_SALE_UNIT < 0) {
        this.modal.warning("Sale Package Unit more than 0.", this.viewref);
      } else {
        this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmEdit);
      }
    } else {
      this.modal.warning("Please Input fill Round No.", this.viewref);
    }
  }

  public ConfirmEdit = () => {
    this.http.post('wPMS', 'SNM137P_UPDATE', this.ParamEdit)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialogEdit.Close();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete record?", this.viewref, () => this.ConfirmDelete(data))
  }
  public ConfirmDelete(data) {
    this.http.post('wPMS', 'SNM137P_Delete', data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          console.log(res)
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      })
  }
  public CancleEdit() {
    this.dialogEdit.Close();
  }
}

