import { SelectionModel } from "@angular/cdk/collections";
import { CdkDragDrop, moveItemInArray } from "@angular/cdk/drag-drop";
import { Component, ElementRef, HostListener, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { MatTable, MatTableDataSource } from "@angular/material/table";
import { Router } from "@angular/router";
//import { AutoTableeorderComponent } from "src/app/components/shared/auto-table-reoder/auto-table-reorder.component";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { SignalrService } from "src/app/core/signalr.service";
import * as moment from "moment";

@Component({
  selector: 'SNWR025',
  templateUrl: 'SNWR025.component.html',
  styleUrls: ['SNWR025.component.css']
})
export class SNWR025Component implements OnInit {

  public mode: number = 2
  public message: string;
  public tableDataLocal: any = [];
  public tableDataImport: any = [];
  public border: any = ["2px solid red", "2px solid blue"];
  public headerList: any = [];
  public displayedColumns: string[] = [];
  public maxHight = '40vh';
  @ViewChild('tableLocal') tableLocal: MatTable<any>;
  @ViewChild('tableImport') tableImport: MatTable<any>;
  //@ViewChild("reOrderPickedCust", { static: false }) reOrderPickedCust: AutoTableeorderComponent;
  private MainInterval: any;
  private Interval1: any;
  private Interval2: any;

  constructor(
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private http: HttpService,
    private el: ElementRef,
    private global: Globals,
    private signalR: SignalrService,
    private router: Router
  ) {
    this.signalR.initiateSignalrConnection();
    this.signalR.proxy.on('receivelocal', (data: any) => this.GetLocalData(data));
    this.signalR.proxy.on('receiveimport', (data: any) => this.GetImportData(data));
    this.signalR.connection.start().done((dataconnect: any) => {
      //this.getTableHeader();
      //this.AutoGetData();
    }).fail((error: any) => {
      console.log('Could not connect ' + error);
    });

    router.events.subscribe((val) => { 
      //console.log("XXXXXXXXXXXX"); 
    });
  }

  @HostListener('window:beforeunload', [ '$event' ])
  beforeUnloadHandler(event) {
    this.DeleteTask();
  }

  ngOnInit(): void {
    //this.StartService();
    this.getTableHeader();

    setTimeout(() => {
      this.AutoGetData();
      this.SetTask();
    }, 1000);
    
   this.MainInterval = setInterval(() => {
      //this.AutoGetData();
      this.SetIntervar();
    },3000)
  }
  
  ngOnDestroy(): void {
    this.signalR.connection.stop();
  }

  getTableHeader() {
    
    this.http.get("System", "GetHeaderGridViewMonitoring", "SNWR025")
      .subscribe(result => {
        this.headerList = result.filter(v => v.VISIBILITY === true);
        this.ArrangeShowColumn(this.headerList);
      })
  }

  private SetTask(){
    this.http.get("WH","SNWR025_SetTask")
    .subscribe(res =>{
    })
  }

  private DeleteTask(){
    this.http.get("WH","SNWR025_DeleteTask")
    .subscribe(res =>{

    })
  }

  ArrangeShowColumn(data) {
    this.displayedColumns = this.headerList.map(v => v.FIELD_NAME);
  }

  private AutodropTable(data: any, table: any) {
    if(data.length > 17){
      moveItemInArray(data, 0, data.length);
      // table.renderRows();
    }
   
  }

  private AutodropTable2(data: any, table: any) {
    if(data.length > 17){
      moveItemInArray(data, 0, data.length);
      // this.tableImport.renderRows();
    }
  }

  dropTable(event: CdkDragDrop<any[]>) {
    // const prevIndex = this.tableNopickCust.findIndex((d) => d === event.item.data);
    // moveItemInArray(this.tableNopickCust, prevIndex, event.currentIndex);
    //this.table.renderRows();
  }

  private AutoGetData() {
      this.signalR.proxy.invoke("GetReceiveLocal")
      this.signalR.proxy.invoke("GetReceiveImport")
  }

  private GetLocalData(data: any) {
    this.tableDataLocal = [];
    this.tableDataLocal = data.Table
                            .sort((a,b) => a.VENDOR_CODE - b.VENDOR_CODE)
                            .sort((a,b) => {
                              if (moment(a.CREATE_DATE).isBefore(b.CREATE_DATE)) return -1;
                              if (moment(b.CREATE_DATE).isBefore(a.CREATE_DATE)) return 1;
                              return 0;
                            });
  }

  private GetImportData(data: any) {
    this.tableDataImport = []
    this.tableDataImport = data.Table;
  }

  private SetIntervar(){
      this.AutodropTable(this.tableDataLocal, this.tableLocal)
      this.AutodropTable2(this.tableDataImport, this.tableImport)
  }

  public CheckBacklogTask(data: any) {
    const now = moment();
    const createDate = moment(data.CREATE_DATE);
    const duration = moment.duration(now.diff(createDate));
    return duration.asDays() > 7;
  }

}
