import { Component, OnInit, ViewContainerRef } from '@angular/core';
import { SelectionModel } from '@angular/cdk/collections';
import { Globals } from '../../../core/global';
import { ModalService } from '../../../core/modal.service';
import { MatDatepicker } from '@angular/material/datepicker';
import * as moment from 'moment';

@Component({
  selector: 'snpr020',
  templateUrl: './snpr020.component.html',
  styleUrls: ['./snpr020.component.css']
})



export class SNPR020Component implements OnInit {


  constructor(
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }
  selection = new SelectionModel<any>(true, []);


  public VendorConfig: any = {
    ColName: "VENDOR_TEXT",
    ColValue: "VENDOR_CODE",
    PlaceholderText: "Select Vendor Code",
    SourceController: "IC",
    SourceMethodName: "SNPR019_FindFVendor",
    NoOfShowRecords: 50,
    MinLengthSearch: 2
  }
  public InvoiceConfig: any = {
    ColName: "INVOICE_NO",
    ColValue: "INVOICE_NO",
    PlaceholderText: "Select Invoice No.",
    SourceController: "IC",
    SourceMethodName: "SNPR019_FindFInvoice",
    NoOfShowRecords: 50,
    MinLengthSearch: 3
  }

  public ConfigPart: any = {
    ColName: "LOOKUP_PART_NAME",
    ColValue: "PART_NO",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 5
  }


  public params: any = {};

  ngOnInit() {
    //let d = new Date()
    this.params.INVOICE_DATE_FROM = moment();
    this.params.INVOICE_DATE_TO = moment();
  }

  SetFVendor(FVendorCode) {
    this.params.VENDOR_CODE_FROM = FVendorCode;
  }
  SetLVendor(TVendorCode) {
    this.params.VENDOR_CODE_TO = TVendorCode;
  }
  SetFInvoice(FInvoiceNo) {
    this.params.INVOICE_NO_FROM = FInvoiceNo;
  }
  SetLInvoice(TInvoiceNo) {
    this.params.INVOICE_NO_TO = TInvoiceNo;
  }

  ValidateDate() {
    if (this.params.INVOICE_DATE_FROM > this.params.INVOICE_DATE_TO) {
      this.modal.warning("Defult First value is not larger than Last value", this.viewref)
    }
  }

  public OpenReport() {

    if (this.params.INVOICE_DATE_FROM > this.params.INVOICE_DATE_TO) {
      this.modal.warning("Defult First value is not larger than Last value", this.viewref)
    } else {
      console.log(' Date from 1 ', this.params.INVOICE_DATE_FROM);
      console.log(' Date To 1 ', this.params.INVOICE_DATE_TO);

      console.log(' Date from ', this.global.ConvertDateToFormatSTD(this.params.INVOICE_DATE_FROM));
      console.log(' Date To ', this.global.ConvertDateToFormatSTD(this.params.INVOICE_DATE_TO));

      this.global.GoToReport("PMS_RPT_WH_SNPR020", {
        P_VENDOR_CODE: this.params.VENDOR_CODE_FROM,
        P_VENDOR_CODE1: this.params.VENDOR_CODE_TO,
        P_INVOICE_NO: this.params.INVOICE_NO_FROM,
        P_INVOICE_NO1: this.params.INVOICE_NO_TO,
        P_INVOICE_DATE: this.SetformatDate(this.params.INVOICE_DATE_FROM),
        P_INVOICE_DATE1: this.SetformatDate(this.params.INVOICE_DATE_TO),
        P_PART_NO: this.params.PART_NO_FROM,
        P_PART_NO1: this.params.PART_NO_TO
      })
    }



  }

  OpendateFromPopup(datepicker: MatDatepicker<any>) {

    datepicker.open();
  }

  OpendateToPopup(datepicker: MatDatepicker<any>) {

    datepicker.open();
  }


  CheckExportDataCondition() {
    if (this.params.VENDOR_CODE_From == null) {
      this.params.VENDOR_CODE_From = this.params.VENDOR_CODE_To;
    }
    if (this.params.INVOICE_NO_FROM == null) {
      this.params.INVOICE_NO_FROM = this.params.INVOICE_NO_TO;
    }
  }


  SetformatDate(date) {
    let d = new Date(date)
    let day = d.getDate();
    let month = d.getMonth() + 1;
    let newmonth = month.toString().length < 2 ? 0 + month.toString() : month.toString();
    let year = d.getFullYear();

    return day + '-' + newmonth + '-' + year;
  }






}

