import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';

@Component({
  selector: 'SNM032E',
  templateUrl: './SNM032E.component.html',
  styleUrls: ['../saleorder.component.css']
})

export class SNM032EComponent implements OnInit {
  public params = {
    EFFECTIVE_DATE: null,
    RATE_TYPE: null,
    CURRENCY_CODE: null 
  }
  public LegitType: string[] = ["4","6","7"];
  public Data: any = {};

  public ConfigRate: any = {
    ColName: "MASTER_TYPE_DES",
    ColValue: "MASTER_TYPE_DES",
    PlaceholderText: "Select Rate",
    SourceController: "SL",
    SourceMethodName: "SNM032E_Get_Rate_Type",
    NoOfShowRecords: 30,
    MinLengthSearch:0,
    UpperCase: false
  }

  public ConfigRateAdd: any = {
    ColName: "MASTER_TYPE_DES",
    ColValue: "MASTER_TYPE_DES",
    PlaceholderText: "Select Rate",
    SourceController: "SL",
    SourceMethodName: "SNM032E_Get_Rate_Type_Add",
    NoOfShowRecords: 30,
    MinLengthSearch: 0,
    UpperCase: false
  }

  public ConfigCurr: any = {
    ColName: "LOOKUP_CURRENCY",
    PlaceholderText: "Select Currency",
    SourceController: "SL",
    SourceMethodName: "SNM032E_Get_Currency",
    MinLengthSearch: 0,
    UpperCase: false
  }

  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private auth: AuthDecode,
    private global: Globals
  ) { }
  
  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialogType", { static: false }) dialogType: any;
  @ViewChild("dialogCurr", { static: false }) dialogCurr: any;

  public MainDatatable: any = [];
  public tableData: any = [];
  public tableDataType: any = [];
  public tableDataCurr: any = [];
  public FrmTitle: any;
  public IsEdit: any = true;
  public hideCols: Array<string> = [];

  //private data: Array<any> = [];
  public files: any = [];
  public FileName: string = ""
  public ListLog : any = [];
  @ViewChild("dialogLog", { static: false }) dialogLog: any;
  
  ngOnInit() {
    this.InitData();
    this.SearchForAc();
  }

  public SearchForAc = () => {
    //if (this.params.EFFECTIVE_DATE != null) {
    //  let newDate = new Date(this.params.EFFECTIVE_DATE);
    //  this.params.EFFECTIVE_DATE = this.GetDateParam(newDate);
    //}
    this.http.post('SL', 'SNM032E_GetExchange_Rate', this.params)
      .subscribe(res => {
        this.MainDatatable = res.Table;
      })
  }

  public GetDateParam(dt) {
    let year = dt.getFullYear();
    let day = dt.getDate().toString().padStart(2, '0'); 
    var monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
    let month = monthNames[dt.getMonth()];
    var p_Date = day + '-' + month + '-' + year;

    return p_Date;
  }

  public InitData() {
    this.global.GetRoleDetail().CHG === "N" ? this.hideCols.push("EDIT") : null;
    this.global.GetRoleDetail().DEL === "N" ? this.hideCols.push("DELETE") : null;
    this.Data = {};
  }

  public GetRate(code) {
    this.Data.RATE_TYPE_DESC = code;
  }

  public GetRateAdd(code) {
    this.Data.RATE_TYPE_DESC = code;
  }

  public GetCurren(code: any) {
    this.Data.CURRENCY_CODE = code.CURRENCY_CODE;
  }

  public GetRateS(code) {
    this.params.RATE_TYPE = code;
  }

  public GetCurrenS(code: any) {
    this.params.CURRENCY_CODE = code.CURRENCY_CODE;
  }

  public editBtnClick(data) {
    this.Data = data;
    this.dialog.Close();

  }

  public editTypeClick(data) {
    this.Data.RATE_TYPE = data.MASTER_TYPE;
    this.dialogType.Close();
  }

  public editCurrClick(data) {
    this.Data.CURRENCY_CODE = data.CURRENCY_CODE;
    this.dialogCurr.Close();
  }

  public SaveAdd() {
    if (this.Data.EFFECTIVE_DATE === null || this.Data.EFFECTIVE_DATE === "" ||  this.Data.EFFECTIVE_DATE === undefined) {
      this.modal.warning('Please select Effective Date', this.viewref)
    } else if (this.Data.RATE_TYPE_DESC === null || this.Data.RATE_TYPE_DESC === "" || this.Data.RATE_TYPE_DESC === undefined) {
      this.modal.warning('Please select Rate Type', this.viewref)
    } else if (this.Data.CURRENCY_CODE === null || this.Data.CURRENCY_CODE === ""  || this.Data.CURRENCY_CODE === undefined) {
      this.modal.warning('Please select Currency', this.viewref)
    } else if (this.Data.EXCHANGE_UNIT === null || this.Data.EXCHANGE_UNIT === "" || this.Data.EXCHANGE_UNIT === undefined) {
      this.modal.warning('Please select Exchange Unit', this.viewref)
    } else {
      this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
    }
  }
  
  public OpenMainDialog() {
    this.Data = {};
    this.InitData();
    this.IsEdit = false;
    this.Data.ModEdit = this.IsEdit;
    // this.Data.RATE_TYPE_DESC = "4 : PRICE REVISION";
    this.Data.EXCHANGE_UNIT = 1; 
    this.dialogMain.title = "New Exchange Rate";
    this.dialogMain.OpenDialog(); 
  }

  public ConfirmSaveAdd = () => {
    this.Data.LAST_IP_ADDR = this.global.GetTokenDetail("ssIPAddress");
    this.http.post('SL', 'SNM032E_Exchange_Rate_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.SearchForAc();
        }
        else if(res.Table[0].STATUS == 'DUPLICATE'){
          this.modal.warning('No change to save found duplicate record', this.viewref);
        }
        else {
          this.modal.error(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  public Edit(P_data) {
    this.InitData();
    this.dialogMain.title = "Edit Exchange Rate";
    let newDate = new Date(P_data.EFFECTIVE_DATE);
    this.Data.EFFECTIVE_DATE = this.GetDateParam(newDate);
    this.Data.RATE_TYPE = P_data.RATE_TYPE_DESC;
    this.Data.CURRENCY_CODE = P_data.CURRENCY_CODE;
    this.http.post("SL", "SNM032E_GetExchange_Rate", this.Data)
      .subscribe(result => {
        if (result.Table.length > 0) {
          this.Data = result.Table[0];
          this.Data.LAST_UPD_NAME = this.Data.UPDATE_BY_NAME;
          this.IsEdit = true;
          this.Data.ModEdit = this.IsEdit;
          this.dialogMain.OpenDialog();
        }
      });
  }

  public Delete(P_data) {
    let newDate = new Date(P_data.EFFECTIVE_DATE);
    this.Data.EFFECTIVE_DATE = this.GetDateParam(newDate);
    this.Data.RATE_TYPE = P_data.RATE_TYPE;
    this.Data.CURRENCY_CODE = P_data.CURRENCY_CODE;
    this.modal.confirm("Do you want to delete record?", this.viewref, this.ConfirmDeleteRow);
  }

  public ConfirmDeleteRow = () => {
    this.Data.LAST_IP_ADDR = this.global.GetTokenDetail("ssIPAddress");
    this.http.post('SL', 'SNM032E_GetExchange_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.SearchForAc();
        }
      });

  }

  public CancelAdd() {
    this.InitData();
    this.dialogMain.Close();
  }

  public OnFileSelect(fileInput) {
    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;
  }

  public UpLoad() {

    if (this.files.length == 0) {
        this.modal.error('Please Open xlsx File First...', this.viewref)
    } else {
       // this.data = []
        let formData = new FormData();
        formData.append('file', this.files[0]);

        this.http.postFile('SL', 'SNM032E_Upload', formData)
            .subscribe(res => {
              //console.log('res',res)
              if(res.Error.length > 0){
                this.ListLog = res.Error;
                this.dialogLog.OpenDialog();
              }else{
                this.modal.info("Upload complete", this.viewref);
                this.SearchForAc();
              }
            },(err) =>{
              this.modal.error("Upload error",this.viewref);
              //console.log('err',err)
            })
    }
  }

  public DownloadTemplate(){
    let link = "assets/templates/ExportExchangeRate_Upload_Template.xlsx"
    window.open(link,'_blank')
  }
 
}
