import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';
import * as moment from 'moment';

@Component({
  selector: 'SNM124',
  templateUrl: './snm124.component.html',
  styleUrls: ['../saleorder.component.css']
})


export class SNM124Component implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public ParamSearch: any = {};
  public ParamEdit: any = {};
  public ParamEdit_Ex: any = {};
  public ParamAdd: any = {};
  public tabledata: any = [];
  public tabledataEdit: any = [];
  public iSupdate: boolean = false;
  public iSupdateEx: boolean = false;
  public currentDate: moment.Moment = moment();

  public hideCols: Array<string> = [];
  SetRoundExportPriceControl = new FormControl('', [Validators.required])

  public ngOnInit() {
    this.Search();
    this.global.GetRoleDetail().CHG === "N" ? this.hideCols.push("EDIT") : null;
    this.global.GetRoleDetail().DEL === "N" ? this.hideCols.push("DELETE") : null;
  }

  public Search() {
    this.tabledata = [];

    this.http.post("SL", "SNM124_GET_DATA", this.ParamSearch)
      .subscribe(res => {
        this.tabledata = res.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public Add() {
    this.ParamAdd = {}
    this.SetRoundExportPriceControl.reset();
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    if (this.ParamAdd.FY) {
      if (this.ParamAdd.ROUND_NO && this.ParamAdd.EFFECTIVE_DATE) {
        this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmSave);
      }
      else {
        this.modal.warning("Please Input fill Round No. and select Effective Date.", this.viewref);
      }
    }
    else {
      this.modal.warning("Please Input fill FY", this.viewref);
    }
  }


  public ConfirmSave = () => {

    this.http.post('SL', 'SNM124_INSERT', this.ParamAdd)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.ParamAdd = this.global.ClearObject(this.ParamAdd);
          this.dialogAdd.Close();
        }
        else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  public EditRow(data) {

    this.ParamEdit = data;
    this.SetRoundExportPriceControl.reset();
    this.dialogEdit.OpenDialog();
    this.tabledataEdit = [];

    this.http.post('SL', 'SNM124_GET_EXCHANGE_RATE', this.ParamEdit)
      .subscribe(res => {
        this.tabledataEdit = res.Table;

        if (this.tabledataEdit.length > 0) {
          this.tabledataEdit.forEach((e) => {
            e.EXCHANGE_UNIT = parseFloat(e.EXCHANGE_UNIT).toFixed(0);
            e.EXCHANGE_RATE = parseFloat(e.EXCHANGE_RATE).toFixed(5);
          });
        }
        //console.log(this.tabledataEdit,'tabledataEdit');
      })
  }

  public SaveEdit() {
    this.modal.confirm("Do you want to save changed you have made?", this.viewref, this.ConfirmEdit)
  }

  public ConfirmEdit = () => {

    this.ParamEdit_Ex.ModEdit = true;

    this.http.post('SL', 'SNM124_UPDATE', this.ParamEdit)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialogEdit.Close();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });

    if (this.tabledataEdit.length > 0) {
    this.http.post('SL', 'SNM124_UPDATE_EXCHANGE_RATE', this.ParamEdit_Ex)
      .subscribe(res => {
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
    }
  }

  public CancleEdit() {
    this.dialogEdit.Close();
  }

  public ExchangeRateChange(data) {
    this.ParamEdit_Ex = data;
    this.ParamEdit_Ex.EXCHANGE_RATE = parseFloat(this.ParamEdit_Ex.EXCHANGE_RATE).toFixed(5);
  }

  public ExchangeUnitChange(data) {
    this.ParamEdit_Ex = data;
    this.ParamEdit_Ex.EXCHANGE_UNIT = parseFloat(this.ParamEdit_Ex.EXCHANGE_UNIT).toFixed(0);
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete record?", this.viewref, () => this.ConfirmDelete(data))
  }

  public ConfirmDelete = data => {
    this.http.post('SL', 'SNM124_DELETE', data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }
}

