import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'SNM145P',
  templateUrl: './SNM145P.component.html',
  styleUrls: ['../saleorder.component.css']
})


export class SNM145PComponent implements OnInit {

  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;

  public params: any = {};
  public paramsAdd: any = {};
  public tableData: any = [];
  public dataIsSelected: any = [];

  public ngOnInit() {
    this.params.ISSUE_STATUS = '';
    this.Search();
  }

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_NAME",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM145P_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  public lookupCustomerCodeAddConfig: any = {
    ColName: "LOOKUP_CUSTOMER_NAME",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNM145P_GetCustomerCodeAdd",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerCodeAdd(value: any) {
    this.paramsAdd.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.paramsAdd.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  public Search() {
    this.tableData = [];
    this.http.post("SL", "SNM145P_GetData", this.params)
      .subscribe(res => {
        this.tableData = res.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public Add() {
    this.paramsAdd = {};
    this.dialogAdd.OpenDialog();
  }

  public CloseDialog() {
    this.paramsAdd = {};
    this.dialogAdd.Close();
  }

  public SaveAdd() {
    if (this.paramsAdd.CUSTOMER_CODE && this.paramsAdd.SENT_BY) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmAdd);
    }
    else {
      this.modal.warning("Please fill Input Customer Code", this.viewref);
    }
  }

  public ConfirmAdd = () => {
    this.http.post('SL', 'SNM145P_SaveAdd', this.paramsAdd)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.paramsAdd = this.global.ClearObject(this.paramsAdd);
          this.dialogAdd.Close();
        }
        else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public UpdateStatus() {
    if (this.dataIsSelected.length > 0) {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmUpdate);
    } else {
      this.modal.warning("No data change!", this.viewref);
    }
  }

  public ConfirmUpdate = () => {
    this.http.post('SL', 'SNM145P_UpdateStatus', this.dataIsSelected)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
        }
        else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public StatusChange(data) {
    if (this.dataIsSelected.some(x => x.CUSTOMER_CODE == data.CUSTOMER_CODE)) {
      this.dataIsSelected.splice(this.dataIsSelected.indexOf(data), 1);
      this.dataIsSelected.push(data);
    } else {
      this.dataIsSelected.push(data);
    }
  }
}
