import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from '../../../core/global';


@Component({
  selector: 'snse031',
  templateUrl: './SNSE031.component.html',
  styleUrls: ['./SNSE031.component.css']
})

export class SNSE031Component implements OnInit {
  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global : Globals
  ) { }

  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  public tableData: any = [];
  public tableDataEdit: any = [];
  public dataSelected: any = [];
  public dataEditSave: any = [];
  public dataEdit: any = {};
  public dataChk: any = {};
  public formData: any = {};
  public params: any = {};
  public iSedit: boolean = false;

  ngOnInit() {
  }

  initData() {
    this.dataEdit.TotalNet = 0; 
  }


  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE025_N_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }


  public SetSelectedCustomerCode(values: any) {
    this.params.CUSTOMER_CODE = values.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = values.CUSTOMER_NAME;

    this.params.BRANCH_CODE = values.BRANCH_CODE;
    this.params.BRANCH_NAME = values.BRANCH_NAME;
    this.iSedit = false;
    this.Search();
  }

  public Search = () => {
    this.tableData = [];

    this.Http.post('SL', 'SNSE031_GetData', this.params)
      .subscribe(res => {
        this.tableData = res.Table;

        if (this.tableData.length > 0) {

          this.tableData.forEach((e) => {
            e.GROSS_WEIGHT = parseFloat(e.GROSS_WEIGHT).toFixed(3);
            e.GROSS_MM = parseFloat(e.GROSS_MM).toFixed(3);
          });

          if (this.iSedit) {
            this.NetWeightCheck(); 
          }
        }

      })
  }


  public ConfirmSave() {
    this.iSedit = false;
    var err = 0;
    var empty = 0;
    var caseNo = '';
    this.tableData.forEach((e) => {
      //if ((e.GROSS_MM != 0 || e.GROSS_WEIGHT != 0) && e.NET_WEIGHT > e.GROSS_WEIGHT) {
      //  err = 1;
      //  caseNo = caseNo + e.CASE_NO + ' ,';
      //}
      if ((e.GROSS_MM > 0 || e.GROSS_WEIGHT > 0) && (e.GROSS_MM_OLD != e.GROSS_MM || e.GROSS_WEIGHT_OLD != e.GROSS_WEIGHT)) {
        empty = empty + 1;

      }
    });
    //console.log(err, empty)
    if (err == 0 && empty > 0) {
      this.modal.confirm('Do you want to update on this data?', this.viewref, this.Save)
    }
    //else if (err == 1) {
    //  this.modal.warning('Net Weight cannot over gross weight.', this.viewref)
    //}
    else if (empty == 0) {
      this.modal.warning('There is no data change.', this.viewref)
    }
  }

  public Save = () => {
    this.dataSelected = [];
    this.tableData.forEach((e) => {
      if ((e.GROSS_MM > 0 || e.GROSS_WEIGHT > 0) && (e.GROSS_MM_OLD != e.GROSS_MM || e.GROSS_WEIGHT_OLD != e.GROSS_WEIGHT)) {
        this.dataSelected.push(e);
      }
    });

    this.Http.post('SL', 'SNSE031_Update', this.dataSelected)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.modal.info('Update data successfully.', this.viewref);
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      })
  }

  public CheckWeight(data) {
    if (data.NET_WEIGHT > data.GROSS_WEIGHT) {
      this.modal.warning(data.CASE_NO + ' : Net Weight cannot over gross weight.', this.viewref)
      data.GROSS_WEIGHT = data.NET_WEIGHT;
    }

  }

  public tempRow: any;

  public EditData(data) {
    this.tempRow = []
    this.dataEdit = data
    this.tempRow = JSON.parse(JSON.stringify(data))

    this.dialogEdit.OpenDialog();
    this.params.CASE_NO = this.dataEdit.CASE_NO;
    this.params.PACK_DATE = this.dataEdit.PACK_DATE;
    this.params.SENT_BY = this.dataEdit.SENT_BY;

    this.tableDataEdit = [];
    this.Http.post('SL', 'SNSE031_NetWeightList', this.dataEdit)
      .subscribe(res => {
        this.tableDataEdit = res.Table;
        if (this.tableDataEdit.length > 0) {
          this.dataEdit.TotalNet = 0;

          this.tableDataEdit.forEach((e) => {
            e.NET_WEIGHT = e.NET_WEIGHT / 1000
            this.dataEdit.TotalNet += e.NET_WEIGHT;
            e.NET_WEIGHT = parseFloat(e.NET_WEIGHT).toFixed(3);
            e.NET_WEIGHT_OLD = e.NET_WEIGHT;
          });
        }
      }, (err) => {
          console.log(err, 'err');
          this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public NetWeightChange(data) {
    this.dataEdit.TotalNet = 0;
    //this.dataEditSave = [];

    this.tableDataEdit.forEach((e) => {
      this.dataEdit.TotalNet += parseFloat(e.NET_WEIGHT);
      e.NET_WEIGHT = parseFloat(e.NET_WEIGHT).toFixed(3);
      //this.dataEditSave.push(e);
    });  
  }

  public CancleEdit() {
    this.dialogEdit.Close();
  }

  public ConfrimSaveEdit() {
    var empty = 0;
    this.tableDataEdit.forEach((e) => {
      if (e.NET_WEIGHT_OLD != e.NET_WEIGHT) {
        empty = empty + 1;
      }
    });
    if (empty > 0) {
      this.modal.confirm('Do you want to update on this data?', this.viewref, this.SaveEdit);
    }
    else if (empty == 0) {
      this.modal.warning('There is no data change.', this.viewref);
    }
  }

  public SaveEdit = () => {

    this.dataEditSave = [];
    this.tableDataEdit.forEach((e) => {
      if (e.NET_WEIGHT_OLD != e.NET_WEIGHT) {
        e.NET_WEIGHT_SAVE = e.NET_WEIGHT * 1000;
        this.dataEditSave.push(e);
      }
    });

    this.Http.post('SL', 'SNSE031_UpdateNetWeight', this.dataEditSave)
      .subscribe(res => {

        if (res.Table[0].STATUS == 'DONE') {
          this.dialogEdit.Close();
          this.iSedit = true;
          
          this.modal.info('Case No. ' + this.dataEdit.CASE_NO + ' update net weight successfully.', this.viewref, this.Search);
          
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
          console.log(err, 'err');
          this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public NetWeightCheck = () => {
    let index = this.tableData.findIndex(x => x.CASE_NO == this.tempRow.CASE_NO)
 
    
    if (this.tableData[index].NET_WEIGHT  > this.tempRow.GROSS_WEIGHT) {
      this.modal.warning("Net Weight cannot over gross weight.", this.viewref)
      this.tableData[index].GROSS_WEIGHT = this.tableData[index].NET_WEIGHT
      this.tableData = this.tableData.slice()
    }
    console.log(this.dataEdit);
  }

  public grossChange(data) {
    data.GROSS_WEIGHT = parseFloat(data.GROSS_WEIGHT).toFixed(3);
  };

  public grossMMChange(data) {
    data.GROSS_MM = parseFloat(data.GROSS_MM).toFixed(3);
  }



}
