import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { FormGroup, FormControl, FormBuilder } from '@angular/forms'
import { Router } from '@angular/router';
import { Observable, config } from 'rxjs';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';

@Component({
  selector: 'snse040_p',
  templateUrl: './SNSE040_P.component.html',
  styleUrls: ['./SNSE040_P.component.css']
})



export class SNSE040_PComponent implements OnInit {

  constructor(
    private http: HttpService,
    private router: Router,
    private modal :ModalService,
    private viewref : ViewContainerRef,
    private global : Globals
  ) { }

  public ConfigCustomer: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Customer.",
    SourceController: "SL",
    SourceMethodName: "SNSE040_P_GetCustomer",
    NoOfShowRecords: 30,
    UpperCase: true,
    MinLengthSearch : 3,
  }

  public ConfigBranch: any = {
    ColName: "LOOKUP",
    ColValue: "",
    PlaceholderText: "Select Branch",
    SourceController: "SL",
    SourceMethodName: "SNSE040_P_GetBranch",
    NoOfShowRecords: 100, 
    MinLengthSearch : 0,
    SourceParams: {
      CUSTOMER_CODE: null 
    }    
  }

  public ConfigInstruction: any = {
    ColName: "INSTRUCTION_NO",
    ColValue: "",
    PlaceholderText: "Select Instruction",
    SourceController: "SL",
    SourceMethodName: "SNSE040_P_GetInstruction",
    NoOfShowRecords: 100, 
    MinLengthSearch : 3,
    SourceParams: {
      CUSTOMER_CODE: null ,
      BRANCH_CODE : null
    }    
  }



  public paramSearch : any = {}
  public tableHeader = [];
  public showDetail : boolean = false
  public params : any = {};
  public iSAdd : boolean = false;
  public tableDetail = [];
  public defectiveList = [];
  public hideCols = []
  public iSSave : boolean = false;
  public sentByList = [];
  public currentData : any;
  public iSEdit : boolean = false;
  public tempDelete = []


  ngOnInit() {
    this.paramSearch.SENT_BY = 'ALL';
    this.http.get("SL","SNSE040_P_GetSentBy")
    .subscribe(res =>{
      this.sentByList = res.Table
    })
    
    this.http.GetSystemDate()
    .subscribe(res =>{
     
      this.currentData = res
    })
  
  }

  public Search(){

    if(!this.paramSearch.CUSTOMER_CODE && !this.paramSearch.PACK_DATE_SEARCH){
      this.modal.warning("Please Customer Code or Pack Date",this.viewref);
      return
    }

    if(this.paramSearch.PACK_DATE_SEARCH){
      this.paramSearch.PACK_DATE = this.FormatDate(this.paramSearch.PACK_DATE_SEARCH)
    }else{
      this.paramSearch.PACK_DATE = null
    }

    this.http.post("SL","SNSE040_P_GetHeader",this.paramSearch)
    .subscribe(res => {
      console.log(res)
      this.tableHeader = res.Table
    })
  }



  public ViewClick(event){
    
    this.showDetail = true
    //this.hideCols = ["EDIT"];
    this.iSAdd = false;
    this.params = event
    this.iSEdit = true
    this.tableDetail = [];

    this.ConfigInstruction.SourceParams.CUSTOMER_CODE = event.CUSTOMER_CODE
    this.ConfigInstruction.SourceParams.BRANCH_CODE = event.BRANCH_CODE

    this.http.post("SL","SNSE040_P_GetDetail",event)
    .subscribe(res => {
      console.log(res)
      this.tableDetail = res.Table
      this.tableDetail.push({ 'EDIT': 'ADD',ITEM_SEQ : this.tableDetail.length + 1})
    })
  }

  Back(){
    this.showDetail = false;
  }


  public Packing(){
    this.showDetail = true
    this.hideCols = [];
    this.iSAdd = true
    this.iSSave = false
    this.iSEdit = false
    this.params = {}
    this.tableDetail = []
    this.params.INVOICE_DATE = new Date(this.currentData)
    this.params.PACK_BY = this.global.GetTokenDetail("ssNameEN");
    this.params.PACK_DATE = new Date(this.currentData);
    let d = new Date()
    this.params.PACK_TIME = d.toLocaleTimeString('en-GB')
    this.ConfigBranch.SourceParams.CUSTOMER_CODE = null
    //this.ConfigDoc.SourceParams.CUSTOMER_CODE 

    this.tableDetail.push({ 'EDIT': 'DEL',ITEM_SEQ : 1},
    { 'EDIT': 'DEL',ITEM_SEQ : 2},
    { 'EDIT': 'DEL',ITEM_SEQ : 3},
    { 'EDIT': 'DEL',ITEM_SEQ : 4},
    { 'EDIT': 'ADD',ITEM_SEQ : 5});
  }

  public SelectCustomer(event){
    this.params.CUSTOMER_CODE = event.CUSTOMER_CODE
    this.params.CUST_NAME_TH = event.CUST_NAME_TH
    this.ConfigBranch.SourceParams.CUSTOMER_CODE = event.CUSTOMER_CODE
    this.ConfigInstruction.SourceParams.CUSTOMER_CODE = event.CUSTOMER_CODE
  }

  public SelectBranch(event){
    this.params.BRANCH_CODE = event.BRANCH_CODE
    this.params.BRANCH_NAME_TH = event.BRANCH_NAME_TH
    this.ConfigInstruction.SourceParams.BRANCH_CODE = event.BRANCH_CODE
  }

  public GetCaseSeq(){

    this.params.PACK_DATE_STRING = this.FormatDate(this.params.PACK_DATE)
    this.http.post("SL","SNSE040_P_FindCaseSeq",this.params)
    .subscribe(res =>{
      this.params.CASE_SEQ = res.CASE_SEQ
    })
  }


  public SelectInstruction(event,data){
  
    console.log(event)

  if(event == undefined){
    data.INSTRUCTION_NO = null
    return
  }

  if(!event.INVOICE_DATE){
    this.modal.warning("ORDER STS Invalid value data,Please check.",this.viewref)
    data.INSTRUCTION_NO = null
    return
  }


  if(this.params.INVOICE_DATE){
    let d1 = new Date(event.INVOICE_DATE)
    let d2 = new Date(this.params.INVOICE_DATE)
 
    if(d1.toDateString() != d2.toDateString()){
     this.modal.warning("invoice date not match",this.viewref)
     data.INSTRUCTION_NO = null
     return
    }
  }
   

   data.INSTRUCTION_NO = event.INSTRUCTION_NO
   let find = this.tableDetail.filter(x => x.INSTRUCTION_NO == event.INSTRUCTION_NO)
   if(find.length > 1){
     this.modal.warning("Duplicate Instruction No. !.....",this.viewref)
     data.INSTRUCTION_NO = null
     return
   }

   data.INSTRUCTION_NO = event.INSTRUCTION_NO
   data.PICK_LOT_DATE = event.PICK_LOT_DATE
   data.PICK_LOT_SEQ = event.PICK_LOT_SEQ
   data.ORDER_NO = event.ORDER_NO,
   data.ORDER_PART_NO = event.ORDER_PART_NO
   data.INPUT_PART = event.INPUT_PART
   data.BAL_QTY = event.A_P
   data.PACK_QTY = 1
   data.ITEM_NO = event.ITEM_NO
   data.SUB_ITEM_NO = event.SUB_ITEM_NO
   data.UPDATE_FLAG = 'INS'
  }


  public Partchange(event,data){

    if(event.target.value != data.INPUT_PART){
        this.modal.warning("PART NO : " + data.INPUT_PART + " : " +  event.target.value + " Invalid value data,Please check.",this.viewref)
        data.ACCEPT_PART_NO = null
        return
    }


  }

  public QtyChange(data){

    if(data.PACK_QTY == null){
      data.PACK_QTY = 0
    }

    if(data.PACK_QTY > data.BAL_QTY){
      this.modal.warning("Pack Qty  Must Be  <=  Bal. Qty",this.viewref)
      data.PACK_QTY = data.BAL_QTY
    }
  }


  public AppendRow() {

    if(this.iSSave){
      return
    }

    for (var i = 0; i < this.tableDetail.length; i++) {
      if (this.tableDetail[i].EDIT === "ADD") {
        this.tableDetail[i].EDIT = 'DEL';
      }
    }
    this.tableDetail.push({ 'EDIT': 'ADD'});
    this.tableDetail.map((x,i)=> {
      x.ITEM_SEQ = i + 1
    })
    this.tableDetail = this.tableDetail.slice();
  }

  public DeleteRow(index,data) {

    if(this.iSSave){
      return
    }

    if(data.UPDATE_FLAG == 'UPD'){
      data.UPDATE_FLAG = 'DEL'
      this.tempDelete.push(data)
    }
    
    this.tableDetail.splice(index, 1);
    this.tableDetail.map((x,i)=> {
      x.ITEM_SEQ = i + 1
    })
    this.tableDetail = this.tableDetail.slice();
 
  }



  public Save(){


    if(this.iSAdd){
      let CompleteData = this.tableDetail.filter(x => x.INSTRUCTION_NO && x.ACCEPT_PART_NO && x.PACK_QTY >= 0)

      if(CompleteData.length == 0){
        this.modal.warning("No detail data for save",this.viewref)
        return
      }else{
        let count = 0;
        CompleteData.forEach(x =>{
          if (x.INSTRUCTION_NO && !x.ACCEPT_PART_NO) {
            this.modal.warning("Please insert part No at Item No. : " + x.ITEM_SEQ, this.viewref);
            count++;
            return
          }
  
          if (x.INSTRUCTION_NO && x.PACK_QTY <= 0) {
            this.modal.warning("Please insert Qty at Item No. : " + x.ITEM_SEQ, this.viewref);
            count++;
            return
          }
        })
  
        if(count != 0){
          return
        }
      }
  
      this.modal.confirm("Do want to save change you have made?",this.viewref,()=> ConfirmSave())
  
      let ConfirmSave = () => {
  
        let Data = {
          CUSTOMER_CODE : this.params.CUSTOMER_CODE,
          BRANCH_CODE : this.params.BRANCH_CODE,
          CASE_NO : this.params.CASE_NO,
          CASE_SEQ : this.params.CASE_SEQ,
          DETAIL : CompleteData
        }
  
        this.http.post("SL","SNSE040_P_Save",Data)
        .subscribe(res => {
  
          if(res.STATUS == 'COMPLETE'){
            this.iSSave = true;
            this.modal.info("Transaction complete record apply and save",this.viewref)
          }else if(res.STATUS == 'DUPLICATE'){
            this.modal.warning(res.MESSAGE,this.viewref)
          }else if(res.STATUS == 'HANDHELD_NOT_FOUND'){
            this.modal.warning(res.MESSAGE,this.viewref)
          }else{
            this.modal.warning("Found an error, please contact support team", this.viewref);
          }
        },err =>{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        })
      }
    }
    else{
        let insertData = this.tableDetail.filter(x => x.INSTRUCTION_NO && x.ACCEPT_PART_NO && x.PACK_QTY >= 0 && x.UPDATE_FLAG == 'INS')
        let updateDate = this.tableDetail.filter(x => x.INSTRUCTION_NO && x.ACCEPT_PART_NO && x.PACK_QTY >= 0 && x.UPDATE_FLAG == 'UPD' && x.TEMP_PACK_QTY != x.PACK_QTY)
        let complateData = [...insertData, ...updateDate, ...this.tempDelete];

        console.log('complateData',complateData)

        if(complateData.length == 0){
          this.modal.warning("No detail data for save",this.viewref)
          return
        }else{
            let count = 0;
            complateData.forEach(x =>{
              if (x.INSTRUCTION_NO && !x.ACCEPT_PART_NO) {
                this.modal.warning("Please insert part No at Item No. : " + x.ITEM_SEQ, this.viewref);
                count++;
                return
              }
      
              if (x.INSTRUCTION_NO && x.PACK_QTY <= 0) {
                this.modal.warning("Please insert Qty at Item No. : " + x.ITEM_SEQ, this.viewref);
                count++;
                return
              }
            })
      
            if(count != 0){
              return
            }
        }

        this.modal.confirm("Do want to save change you have made?",this.viewref,()=> ConfirmSave())
  
      let ConfirmSave = () => {
  
        let Data = {
          CUSTOMER_CODE : this.params.CUSTOMER_CODE,
          BRANCH_CODE : this.params.BRANCH_CODE,
          CASE_NO : this.params.CASE_NO,
          CASE_SEQ : this.params.CASE_SEQ,
          PACK_DATE : this.FormatDate(this.params.PACK_DATE),
          DETAIL : complateData
        }
  
        this.http.post("SL","SNSE040_P_Save_Update",Data)
        .subscribe(res => {
  
          if(res.STATUS == 'COMPLETE'){
            this.iSSave = true;
            this.modal.info("Transaction complete record apply and save",this.viewref)
          }else if(res.STATUS == 'DUPLICATE'){
            this.modal.warning(res.MESSAGE,this.viewref)
          }else if(res.STATUS == 'HANDHELD_NOT_FOUND'){
            this.modal.warning(res.MESSAGE,this.viewref)
          }else{
            this.modal.warning("Found an error, please contact support team", this.viewref);
          }
        },err =>{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        })
      }

    }

  }


  FormatDate(date) {

    let d = new Date(date);
    let Day = d.getDate().toString();
    let NewDay = Day.toString().length < 2 ? '0' + Day.toString() : Day.toString()
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    return Year + NewMonth + NewDay;
}

}
