import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';



@Component({
  selector: 'SNSE215',
  templateUrl: './SNSE215.component.html',
  styleUrls: ['../saleorder.component.css']
})



export class SNSE215Component implements OnInit {

  public params: any = {}
  public Data: any = {};
  public Datatable: any = [];

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
  ) { }

  @ViewChild("dialogMain", { static: false }) dialogMain: any;

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE215_GetCustomer",
    NoOfShowRecords: 50,
    MinLengthSearch: 0
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  public lookupOrderNoConfig: any = {
    ColName: "ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "SL",
    SourceMethodName: "SNSE215_GetOrderNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params
    }
  }

  public SetSelectedOrderNo(value: any) {
    this.params.ORDER_NO = value.ORDER_NO;
  }

  public lookupCustomerCodeMasterConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE215_GetCustomer2",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedCustomerMasterCode(value: any) {
    this.Data.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.Data.CUSTOMER_NAME = value.CUSTOMER_NAME;
  }

  ngOnInit() {
    this.Search();
  }

  public Search = () => {
    this.http.post('SL', 'SNSE215_GetData', this.params)
      .subscribe(res => {
        this.Datatable = res.Table;
      })
  }

  private InitData() {
    this.Data = {};
  }

  public CreateClick() {
    this.InitData();
    this.dialogMain.OpenDialog();
  }

  public ConfirmDelete(row) {
    this.Data = {};
    this.Data = row
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.Delete);
  }

  private Delete = () => {
    this.http.post('SL', 'SNSE215_Delete', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.Search();
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public ConfirmSave() {
    if (this.Data.CUSTOMER_CODE && this.Data.ORDER_NO) {
        this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.Save)
      }
      else {
        this.modal.warning('Please complete all data.', this.viewref)
      }
  }

  private Save = () => {
    this.http.post('SL', 'SNSE215_Save', this.Data)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.Search();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public Cancel() {
    this.InitData();
    this.dialogMain.Close();
  }
}

