import { Component, OnInit, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';


@Component({
  selector: 'snse023_001',
  templateUrl: './SNSQ023_001.component.html',
  styleUrls: ['./SNSQ023_001.component.css']
})


export class SNSQ023_001Component implements OnInit {

  public params: any = {};

  public tableData: any = [];

  public formData = {
    CNT_REC: 0,
    CNT_FIRST_ORDER: 0,
    AVG_LT_ALLOCATED_PICK: '0',
    CNT_FIRST_ALLOCATE_LINE: 0,
    AVG_LT_PICK_INVOICE: '0',
    V_FILL_RATE: '0'
  }

  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  ngOnInit() {
    var date = new Date();
    this.params.FROM_DATE = new Date(date.getFullYear(), date.getMonth(), 1, 7);
    this.params.TO_DATE = new Date();
  }

  InitData() {
    this.formData.CNT_REC = 0;
    this.formData.CNT_FIRST_ORDER = 0;
    this.formData.AVG_LT_ALLOCATED_PICK = '0';
    this.formData.CNT_FIRST_ALLOCATE_LINE = 0;
    this.formData.AVG_LT_PICK_INVOICE = '0';
    this.formData.V_FILL_RATE = '0';
  }

  public lookupCustomerCodeConfig: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSQ023_GetCustomerCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params
    },
    UpperCase: true
  }

  public lookupOrderNoConfig: any = {
    ColName: "LOOKUP_ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "SL",
    SourceMethodName: "SNSQ023_GetOrderNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params
    },
    UpperCase: true
  }

  public lookupOrderPartNoConfig: any = {
    ColName: "LOOKUP_PART_NO",
    PlaceholderText: "Select Order Part No",
    SourceController: "SL",
    SourceMethodName: "SNSQ023_GetOrderPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params
    },
    UpperCase: true
  }

  public SetSelectedCustomerCode(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE;
    this.params.CUSTOMER_NAME = value.CUST_NAME_EN;
    this.params.ORDER_NO = null;
    this.params.ORDER_PART_NO = null;
  }

  public SetSelectedOrderNo(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE
    this.params.CUSTOMER_NAME = value.CUST_NAME_EN;
    this.params.ORDER_NO = value.ORDER_NO;
    this.params.ORDER_PART_NO = null;
  }

  public SetSelectedOrderPartNo(value: any) {
    this.params.CUSTOMER_CODE = value.CUSTOMER_CODE
    this.params.CUSTOMER_NAME = value.CUST_NAME_EN;
    this.params.ORDER_NO = value.ORDER_NO
    this.params.ORDER_PART_NO = value.ORDER_PART_NO;
  }

  public Search() {
    this.tableData = [];
    this.InitData();
    this.Http.post("SL", "SNSQ023_GetData", this.params)
      .subscribe(res => {
        this.tableData = res.Table;
        if (this.tableData.length > 0) {
          this.formData.CNT_REC = this.tableData.length;
          //this.formData.CNT_FIRST_ORDER = this.tableData[0].CNT_FIRST_ORDER;
          //this.formData.CNT_FIRST_ALLOCATE_LINE = this.tableData[0].CNT_FIRST_ALLOCATE_LINE;
          //this.formData.AVG_LT_ALLOCATED_PICK = this.tableData[0].AVG_LT_ALLOCATED_PICK;
          //this.formData.AVG_LT_PICK_INVOICE = this.tableData[0].AVG_LT_PICK_INVOICE;
          //this.formData.V_FILL_RATE = this.tableData[0].FILL_RATE;

          var custCode = null;
          var orderNo = null;
          var orderPartNo = null;
          var itemNo = null;
          var bo_qty = 0;
          this.tableData.forEach((e) => {
            if (custCode == e.CUSTOMER_CODE && orderNo == e.ORDER_NO && itemNo == e.ITEM_NO && orderPartNo == e.ORDER_PART_NO) {
              e.CUSTOMER_CODE = null;
              e.CUST_NAME_EN = null;
              e.ORDER_NO = null;
              e.ORDER_DATE = null;
              e.ORDER_PART_NO = null;
              e.V_VENDOR_CODE = null;
              e.P_VENDOR_CODE = null;
              e.ORDER_QTY = null;

              if (e.ALLOCATE_DATE == e.FIRST_ALLOCATE_DATE && e.ALLOCATE_DATE == e.CHK_FIRST_ALLOCATE) {
                e.V_F_ACCEPT_QTY = e.ACCEPT_QTY;
                e.V_AFTER_F_ACCEPT_QTY = null;
                e.V_BO_QTY = bo_qty - e.V_F_ACCEPT_QTY;
              }
              else {
                e.V_F_ACCEPT_QTY = null;
                e.V_AFTER_F_ACCEPT_QTY = e.ACCEPT_QTY;
                e.V_BO_QTY = bo_qty - e.V_AFTER_F_ACCEPT_QTY;
              };

              e.V_PICK_LOT_DATE = e.V_PICK_LOT_DATE_SUB;
              e.V_LT_ORDER_PICK = e.V_LT_ORDER_PICK_SUB;
              e.INVOICE_DATE = e.V_INVOICE_DATE_SUB;
              e.V_LT_PICK_INVOICE = e.V_LT_PICK_INVOICE_SUB;
             
              bo_qty = e.V_BO_QTY;

              if (e.V_F_ACCEPT_QTY == 0) {
                e.V_F_ACCEPT_QTY = null;
              }
            } else {        
              custCode = e.CUSTOMER_CODE;
              orderNo = e.ORDER_NO;
              orderPartNo = e.ORDER_PART_NO;
              itemNo = e.ITEM_NO;
              bo_qty = e.V_BO_QTY;
            }
          });
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      })
  }

  private WriteContents(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = "SNSQ023.csv";
    a.click();
  }

  public ExportExcel() {
    if (this.tableData.length > 0) {
      this.Http.Download("SL", "SNSQ023_getDataForExport", JSON.stringify(this.params))
        .subscribe(result => {
          if (result) {
            this.download(result)
          }
        }, (err) => {
          console.log(err, 'err')
        })
    }
    else {
      this.modal.warning('No data for export to excel', this.viewref)
    }
  }

  private download(content) {
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type })
    a.href = URL.createObjectURL(file);
    a.download = "SNSQ023.csv"
    a.click()
  }
}
