import { SelectionModel } from '@angular/cdk/collections';
import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { Globals } from 'src/app/core/global';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';


@Component({
    selector: 'snm105p',
    templateUrl: './SNM105P.component.html',
    styleUrls: ['./SNM105P.component.css']
})

export class SNM105PComponent implements OnInit {

    constructor(
        private http: HttpService,
        private modal:ModalService,
        private viewref : ViewContainerRef,
        private global : Globals
    ) { }


    public CustomerList: any;
    public tableData: any = [];
    public files: any = [];
    public FileName: string = ""
    public selection = new SelectionModel<any>(true, []);
    public checkBasePrice : boolean = true;
    public checkStandardCost : boolean = false;
    public checkBasePricePo : boolean = true;
    public checkPricingCode : boolean = true;
    public total_error : number;
    public error_list : any = [];
    @ViewChild("dialog",{static: false}) dialog : any;
    ngOnInit() {

    }

    
    public fileChange(fileInput) {
        this.selection.clear()
        this.files = fileInput.target.files;
        this.FileName = this.files[0].name;

        this.UpLoad()
    }

    private UpLoad(){
        this.tableData = [];
        let formData = new FormData();
        formData.append('file', this.files[0]);

        this.http.postFile('TI', 'SNM105P_UploadExcel', formData)
        .subscribe(res => {
          //console.log('res',res)
          this.total_error = 0;
          this.tableData = res.filter(x => x.PART_NO);
          let countrow = 0;
          this.total_error = this.tableData.filter((x,i) => {
              if(x.ERR_FLAG == 'Y'){
                 countrow += 1;
                  this.error_list.push({
                      Records : i + 1
                  })
              } 
        })
        this.total_error = countrow;
        
        this.BPChange({checked : this.checkBasePrice})
        this.STDChange({checked : this.checkStandardCost})

        },(err) =>{
          this.modal.error("Upload error",this.viewref);
          //console.log('err',err)
        })
    }

    public UpdatePrice(){
        this.modal.confirm("Do you want to save change you have made?",this.viewref,this.ConfirmUpdatePrice)
    }
    private ConfirmUpdatePrice = () =>{
       
        let isBasePrice = this.checkBasePrice == true ? 'Y' : 'N';
        let isStandardCost = this.checkStandardCost == true ? 'Y' : 'N';
        
        let datafilter = []
        if(this.checkBasePrice && this.checkStandardCost){
            datafilter = this.selection.selected.filter(x => x.ERR_FLAG_BP == 'N' && x.ERR_FLAG_STD == 'N')
        }else{
            if(this.checkBasePrice){
                datafilter = this.selection.selected.filter(x => x.ERR_FLAG_BP == 'N')
            }else{
                datafilter = this.selection.selected.filter(x => x.ERR_FLAG_STD == 'N')
            }
        }

        let data  = {
        IS_BASE_PRICE : isBasePrice,
        IS_STANDARD_COST : isStandardCost,
        DETAIL : datafilter
        }
        this.http.post("TI","SNM105P_UpdatePrice",data)
        .subscribe(res => {
            this.modal.info('Transaction complete: ' +  datafilter.length  +' record applied and saved.', this.viewref);
            this.selection.clear();
        },(err)=>{
            this.modal.error('No change to save. Something went wrong.', this.viewref);
        })
    }


    public DownLoad(){
        if(this.tableData.length == 0){
            this.modal.warning("No data for Export",this.viewref)
            return
        }


        this.http.Download("TI","SNM105P_Download",this.tableData)
        .subscribe(res =>{
            this.WriteContents(res)
        })
    }


    private WriteContents(content) {

        const a = document.createElement('a');
        const file = new Blob([content], { type: content.type });
        a.href = URL.createObjectURL(file);
        a.download = "SNM105P" + '_' + this.global.CurrentDate() + ".csv";
        a.click();
      }

      public isAllSelected() {
        const numSelected = this.selection.selected.length;
        const numRows = this.tableData.filter(x => x.ERR_FLAG_BP == 'N' || x.ERR_FLAG_STD == 'N').length;
        return numSelected === numRows;
      }
    
      public Toggle() {
        this.isAllSelected() ?
          this.selection.clear() :
          this.tableData.forEach(row => {
            if (row.ERR_FLAG_BP == 'N' || row.ERR_FLAG_STD == 'N') {
              this.selection.select(row)
            }
          });
      }

      public STDChange(event){

       
          if(this.tableData.length == 0)return;

          let msg_err_stock = 'Does not exist on @TPM_STOCK_MASTER!'
          let msg_err_part = 'Does not exist on @TPM_PARTS_MASTER!'

          if(event.checked){
              
              this.tableData.map(x =>{
                  if(x.ERR_FLAG_PART == 'Y'){
                      x.MSG = msg_err_stock
                  }else if (x.ERR_FLAG_STD == 'Y'){
                      x.MSG = x.ERR_MSG_STD
                  }
              })
          }else{
              if(this.checkBasePrice){   
                this.tableData.map(x =>{
                    if(x.ERR_FLAG_PART == 'Y'){
                        x.MSG = msg_err_part
                    }else if(x.ERR_FLAG_BP == 'Y'){
                        x.MSG = x.ERR_MSG_BP
                    }else if(x.ERR_FLAG_PART != 'Y' && x.ERR_FLAG_BP != 'Y'){
                        x.MSG = null
                    }
                })
              }else{
                this.tableData.map(x =>{
                        x.MSG = null
                })
              }
          }

          this.tableData = this.tableData.slice();
          this.CheckSelect();
      }

      public BPChange(event){
        if(this.tableData.length == 0)return;
        
        let msg_err_stock = 'Does not exist on @TPM_STOCK_MASTER!'
        let msg_err_part = 'Does not exist on @TPM_PARTS_MASTER!'
          
        if(event.checked){
            this.tableData.map(x =>{
                if(x.ERR_FLAG_PART == 'Y'){
                    x.MSG = msg_err_part
                }else if (x.ERR_FLAG_BP == 'Y'){
                    x.MSG = x.ERR_MSG_BP
                }
            })
        }else{
            if(this.checkStandardCost){
              
              this.tableData.map(x =>{
                if(x.ERR_FLAG_PART == 'Y'){
                    x.MSG = msg_err_stock
                }else if(x.ERR_FLAG_STD == 'Y'){
                    x.MSG = x.ERR_MSG_STD
                }else if(x.ERR_FLAG_PART != 'Y' && x.ERR_FLAG_STD != 'Y'){
                    x.MSG = null
                }
            })
            }else{
              this.tableData.map(x =>{
                      x.MSG = null
              })
            }
        }

        this.tableData = this.tableData.slice();
        this.CheckSelect();
    }


    public CheckErr(){
        return this.tableData.filter(x => x.ERR_FLAG == 'N').length
    }

    public CheckSelect(){
        if(!this.checkBasePrice && !this.checkStandardCost){
            this.selection.clear()
        }
    }

    public DownLoadTemplate(){

        let link =  "assets/templates/SNM105P_Upload_Template.xlsx"
        window.open(link,'_blank')
    }


}
