import { BooleanInput } from "@angular/cdk/coercion";
import { SelectionModel } from "@angular/cdk/collections";
import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { find } from "rxjs/operators";
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from "src/app/core/modal.service";


@Component({
    selector: 'snm113p',
    templateUrl: './SNM113P.component.html',
    styleUrls: ['./SNM113P.component.css']

})


export class SNM113PComponent implements OnInit {

    constructor(
        private http: HttpService,
        private modal : ModalService,
        private viewref : ViewContainerRef
    ) { }

    public ConfigCondition: any = {
        ColName: "LOOKUP",
        ColValue: "",
        PlaceholderText: "Select Condition Key",
        SourceController: "TI",
        SourceMethodName: "SNM113P_CONDITION_KEY",
        NoOfShowRecords: 50,
        MinLengthSearch: 0,
        UpperCase: false
      }

      public ConfigCustomer: any = {
        ColName: "LOOKUP",
        ColValue: "",
        PlaceholderText: "Select Customer Code",
        SourceController: "TI",
        SourceMethodName: "SNM113P_CUSTOMER_CODE",
        NoOfShowRecords: 50,
        MinLengthSearch: 2,
        UpperCase: true
      }

      public ConfigBranch: any = {
        ColName: "LOOKUP",
        ColValue: "",
        PlaceholderText: "Select Brach",
        SourceController: "TI",
        SourceMethodName: "SNM113P_BRANCH_CODE",
        NoOfShowRecords: 100,
        MinLengthSearch: 0,
        SourceParams: {
          CUSTOMER_CODE: null
        }    
      }

    public ConfigPart: any = {
        ColName: "LOOKUP",
        PlaceholderText: "Select Part No.",
        SourceController: "TI",
        SourceMethodName: "SNM113P_GETPART",
        NoOfShowRecords: 30
    }

    public CustomerList: any;
    public tableDataCus: any = [];
    public tableDataPart: any = [];
    public SelectList : any = []
    public Param : any = {};
    public IsSearch : boolean = true;
    public ParamSearch : any = {};
    public tableMain : any = [];
    @ViewChild("dialog",{static: false}) dialog : any;
    public selectionCus = new SelectionModel<any>(true, []);
    public selectionPart = new SelectionModel<any>(true, []);
    public IsAdd : boolean = false;
    public IsEdit : boolean = false;
    public CustomerName : string = ""

    ngOnInit() {

    }

    public Search(){
        this.http.post("TI","SNM113P_SEARCH",this.ParamSearch)
        .subscribe(res =>{
            this.tableMain = res.Table;
        })
    }

    public Add(){
    this.tableDataCus = []
    this.tableDataPart = []

    for(let i = 0; i <= 5; i++){

      this.tableDataCus.push({
          EDIT : 'DEL',
          FLAG : 'N'
      })

      this.tableDataPart.push({
          EDIT : 'DEL',
          FLAG : 'N'
      })
    }

    this.tableDataCus[5].EDIT = 'ADD'
    this.tableDataPart[5].EDIT = 'ADD'
    

    this.IsAdd = true;
    this.IsEdit = false;
    this.Param = {};
   
    this.dialog.OpenDialog();

    
    }

    public EditRow(data){

        this.tableDataCus = []
        this.tableDataPart = []
        this.IsAdd =false;
        this.IsEdit = true;
        
        this.http.post("TI","SNM113P_GETDETAIL",data)
        .subscribe(res =>{
            this.Param.CONDITION_KEY = data.CONDITION_KEY
            this.Param.DESCRIPTION = data.DESCRIPTION
            this.tableDataCus = res.Table
            this.tableDataPart = res.Table1
            this.Param.LAST_UPD_BY = res.Table2[0].LAST_UPD_BY
            this.Param.LAST_UPD_DATE = res.Table2[0].LAST_UPD_DATE

            this.tableDataCus.push({
                EDIT : 'ADD',
                FLAG : 'N'
            })

            this.tableDataPart.push({
                EDIT : 'ADD',
                FLAG : 'N'
            })
            this.dialog.OpenDialog()
            //console.log(res)
        })
    }

    public CheckConditionKey(){

      this.http.post("TI","SNM113P_CHECK_CONDITION_KEY",this.Param)
      .subscribe(res =>{
        console.log(res)
        if(res.STATUS == 'Y'){
          this.modal.warning("Cannot add " + this.Param.CONDITION_KEY + " found duplicate condition key",this.viewref)
          this.Param.CONDITION_KEY = null
        }
      })
    }

    public Delete(data){

      this.modal.confirm("Do you want to delete record!",this.viewref,() => confirm())


      let confirm = () => {

        this.http.post("TI","SNM113P_DELETE",data)
         .subscribe(res =>{

          if(res.STATUS == 'DONE'){
            this.modal.info("Process complete",this.viewref)
          }else{
            this.modal.warning("Found an error, please contact support team", this.viewref);
          }
        },err =>{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        })

      }
      
    }

    public Save(){

        let find = this.tableDataCus.some(x => x.CUSTOMER_CODE && !x.BRANCH_CODE)
        if(find){
          this.modal.warning("Please input branch code",this.viewref)
          return
        }

        this.modal.confirm("Do you want to save change you have made?",this.viewref,this.ConfirmSave)
    }

    private ConfirmSave = () => {

      let Object = {
        CONDITION_KEY : this.Param.CONDITION_KEY,
        DESCRIPTION : this.Param.DESCRIPTION,
        FLAG : this.IsAdd ? 'A' : 'E',
        CustomerList : this.tableDataCus.filter(x => x.CUSTOMER_CODE && x.FLAG == 'N'),
        PartList : this.tableDataPart.filter(x => x.PART_NO && x.FLAG == 'N'),
        DeleteCustomer : this.selectionCus.selected,
        DeletePart : this.selectionPart.selected
      }

      this.http.post("TI","SNM113P_SAVE",Object)
      .subscribe(res => {
       
        if(res.STATUS == 'DONE'){
          this.modal.info("Process complete",this.viewref)
          this.selectionCus.clear()
          this.selectionPart.clear()
          this.Search();
        }else{
          this.modal.warning("Found an error, please contact support team", this.viewref);
        }
      },err => {
        this.modal.warning("Found an error, please contact support team", this.viewref);
      })

    }


    public Cancel(){
        this.dialog.Close();
    }

    public rowClick(event){
      this.CustomerName = event.CUST_NAME_TH
    }

    public SetselectCondition(event){
        this.Param.CONDITION_KEY = event.CONDITION_KEY
        this.Param.DESCRIPTION = event.DESCRIPTION
    }
    
    public SetSourceBranch(data) {
        this.ConfigBranch.SourceParams.CUSTOMER_CODE = data.CUSTOMER_CODE
    
    }

    public SetBranch(data,event){
      let findDup = this.tableDataCus.filter(x => x.CUSTOMER_CODE = data.CUSTOMER_CODE && x.BRANCH_CODE == event.BRANCH_CODE)

      if(findDup.length > 1){
        this.modal.warning("Customer " + data.CUSTOMER_CODE + ' ' + event.BRANCH_CODE + " is duplicate",this.viewref)
        data.BRANCH_CODE = null
        return;
      }

      data.BRANCH_CODE = event.BRANCH_CODE
    }

    public SetPart(data,event){

      let findDup = this.tableDataPart.filter(x => x.PART_NO == event.PART_NO)

      //console.log(findDup)
      if(findDup.length > 1){
        this.modal.warning("Part no " + event.PART_NO + " is duplicate",this.viewref)
        data.PART_NO = null;
        data.PART_NAME_EN = null;
        data.MAIN_MODEL = null
        return;
      }

      data.PART_NO = event.PART_NO;
      data.PART_NAME_EN = event.PART_NAME_EN;
      data.MAIN_MODEL = event.MAIN_MODEL
    }

    public AppendRowCus() {
        for (var i = 0; i < this.tableDataCus.length; i++) {
          if (this.tableDataCus[i].EDIT === "ADD") {
            this.tableDataCus[i].EDIT = 'DEL';
          }
        }
        this.tableDataCus.push({ 'EDIT': 'ADD', 
            CUSTOMER_CODE: null,
            BRANCH_CODE: null, 
            FLAG : 'N'                       
        });
    
        this.tableDataCus = this.tableDataCus.slice();
    
        
      }

      public AppendRowPart() {
        for (var i = 0; i < this.tableDataPart.length; i++) {
          if (this.tableDataPart[i].EDIT === "ADD") {
            this.tableDataPart[i].EDIT = 'DEL';
          }
        }
        this.tableDataPart.push({ 'EDIT': 'ADD', 
            PART_NO: null,
            FLAG : 'N'                       
        });
    
        this.tableDataPart = this.tableDataPart.slice();
    
      }

      public DeleteRowCus(index,data) {

          let indexx = this.tableDataCus.findIndex(x => x == data)
          this.tableDataCus.splice(indexx, 1);
          this.tableDataCus = this.tableDataCus.slice();
     
      }

      public DeleteRowPart(index,data) {

          let indexx = this.tableDataPart.findIndex(x => x == data)
          this.tableDataPart.splice(index, 1);
          this.tableDataPart = this.tableDataPart.slice();
     
      }


    public isAllSelectedCus() {
        const numSelected = this.selectionCus.selected.length;
        const numRows = this.tableDataCus.filter(x => x.ERR_FLAG == 'O').length;
        return numSelected === numRows;
      }
    
    public ToggleCus() {
        this.isAllSelectedCus() ?
          this.selectionCus.clear() :
          this.tableDataCus.forEach(row => {
            if (row.LAG == 'O') {
              this.selectionCus.select(row)
            }
          });
      }

    public isAllSelectedPart() {
        const numSelected = this.selectionPart.selected.length;
        const numRows = this.tableDataPart.filter(x => x.ERR_FLAG == 'O').length;
        return numSelected === numRows;
      }
    
    public TogglePart() {
        this.isAllSelectedPart() ?
          this.selectionPart.clear() :
          this.tableDataPart.forEach(row => {
            if (row.LAG == 'O') {
              this.selectionPart.select(row)
            }
          });
      }

}

