import { Component, ElementRef, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snm131p',
  templateUrl: './SNM131P.component.html',
  styleUrls: ['./SNM131P.component.css']
})



export class SNM131PComponent implements OnInit {


  constructor(
    private http: HttpService,
    private router: Router,
    private modal : ModalService,
    private viewref : ViewContainerRef
  ) { }
  public ConfigDesign: any = {
    ColName: "DESIGN_NOTE_NUMBER",
    ColValue: "DESIGN_NOTE_NUMBER",
    PlaceholderText: "Select Design Note",
    SourceController: "TI",
    SourceMethodName: "SNM131P_DesignNote",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: false
  }

  @ViewChild("dialog",{static: false}) dialog : any;
  public ParamSearch :any ={};
  public Params :any ={};
  public categoryList: any = [];
  public tabledata : any = [];
  public isAdd : boolean = false;
  public CategoryControl = new FormControl('',[Validators.required])
  public FileList: Array<any> = [];
  public FileName: string = "";
  @ViewChild("fileInput") fileInput: ElementRef;
  public FileNameReturn : string = "";
  public CheckUpload : boolean = true;

  ngOnInit() {

  }

  Search(){
    this.tabledata = []
    this.http.post("TI","SNM131P_Search",this.ParamSearch)
    .subscribe(res =>{
      //console.log(res)
          this.tabledata = res.Table
    })

  }

  openFileBrowser() {
    this.FileName = "";
    this.fileInput.nativeElement.value = '';
    this.fileInput.nativeElement.click();
  }

  selectFile(evt) {
    this.CheckUpload = false;
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
        let File: File = Files[0];
        this.FileName = File.name;
        
        let formData = new FormData();
        formData.append('file', File);
        formData.append('DESIGN_NOTE', this.Params.DESIGN_NOTE_NUMBER);

        this.http.postFile("TI", "SNM131P_UploadFile", formData)
            .subscribe(result => {          
                this.modal.info("Upload " + result + " completed.", this.viewref);
                this.Params.FILE_NAME = result;
                this.CheckUpload = true;
            }, err => {
              this.modal.error("Upload not complete", this.viewref);
            })

    }
  }

  Download(data) {
    if (!data.FILE_NAME) {
      this.modal.warning("File not found!", this.viewref)
      return
    }
    this.http.Download('TI', 'SNM131P_Download',{filename : data.FILE_NAME})
        .subscribe(res => {
            if (res) {
              this.writeContents(res,data.FILE_NAME)     
            }              
        },err => {
          this.modal.warning("File not found",this.viewref)
        })
  }

  writeContents(content,filename) {
    
    const a = document.createElement('a');
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = filename;
    a.click();
  }

  Add(){
    this.Params = {}
    this.Params.CAR_MARKET_TYPE = 'C';
    this.Params.CAR_PROD_TYPE = 'O';
    this.Params.ITEM_NO = 0
    this.isAdd = true
    this.FileName = "";
    //this.fileInput.nb Za.value = '';
    this.dialog.title = "Add Design Note"
    this.dialog.OpenDialog();
  }

  EditRow(data){
  
    this.http.post("TI","SNM131P_Search",data)
    .subscribe(res =>{
      if(res.Table.length > 0){
        this.isAdd = false;
        this.Params = res.Table[0];
        this.FileName = this.Params.FILE_NAME
        this.dialog.title = "Edit Design Note"
        this.dialog.OpenDialog();
      }else{
        this.modal.warning("Design Note not found",this.viewref)
      }
    })
  }

  Delete(data){

    this.modal.confirm("Do you want to delete racord?",this.viewref,() =>{
      
      this.http.post("TI","SNM131P_Delete",data)
      .subscribe(res =>{
        
        if (res.STATUS_MSG == 'DONE') {
          this.modal.info('Transaction complete: 1 record applied and deleted.', this.viewref);
          this.Search();
          this.dialog.Close();           
        }
        else if(res.STATUS_MSG == 'IS_USE'){
          this.modal.warning("This Design Note Number is used by another parts,plese remove Design Note Number from parts before delete",this.viewref)
        }
        else {
            this.modal.warning('Delete fail.', this.viewref);
        }
      },err => {
        this.modal.warning('Delete fail.', this.viewref);
      })
    })
  }


  Save(){
    this.modal.confirm("Do you want to save change you have made?",this.viewref,this.ConfirmSave)
  }

  ConfirmSave = () =>{

    if(this.isAdd){
      this.http.post("TI","SNM131P_Insert",this.Params)
      .subscribe(res => {
        if (res.STATUS_MSG == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          if(this.tabledata.length > 0){
            this.Search();
          }
          this.dialog.Close();                                 
        } else if (res.STATUS_MSG == 'DUPPLICATE') {
            this.modal.info('No change to save. Found duplicate record.', this.viewref);
        }
        else {
            this.modal.warning('No change to save. Add fail.', this.viewref);
        }
      },(err)=>{
        this.modal.warning('No change to save. Add fail.', this.viewref);
      })
    }else{
      this.http.post("TI","SNM131P_Update",this.Params)
      .subscribe(res => {
        if (res.STATUS_MSG == 'DONE') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialog.Close();
            
        } else {
            this.modal.warning('No change to save. Update fail.', this.viewref);
        }
      },err => {
        this.modal.warning('No change to save. Update fail.', this.viewref);
      })
    }

  }
  Cancel(){
    this.dialog.Close();
  }
  
  

}
