import { SelectionModel } from '@angular/cdk/collections';
import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { Globals } from 'src/app/core/global';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';


@Component({
    selector: 'snm135p',
    templateUrl: './SNM135P.component.html',
    styleUrls: ['./SNM135P.component.css']
})

export class SNM135PComponent implements OnInit {

    constructor(
        private http: HttpService,
        private modal:ModalService,
        private viewref : ViewContainerRef,
        private global : Globals
    ) { }


    public CustomerList: any;
    public tableData: any = [];
    public files: any = [];
    public FileName: string = ""
    public selection = new SelectionModel<any>(true, []);
    public checkBasePrice : boolean = true;
    public checkStandardCost : boolean = true;
    public checkBasePricePo : boolean = true;
    public checkPricingCode : boolean = true;
    public checkRetailPrice : boolean = true;
    public total_error : number;
    public error_list : any = [];

    public tableDataBatch: any = [];
    public filesBatch: any = [];
    public FileNameBatch: string = ""
    public total_errorBatch : number;
    public selectionBatch = new SelectionModel<any>(true, []);
    
    @ViewChild("dialog",{static: false}) dialog : any;
    ngOnInit() {

    }

    
    fileChange(fileInput) {
        this.selection.clear()
        this.files = fileInput.target.files;
        this.FileName = this.files[0].name;

        this.UpLoad()
    }

    UpLoad(){
        this.tableData = [];
        let formData = new FormData();
        formData.append('file', this.files[0]);

        this.http.postFile('TI', 'SNM135P_UploadExcel', formData)
        .subscribe(res => {
          //console.log('res',res)
          this.total_error = 0;
          this.tableData = res.filter(x => x.PART_NO);
          let countrow = 0;
          this.tableData.map((x,i) => {
              if(x.ERR_FLAG == 'Y'){
                 countrow += 1;
                  this.error_list.push({
                      Records : i + 1
                  })
              } 
        })
        this.total_error = countrow;
    
        },(err) =>{
          this.modal.error("Upload error",this.viewref);
          console.log('err',err)
        })
    }

    public UpdatePrice(){
        this.modal.confirm("Do you want to save change you have made?",this.viewref,this.ConfirmUpdatePrice)
    }

    private ConfirmUpdatePrice = () =>{

        let isBasePrice = this.checkBasePrice == true ? 'Y' : 'N';
        let isStandardCost = this.checkStandardCost == true ? 'Y' : 'N';
        let isRetailPrice = this.checkRetailPrice == true ? 'Y' : 'N';
        let isPricingCode = this.checkPricingCode == true ? 'Y' : 'N';
        let isPOUnitPrice = this.checkBasePricePo == true ? 'Y' : 'N';

        let data  = {
            IS_BASE_PRICE : isBasePrice,
            IS_STANDARD_COST : isStandardCost,
            IS_RETAIL_PRICE : isRetailPrice,
            IS_PRICING_CODE : isPricingCode,
            IS_PO_UNIT_PRICE : isPOUnitPrice,
            DETAIL : this.selection.selected
        }
        this.http.post("TI","SNM135P_UpdatePrice",data)
        .subscribe(res => {
            if(res.STATUS == 'DONE'){
                this.modal.info('Transaction complete: ' +  this.selection.selected.length  +' record applied and saved.', this.viewref);
                this.selection.clear();
            }else{
                this.modal.error('No change to save. Something went wrong.', this.viewref);
            }
        },(err)=>{
            this.modal.error('No change to save. Something went wrong.', this.viewref);
        })
    }


    public DownLoad(){
        if(this.tableData.length == 0){
            this.modal.warning("No data for Export",this.viewref)
            return
        }

      if (this.checkBasePricePo != true) {
        this.http.Download("TI","SNM135P_Download",this.tableData)
        .subscribe(res =>{
            this.WriteContents(res)
        })
      } else {
        this.http.Download("TI", "SNM135P_Download", this.tableData)
          .subscribe(res => {
            this.WriteContents(res)
          })
        this.http.Download("TI", "SNM135P_Download_PO", this.tableData)
          .subscribe(res => {
            this.WriteContentsPO(res)
          })
      }  
    }


    WriteContents(content) {

        const a = document.createElement('a');
        const file = new Blob([content], { type: content.type });
        a.href = URL.createObjectURL(file);
        a.download = "SNM135P" + '_' + this.global.CurrentDate() + ".csv";
        a.click();
    }
    WriteContentsPO(content) {
        const a = document.createElement('a');
        const file = new Blob([content], { type: content.type });
        a.href = URL.createObjectURL(file); 
        a.download = "SNM135P_PO" + '_' + this.global.CurrentDate() + ".csv";
        a.click();
      }

    isAllSelected() {
        const numSelected = this.selection.selected.length;
        const numRows = this.tableData.filter(x => x.ERR_FLAG == 'N').length;
        return numSelected === numRows;
      }
    
      Toggle() {
        this.isAllSelected() ?
          this.selection.clear() :
          this.tableData.forEach(row => {
            if (row.ERR_FLAG == 'N') {
              this.selection.select(row)
            }
          });
      }

      public STDChange(event){
          if(this.tableData.length == 0)return;

          let msg_err_stock = 'Does not exist on @TPM_STOCK_MASTER!'
          let msg_err_part = 'Does not exist on @TPM_PARTS_MASTER!'

          if(event.checked){
              this.tableData.map(x =>{
                  if(x.ERR_FLAG_PART == 'Y'){
                      x.MSG = msg_err_stock
                  }
              })
          }else{
              if(this.checkBasePrice){                
                this.tableData.map(x =>{
                    if(x.ERR_FLAG_PART == 'Y'){
                        x.MSG = msg_err_part
                    }
                })
              }else{
                this.tableData.map(x =>{
                    if(x.ERR_FLAG_PART == 'Y'){
                        x.MSG = ""
                    }
                })
              }
          }

          this.tableData = this.tableData.slice();
          //this.CheckSelect();
      }

      public BPChange(event){
        if(this.tableData.length == 0)return;
        
        let msg_err_stock = 'Does not exist on @TPM_STOCK_MASTER!'
        let msg_err_part = 'Does not exist on @TPM_PARTS_MASTER!'
          
        if(event.checked && !this.checkStandardCost){
            this.tableData.map(x =>{
                if(x.ERR_FLAG_PART == 'Y'){
                    x.MSG = msg_err_part
                }
            })
        }else{
            if(this.checkStandardCost){
              
              this.tableData.map(x =>{
                  if(x.ERR_FLAG_PART == 'Y'){
                      x.MSG = msg_err_stock
                  }
              })
            }else{
              this.tableData.map(x =>{
                  if(x.ERR_FLAG_PART == 'Y'){
                      x.MSG = ""
                  }
              })
            }
        }

        this.tableData = this.tableData.slice();
        //this.CheckSelect();
    }


    private CheckErr(){
        return this.tableData.filter(x => x.ERR_FLAG == 'N').length
    }

    private CheckSelect(){
        if(!this.checkBasePrice && !this.checkStandardCost){
            this.selection.clear()
        }
    }

    public DownLoadTemplate(type){

        let link =  ""

        if(type == 1 ){
            link =  "assets/templates/SNM135P_Upload_Template.xlsx"
        }else{
            link =  "assets/templates/SNM135P_Upload_Template_Batch.xlsx"
        }

        window.open(link,'_blank')
    }


    fileChangeBatch(fileInput) {
        this.selectionBatch.clear()
        this.filesBatch = fileInput.target.files;
        this.FileNameBatch = this.filesBatch[0].name;

        this.UpLoadBatch()
    }

    UpLoadBatch(){
        this.total_errorBatch = 0;
        this.tableDataBatch = [];
        let formData = new FormData();
        formData.append('file', this.filesBatch[0]);

        this.http.postFile('TI', 'SNM135P_UploadExcel', formData)
        .subscribe(res => {
          //console.log('res',res)
          this.total_error = 0;
          this.tableDataBatch = res.filter(x => x.PART_NO);
          let countrow = 0;
          this.tableDataBatch.map((x,i) => {
              if(x.ERR_FLAG == 'Y'){
                 countrow += 1;
                  this.error_list.push({
                      Records : i + 1
                  })
              } 
        })
        this.total_errorBatch = countrow;
    
        },(err) =>{
          this.modal.error("Upload error",this.viewref);
          console.log('err',err)
        })
    }


    public SaveData(){
        this.modal.confirm("Do you want to save",this.viewref,()=>ConfirmSave())
        let ConfirmSave =()=>{
        this.http.post("TI","SNM135P_SaveDataForBatch",this.selectionBatch.selected)
        .subscribe(res =>{
            this.modal.info("Process complete",this.viewref)
                this.selectionBatch.clear()
        },err => {
            this.modal.error("Process not complete",this.viewref)
        })
        }
    }



    isAllSelectedBatch() {
        const numSelected = this.selectionBatch.selected.length;
        const numRows = this.tableDataBatch.filter(x => x.ERR_FLAG == 'N').length;
        return numSelected === numRows;
      }
    
      ToggleBatch() {
        this.isAllSelectedBatch() ?
          this.selectionBatch.clear() :
          this.tableDataBatch.forEach(row => {
            if (row.ERR_FLAG == 'N') {
              this.selectionBatch.select(row)
            }
          });
      }


}
