import {
  Component,
  ElementRef,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { Router } from "@angular/router";
import { FormControl, Validators } from "@angular/forms";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "SNM142P",
  templateUrl: "./SNM142P.component.html",
  styleUrls: ["./SNM142P.component.css"],
})
export class SNM142PComponent implements OnInit {
  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) {}

  public ConfigPart: any = {
    ColName: "LOOKUP",
    PlaceholderText: "Select Part No.",
    SourceController: "TI",
    SourceMethodName: "SNM142P_GetPart",
    NoOfShowRecords: 30,
  };

  public ConfigPartKIT: any = {
    ColName: "LOOKUP",
    PlaceholderText: "Select Part No.",
    SourceController: "TI",
    SourceMethodName: "SNM142P_GetPartKit",
    NoOfShowRecords: 30,
  };

  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialogMultiPart", { static: false }) dialogMultiPart: any;
  public paramSearch: any = {};
  public params: any = {};
  public categoryList: any = [];
  public tabledata: any = [];
  public isAdd: boolean = false;
  public categoryControl = new FormControl("", [Validators.required]);
  public tableDataDetail: any = [];
  public tableDataMultipart: any = [];
  public tempData: any = [];
  public fileName: string = "";
  public fileNameMultipart: string = "";
  private File: any;
  @ViewChild("fileInput") fileInput: ElementRef;
  @ViewChild("fileInputMultipart") fileInputMultipart: ElementRef;

  ngOnInit() {}

  public Search() {
    this.tabledata = [];
    this.http
      .post("TI", "SNM142P_SearchHeader", this.paramSearch)
      .subscribe((res) => {
        //console.log(res)
        this.tabledata = res.Table;
      });
  }

  public Download() {
    if (!this.params.FILE_NAME) {
      this.modal.warning("File not found!", this.viewref);
      return;
    }

    this.http
      .Download("TI", "SNM130P_Download", { fileName: this.params.FILE_NAME })
      .subscribe((res) => {
        if (res) {
          this.writeContents(res, this.params.FILE_NAME);
        }
      });
  }

  private writeContents(content, fileName) {
    const a = document.createElement("a");
    const file = new Blob([content], { type: content.type });
    a.href = URL.createObjectURL(file);
    a.download = fileName;
    a.click();
  }

  public DownloadTemplate(template: number) {
    let link = "";
    if (template == 1) {
      link = "assets/templates/Accessories_Kit_Multipart_Template.xlsx";
    }
    window.open(link, "_blank");
  }

  public openFileBrowser() {
    this.fileName = "";
    this.fileInput.nativeElement.value = "";
    this.fileInput.nativeElement.click();
  }

  public selectFile(evt) {
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
      let File: File = Files[0];
      this.fileName = File.name;

      let formData = new FormData();
      formData.append("file", File);

      this.http.postFile("TI", "SNM142P_UploadFile", formData).subscribe(
        (result) => {
          this.modal.info("Upload complete.", this.viewref);

          if (
            this.tableDataDetail.filter((x) => x.PART_COMPONENT_NO).length > 0
          ) {
            let newobject = [];
            this.tableDataDetail = this.tableDataDetail.filter(
              (x) => x.PART_COMPONENT_NO
            );

            result.forEach((x) => {
              let find = this.tableDataDetail.find(
                (n) => n.PART_COMPONENT_NO == x.PART_COMPONENT_NO
              );

              if (find == undefined) {
                if (x.MSG == "OK") {
                  this.tableDataDetail.push(x);
                  this.tempData.push(x);
                }
              }
            });
            //Array.prototype.push.apply(this.tableDataDetail,newobject);

            this.tableDataDetail.push({
              EDIT: "ADD",
              PART_COMPONENT_NO: null,
              COMPONENT_NAME_EN: null,
              COMPONENT_NAME_TH: null,
            });
          } else {
            result = result.filter((x) => x.MSG == "OK");
            this.tableDataDetail = result;
            result.forEach((x) => {
              this.tempData.push(x);
            });
            this.tableDataDetail.push({
              EDIT: "ADD",
              PART_COMPONENT_NO: null,
              COMPONENT_NAME_EN: null,
              COMPONENT_NAME_TH: null,
            });
          }
        },
        (err) => {
          this.modal.error("Upload not complete.", this.viewref);
        }
      );
    }
  }

  public Add() {
    this.tableDataDetail = [];
    this.params = {};
    this.isAdd = true;
    this.dialog.title = "Add New Accessories Kit";

    this.tableDataDetail.push({
      EDIT: "ADD",
      PART_COMPONENT_NO: null,
      COMPONENT_NAME_EN: null,
      COMPONENT_NAME_TH: null,
    });

    this.dialog.OpenDialog();
  }

  public EditRow(datarow) {
    this.http
      .post("TI", "SNM142P_SearchHeader", { PART_NO: datarow.PART_KIT_NO })
      .subscribe((res) => {
        if (res.Table.length > 0) {
          this.isAdd = false;
          this.params = res.Table[0];
          this.SearchDetail(this.params);
          this.dialog.title = "Edit Accessories Kit";
          this.dialog.OpenDialog();
        } else {
          this.modal.warning("Accessories Kit not found.", this.viewref);
        }
      });
  }

  public SearchDetail(data) {
    this.tableDataDetail = [];
    this.http.post("TI", "SNM142P_SearchDetail", data).subscribe((res) => {
      this.tableDataDetail = res.Table;
      this.tableDataDetail.push({
        EDIT: "ADD",
        PART_COMPONENT_NO: null,
        COMPONENT_NAME_EN: null,
        COMPONENT_NAME_TH: null,
      });
    });
  }

  public SelectPart(event) {
    if (event) {
      this.params.PART_KIT_NO = event.PART_NO;
      this.params.PART_NAME_EN = event.PART_NAME_EN;
      this.params.PART_NAME_TH = event.PART_NAME_TH;
    } else {
      this.params.PART_KIT_NO = null;
      this.params.PART_NAME_EN = null;
      this.params.PART_NAME_TH = null;
    }
  }

  public CancelPart(event) {
    if (this.params.PART_KIT_NO) {
      this.http
        .get("TI", "SNM142P_GetPart", this.params.PART_KIT_NO, "20")
        .subscribe((res) => {
          if (res.Table.length > 0) {
            this.params.PART_KIT_NO = res.Table[0].PART_NO;
            this.params.PART_NAME_EN = res.Table[0].PART_NAME_EN;
            this.params.PART_NAME_TH = res.Table[0].PART_NAME_TH;
          } else {
            this.params.PART_KIT_NO = null;
            this.params.PART_NAME_EN = null;
            this.params.PART_NAME_TH = null;
          }
        });
    } else {
      this.params.PART_KIT_NO = null;
      this.params.PART_NAME_EN = null;
      this.params.PART_NAME_TH = null;
    }
  }

  public SelectKitPart(event, data, index) {
    if (event) {
      if (event.PART_NO == this.params.PART_KIT_NO) {
        this.modal.warning(
          "Component part can not be like Kit part",
          this.viewref
        );
        data.PART_COMPONENT_NO = null;
        data.COMPONENT_NAME_EN = null;
        data.COMPONENT_NAME_TH = null;
        data.ITEM_QTY = null;
        data.STOCK_TYPE = null;
        data.REMARKS = null;
        return;
      }

      let duppart = this.tableDataDetail.filter(
        (x, i) => x.PART_COMPONENT_NO == event.PART_NO && i != index
      );
      if (duppart.length > 0) {
        this.modal.warning(
          "Part No" + " - " + event.PART_NO + " is dupplicate.",
          this.viewref
        );
        data.PART_COMPONENT_NO = null;
        data.COMPONENT_NAME_EN = null;
        data.COMPONENT_NAME_TH = null;
        data.ITEM_QTY = null;
        data.STOCK_TYPE = null;
        data.REMARKS = null;
        return;
      }

      data.PART_COMPONENT_NO = event.PART_NO;
      data.COMPONENT_NAME_EN = event.PART_NAME_EN;
      data.COMPONENT_NAME_TH = event.PART_NAME_TH;

      if (!this.isAdd) {
        let ind = this.tempData.findIndex(
          (x) =>
            x.PART_COMPONENT_NO == data.PART_COMPONENT_NO && data.FLAG == "D"
        );
        if (ind > -1) {
          this.tempData.splice(ind, 1);
        }

        data.FLAG = "A";
        this.tempData.push(data);
      }
    } else {
      data.PART_COMPONENT_NO = data.PART_COMPONENT_NO.toUpperCase();
      let duppart = this.tableDataDetail.filter(
        (x, i) => x.PART_COMPONENT_NO == event.PART_NO && i != index
      );
      if (duppart.length > 0) {
        this.modal.warning(
          "Part No" + " - " + event.PART_NO + " is dupplicate",
          this.viewref
        );
        data.PART_COMPONENT_NO = null;
        data.COMPONENT_NAME_EN = null;
        data.COMPONENT_NAME_TH = null;
        data.ITEM_QTY = null;
        data.STOCK_TYPE = null;
        data.REMARKS = null;
        return;
      }
      if (!this.isAdd) {
        let ind = this.tempData.findIndex(
          (x) =>
            x.PART_COMPONENT_NO == data.PART_COMPONENT_NO && data.FLAG == "D"
        );
        if (ind > -1) {
          this.tempData.splice(ind, 1);
        }

        data.FLAG = "A";
        this.tempData.push(data);
      }
    }
  }

  public CancelCallback(event, data, index) {
    data.PART_COMPONENT_NO = null;
    data.COMPONENT_NAME_EN = null;
    data.COMPONENT_NAME_TH = null;
    data.ITEM_QTY = null;
    data.STOCK_TYPE = null;
    data.REMARKS = null;
  }

  public CheckQty(event, data) {
    if (event.target.valueAsNumber < 1) {
      this.modal.warning("Quantity must not be less than 1", this.viewref);
      data.ITEM_QTY = 1;
    }
    //console.log(event.target.valueAsNumber)
  }

  public CheckRowData() {
    let filter = this.tableDataDetail.filter((x) => x.PART_COMPONENT_NO);
    let valid = filter.length > 0 ? true : false;

    return valid;
  }

  public Delete(data) {
    this.modal.confirm("Do you want to delete record?", this.viewref, () => {
      this.http.post("TI", "SNM142P_Delete", data).subscribe((res) => {
        if (res.STATUS_MSG == "DONE") {
          this.modal.info(
            "Transaction complete: 1 record applied and deleted.",
            this.viewref
          );
          this.Search();
          this.dialog.Close();
        } else {
          this.modal.warning("Delete fail.", this.viewref);
        }
      });
    });
  }

  public Save() {
    if (this.tableDataDetail.some((x) => x.PART_COMPONENT_NO == this.params.PART_KIT_NO)) {
      this.modal.warning(
        "Component part can not be like Kit part.",
        this.viewref
      );
      return;
    }

    if (this.tableDataDetail.some((x) => x.PART_COMPONENT_NO && (x.ITEM_QTY < 1 || !x.ITEM_QTY))) {
      this.modal.warning("Quantity must not be less than 1 digit.", this.viewref);
      return;
    }

    if (this.tableDataDetail.some((x) => x.PART_COMPONENT_NO && (!x.STOCK_TYPE || x.STOCK_TYPE === null || x.STOCK_TYPE.match(/^ *$/) !== null))) {
      this.modal.warning("Stock type must not be null.", this.viewref);
      return;
    }

    if (this.tableDataDetail.some((x) => x.PART_COMPONENT_NO && !x.COMPONENT_NAME_EN && !x.REMARKS)) {
      this.modal.warning(
        "Please write remarks for parts is not register in parts master.",
        this.viewref
      );
      return;
    }

    this.modal.confirm(
      "Do you want to save change you have made?",
      this.viewref,
      this.ConfirmSave
    );
  }

  private ConfirmSave = () => {
    if (this.isAdd) {
      this.params.Detail = this.tableDataDetail.filter(
        (x) => x.PART_COMPONENT_NO && x.ITEM_QTY
      );

      this.http.post("TI", "SNM142P_Insert", this.params).subscribe(
        (res) => {
          if (res.STATUS_MSG == "DONE") {
            this.modal.info(
              "Transaction complete: applied and saved.",
              this.viewref
            );
            if (this.tabledata.length > 0) {
              this.Search();
            }
            this.dialog.Close();
          } else if (res.STATUS_MSG == "DUPLICATE") {
            this.modal.info(
              "No change to save. Found duplicate record.",
              this.viewref
            );
          } else {
            this.modal.warning("No change to save. Add fail.", this.viewref);
          }
        },
        (err) => {
          this.modal.warning("No change to save. Add fail.", this.viewref);
        }
      );
    } else {

      this.tableDataDetail.map((x) => {
        if (x.FLAG == "U") {
          this.tempData.push(x);
        }
      });

      this.params.Detail = this.tempData;

      this.http.post("TI", "SNM142P_Update", this.params).subscribe(
        (res) => {
          if (res.STATUS_MSG == "DONE") {
            this.tempData = [];
            this.modal.info(
              "Transaction complete: applied and saved.",
              this.viewref
            );
            this.Search();
            this.dialog.Close();
          } else {
            this.modal.error("No change to save. Update fail.", this.viewref);
          }
        },
        (err) => {
          this.modal.error("No change to save. Update fail.", this.viewref);
        }
      );
    }
  };
  public Cancel() {
    this.dialog.Close();
  }

  public AppendRow() {
    for (var i = 0; i < this.tableDataDetail.length; i++) {
      if (this.tableDataDetail[i].EDIT === "ADD") {
        this.tableDataDetail[i].EDIT = "DEL";
      }
    }
    this.tableDataDetail.push({
      EDIT: "ADD",
      PART_COMPONENT_NO: null,
      COMPONENT_NAME_EN: null,
      COMPONENT_NAME_TH: null,
    });

    this.tableDataDetail = this.tableDataDetail.slice();
  }
  public DeleteRow(index, data) {
    if (!this.isAdd) {
      let ind = this.tempData.findIndex(
        (x) => x.PART_KIT_NO == data.PART_KIT_NO && data.FLAG == "A"
      );
      if (ind > -1) {
        this.tempData.splice(ind, 1);
      }
      data.FLAG = "D";
      this.tempData.push(data);
    }

    //let index = this.tableDataDetail.findIndex(x => x == Item)
    this.tableDataDetail.splice(index, 1);
    this.tableDataDetail = this.tableDataDetail.slice();
  }

  public UploadMultipart() {
    this.tableDataMultipart = [];
    this.fileNameMultipart = "";
    this.dialogMultiPart.title = "Add New Accessories Kit";
    this.dialogMultiPart.OpenDialog();
  }

  public SaveMultipart() {
    if (this.tableDataMultipart.some((x) => x.MSG)) {
      this.modal.warning(
        "This data have error, Please recheck data",
        this.viewref
      );
      return;
    }

    this.modal.confirm("Do you want to save", this.viewref, () =>
      confirmSave()
    );

    let confirmSave = () => {
      this.http
        .post("TI", "SNM142P_SaveMultipart", this.tableDataMultipart)
        .subscribe(
          (res) => {
            if (res.STATUS_MSG == "DONE") {
              this.modal.info(
                "Transaction complete: applied and saved.",
                this.viewref
              );
              this.Search();
              this.dialogMultiPart.Close();
            } else {
              this.modal.error("No change to save. fail.", this.viewref);
            }
          },
          (err) => {
            this.modal.error("No change to save. fail.", this.viewref);
          }
        );
    };
  }

  public OpenFileBrowserMultipart() {
    this.fileNameMultipart = "";
    this.fileInputMultipart.nativeElement.value = "";
    this.fileInputMultipart.nativeElement.click();
  }

  public SelectFileMultipart(evt) {
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
      //let File: File = Files[0];
      this.File = Files[0];
      this.fileNameMultipart = this.File.name;
    }
  }

  public PreView() {
    if (!this.fileNameMultipart) {
      this.modal.warning("Please Open File First...", this.viewref);
      return;
    }

    let formData = new FormData();
    formData.append("file", this.File);
    this.tableDataMultipart = [];
    this.http.postFile("TI", "SNM142P_UploadMultiParts", formData).subscribe(
      (result) => {
        this.modal.info("Upload complete", this.viewref);

        this.tableDataMultipart = result.filter((x) => x.PART_KIT_NO);

        this.tableDataMultipart.map((data, index) => {
          let find = this.tableDataMultipart.some(
            (x, i) =>
              x.PART_KIT_NO == data.PART_KIT_NO &&
              x.PART_COMPONENT_NO == data.PART_COMPONENT_NO &&
              i != index
          );
          if (find) {
            if (data.MSG) {
              data.MSG =
                data.MSG +
                " , " +
                data.PART_COMPONENT_NO +
                " duplicate in file";
            } else {
              data.MSG = data.PART_COMPONENT_NO + " duplicate in file";
            }
          }
        });
      },
      (err) => {
        this.modal.error("Upload not complete.", this.viewref);
      }
    );
  }
}
