import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { ElementRef } from '@angular/core';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snse145',
  templateUrl: './snse145.component.html',
  styleUrls: ['./snse145.component.css']
})



export class SNSE145Component implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewref : ViewContainerRef
  ) { 

  }

  public data: any;
  public FileName: string = "";
  public FileNameReturn: string;
  public DataUploadList: Array<any> = [];
  public FileList: Array<any> = [];
  public File : any ;
  public LogList : any = [];
  public StartDate : any;
  @ViewChild("fileUpload") fileUpload: ElementRef;
  @ViewChild("dialog",{static: false}) dialog : any;
  ngOnInit() {

    

  }

  public OpenFileBrowser() {
    this.FileName = "";
    this.fileUpload.nativeElement.value = '';
    this.fileUpload.nativeElement.click();
  }

  private GetData(FileName) {
      this.http.get("TI", "SNSE145_GetDetailByFileName", FileName)
          .subscribe(result => {
              this.DataUploadList = result.Table;
              this.CreateLog([]);
          }, err => {
              console.error(err);
          })
  }

  public SelectFile(evt) {
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
        //let File: File = Files[0];
        this.File = Files[0];
        this.FileName = this.File.name;
    }
  }

  public PreView(){


    if(!this.FileName){
        this.modal.warning("Please Open TSV File First...",this.viewref)
        return;
    }
    
    let formData = new FormData();
    formData.append('file', this.File);
    this.StartDate = +new Date;

    
     this.http.postFile("TI", "SNSE145_UploadFile", formData)
      .subscribe(result => {
        
        if(result == "DUPLICATE"){
          this.modal.warning("This file is duplicate please rename before upload",this.viewref)
        }else{
          this.FileNameReturn = result;
          this.GetData(this.FileNameReturn);
          this.modal.info("Upload " + this.FileName + " completed.", this.viewref);   
        }
         
    }, err => {
        this.modal.error("Found eror, please contact support team.", this.viewref);
    })

    
    
  }

  public TransferData() {

    this.modal.confirm("Do you want to save change you have made?",this.viewref,() => ConfirmTranfer())


    let ConfirmTranfer = () => {
      this.http.post("TI", ["SNSE145_TransferData", this.FileNameReturn], {})
      .subscribe(result => {
          this.modal.info("Transfer data completed.", this.viewref);
          this.GetProcessLog()
          this.DataUploadList = [];
          //this.FileNameReturn = null;
          this.FileName = null;
          this.fileUpload.nativeElement.value = '';
          
      }, err => {
          this.modal.error("Unable to Transfer data.", this.viewref);
          console.error(err);
      })
    }
   
  }

  private GetProcessLog(){
    this.http.post("TI","SNSE145_GetProcessLog",{file_name : this.FileNameReturn})
    .subscribe(res => {
      this.CreateLog(res.Table)
    })
  }


  private CreateLog(array : Array<any>){

    if(array.length > 0){
      this.LogList.push({
        LOG_DESCRIPTION : null
      })

      array.forEach(x =>{
        this.LogList.push({
          LOG_DESCRIPTION : x.LOG_DESCRIPTION
        })
      })

    }else{
      //console.log('vvvvv',this.DataUploadList)
      let row_complete = this.DataUploadList.filter(x => x.NP_STATUS == 'OK').length
      let row_error = this.DataUploadList.filter(x => x.NP_STATUS != 'OK').length
      let messageLog = 'Preview TSV start on '+this.SetFormateDate(this.StartDate);

      if(this.LogList.length > 0){
        this.LogList.push({
          LOG_DESCRIPTION : null
        })
      }
  
      this.LogList.push({
        LOG_DESCRIPTION : messageLog
      },
      {
        LOG_DESCRIPTION : 'Load complete '+ row_complete+' records...'
      },
      {
        LOG_DESCRIPTION : 'Load error '+ row_error+' records...'
      },
      {
        LOG_DESCRIPTION : 'Preview TSV stop on '+this.SetFormateDate(+new Date)
      })
    }
  }



  public ShowLog(){
    this.dialog.OpenDialog();
  }

  private SetFormateDate(d_date) {

    let d = new Date(d_date);
    let Day = d.getDate().toString().length < 2 ? '0' + d.getDate().toString() : d.getDate().toString();
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    let hours = d.getHours();
    let minute = d.getMinutes().toString().length < 2 ? '0' + d.getMinutes().toString() : d.getMinutes().toString();
    let seconds = d.getSeconds().toString();

    return Day + '/' + NewMonth + '/' + Year + ' ' + hours + ':' + minute + ':' + seconds;
  }


}
