import { Component, ElementRef, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';
import { ActivatedRoute } from '@angular/router';
import { SelectionModel } from "@angular/cdk/collections";


@Component({
  selector: 'snse197_m',
  templateUrl: './SNSE197_M.component.html',
  styleUrls: ['./SNSE197_M.component.css']
})

export class SNSE197_MComponent implements OnInit {
  private data: Array<any> = [];
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private route: ActivatedRoute,
    private global: Globals
  ) {
  }

  public ConfigPartNo: any = {
    ColName: "LOOKUP",
    ColValue : '',
    PlaceholderText: "Select Part No",
    SourceController: "TI",
    SourceMethodName: "SNSE197_M_GetPartNo",
    NoOfShowRecords: 20,
    MinLengthSearch: 0,
    UpperCase: true
  }


  public CustomerList: any;
  public tableData: any = [];
  public tableDataUpload: any = [];
  public tableData1: any = [{}];
  public SelectList: any = [];
  private files: any = [];
  public FileName: string = ""
  public totalUpload: number = 0;
  public totalError: number = 0;
  public DataUploadList: any;
  public File : any ;
  public errorMsg : string = "";

  public paramPart: any = {}
  @ViewChild("fileUpload") fileUpload: ElementRef;
  public selection = new SelectionModel<any>(true, []);
  ngOnInit() {
    
  }

  public SetPartNo(values: any) {

    if(values){
      this.paramPart.PART_NO = values.PART_NO;
      this.paramPart.PART_NAME = values.PART_NAME;
      this.paramPart.VENDOR_CODE   = values.VENDOR_CODE

    }else{
      this.paramPart.PART_NO = null;
      this.paramPart.PART_NAME = null;
      this.paramPart.VENDOR_CODE = null
    }

  
  }

  public CancelPartNo(){
      if(this.paramPart.PART_NO){
        this.http.get("TI","SNSE197_M_GetPartNo",this.paramPart.PART_NO,'20')
        .subscribe(res =>{
          if(res.Table.length > 0){
                this.paramPart.PART_NO = res.Table[0].PART_NO;
                this.paramPart.PART_NAME = res.Table[0].PART_NAME;
                this.paramPart.VENDOR_CODE   = res.Table[0].VENDOR_CODE
          }else{
            this.paramPart.PART_NO = null;
            this.paramPart.PART_NAME = null;
            this.paramPart.VENDOR_CODE = null;
          }
        })
      }
  }

  public TabChanged(event){

    console.log(event)
    this.paramPart = {}
    this.tableData =[]
    this.tableDataUpload = []
    this.FileName = ""
    this.selection.clear()
    this.errorMsg = ""

  }

  public GetActiveList(){
    this.http.post("TI","SNSE197_M_GetactiveList",this.paramPart)
    .subscribe(res =>{
      this.tableData = res.Table
    })
  }



  public Disable(data){
    this.modal.confirm("Are you sure to disable this NET PRICE?",this.viewref,() => confirmDisable())

    let confirmDisable = () =>{
        this.http.post("TI","SNSE197_M_DisableNetPrice",data)
        .subscribe(res => {
          this.modal.info("Net Price disabled completed!",this.viewref);
          this.GetActiveList();
        },err =>{
          this.modal.error("Found eror, please contact support team.", this.viewref);
        })
    }
  }

  public ValidateSaveNetPrice() {
   
    if (!this.paramPart.PART_NO){
      this.modal.warning('Please input part no.!',this.viewref);
      return;
    }

    if(!this.paramPart.VENDOR_CODE){
      this.modal.warning("Please input vendor by selecting part no.!",this.viewref)
      return;
    }

    if(!this.paramPart.NET_PRICE1){
      this.modal.warning("Please input Net Price 1!",this.viewref)
      return;
    }

    if(!this.paramPart.NET_PRICE2){
      this.modal.warning("Please input Net Price 2!",this.viewref)
      return;
    }

    if(!this.paramPart.EFFECTIVE_DATE){
      this.modal.warning("Please input effective date!",this.viewref)
      return;
    }

    if(!this.paramPart.DISABLE_DATE){
      this.modal.warning("Please input disable date!",this.viewref)
      return;
    }

    this.http.post("TI","SNSE197_M_CheckOverlap",this.paramPart)
    .subscribe(res =>{

      if(res.STATUS_MSG == 'FAIL'){
        var splitMsg = res.ERROR_MSG.split(",");
        this.modal.warning(splitMsg[0]+'<br>'+splitMsg[1]+'<br>'+splitMsg[2]+'<br>'+splitMsg[3]+'<br>'+splitMsg[4]+'<br>'+splitMsg[5],this.viewref,() =>{},{width : '600px',height:'400px'})
      }else{
        this.modal.confirm("Do you want to save changed you have made?",this.viewref,this.SaveByPart)
      }
 
    })
  }


  public SaveByPart = () => {
    this.http.post("TI","SNSE197_M_SaveByPart",this.paramPart)
    .subscribe(res =>{
      if(res.STATUS == 'DONE'){
        this.modal.info("Net Price setting completed!",this.viewref)
      }else{
        this.modal.error("Found eror, please contact support team.", this.viewref);
      }
          
    },err => {
      this.modal.error("Found eror, please contact support team.", this.viewref);
    })
  }


  public SaveByUpload (){

    if(this.selection.selected.length == 0){
      this.modal.warning("No record seleted!",this.viewref)
      return;
    }

    this.modal.confirm("Do you want to upload selected Net Price?",this.viewref,() => ConfirmSave())


    let ConfirmSave = () =>{
        this.http.post("TI","SNSE197_M_SaveByUpload",this.selection.selected)
        .subscribe(res =>{
          console.log(res)
          if(res.STATUS_MSG == 'DONE'){
            this.modal.info("Upload Net Price completed!",this.viewref)
            this.selection.clear();
            this.DataUploadList = [];
            this.FileName ='';
          }else{
            this.modal.error("Found eror, please contact support team.", this.viewref);
          }
        },err => {
          this.modal.error("Found eror, please contact support team.", this.viewref);
        })
    }

  }

  public DownLoadTemplate(){
      let link = 'assets/templates/Upload_Netprice_Templete.xlsx'
      window.open(link,'_blank')
   
  }


  public OpenFileBrowser() {
    this.FileName = "";
    this.fileUpload.nativeElement.value = '';
    this.fileUpload.nativeElement.click();
  }

  public SelectFile(evt) {
    let Files: Array<File> = evt.target.files;

    if (Files.length > 0) {
        //let File: File = Files[0];
        this.File = Files[0];
        this.FileName = this.File.name;
    }
  }

  public PreView(){


    if(!this.FileName){
        this.modal.warning("Please Select File First...",this.viewref)
        return;
    }
    
    let formData = new FormData();
    formData.append('file', this.File);

     this.http.postFile("TI", "SNSE197_M_UploadFile", formData)
      .subscribe(result => {

        this.tableDataUpload = result.filter(x => x.PART_NO);
           
    }, err => {
        this.modal.error("Found eror, please contact support team.", this.viewref);
    })
  }
  

  CheckNumber(data){
    let isnumber : boolean = false;
    if(typeof data == 'number'){
      isnumber = true;
    }

    return isnumber;
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.tableDataUpload.filter(x => x.STATUS == 'OK').length;
    return numSelected === numRows;
  }

  Toggle() {
    this.isAllSelected() ?
      this.selection.clear() :
      this.tableDataUpload.forEach(row => {
        if (row.STATUS == 'OK') {
          this.selection.select(row)
        }
      });
  }

  RowClick(event){
    this.errorMsg = event.ERROR_MSG
  }

  GetErrorRecord(){
    return this.tableDataUpload.filter(x => x.STATUS != "OK").length
  }

  
}


