import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from '../../../core/modal.service';
import { Globals } from '../../../core/global';

@Component({
  selector: 'snm025p',
  templateUrl: './SNM025P.component.html',
  styleUrls: ['./SNM025P.component.css']
})

export class SNM025PComponent implements OnInit {

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) { }
  @ViewChild("dialogMain", { static: false }) dialogMain: any;
  @ViewChild("dialog", { static: false }) dialog: any;

  public data: any = [];
  public tableData: any = [];
  public params: any = {};
  public pageMode: string = "view";
  public rackData: any = {};
  public valid: boolean = false;
  public mainDatatable: any;

  public ConfigRack: any = {
    ColName: "RACK_CODE",
    ColValue: "RACK_CODE",
    PlaceholderText: "Select Rate",
    SourceController: "WH",
    SourceMethodName: "SNM025P_GetRack",
    NoOfShowRecords: 30,
    MinLengthSearch: 1,
    UpperCase: false
  }
 
  ngOnInit() {
    this.InitData();
  }

  public GetRackS(code) {
    this.params.RACK_CODE = code;
  }

  private InitData() {
    this.rackData = {};
  }

  public SearchForAc(){
    this.http.get('WH', 'SNM025P_GetRack_List', this.params.RACK_CODE)
      .subscribe(res => {
        this.mainDatatable = res.Table;
      }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public AddNew() {
    this.pageMode = "add";
    this.InitData();
    this.dialogMain.OpenDialog();
  }

  public DeleteRow(data) {
    this.rackData.RACK_CODE = data.RACK_CODE;
    this.modal.confirm("Do you want to delete this record?", this.viewref, this.ConfirmDeleteRow);
  }

  private ConfirmDeleteRow = () => {
    this.rackData.LAST_IP_ADDR = this.global.GetTokenDetail("ssIPAddress");
    this.http.post('WH', 'SNM025P_delete_Rack', this.rackData)
      .subscribe(res => {
        if (res.Table[0].STATUS === "DONE") {
          this.modal.info('Transaction complete: 1 record deleted.', this.viewref);
          this.SearchForAc();
        }
      },(err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public EditRow(data) {
    //this.InitData();
    this.pageMode = "edit";
       
    this.http.get("WH", "SNM025P_GetRack_Info", data.RACK_CODE)
      .subscribe(result => {
        if (result.Table.length > 0) {
          this.rackData = result.Table[0];
          this.dialogMain.OpenDialog();
          this.SearchForAc();
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public GetRackDate(RackCode) {
    this.rackData.RACK_CODE = RackCode;
    this.http.get("WH", "SNM025P_GetRack_Info", RackCode)
      .subscribe(result => {
        this.rackData = result.Table[0];
        this.dialog.Close();
        this.SearchForAc();
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public SaveAdd() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
  }

  private ConfirmSaveAdd = () => {
    this.rackData.LAST_IP_ADDR = this.global.GetTokenDetail("ssIPAddress");
    this.http.post('WH', 'SNM025P_Rack_Save', this.rackData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'DONE') {
          this.InitData();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.dialogMain.Close();
          this.SearchForAc();
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public CancelAdd() {   
    this.dialogMain.Close();
  }

  public openDialog() {
    this.tableData = [];
    this.http.get("WH", "SNM025P_GetRack_Info", "@@@")
      .subscribe(result => {
        this.tableData = result.Table;
        this.dialog.OpenDialog()
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public editBtnClick(data) {
    this.GetRackDate(data.RACK_CODE) 
  }

  public Cancel() {
    this.dialog.Close();
  }

}

