import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { ModalService } from '../../../core/modal.service';
import { FormControl, Validators } from '@angular/forms';

@Component({
    selector: 'snm026p',
    templateUrl: './SNM026P.component.html',
    styleUrls: ['./SNM026P.component.css']
})

export class SNM026PComponent implements OnInit {

    constructor(
        private http: HttpService,
        private router: Router,
        private modal: ModalService,
        private viewref: ViewContainerRef
  
    ) { }
  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("dialog2", { static: false }) dialog2: any;

  public data: any = [];
  public opened: boolean;
  public params: any = {};
  public categoryList: any = [];
  public addData: any = {
    RACK_CODE: null,
    PART_CATEGORY: null,
    RACK_GROUP_CODE: null,
  }

  public editData: any = {
    RACK_CODE: null,
    PART_CATEGORY: null,
    RACK_GROUP_CODE: null,
  }

  public RackCodeControl = new FormControl('', [Validators.required])
  public PartCatControl = new FormControl('', [Validators.required])

  public  ConfigRack: any = {
    ColName: "RACK_CODE",
    ColValue: "RACK_CODE",
    PlaceholderText: "Select Rack",
    SourceController: "WH",
    SourceMethodName: "SNM026P_GetRack",
    NoOfShowRecords: 30,
    MinLengthSearch : 1,
    UpperCase : false
  }

  ngOnInit() {
    this.GetCat();
  }

  public GetCat() {
    this.http.get('WH', 'SNM026P_GetPartCategory')
      .subscribe(res => {
        this.categoryList = res.Table
      }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contact support team", this.viewref);
      })
  }

  public openDialog() {
    this.dialog.OpenDialog()
  }

  public Search() {
    this.http.post("WH", "SNM026P_GetData", this.params)
      .subscribe(res => {
        this.data = res.Table
      }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contact support team", this.viewref);
      });
  }

  public SaveAdd() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveAdd)
  }

  private ConfirmSaveAdd = () => {
    this.http.post('WH', 'SNM026P_Insert', this.addData)
      .subscribe(res => {
        console.log(res)
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialog.Close();
          this.ClearData();
        }
        else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.info('No change to save. Found duplicate record.', this.viewref);
        }
        else {
          this.modal.warning(res.Table[0].MESSAGE, this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });             
  }

  public CancelAdd() {
    this.dialog.Close();
  }

  public EditRow(data) {
    this.editData.RACK_CODE = data.RACK_CODE
    this.editData.PART_CATEGORY = data.PART_CATEGORY
    this.editData.RACK_GROUP_CODE = data.RACK_GROUP_CODE

    this.dialog2.OpenDialog();
  }

  public SaveEdit() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSaveEdit)
  }

  private ConfirmSaveEdit = () => {
    this.http.post('WH', 'SNM026P_Update', this.editData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
          this.Search();
          this.dialog2.Close();
        }
        else {
          this.modal.warning('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewref);
      });  
    }

  public Delete(data) {
    this.modal.confirm('Do you want to delete record?', this.viewref, () =>{
      this.http.post('WH', 'SNM026P_Delete', data)
        .subscribe(res => {
          if (res.Table[0].STATUS == 'Done') {
            this.modal.info('Transaction complete: applied and saved.', this.viewref);
            this.Search();
          } else {
            this.modal.warning('No change to save. Something went wrong.', this.viewref);
          }
        }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contact support team", this.viewref);
        });
    })
  }

  public CancelEdit() {
    this.dialog2.Close();
  }

  public ClearData(){
    this.addData.RACK_CODE = null
    this.addData.RACK_GROUP_CODE = null
    this.addData.PART_CATEGORY = null  
  }

  public RackCodeErrorMessage(){
    return  this.RackCodeControl.hasError('required') ? 'Field must be entered' :''    
  }

  public PartCatErrorMessage(){
    return  this.PartCatControl.hasError('required') ? 'Field must be entered' :''
  }

    


}

