import { Component, OnInit, ViewChild, ViewContainerRef, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snm102p',
  templateUrl: './SNM102P.component.html',
  styleUrls: ['./SNM102P.component.css']
})


export class SNM102PComponent implements OnInit {

  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }

  @ViewChild('fileUpload', { static: false }) fileUpload: ElementRef;
  public tableData: any;
  public FileName: string = '';
  public files: any;
  public isSaved: any = false;

  ngOnInit() {

  }

  public OnFileSelect(fileInput: any) {
    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;
  }

  public UpLoad() {
    if (this.files.length == 0) {
      this.modal.error('Please Open CSV File First...', this.viewref)
    } else {
      this.tableData = [];
      let formData = new FormData();
      formData.append('file', this.files[0]);

      this.http.postFile('WH', 'SNM102P_UploadandGet', formData)
        .subscribe(res => {
          this.tableData = res.Table;
          this.isSaved = false;
        })
    }
  }

  public Save() {
    this.modal.confirm('Do you want to save changed you have made?', this.viewref, this.ConfirmSave)
  }

  public ConfirmSave = () => {
    let data = [];
    data = this.tableData.filter(x => x.RESULT == 'OK')
    if (data.length > 0) {
      this.http.post("WH", "SNM102P_Update", data)
        .subscribe(res => {
          if (res.Table[0].STATUS == 'DONE') {
            this.modal.info('Transaction complete: ' + data.length + ' record applied and saved.', this.viewref);
            this.isSaved = true;
          } else {
            this.modal.warning('No change to save. Something went wrong.', this.viewref);
          }
        })
    }
    else {
      this.modal.info("No data", this.viewref)
    }
  }

  public Clear() {
    this.tableData = []
    this.FileName = ''
    this.fileUpload.nativeElement.value = null;
  }
}

