import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "../../../core/modal.service";
import { Globals } from "src/app/core/global";
import { FormControl, Validators } from "@angular/forms";

@Component({
  selector: "SNM141",
  templateUrl: "./SNM141.component.html",
  styleUrls: ["./SNM141.component.css"],
})
export class SNM141Component implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) {}

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialogEdit", { static: false }) dialogEdit: any;

  SetPartNoControl = new FormControl("", [Validators.required]);

  public lookupConfigPart: any = {
    ColName: "LOOKUP_PART_NO_VALUE",
    PlaceholderText: "Select Part",
    SourceController: "wPMS",
    SourceMethodName: "SNM137P_GET_PART_NO_NON_DUP",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
  };

  public tabledata: any = [];
  public paramSearch: any = {};
  public paramsAdd: any = {};
  public paramsEdit: any = {};

  ngOnInit() {
    this.Search();
  }

  SetSelectedPart(value: any) {
    if (value) {
      this.paramSearch.PART_NO = value.PART_NO;
    } else {
      this.paramSearch.PART_NO = "";
    }
  }

  SetSelectedPartAdd(value: any) {
    if (value) {
      this.paramsAdd.PART_NO = value.PART_NO;
      this.paramsAdd.PART_NAME_EN = value.PART_NAME_EN;
      this.paramsAdd.PART_NAME_TH = value.PART_NAME_TH;
    } else {
      this.paramsAdd.PART_NO = "";
      this.paramsAdd.PART_NAME_EN = "";
      this.paramsAdd.PART_NAME_TH = "";
    }
  }

  Search() {
    this.tabledata = [];
    this.http
      .post("WH", "SNM141_Get_Data", this.paramSearch)
      .subscribe((res) => {
        this.tabledata = res.Table;
      });
  }

  openDialog() {
    this.paramsAdd = {};
    this.paramsAdd.PACKAGE_QTY = 0;
    this.dialogAdd.OpenDialog();
  }

  SaveAdd() {
    this.modal.confirm(
      "Do you want to save changed you have made?",
      this.viewref,
      this.ConfirmSaveAdd
    );
  }

  ConfirmSaveAdd = () => {
    if (this.paramsAdd.PACKAGE_QTY < 1) {
      this.modal.warning("Package QTY must be more than 1.", this.viewref);
    } else {
      this.http.post("WH", "SNM141_Insert", this.paramsAdd).subscribe(
        (res) => {
          if (res.Table[0].STATUS == "Done") {
            this.modal.info(
              "Transaction complete: applied and saved.",
              this.viewref
            );
            this.paramSearch = {};
            this.Search();
            this.dialogAdd.Close();
          } else if (res.Table[0].STATUS == "Duplicate") {
            this.modal.warning(
              "No change to save. Found duplicate record.",
              this.viewref
            );
          } else {
            this.modal.error(
              "No change to save. Something went wrong.",
              this.viewref
            );
          }
        },
        (err) => {
          this.modal.warning(
            "Found an error, please contract support team",
            this.viewref
          );
        }
      );
    }
  };

  CancelAdd() {
    this.dialogAdd.Close();
  }

  EditRow(data) {
    this.paramsEdit = { ...data };
    this.dialogEdit.OpenDialog();
  }

  SaveEdit() {
    this.modal.confirm(
      "Do you want to save changed you have made?",
      this.viewref,
      this.ConfirmSaveEdit
    );
  }

  ConfirmSaveEdit = () => {
    if (this.paramsEdit.PACKAGE_QTY < 1) {
      this.modal.warning("Package QTY must be more than 1.", this.viewref);
    } else {
      this.http
        .post("WH", "SNM141_Update", this.paramsEdit)
        .subscribe((res) => {
          if (res.Table[0].STATUS == "Done") {
            this.modal.info(
              "Transaction complete: applied and saved.",
              this.viewref
            );
            this.Search();
            this.dialogEdit.Close();
          } else {
            this.modal.warning(
              "Found an error, please contract support team",
              this.viewref
            );
          }
        });
    }
  };

  CancelEdit() {
    this.dialogEdit.Close();
  }
}
