import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "../../../core/modal.service";
import { Globals } from "src/app/core/global";
import { FormControl, Validators } from "@angular/forms";

@Component({
  selector: "SNM142",
  templateUrl: "./SNM142.component.html",
  styleUrls: ["./SNM142.component.css"],
})
export class SNM142Component implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals
  ) {}

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;

  SetPartNoControl = new FormControl("", [Validators.required]);

  public lookupConfigPart: any = {
    ColName: "LOOKUP_PART_NO_VALUE",
    PlaceholderText: "Select Part",
    SourceController: "wPMS",
    SourceMethodName: "SNM137P_GET_PART_NO_NON_DUP",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
  };

  public tabledata: any = [];
  public paramSearch: any = {};
  public paramsAdd: any = {};
  public paramsEdit: any = {};

  ngOnInit() {
    this.Search();
  }

  SetSelectedPart(value: any) {
    if (value) {
      this.paramSearch.PART_NO = value.PART_NO;
    } else {
      this.paramSearch.PART_NO = "";
    }
  }

  SetSelectedPartAdd(value: any) {
    if (value) {
      this.paramsAdd.PART_NO = value.PART_NO;
      this.paramsAdd.PART_NAME_EN = value.PART_NAME_EN;
      this.paramsAdd.PART_NAME_TH = value.PART_NAME_TH;
    } else {
      this.paramsAdd.PART_NO = "";
      this.paramsAdd.PART_NAME_EN = "";
      this.paramsAdd.PART_NAME_TH = "";
    }
  }

  Search() {
    this.tabledata = [];
    this.http
      .post("WH", "SNM142_Get_Data", this.paramSearch)
      .subscribe((res) => {
        this.tabledata = res.Table;
      });
  }

  openDialog() {    
    this.paramsAdd = {};
    this.dialogAdd.OpenDialog();
  }

  SaveAdd() {
    this.modal.confirm(
      "Do you want to save changed you have made?",
      this.viewref,
      this.ConfirmSaveAdd
    );
  }

  ConfirmSaveAdd = () => {
    this.http.post("WH", "SNM142_Insert", this.paramsAdd).subscribe(
      (res) => {
        if (res.Table[0].STATUS == "Done") {
          this.modal.info(
            "Transaction complete: applied and saved.",
            this.viewref
          );    
          this.paramSearch = {};          
          this.Search();          
          this.dialogAdd.Close();
        } else if (res.Table[0].STATUS == "Duplicate") {
          this.modal.warning(
            "No change to save. Found duplicate record.",
            this.viewref
          );
        } else {
          this.modal.error(
            "No change to save. Something went wrong.",
            this.viewref
          );
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team",
          this.viewref
        );
      }
    );
  };

  CancelAdd() {
    this.dialogAdd.Close();
  }

  DeleteRow(data: any) {
    this.modal.confirm("Do you want to delete this record?", this.viewref, () =>
      this.ConfirmDelete(data)
    );
  }

  private ConfirmDelete(data) {
    this.http.post("WH", "SNM142_Delete", data).subscribe(
      (res) => {
        if (res.Table[0].STATUS == "Done") {
          this.modal.info("Delete record complete", this.viewref);
          this.Search();
        } else {
          this.modal.error(
            "Found an error, please contact support team",
            this.viewref
          );
        }
      },
      (err) => {
        this.modal.error(
          "Found an error, please contact support team",
          this.viewref
        );
      }
    );
  }
}
