import {
  Component,
  OnInit,
  ViewChild,
  ViewContainerRef,
  ElementRef,
  HostListener,
  ViewChildren,
  QueryList,
} from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { debounceTime, delay, map, take, tap } from "rxjs/operators";
import { Globals } from "src/app/core/global";
import * as moment from "moment";
import { pipe, Subject } from "rxjs";
import { SelectionModel } from "@angular/cdk/collections";

@Component({
  selector: "snpe015_P",
  templateUrl: "./SNPE015_P.component.html",
  styleUrls: ["../warehouse.component.css"],
})
export class SNPE015_PComponent implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private el: ElementRef,
    private global: Globals
  ) {}

  @ViewChild("selectPODialog", { static: true }) selectPODialog: any;
  @ViewChild("selectPOForPendingDialog", { static: true })
  selectPOForPendingDialog: any;
  @ViewChild("pendingDialog", { static: true }) pendingDialog: any;
  @ViewChildren("acceptPartNoInput") partInputs: QueryList<ElementRef>;
  @ViewChildren("qtyInput") qtyInput: QueryList<ElementRef>;
  private changeLineSubject: Subject<boolean> = new Subject<boolean>();

  private currentDate: moment.Moment = moment();
  private matchedPO: any[] = [];
  private duplicateCheck: Subject<boolean> = new Subject<boolean>();
  public invoiceHeader: any = {};
  public invoiceDetail: any[] = [];
  public paramsSearch: any = {};
  public invoiceList: any[] = [];
  public ShowList: boolean = false;
  public invoiceType: string;
  public hideColumnDetail: string[] = [];
  public mtPoClass: any = [];
  public pendingList = new SelectionModel<any>(true, []);
  public poNotFound: boolean = false;
  private currentElement: any;
  private currentPartNo: string;
  private currentRow: any;
  private pendingDataForPODialog: any;

  public tblFooter: any = [
    {
      fieldName: "PART_NAME",
      value: "Total",
      show: true,
    },

    {
      fieldName: "STICKER_UNIT",

      show: true,
    },
    {
      fieldName: "ORDER_QTY",

      show: true,
    },
    {
      fieldName: "QTY",

      show: true,
    },
    {
      fieldName: "NET_AMT_B",

      show: true,
    },
  ];

  public vendorLookupConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetVendorCode",
    NoOfShowRecords: 50,
    UpperCase: true,
    MinLengthSearch: 1,
    SourceParams: {
      INV_TYPE: "L_PO",
    },
  };

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 20,
  };

  public ConfigInvoiceNo: any = {
    ColName: "INVOICE_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetInvList",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      INVOICE_STATUS: "P",
    },
  };

  ngOnInit() {
    this.InitAdd();
    this.GetPOClass();

    this.duplicateCheck
      .pipe(debounceTime(200))
      .subscribe(
        (v) =>
          this.invoiceHeader.VENDOR_CODE &&
          this.invoiceHeader.INVOICE_NO &&
          this.CheckDuplicate()
      );
  }

  private InitAdd(invType?: string) {
    invType = invType || "L_PO";

    this.invoiceHeader = {
      VENDOR_CODE: null,
      INVOICE_NO: null,
      INVOICE_DATE: this.currentDate,
      INVOICE_STATUS: "N",
      INVOICE_STATUS_DESC: "New",
      INVOICE_TYPE: invType,
    };

    this.invoiceDetail = [];
    this.pendingList.clear();
    this.matchedPO = [];
    this.CreateDetailRow(0);

    this.invoiceType = invType;
    this.vendorLookupConfig.SourceParams.INV_TYPE = invType;

    this.SetHideColumns(invType);
  }

  private GetPOClass() {
    this.http
      .get("System", "QueryData", "SP_NP_MT_PO_CLASS_LIST")
      .subscribe((result) => {
        this.mtPoClass = result.Table;
      });
  }

  public Search() {
    this.invoiceList = [];
    this.http.post("WH", "SNPE015_P_GetData", this.paramsSearch).subscribe(
      (res) => {
        this.invoiceList = res.Table;
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  public ViewDetail(invoiceData) {
    this.invoiceDetail = [];
    this.pendingList.clear();

    this.ShowList = false;
    this.http.post("WH", "SNPE015_P_GetInvoiceDetail", invoiceData).subscribe(
      (res) => {
        this.invoiceHeader = res.Table[0] || {};
        this.invoiceDetail = res.Table1;

        if (this.invoiceHeader.INVOICE_NO) {
          this.invoiceHeader.VENDOR = this.invoiceHeader.VENDOR_CODE;
          this.invoiceType = this.invoiceHeader.INVOICE_TYPE;
          this.SetHideColumns(this.invoiceType, ["CHECK"]);

          let pendingList = this.invoiceDetail
            .filter((item) => item.PENDING_TYPE)
            .map((item) => {
              item._SELECT_PO = item.PO_NO ? false : true;

              return item;
            });
          this.pendingList = new SelectionModel<any>(true, pendingList);

          //if(this.invoiceType.substr(-2) == 'WO') {
          //  this.invoiceHeader.ORDER_CLASS = this.invoiceDetail[0].ORDER_CLASS;
          //  this.invoiceHeader.ORDER_TYPE = this.invoiceDetail[0].ORDER_TYPE;
          //}
        }

        this.invoiceDetail.forEach((v) => {
          v.ACCEPT_PART_NO = v.PART_NO;
          v._PART_NO = v.PART_NO;
        });
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  public Add() {
    this.ShowList = false;
    this.InitAdd();
  }

  public Save() {
    var data = Object.assign({}, this.invoiceHeader);
    data.PART_LIST = this.invoiceDetail
      .slice()
      .filter((v) => v.ACCEPT_PART_NO)
      .map((v) => {
        if (this.invoiceHeader.INVOICE_TYPE.substr(-2) === "WO") {
          v.ORDER_CLASS = this.invoiceHeader.ORDER_CLASS;
          v.ORDER_TYPE = this.invoiceHeader.ORDER_TYPE;
        }
        return v;
      });

    let findLargeLotStickers = data.PART_LIST.find((v) => v.STICKER_UNIT > 200);
    if (findLargeLotStickers) {
      this.modal.warning("Please input correct Receive pck.", this.viewRef);
      return;
    }

    let findPendingWOType = this.pendingList.selected.find(
      (item) => !item.PENDING_TYPE
    );
    if (findPendingWOType) {
      this.modal.warning("Please choose pending type.", this.viewRef, () => {
        this.pendingDialog.OpenDialog();
      });
      return;
    }

    this.modal.confirm(
      "Are you sure to confirm pre-receive",
      this.viewRef,
      () => {
        this.http.post("WH", "SNPE015_P_Save", data).subscribe(
          (res) => {
            if (res.Result == "Done") {
              this.invoiceHeader.DOCUMENT_NO = res.DocNo;
              this.modal.info(
                "Transaction complete: applied and saved.",
                this.viewRef
              );
              this.ViewDetail(this.invoiceHeader);
            } else if (res.Result == "Duplicate invoice") {
              this.modal.warning(
                "Duplicate invoice : " + this.invoiceHeader.INVOICE_NO,
                this.viewRef
              );
            } else {
              this.modal.warning(
                "Found an error, please contact support team.",
                this.viewRef
              );
            }
          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
      },
      () => {}
    );
  }

  public CheckDuplicate() {
    this.http
      .post("WH", "SNPE015_P_CheckDuplicateInvoice", {
        VENDOR_CODE: this.invoiceHeader.VENDOR_CODE,
        INVOICE_NO: this.invoiceHeader.INVOICE_NO,
        INVOICE_DATE: this.invoiceHeader.INVOICE_DATE,
      })
      .subscribe((res) => {
        if (res !== "Done") {
          this.modal.warning("Duplicate invoice.", this.viewRef);
          this.InitAdd(this.invoiceHeader.INVOICE_TYPE);
        }
      });
  }

  public InvoiceDateChange() {
    this.duplicateCheck.next(true);
  }

  public CheckValidSave() {
    var validItems = this.invoiceDetail.filter((d, i) =>
      this.CheckRequiredFields(i, false)
    );
    var filledItems = this.invoiceDetail.filter((v) => v.ACCEPT_PART_NO);
    return (
      this.invoiceHeader.INVOICE_STATUS === "N" &&
      validItems.length === filledItems.length &&
      validItems.length > 0
    );
  }

  public CheckRequiredFields(tblIndex, allowUnChange: boolean = true) {
    let valid: boolean =
      this.invoiceHeader.VENDOR_CODE &&
      this.invoiceHeader.INVOICE_NO &&
      this.invoiceHeader.INVOICE_DATE
        ? true
        : false;

    if (valid) {
      let thisRow = this.invoiceDetail[tblIndex];
      let countChangeItems: number = 0;

      if (thisRow.ACCEPT_PART_NO) countChangeItems++;
      if (thisRow.QTY) countChangeItems++;

      valid =
        (countChangeItems > 0 && thisRow.ACCEPT_PART_NO && thisRow.QTY) ||
        (allowUnChange && countChangeItems === 0)
          ? true
          : false;
    }

    return valid;
  }

  public CheckActiveRow(tblIndex) {
    let valid: boolean =
      this.invoiceHeader.VENDOR_CODE &&
      this.invoiceHeader.INVOICE_NO &&
      this.invoiceHeader.INVOICE_DATE
        ? true
        : false;

    if (valid) {
      valid =
        !this.invoiceHeader.INVOICE_STATUS ||
        this.invoiceHeader.INVOICE_STATUS === "N";
    }

    if (valid && tblIndex > 0) {
      let prevRow = this.invoiceDetail[tblIndex - 1];

      if (!prevRow.ACCEPT_PART_NO) valid = false;
      if (!prevRow.QTY) valid = false;
      // if (
      //   this.invoiceHeader.INVOICE_TYPE.substr(-2) === "PO" &&
      //   prevRow.PENDING_TYPE !== "T_PO" &&
      //   !prevRow.PO_NO
      // )
      //   valid = false;
    }

    if (valid) {
      this.CreateDetailRow(tblIndex);
    }

    return valid;
  }

  public CreateDetailRow(tblIndex) {
    let currentMaxRow: number = this.invoiceDetail.length || 0;
    let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

    if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.invoiceDetail.push({});
      }

      this.invoiceDetail = this.invoiceDetail.slice();
    }
  }

  public DeleteRow(index) {
    this.modal.confirm(
      "Are you sure to remove this row?",
      this.viewRef,
      () => {
        const row = this.invoiceDetail[index];
        this.pendingList.isSelected(row) && this.pendingList.deselect(row);

        this.invoiceDetail.splice(index, 1);
        this.invoiceDetail = [...this.invoiceDetail];
      },
      () => {}
    );
  }

  public async CheckPO(event, elemID, index, data) {
    this.currentElement = this.el.nativeElement.querySelector("#" + elemID);
    this.currentPartNo = event.PART_NO;
    this.currentRow = data;

    //With P/O
    if (
      this.invoiceHeader.INVOICE_TYPE.substr(-2) === "PO" &&
      !this.poNotFound
    ) {
      await this.GetWithPO(
        this.invoiceHeader.INVOICE_NO,
        this.invoiceHeader.VENDOR_CODE,
        event.PART_NO
      ).then(
        (res) => {
          this.matchedPO = res.Table;
          if (this.matchedPO.length > 0) {
            if (
              res.Table.find((v) => v.LOCATION_NO && v.LOCATION_NO.length > 0)
            ) {
              if (this.matchedPO.length === 1) {
                this.SetPOMatchOne(
                  data,
                  index,
                  this.matchedPO[0],
                  this.currentElement
                );
              } else if (this.matchedPO.length > 1) {
                this.matchedPO.forEach(
                  (d) => (d.CHECK = this.CheckIfExists(d))
                );
                this.selectPODialog.OpenDialog();
              }
            } else {
              data._PART_NO = null;
              this.currentElement.focus();
              this.modal.error(
                "Location hasn't configured for this part.",
                this.viewRef
              );
            }
          } else {
            console.log("isselected, ", this.pendingList.isSelected(data));
            data._PART_NO = null;
            this.currentElement.focus();
            this.modal.error("Unable to find matched PO.", this.viewRef);
          }
        },
        (err) => {
          this.modal.error(err.message, this.viewRef);
        }
      );
    } else if (this.poNotFound) {
      await this.SetWithoutPO({ PENDING_TYPE: "T_PO" });
      this.poNotFound = false;
    } else {
      await this.SetWithoutPO();
    }
  }

  private async GetWithPO(
    invoiceNo: string,
    vendorCode: string,
    partNo: string
  ) {
    return await this.http
      .post("WH", "SNPE015_P_CheckPOByPartNo", {
        VENDOR_CODE: vendorCode,
        INVOICE_NO: invoiceNo,
        PART_NO: partNo,
      })
      .toPromise();
  }

  private async SetWithoutPO(additionalProp?: any) {
    await this.http
      .post("WH", "SNPE015_P_CheckWOPartNo", {
        VENDOR_CODE: this.invoiceHeader.VENDOR_CODE,
        PART_NO: this.currentPartNo,
      })
      .toPromise()
      .then((res) => {
        this.matchedPO = res.Table;

        if (this.matchedPO.length === 1) {
          if (
            res.Table.find((v) => v.LOCATION_NO && v.LOCATION_NO.length > 0)
          ) {
            this.currentRow = Object.assign(
              this.currentRow,
              this.SetPartRow(this.currentRow, this.matchedPO[0]),
              additionalProp
            );
            if (this.poNotFound) this.pendingList.select(this.currentRow);
          } else {
            this.currentRow._PART_NO = null;
            this.currentElement.focus();
            this.modal.error(
              "Location hasn't configured for this part.",
              this.viewRef
            );
          }
        } else {
          this.currentRow._PART_NO = null;
          this.currentElement.focus();
          this.modal.error("Unable to find part no.", this.viewRef);
        }

        this.changeLineSubject.next(true);
      });
  }

  public async AcceptSelectedPO() {
    if (this.poNotFound) {
      await this.SetWithoutPO({ PENDING_TYPE: "T_PO" });
      this.poNotFound = false;
    } else {
      let ActiveRow = this.invoiceDetail.filter((d, i) =>
        this.CheckRequiredFields(i, false)
      );
      let SelectedRow = this.matchedPO.filter((d) => {
        const found = ActiveRow.find(
          (v) =>
            v.ACCEPT_PART_NO === d.ACCEPT_PART_NO &&
            v.PO_NO === d.PO_NO &&
            v.ITEM_NO === d.ITEM_NO &&
            v.SUB_ITEM_NO === d.SUB_ITEM_NO
        );

        return d.CHECK && !found;
      });

      if (SelectedRow.length > 0) {
        SelectedRow = SelectedRow.map((d) => this.SetPartRow({}, d));
        this.invoiceDetail = ActiveRow.concat(SelectedRow);
      }
    }
    this.invoiceDetail = this.invoiceDetail.slice();
    this.selectPODialog.Close();
  }

  public SetPOMatchOne(
    data: any,
    index: number,
    result: any,
    currentElement: any
  ) {
    let foundDuplicate = this.invoiceDetail.find((d, i) => {
      return (
        index !== i &&
        result.ACCEPT_PART_NO === this.invoiceDetail[i].ACCEPT_PART_NO &&
        result.PO_NO === this.invoiceDetail[i].PO_NO &&
        result.ITEM_NO === this.invoiceDetail[i].ITEM_NO &&
        result.SUB_ITEM_NO === this.invoiceDetail[i].SUB_ITEM_NO
      );
    });

    if (foundDuplicate) {
      data._PART_NO = null;
      currentElement.focus();
      this.modal.error("Unable to insert duplicated record.", this.viewRef);
      return;
    }

    // data.CHECK = true;
    data = Object.assign(data, this.SetPartRow(data, result));
    this.changeLineSubject.next(true);
  }

  private SetPartRow(row: any, data: any) {
    row = data;
    row._PART_NO = data.ACCEPT_PART_NO;
    return row;
  }

  //------------------------------------- Add mode ------------------------------------------------------
  public SetVendorCodeSearch(value) {
    this.paramsSearch.VENDOR_CODE = value.VENDOR_CODE;
  }

  public SetVendorCodeLookup(value) {
    if (value.VENDOR_CODE) {
      this.invoiceHeader.VENDOR = value.VENDOR_CODE;
      this.invoiceHeader.VENDOR_CODE = value.VENDOR_CODE;
      this.invoiceHeader.VENDOR_NAME = value.VENDOR_NAME_EN;

      // this.duplicateCheck.next(true)
    } else {
      this.invoiceHeader.VENDOR = null;
      this.invoiceHeader.VENDOR_CODE = null;
      this.el.nativeElement.querySelector("#vendorcode").focus();
      this.modal.error("Unable to find vendor.", this.viewRef);
    }
  }

  public SetInvoice() {
    this.invoiceHeader.INVOICE_NO = this.invoiceHeader.INVOICE_NO.toUpperCase();
    this.duplicateCheck.next(true);
  }

  public QuantityChange(data) {
    if (data.QTY > data.ACCEPT_QTY) {
      data.QTY = data.ACCEPT_QTY;
    }

    if (data.RECEIVE_HH_ON !== "Y") {
      data.RECEIVE_PACKAGE = data.QTY;
    } else if (data.PENDING_TYPE) {
      data.RECEIVE_PACKAGE = data.QTY;
    } else if (
      this.invoiceHeader.INVOICE_TYPE.substring(-2) === "WO" &&
      data.RECEIVE_STICKER_1_1_ON === "Y" &&
      data.RECEIVE_PACKAGE === 0
    ) {
      data.RECEIVE_PACKAGE = data.QTY;
    }

    if (data.QTY % data.RECEIVE_PACKAGE > 0) {
      this.modal.warning(
        "Receive QTY not matched by Receive package.",
        this.viewRef
      );
    }

    data.STICKER_UNIT = Math.ceil(data.QTY / data.RECEIVE_PACKAGE);
  }

  public InvoiceTypeChange(select) {
    if (this.invoiceHeader.VENDOR_CODE) {
      this.modal.confirm(
        "All change will be reset.<br/>Are you sure to change invoice type?",
        this.viewRef,
        () => this.InitAdd(select.value),
        () => (this.invoiceType = this.vendorLookupConfig.SourceParams.INV_TYPE)
      );
    } else {
      this.InitAdd(select.value);
    }
  }

  public CheckIfExists(data) {
    return this.invoiceDetail.find((d) => {
      return (
        d.ACCEPT_PART_NO === data.ACCEPT_PART_NO &&
        d.PO_NO === data.PO_NO &&
        d.ITEM_NO === data.ITEM_NO &&
        d.SUB_ITEM_NO === data.SUB_ITEM_NO
      );
    });
  }

  public CancelPartNo(data: any) {
    data._PART_NO = data.ACCEPT_PART_NO;
  }

  public TotalQty() {
    return this.invoiceDetail.reduce((a, b) => a + (b.QTY || 0), 0);
  }

  public TotalAmount() {
    return this.invoiceDetail.reduce(
      (a, b) => a + (b.QTY || 0) * (b.UNIT_PRICE_B || 0),
      0
    );
  }

  public TotalItem() {
    return this.invoiceDetail.reduce(
      (a, b) => a + ((b.QTY || 0) / b.RECEIVE_PACKAGE || 0),
      0
    );
  }

  public GetMatchedPO() {
    return this.matchedPO;
  }

  public BackToSearch() {
    this.Search();
    this.ShowList = true;
  }

  public OpenReport_Pre() {
    //this.global.GoToReport("PMS_RPT_WMS_SNPE015",
    //  {
    //    P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
    //    P_DOCUMENT_NO: this.invoiceHeader.DOCUMENT_NO
    //  }
    //);
    this.global.GoToReport("PMS_RPT_WMS_SNPR020_v3", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_DOCUMENT_NO: this.invoiceHeader.DOCUMENT_NO,
    });
  }

  public OpenReport_PreNew() {
    this.global.GoToReport("PMS_RPT_WMS_SNPE015", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_DOCUMENT_NO: this.invoiceHeader.DOCUMENT_NO,
    });
  }

  private SetHideColumns(invType: string, fixCols?: string[]) {
    let hideCols = [];
    if (invType.substring(-2) === "WO") {
      hideCols = ["CHECK_2", "PO_NO", "PO_DATE", "ITEM_NO_SHOW", "ORDER_QTY"];
    }
    if (this.invoiceHeader.INVOICE_STATUS !== "N") {
      hideCols = hideCols.concat(["CHECK_2"]);
    }
    this.hideColumnDetail = hideCols.concat(fixCols || []);
  }

  public OrderClassChanged(selected) {
    switch (parseInt(selected.value)) {
      case 1:
      case 2:
        this.invoiceHeader.ORDER_TYPE = "1";
        break;
      case 3:
      case 4:
        this.invoiceHeader.ORDER_TYPE = "3";
        break;
      case 5:
        this.invoiceHeader.ORDER_TYPE = "4";
        break;
      case 7:
        this.invoiceHeader.ORDER_TYPE = "6";
        break;
    }
  }

  public onPartInputEnter(event: any, index: number) {
    const nextIndex = index + 1;
    const nextPartInput = this.partInputs.toArray()[nextIndex];
    const currentQtyInput = this.qtyInput.toArray()[index];

    const subject = this.changeLineSubject
      .pipe(
        take(1),
        delay(300),
        map((v) => v)
      )
      .subscribe((v) => {
        if (nextPartInput.nativeElement.disabled)
          currentQtyInput.nativeElement.focus();

        if (!nextPartInput.nativeElement.disabled)
          nextPartInput.nativeElement.focus();
        subject.unsubscribe();
      });

    if (this.changeLineSubject) {
      this.changeLineSubject.next(true);
    }
  }

  public onQtyInputEnter(event: any, index: number) {
    const nextIndex = index + 1;
    const nextPartInput = this.partInputs.toArray()[nextIndex];

    setTimeout(() => {
      if (!nextPartInput.nativeElement.disabled)
        nextPartInput.nativeElement.focus();
    }, 100);
  }

  public CancelPreReceive() {
    this.modal.confirm(
      "Are you sure to cancel pre-receive?",
      this.viewRef,
      () => {
        this.http.post("WH", "SNPE015_P_Cancel", this.invoiceHeader).subscribe(
          (res) => {
            this.modal.info(
              "Transaction complete: applied and saved.",
              this.viewRef
            );
            this.BackToSearch();
          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
      },
      () => {}
    );
  }

  public CompletePending() {
    const foundUnfinished = this.pendingList.selected.find(
      (item) => item.PENDING_TYPE === "T_PO" && !item.PO_NO
    );
    if (foundUnfinished) {
      this.modal.warning("Please finish pending detail.", this.viewRef);
      return;
    }

    const pendingPO = this.pendingList.selected.filter(
      (item) => item._SELECT_PO
    );
    const data = { header: this.invoiceHeader, pendingPO: pendingPO };

    this.modal.confirm(
      "Are you sure to complete pending?",
      this.viewRef,
      () => {
        this.http.post("WH", "SNPE015_P_PendingComplete", data).subscribe(
          (res) => {
            if (res === "Done") {
              this.modal.info(
                "Transaction complete: applied and saved.",
                this.viewRef
              );

              this.pendingDialog.Close();
              this.ViewDetail(this.invoiceHeader);
            } else {
              this.modal.warning(res, this.viewRef);
            }
          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
      }
    );
  }

  public SetPendingList(evt: any, data: any, index: number) {
    evt ? this.pendingList.toggle(data) : null;

    if (evt.checked && !this.CheckRequiredFields(index, false)) {
      this.modal.confirm(
        "Is PO not found?",
        this.viewRef,
        () => {
          this.poNotFound = true;
        },
        () => {}
      );
    } else if (!evt.checked && this.poNotFound) {
      this.poNotFound = false;
    }
  }

  public GetMatchedPOForPending() {
    return this.matchedPO.filter(
      (v) =>
        !this.invoiceDetail.find(
          (d) =>
            v.ACCEPT_PART_NO === d.ACCEPT_PART_NO &&
            v.PO_NO === d.PO_NO &&
            v.ITEM_NO === d.ITEM_NO &&
            v.SUB_ITEM_NO === d.SUB_ITEM_NO
        )
    );
  }

  public async SelectPOForPending(pendingData: any) {
    this.pendingDataForPODialog = undefined;
    await this.GetWithPO(
      this.invoiceHeader.INVOICE_NO,
      this.invoiceHeader.VENDOR_CODE,
      pendingData.ACCEPT_PART_NO
    ).then(
      (res) => {
        this.matchedPO = res.Table;

        this.matchedPO.forEach((d) => (d.CHECK = this.CheckIfExists(d)));
        this.pendingDataForPODialog = pendingData;
        this.selectPOForPendingDialog.OpenDialog();
      },
      (err) => {
        this.modal.error(err.message, this.viewRef);
      }
    );
  }

  public AcceptSelectedPOForPending(selectedRow: any) {
    const foundUsedPO = this.invoiceDetail.find(
      (item) =>
        item.ACCEPT_PART_NO === selectedRow.ACCEPT_PART_NO &&
        item.PO_NO === selectedRow.PO_NO &&
        item.ITEM_NO === selectedRow.ITEM_NO &&
        item.SUB_ITEM_NO === selectedRow.SUB_ITEM_NO
    );

    if (foundUsedPO) {
      this.modal.warning("This PO used by another item.", this.viewRef);
      return;
    }

    this.pendingDataForPODialog = Object.assign(
      this.pendingDataForPODialog,
      selectedRow
    );

    this.invoiceDetail = this.invoiceDetail.slice();
    this.selectPOForPendingDialog.Close();
  }
}
