import { Component, OnInit, ViewChild, ViewContainerRef, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { tap } from 'rxjs/operators';
import { Globals } from 'src/app/core/global';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import * as moment from 'moment';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'snpe015_w',
  templateUrl: './SNPE015_W.component.html',
  styleUrls: ['../warehouse.component.css']
})

export class SNPE015_WComponent implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private el: ElementRef,
    private global: Globals
  ) { }

  @ViewChild("dialog", { static: true }) dialog: any;
  @ViewChild("autoCompleteVendor", { static: true }) autoCompleteVendor: AutoCompleteQueryComponent;

  private currentDate: moment.Moment = moment();


  public invoiceHeader: any = {};
  public invoiceDetail: any[] = [];
  public invoiceItem: any[] = [];
  public invoiceData: any = {};

  public documentNo: string = '';

  public paramsSearch: any = {};
  public ShowList: boolean = true;
  public docList: Array<any> = [];
  public docList2: Array<any> = [];
  public docList3: Array<any> = [];
  public tempData: any = {};
  public iSCancel: boolean = false;

  public rowSave: any = [];
  public selection = new SelectionModel<any>(true, []);
  public dataSelected: any[] = [];
  public tabSelected: number = 0;

  private matchedPO: any[] = [];

  public tblFooter: any = [
    {
      fieldName: "ACCEPT_PART_NO",
      value: "Total",
      show: true
    },
    {
      fieldName: "ORDER_QTY",
      show: true
    },
    {
      fieldName: "QTY",

      show: true
    },
    {
      fieldName: "AMOUNT",

      show: true
    }]


  public vendorLookupConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor",
    SourceController: "AC",
    SourceMethodName: "SNPE015_GetVendorCode",
    NoOfShowRecords: 50,
    UpperCase: true
  }

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 20
  }

  public ConfigDocNo: any = {
    ColName: "PRE_RECEIVE_TEXT",
    PlaceholderText: "Select Pre Receiving No.",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetPre_ReceivingList",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      INVOICE_STATUS: "S"
    }
  }

  public ConfigInvoiceNo: any = {
    ColName: "INVOICE_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetInvList",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      INVOICE_STATUS: "S"
    }
  }

  ngOnInit() {
    this.paramsSearch.DOCUMENT_DATE_FROM = moment();
    this.paramsSearch.DOCUMENT_DATE_TO = moment();
    this.paramsSearch.DOC_DATE_FROM = moment();
    this.paramsSearch.DOC_DATE_TO = moment();
    this.paramsSearch.DOC_DATE_FROM3 = moment();
    this.paramsSearch.DOC_DATE_TO3 = moment();
  }


  public SetVendorCodeSearch(value) {
    this.paramsSearch.VENDOR_CODE = value.VENDOR_CODE;
  }


  public Search() {
    this.http.post("WH", "SNPE015_W_GetData",
      {
        DOCUMENT_DATE_FROM: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        DOCUMENT_DATE_TO: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
        PRE_RECEIVING_NO: this.paramsSearch.PRE_RECEIVING_NO,
        VENDOR_CODE: this.paramsSearch.VENDOR_CODE,
        INVOICE_NO: this.paramsSearch.INVOICE_NO
      })
      .subscribe(result => {
        this.docList = result.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      });
  }

  public Search2() {
    this.http.post("WH", "SNPE015_W_GetData_Comfirmed",
      {
        DOCUMENT_DATE_FROM: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_FROM),
        DOCUMENT_DATE_TO: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_TO),
        RECEIVE_DOC_NO: this.paramsSearch.RECEIVE_DOC_NO,
        VENDOR_CODE: this.paramsSearch.VENDOR_CODE2,
        INVOICE_NO: this.paramsSearch.INVOICE_NO2
      })
      .subscribe(result => {
        this.docList2 = result.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      });
  }

  public Search3() {
    this.http.post("WH", "SNPE015_W_GetData_OH",
      {
        DOCUMENT_DATE_FROM: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_FROM3),
        DOCUMENT_DATE_TO: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_TO3),
        RECEIVE_DOC_NO: this.paramsSearch.RECEIVE_DOC_NO3,
        VENDOR_CODE: this.paramsSearch.VENDOR_CODE3,
        INVOICE_NO: this.paramsSearch.INVOICE_NO3
      })
      .subscribe(result => {
        this.docList3 = result.Table;
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contact support team", this.viewRef);
      });
  }


  public ViewDetail(invoiceData) {
    this.invoiceHeader = {};
    this.invoiceDetail = [];
    this.dataSelected = [];
    this.selection.clear();

    this.invoiceData = invoiceData;
    this.ShowList = false;
    this.http.post("WH", "SNPE015_R_WEB_GetInvoiceDetail", invoiceData).subscribe(
      (res) => {
        this.invoiceHeader = res.Table[0] || {};;
        this.invoiceDetail = res.Table1;
        this.invoiceItem = res.Table2;
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  QuantityChange(data) {
    if (data.QTY > data.ACCEPT_QTY) {
      data.QTY = data.ACCEPT_QTY
    }

    let invoiceQty = this.GetInvoiceItemsByPO(data).length
    let shelvedQty = this.ShelvedQty(data);
    if (data.QTY > invoiceQty) {
      this.modal.warning("Shelve QTY cannot more than Invoice QTY.", this.viewRef);
      data.QTY = invoiceQty;
      return;
    }

    this.UpdateSelectedItem();
  }

  SetNewPartNo(part: any, data: any) {
    this.tempData.Part = part;
    data.ACCEPT_PART_NO = part.PART_NO;
  }


  public TotalQty() {
    return this.invoiceDetail.reduce((a, b) => a + (b.QTY || 0), 0);
  }

  public TotalAmount() {
    return this.invoiceDetail.reduce((a, b) => a + ((b.QTY || 0) * (b.UNIT_PRICE || 0)), 0);
  }

  public TotalItem() {
    return this.invoiceDetail.reduce(
      (a, b) => a + (((b.QTY || 0) / b.RECEIVE_PACKAGE) || 0),
      0
    );
  }

  public BackToGrid() {
    //this.Search();
    this.iSCancel = false;
    this.ShowList = true;
    this.TabChanged(this.tabSelected);
  }


  public ConfirmSaveDocument() {
    if (this.invoiceDetail.length > 0) {
      this.invoiceHeader.PART_LIST = this.invoiceItem;

      this.modal.confirm("Are you sure to confirm?", this.viewRef
        , async () => {
          try {

            await this.Confirm(); //Wait save and call confirm
          } catch (e) {
            this.modal.warning(
              'Found an error, please contact support team',
              this.viewRef
            );
          }
        }, () => { })
    } else {
      this.modal.error("No data to process.", this.viewRef);
    }
  }


  private async Confirm() {
    let data = Object.assign({}, this.invoiceHeader); //New object ref.

    await this.http.post("WH", "SNPE015_W_ConfirmShelve", data)
      .toPromise() //awaitable
      .then(res => {

        this.documentNo = res.DOCUMENT_NO;

        if (this.documentNo && this.documentNo != "") {
          this.modal.info("The document has been confirmed shelve : " + this.documentNo, this.viewRef)
          this.ViewDetail(this.invoiceData);
        } else {
          this.modal.warning(res.Message, this.viewRef);
        }
      })
      .catch((e) => {
        throw (e);
      });
  }
  /** */

  public ReturnAction() {
    if (this.invoiceDetail.length > 0) {
      this.invoiceHeader.PART_LIST = this.invoiceItem;

      this.modal.confirm("Are you sure to return this invoice?", this.viewRef
        , async () => {
          try {

            await this.Return(); //Wait save and call confirm
          } catch (e) {
            this.modal.warning(
              'Found an error, please contact support team',
              this.viewRef
            );
          }
        }, () => { })
    } else {
      this.modal.error("No data to process.", this.viewRef);
    }
  }

  private Return() {
    let data = Object.assign({}, this.invoiceHeader); //New object ref.

    this.http.post("WH", "SNPE015_W_Cancel", data)
      .toPromise() //awaitable
      .then(res => {
        if (res.Result === "Done") {
          this.modal.info("The invoice has been returned.", this.viewRef)
          //this.ViewDetail(this.invoiceData);
          this.BackToGrid();
        } else {
          this.modal.warning(res.Message, this.viewRef);
        }
      })
      .catch((e) => {
        throw (e);
      });
  }

  public OpenReport_Pre() {
    this.global.GoToReport("PMS_RPT_WH_SNPR028",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_DOCUMENT_NO: this.invoiceHeader.RECEIVE_DOC_NO,
        P_DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.CREATE_DATE),
        P_VENDOR_CODE: this.invoiceHeader.VENDOR,
        P_INVOICE_NO: this.invoiceHeader.INVOICE_NO,
        P_INVOICE_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.INVOICE_DATE),
      }
    );
  }

  public OpenReport() {
    this.global.GoToReport("PMS_RPT_WH_SNPR028",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_DOCUMENT_NO: this.invoiceHeader.RECEIVE_DOC_NO,
        P_DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.CREATE_DATE),
        P_VENDOR_CODE: this.invoiceHeader.VENDOR,
        P_INVOICE_NO: this.invoiceHeader.INVOICE_NO,
        P_INVOICE_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.INVOICE_DATE),
      }
    );
  }

  public Report2() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_DAILY",
      { 
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOC_DATE_TO),
      }
    );
  }

  public ReportR() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_L_R",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }

  public ReportS() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_L_S",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }
  public ReportR_By_Part() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_L_R_BY_PART",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }
 

  public ShelvedQty(data) {
    return this.GetInvoiceItemsByPO(data).filter(d => d.SHELVE_FLAG === 'Y').length;
  }

  /**Checkbox section */
  public isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.invoiceDetail.length;
    return numSelected === numRows;
  }

  public Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
    } else {
      this.invoiceDetail.forEach(row => {
        if (this.IsShelvedChecked(row) !== 1) {
          this.selection.select(row);
        }
      });
    }

    this.UpdateSelectedItem();
  }

  private GetInvoiceItemsByPO(data) {
    return this.invoiceItem
      .filter(d => data.PO_NO === d.ORDER_NO && data.ITEM_NO === d.ORDER_ITEM_NO && data.SUB_ITEM_NO === d.ORDER_SUB_ITEM_NO);
  }

  public UpdateSelectedItem() {
    this.dataSelected = [];
    this.selection.selected.forEach(v => {
      let invoiceItems: any[] = this.GetInvoiceItemsByPO(v).filter(d => d.SHELVE_FLAG === 'N').slice(0, v.QTY);
      this.dataSelected.push.apply(this.dataSelected, invoiceItems);
    })
  }

  public IsShelvedChecked(data) {
    let allInvoiceItems: number = this.GetInvoiceItemsByPO(data).length;
    let shelvedInvoiceItems: number = this.GetInvoiceItemsByPO(data).filter(d => d.SHELVE_FLAG === 'Y').length;

    if (allInvoiceItems === shelvedInvoiceItems) {
      return 1; //Checked
    } else if (shelvedInvoiceItems > 0 && allInvoiceItems !== shelvedInvoiceItems) {
      return 2; //Indeterminate
    } else {
      return 0; //Unchecked
    }
  }

  /** */
  public CheckActiveRow(tblIndex) {
    let valid: boolean =
      this.invoiceHeader.VENDOR_CODE &&
        this.invoiceHeader.INVOICE_NO &&
        this.invoiceHeader.INVOICE_DATE
        ? true
        : false;

    if (valid) {
      valid =
        !this.invoiceHeader.INVOICE_STATUS ||
        this.invoiceHeader.INVOICE_STATUS === "N";
    }

    if (valid && tblIndex > 0) {
      let prevRow = this.invoiceDetail[tblIndex - 1];

      valid =
        prevRow.ACCEPT_PART_NO && prevRow.QTY //&& prevRow.UNIT_PRICE_B
          ? true
          : false;
    }

    if (valid) {
      this.CreateDetailRow(tblIndex);
    }

    return valid;
  }

  public CheckPO(event, elemID, index, data) {
    let currentElement = this.el.nativeElement.querySelector("#" + elemID);

    this.http
      .post("WH", "SNPE015_P_CheckPOByPartNo", {
        VENDOR_CODE: this.invoiceHeader.VENDOR_CODE,
        INVOICE_NO: this.invoiceHeader.INVOICE_NO,
        PART_NO: event.PART_NO,
      })
      .subscribe(
        (res) => {
          this.matchedPO = res.Table.filter(
            (v) => (v.ORDER_CLASS == "2" || moment(v.PO_DATE) < moment(this.invoiceHeader.INVOICE_DATE)) //Order class 2 or
          );
          if (this.matchedPO.length > 0) {

            if (res.Table.find(v => v.LOCATION_NO && v.LOCATION_NO.length > 0)) {

              if (this.matchedPO.length === 1) {
                this.SetPOMatchOne(data, index, this.matchedPO[0], currentElement);
              } else if (this.matchedPO.length > 1) {
                this.matchedPO.forEach((d) => (d.CHECK = this.CheckIfExists(d)));
                this.dialog.OpenDialog();
              }

            } else {
              data._PART_NO = null;
              currentElement.focus();
              this.modal.error("Location hasn't configured for this part.", this.viewRef);
            }

          } else {
            data._PART_NO = null;
            currentElement.focus();
            this.modal.error("Unable to find matched PO.", this.viewRef);
          }
        },
        (err) => {
          this.modal.error(err.message, this.viewRef);
        }
      );
  }

  public AcceptSelectedPO() {
    let thisPartNo = this.matchedPO[0].ACCEPT_PART_NO;

    let ActiveRow = this.invoiceDetail.filter((d, i) =>
      this.CheckRequiredFields(i, false)
    );
    let SelectedRow = this.matchedPO.filter((d) => {
      
      //Add checking exists row for do not concat
      const found = ActiveRow.find(v => v.ACCEPT_PART_NO === d.ACCEPT_PART_NO && 
                                        v.PO_NO === d.PO_NO &&
                                        v.ITEM_NO === d.ITEM_NO &&
                                        v.SUB_ITEM_NO === d.SUB_ITEM_NO);
      
      return d.CHECK && !found
    });

    SelectedRow = SelectedRow.map((d) => this.SetPartRow({}, d));

    this.invoiceDetail = ActiveRow.concat(SelectedRow);

    this.invoiceDetail = this.invoiceDetail.slice();
    this.dialog.Close();
  }

  public SetPOMatchOne(
    data: any,
    index: number,
    result: any,
    currentElement: any
  ) {
    let foundDuplicate = this.invoiceDetail.find((d, i) => {
      return (
        index !== i &&
        result.ACCEPT_PART_NO === this.invoiceDetail[i].ACCEPT_PART_NO &&
        result.PO_NO === this.invoiceDetail[i].PO_NO &&
        result.ITEM_NO === this.invoiceDetail[i].ITEM_NO &&
        result.SUB_ITEM_NO === this.invoiceDetail[i].SUB_ITEM_NO
      );
    });

    if (foundDuplicate) {
      data._PART_NO = null;
      currentElement.focus();
      this.modal.error("Unable to insert duplicated record.", this.viewRef);
      return;
    }

    // data.CHECK = true;
    data = Object.assign(data, this.SetPartRow(data, result));
  }

  private SetPartRow(row: any, data: any) {
    row = data;
    row._PART_NO = data.ACCEPT_PART_NO;
    return row;
  }

  public CreateDetailRow(tblIndex) {
    let currentMaxRow: number = this.invoiceDetail.length || 0;
    let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

    if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.invoiceDetail.push({});
      }

      this.invoiceDetail = this.invoiceDetail.slice();
    }
  }

  public CheckIfExists(data) {
    return this.invoiceDetail.find((d) => {
      return (
        d.ACCEPT_PART_NO === data.ACCEPT_PART_NO &&
        d.PO_NO === data.PO_NO &&
        d.ITEM_NO === data.ITEM_NO &&
        d.SUB_ITEM_NO === data.SUB_ITEM_NO
      );
    });
  }

  public CheckRequiredFields(tblIndex, allowUnChange: boolean = true) {
    let valid: boolean =
      this.invoiceHeader.VENDOR_CODE &&
        this.invoiceHeader.INVOICE_NO &&
        this.invoiceHeader.INVOICE_DATE
        ? true
        : false;

    if (valid) {
      let thisRow = this.invoiceDetail[tblIndex];
      let countChangeItems: number = 0;

      if (thisRow.ACCEPT_PART_NO) countChangeItems++;
      if (thisRow.QTY) countChangeItems++;
      if (thisRow.ACCEPT_PART_NO) countChangeItems++;

      valid =
        (countChangeItems > 0 &&
          thisRow.ACCEPT_PART_NO &&
          thisRow.QTY &&
          thisRow.UNIT_TYPE) ||
          (allowUnChange && countChangeItems === 0)
          ? true
          : false;
    }

    return valid;
  }

  public UpdatePO() {

    this.modal.confirm(
      "Are you sure to update new PO data to this invoice?",
      this.viewRef,
      () => {


        var data = Object.assign({}, this.invoiceHeader);
        data.PART_LIST = this.invoiceDetail.slice().filter((v) => v.PART_NO)
          .map(v => {
            return v;
          });

        this.http.post("WH", "SNPE015_W_Update_PO", data).subscribe(
          (res) => {
            if (res.Result == "Done") {
              this.invoiceHeader.DOCUMENT_NO = res.DocNo;
              this.modal.info(
                "Transaction complete: applied and saved.",
                this.viewRef
              );
              this.ViewDetail(this.invoiceHeader);
            } else {
              this.modal.warning("Found an error, please contact support team.", this.viewRef);
            }

          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );


      },
      () => { }
    );

  }

  public UpdateInvoice() {
    this.modal.confirm(
      "Are you sure to update the new invoice?",
      this.viewRef,
      () => {
        //-------------------------------------------------------------------------------------
        this.http.post("WH", "SNPE015_W_Update_INVOICE", this.invoiceHeader).subscribe(
          (res) => {
            if (res.Result == "Done") {
              this.invoiceHeader.DOCUMENT_NO = res.DocNo;
              this.modal.info(
                "The invoice has been updated.",
                this.viewRef
              );
              this.ViewDetail(this.invoiceHeader);
            } else {
              this.modal.warning("Found an error, please contact support team.", this.viewRef);
            }

          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
        //-------------------------------------------------------------------------
      },
      () => { }
    );

  }

  public SaveInvoiceDate() {
    this.modal.confirm(
      "Are you sure to update the new invoice date?",
      this.viewRef,
      () => {
        //-------------------------------------------------------------------------------------
        this.http.post("WH", "SNPE015_W_Update_Invoice_Date", this.invoiceHeader).subscribe(
          (res) => {
            if (res.Result == "Done") {
              this.invoiceHeader.DOCUMENT_NO = res.DocNo;
              this.invoiceHeader.INVOICE_NO = this.invoiceHeader.UPDATE_INVOICE_NO;
              this.modal.info(
                "The invoice has been updated.",
                this.viewRef
              );
              this.ViewDetail(this.invoiceHeader);
            } else {
              this.modal.warning("Found an error, please contact support team.", this.viewRef);
            }

          },
          (err) => {
            this.modal.warning(
              "Found an error, please contact support team",
              this.viewRef
            );
          }
        );
        //-------------------------------------------------------------------------
      },
      () => { }
    );

  }

  public TabChanged(index) {
    this.tabSelected = index;
    switch(index) {
      case 0:
        this.Search();
        break;
      case 1:
        this.Search2();
        break;
      case 2:
        this.Search3();
        break;
    }

  }
  
}

