import { Component, OnInit, ViewContainerRef, ViewChild, ElementRef } from '@angular/core';
import { ModalService } from 'src/app/core/modal.service';
import { HttpService } from 'src/app/core/http.service';
import { tap } from 'rxjs/operators';
import { Globals } from 'src/app/core/global';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import * as moment from 'moment';

@Component({
    selector: 'SNPE023',
    templateUrl: 'SNPE023.component.html',
    styleUrls: ['SNPE023.component.css']
})

export class SNPE023Component implements OnInit {
    constructor(
        private http: HttpService,
        private modal: ModalService,
        private viewref: ViewContainerRef,
        private el: ElementRef,
        private global: Globals
    ) { }

    @ViewChild("autoCompleteOrderClass", { static: false }) autoCompleteOrderClass: any;
    @ViewChild("autoCompleteVendor", { static: true }) autoCompleteVendor: AutoCompleteQueryComponent;

    private currentDate: moment.Moment = moment();
    public DocNo: string;
    public DocDate: moment.Moment;
    public Vendor: string;
    public VendorCode: string;
    public OrderClass: string;
    public OrderClassDesc: string;
    public OrderType: string;
    public InvoiceNo: string;
    public InvoiceDate: moment.Moment;
    public InvoiceStatus: string;
    public InvoiceProcess: string;
    public Invoice: any = {};
    public SentBy: string;
    public UpdateBy: string;
    public UpdateDate: moment.Moment;
    public ConfirmDate :any;

    public TableData: Array<any> = [];
    public MatchedPO: Array<any> = [];
    public ParamsSearch: any = {
        DOCUMENT_DATE: moment()
    };
    public ShowList: boolean = true;
    public DocList: Array<any> = [];
    public tempData: any = {};
    public iSCancel : boolean = false;

    public vendorLookupConfig: any = {
        ColName: "LOOKUP_VENDOR_NAME",
        PlaceholderText: "Select Vendor",
        SourceController: "AC",
        SourceMethodName: "SNPE015_GetVendorCode",
        NoOfShowRecords: 50,
        UpperCase: true
    }

    public orderClassLookupConfig: any = {
        ColName: "LOOKUP_NAME",
        PlaceholderText: "Select Order Class",
        SourceController: "wPMS",
        SourceMethodName: "GetOrderClass",
        NoOfShowRecords: 50,
        MinLengthSearch: 0,
        SourceParams: {
            JOB_TYPE: '2'
        }
    }

    public PartNoLookupConfig: any = {
        ColName: "LOOKUP_PART_NAME",
        PlaceholderText: "Select Part No.",
        SourceController: "wPMS",
        SourceMethodName: "GetPartNo",
        NoOfShowRecords: 20
        // SourceParams: {
        //     VENDOR_CODE: this.VendorCode
        // }
    }

    public ConfigDocNo: any = {
        ColName: "DOCUMENT_NO",
        PlaceholderText: "Select Document No.",
        SourceController: "AC",
        SourceMethodName: "SNPE023_GetDocList",
        NoOfShowRecords: 50,
        MinLengthSearch: 2,
        UpperCase: true
    }

    public ConfigInvoiceNo: any = {
        ColName: "REF_INV_NO",
        PlaceholderText: "Select Document No.",
        SourceController: "AC",
        SourceMethodName: "SNPE023_GetInvList",
        NoOfShowRecords: 50,
        MinLengthSearch: 2
    }

    ngOnInit() {
    }

    GetDate() {
        return this.http.GetSystemDate()
            .pipe(tap((result: any) => {
                if (result) this.currentDate = result;
            }))
    }

    Search() {
        this.http.post("AC", "SNPE023_GetDocumentList",
        {
            DocumentNo: this.ParamsSearch.DOCUMENT_NO,
            DocumentDate: this.global.ConvertDateToFormatSTD(this.ParamsSearch.DOCUMENT_DATE),
            InvoiceNo: this.ParamsSearch.INVOICE_NO,
            InvoiceDate: this.global.ConvertDateToFormatSTD(this.ParamsSearch.INVOICE_DATE)
        })
            .subscribe(result => {
                this.DocList = result.Table;
            })
    }

    ViewClick(data) {
        this.DocNo = data.DOCUMENT_NO;
        this.DocDate = data.DOCUMENT_DATE;
        this.InvoiceNo = data.REF_INV_NO;
        this.InvoiceStatus = "";
        this.InvoiceProcess = '';
        this.InvoiceDate = data.REF_INV_DATE;
        this.Vendor = data.VENDOR;
        this.VendorCode = data.VENDOR_CODE;
        this.SentBy = 'N';
        this.OrderClass = null;
        this.OrderClassDesc = null;
        this.OrderType = null;
        this.UpdateBy = null;
        this.UpdateDate = null;

        this.TableData = [];
        this.ShowList = false;
        this.iSCancel = false;
        this.GetDocumentDetail();
    }

    AddClick() {
        this.DocNo = null;
        this.DocDate = this.currentDate;
        this.InvoiceNo = null;
        this.InvoiceStatus = "New";
        this.InvoiceProcess = 'N';
        this.InvoiceDate = this.currentDate;
        this.Vendor = null;
        this.VendorCode = null;
        this.SentBy = 'N';
        this.OrderClass = null;
        this.OrderClassDesc = null;
        this.OrderType = null;
        this.UpdateBy = null;
        this.UpdateDate = null;

        this.TableData = [];
        this.ShowList = false;
        this.tempData = {};

        this.CreateDetailRow(0);
    }

    // NewDocument() {
    //     this.modal.confirm("Are you sure to new document?<br>All changes will be cleared.", this.viewref
    //         , () => {
    //             this.ngOnInit();
    //         }, () => { })
    // }

    SaveDocument() {
        let saveRows = this.TableData.filter((d, i) => {
            d.DOCUMENT_NO = this.DocNo;
            d.VENDOR_CODE = this.VendorCode;
            d.SENT_BY = this.SentBy;
            d.ORDER_CLASS = this.OrderClass;
            d.ORDER_TYPE = this.OrderType;
            d.INVOICE_NO = this.InvoiceNo;
            d.INVOICE_DATE = this.global.ConvertDateToFormatSTD(this.InvoiceDate);
            return this.CheckRequiredFields(i, false) && d.INVOICE_PROCESS === 'N';
        })

        if (saveRows.length > 0) {
            this.modal.confirm("Are you sure to confirm?", this.viewref
                , () => {
                    this.http.post("AC", "SNPE023_SaveDocument", saveRows)
                        .subscribe(result => {
                            this.modal.info("Transaction complete: applied and saved.", this.viewref);
                            this.CheckDocument();
                        }, err => {
                            this.modal.error(err.message, this.viewref);
                        })
                }, () => { })
        } else {
            this.modal.error("Unable to process.", this.viewref);
        }
    }

    DeleteDocument() {
        this.modal.confirm("Are you sure to cancel this document?", this.viewref
            , () => {
                this.http.post("AC", "SNPE023_DeleteDocument", {
                    VendorCode: this.VendorCode, 
                    InvoiceNo: this.InvoiceNo, 
                    DocumentNo: this.DocNo
                })
                    .subscribe(result => {
                        if (result === "Done") {
                            this.modal.info("Transaction complete: applied and saved.", this.viewref);
                            this.ngOnInit();
                            this.iSCancel = true;
                        }
                        else if(result === "INVOICE_ERROR"){
                            this.modal.warning("Unable to process this invoice",this.viewref);
                        }
                        else if(result === "CONFIRMED"){
                            this.modal.warning("Can not cancel document because this document confirmed",this.viewref);
                        }
                        else{
                            this.modal.error('Found an error, please contact support team', this.viewref);
                        }
                    }, err => {
                        this.modal.error('Found an error, please contact support team', this.viewref);
                    })
            }, () => {

            });

    }

    GetDocumentDetail() {
        if (this.VendorCode && this.InvoiceNo && this.DocNo) {
            this.http.get("AC", "SNPE023_CheckDocument", this.DocNo)
                .subscribe(result => {
                    if (result.Table.length > 0) {
                        let tableHeader = result.Table[0];

                        this.VendorCode = tableHeader.VENDOR_CODE;
                        this.SentBy = tableHeader.SENT_BY;
                        this.DocNo = tableHeader.DOCUMENT_NO;
                        this.DocDate = tableHeader.DOCUMENT_DATE;
                        this.InvoiceNo = tableHeader.INVOICE_NO;
                        this.InvoiceDate = tableHeader.INVOICE_DATE;
                        this.InvoiceStatus = tableHeader.INVOICE_STATUS;
                        this.InvoiceProcess = tableHeader.INVOICE_PROCESS;
                        this.UpdateBy = tableHeader.LAST_UPD_BY;
                        this.UpdateDate = tableHeader.LAST_UPD_DATE;
                        this.ConfirmDate = tableHeader.CONFIRM_DATE;
                        this.TableData = result.Table1;

                        if (this.TableData.length > 0) {
                            this.OrderClass = this.TableData[0].ORDER_CLASS;
                            this.OrderType = this.TableData[0].ORDER_TYPE;
                            this.OrderClassDesc = this.TableData[0].ORDER_CLASS_DESC;
                        }

                        // for (let i = this.TableData.length; i < 10; i++) {
                        //     this.TableData.push({
                        //         ITEM_NO: i + 1,
                        //         INVOICE_PROCESS: this.InvoiceProcess
                        //     });
                        // }

                    }
                })
        }
    }

    CheckDocument() {
        if (this.VendorCode && this.InvoiceNo) {
            this.http.post("AC", "SNPE023_CheckDocument", {
                VendorCode: this.VendorCode, 
                InvoiceNo: this.InvoiceNo
            })
                .subscribe(result => {
                    if (result.Table.length > 0) {
                        let tableHeader = result.Table[0];

                        this.VendorCode = tableHeader.VENDOR_CODE;
                        this.SentBy = tableHeader.SENT_BY;
                        this.DocNo = tableHeader.DOCUMENT_NO;
                        this.DocDate = tableHeader.DOCUMENT_DATE;
                        this.InvoiceNo = tableHeader.INVOICE_NO;
                        this.InvoiceDate = tableHeader.INVOICE_DATE;
                        this.InvoiceStatus = tableHeader.INVOICE_STATUS;
                        this.InvoiceProcess = tableHeader.INVOICE_PROCESS;
                        this.UpdateBy = tableHeader.LAST_UPD_BY;
                        this.UpdateDate = tableHeader.LAST_UPD_DATE;
                        this.ConfirmDate = tableHeader.CONFIRM_DATE;
                        this.TableData = result.Table1;

                        if (this.TableData.length > 0) {
                            this.OrderClass = this.TableData[0].ORDER_CLASS;
                            this.OrderType = this.TableData[0].ORDER_TYPE;

                            this.autoCompleteOrderClass.textInput = this.OrderClass;
                            this.autoCompleteOrderClass.query();
                        }

                        for (let i = this.TableData.length; i < 10; i++) {
                            this.TableData.push({
                                ITEM_NO: i + 1,
                                INVOICE_PROCESS: this.InvoiceProcess
                            });
                        }

                    }
                })
        }
    }

    SetVendorCodeLookup(value) {
        if (value.VENDOR_CODE) {
            this.Vendor = value.LOOKUP_VENDOR_NAME;
            this.VendorCode = value.VENDOR_CODE;

            // this.PartNoLookupConfig.SourceParams.VENDOR_CODE = this.VendorCode;
        } else {
            this.Vendor = null;
            this.VendorCode = null;
            this.el.nativeElement.querySelector("#vendorcode").focus();
            this.modal.error("Unable to find vendor.", this.viewref);
        }
    }

    SetOrderClassLookup(value) {
        if (value.ORDER_CLASS) {
            this.OrderClass = value.ORDER_CLASS;
            this.OrderClassDesc = value.ORDER_DESCRIPT;
        } else {
            this.OrderClass = null;
            this.OrderClassDesc = null;
            this.el.nativeElement.querySelector("#orderclass").focus();
            this.modal.error("Unable to find order class.", this.viewref);
        }
    }

    SetOrderType() {
        switch (this.OrderClass) {
            case "1":
                this.OrderType = "1"
                break;
            case "2":
                this.OrderType = "1"
                break;
            case "3":
                this.OrderType = "3"
                break;
            case "4":
                this.OrderType = "3"
                break;
            case "5":
                this.OrderType = "4"
                break;
            case "7":
                this.OrderType = "6"
                break;
            default:
                this.OrderType = undefined;
                break;
        }
    }

    SetNewPartNo(part: any, data: any, index: number) {
        this.tempData.Part = part;
        if(this.TableData.filter((v,i) => index !== i && v.ACCEPT_PART_NO && v.ACCEPT_PART_NO === part.PART_NO).length > 0) {
            this.modal.warning("Duplicate part number", this.viewref);
            data.ACCEPT_PART_NO = null;
            return;
        }
        data.ACCEPT_PART_NO = part.PART_NO;
        data.PART_NAME = part.PART_NAME;
        data.UNIT_PRICE = parseFloat(part.BASE_PRICE);

        this.GetBasePrice(part.PART_NO, data);
    }

    CancelPartNo(data: any) {
        data.ACCEPT_PART_NO = this.tempData.Part.PART_NO;
    }


    GetBasePrice(part: string, data: any) {
        this.http.get("AC", "SNPE023_GetBasePrice", part)
            .subscribe(result => {
                data.UNIT_PRICE = 0;
                if (result.Table.length > 0) {
                    data.UNIT_PRICE = parseFloat(result.Table[0].BASE_PRICE);
                }
            })
    }

    CheckValidSave() {
        return this.TableData.find((d, i) => this.CheckRequiredFields(i, false) && d.INVOICE_PROCESS === 'N');
    }

    CheckRequiredFields(tblIndex, allowUnChange: boolean = true) {
        let valid: boolean = (this.VendorCode && this.SentBy && this.OrderClass && this.OrderType && this.InvoiceNo && this.InvoiceDate) ? true : false;

        if (valid) {
            let thisRow = this.TableData[tblIndex];
            let countChangeItems: number = 0;

            if (thisRow.ACCEPT_PART_NO) countChangeItems++;
            if (thisRow.ITEM_QTY) countChangeItems++;
            if (thisRow.ACCEPT_PART_NO) countChangeItems++;

            valid = (countChangeItems > 0 && thisRow.ACCEPT_PART_NO && thisRow.ITEM_QTY && thisRow.UNIT_PRICE) || (allowUnChange && countChangeItems === 0) ? true : false;
        }

        return valid;
    }

    CheckActiveRow(tblIndex) {
        let valid: boolean = (this.VendorCode && this.SentBy && this.OrderClass && this.OrderType && this.InvoiceNo && this.InvoiceDate) ? true : false;

        if (valid) {
            valid = (!this.TableData[tblIndex].INVOICE_PROCESS || this.TableData[tblIndex].INVOICE_PROCESS === 'N') && !this.DocNo;
        }

        if (valid && tblIndex > 0) {
            let prevRow = this.TableData[tblIndex - 1];

            valid = (prevRow.ACCEPT_PART_NO && prevRow.ITEM_QTY && prevRow.UNIT_PRICE) ? true : false;
        }

        if (valid) {
            this.CreateDetailRow(tblIndex);
        }

        return valid;
    }

    CreateDetailRow(tblIndex) {
        let currentMaxRow: number = this.TableData.length || 0;
        let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

        if (currentMaxRow - tblIndex <= 2) {
            for (let i = currentMaxRow; i < nextMaxRow; i++) {
                this.TableData.push({
                    ITEM_NO: i + 1,
                    INVOICE_PROCESS: 'N'
                });
            }

            this.TableData = this.TableData.slice();
        }

    }

    DeleteRow(index) {
      this.modal.confirm("Are you sure to remove this row?", this.viewref
        , () => {
          this.TableData.splice(index, 1);
          this.TableData = [...this.TableData];
        }, () => { })
    }

    TotalQty() {
        return this.TableData.reduce((a, b) => a + (b.ITEM_QTY || 0), 0);
    }

    TotalAmount() {
        return this.TableData.reduce((a, b) => a + ((b.ITEM_QTY || 0) * (b.UNIT_PRICE || 0)), 0);
    }

    BackToGrid() {
        this.ShowList = true;
    }

    CheckDeleteButton() {
        return this.InvoiceProcess === 'C' && this.ConfirmDate != 'N'
    }
}
