import { SelectionModel } from "@angular/cdk/collections";
import {
  Component,
  ElementRef,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { MatTabGroup } from "@angular/material/tabs";
import * as moment from "moment";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { COMMA, ENTER } from "@angular/cdk/keycodes";
import { MatAutocompleteSelectedEvent } from "@angular/material/autocomplete";
import { MatChipInputEvent } from "@angular/material/chips";
import { FormControl } from "@angular/forms";

@Component({
  selector: "SNPE047",
  templateUrl: "SNPE047.component.html",
  styleUrls: ["SNPE047.component.css"],
})
export class SNPE047Component implements OnInit {
  constructor(
    private http: HttpService,
    public global: Globals,
    private viewRef: ViewContainerRef,
    private modal: ModalService
  ) {}

  public ConfigDocNo: any = {
    ColName: "PRE_RECEIVE_TEXT",
    PlaceholderText: "Select Pre Receiving No.",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetPre_ReceivingList",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      INVOICE_STATUS: "S",
    },
  };

  public ConfigInvoiceNo: any = {
    ColName: "INVOICE_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE015_P_GetInvList",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      INVOICE_STATUS: "S",
    },
  };

  public vendorLookupConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor",
    SourceController: "AC",
    SourceMethodName: "SNPE015_GetVendorCode",
    NoOfShowRecords: 50,
    UpperCase: true,
  };

  @ViewChild("docTab") docTab: MatTabGroup;
  @ViewChild("caseInput") caseInput: ElementRef<HTMLInputElement>;
  private lastUrl: string;
  public tabList = [
    { label: "Wait to confirm", searchUrl: "SNPE047_GetList" },
    { label: "Confirmed shelve", searchUrl: "SNPE047_GetList_Confirmed" },
    { label: "Confirmed O/H", searchUrl: "SNPE047_GetList_OH" },
  ];
  public paramsSearch = {
    DOCUMENT_DATE_FROM: null,
    DOCUMENT_DATE_TO: null,
    PRE_RECEIVING_NO: null,
    INVOICE_NO: null,
    VENDOR_CODE: null,
  };
  public invoiceList = [];
  public IsSearch = true;
  public IsAccountCompleted = false;
  public invoiceHeader: any = {};
  public invoiceDetail = [];
  public invoiceDetailTemp = [];
  public selection = new SelectionModel<any>(true, []);
  public documentNo: string = null;
  public docList: string[];
  public documentSelected: string = "All";

  public partialSelection = {
    separatorKeyCode: [ENTER, COMMA],
    selectable: true,
    removable: true,
    parts: [],
    cases: [],
    selected: [],
    itemType: "cases",
    items: [],
  };

  ngOnInit() {
    this.initial();
    this.TabChanged(0);
    // this.ViewDetail({
    //   VENDOR_CODE: "014",
    //   VENDOR_NAME: "NML OPTION PARTS (JAPAN)",
    //   INVOICE_NO: "F4D77",
    //   INVOICE_DATE: "2020-05-22T00:00:00",
    //   PRE_RECEIVE_DOC_NO: "PRC-21-0000123",
    //   INVOICE_STATUS_DESC: "Shelved",
    //   RECEIVE_BY: "Kaison Ounpaey",
    //   RECEIVE_DATE: "2021-07-09T14:29:10",
    // });
  }

  private initial(resetSearch: boolean = true) {
    this.documentNo = null;
    this.IsSearch = true;
    if (resetSearch)
      this.paramsSearch = {
        DOCUMENT_DATE_FROM: moment(),
        DOCUMENT_DATE_TO: moment(),
        PRE_RECEIVING_NO: null,
        INVOICE_NO: null,
        VENDOR_CODE: null,
      };
    this.invoiceHeader = {};
    this.invoiceDetail = [];
    this.invoiceDetailTemp = [];
    this.docList = [];
    this.partialSelection = {
      separatorKeyCode: [ENTER, COMMA],
      selectable: true,
      removable: true,
      parts: [],
      cases: [],
      selected: [],
      itemType: "cases",
      items: [],
    };
  }

  public TabChanged(index) {
    let url = this.tabList[index].searchUrl;
    this.Search(url);
  }

  public Search(url) {
    this.invoiceList = [];
    this.lastUrl = url;
    this.http.post("WH", url, this.paramsSearch).subscribe(
      (result) => {
        this.invoiceList = result.Table;
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  private GetInvoiceDetail(invoiceHeader) {
    this.invoiceDetail = [];
    this.invoiceDetailTemp = [];
    this.docList = [];
    this.selection = new SelectionModel<any>(true, []);
    this.IsAccountCompleted = false;
    this.http
      .post("WH", "SNPE047_GetDetail", invoiceHeader)
      .subscribe((res) => {
        this.invoiceDetail = res.Table;
        this.invoiceHeader.INVOICE_STATUS =
          this.invoiceDetail[0].INVOICE_STATUS;
        this.invoiceHeader.INVOICE_STATUS_DESC =
          this.invoiceDetail[0].INVOICE_STATUS_DESC;

        this.CheckAccountComplete();
        this.GetDocList();
        this.PreparePartial();
      });
  }
  private CheckAccountComplete() {
    this.IsAccountCompleted = this.invoiceDetail.find(
      (d) => d.ACCOUNT_COMPLETED === "N"
    )
      ? false
      : true; //Find not complete data

    //Block selection if not completed
    if (!this.IsAccountCompleted) {
      setTimeout(() => this.AllToggle(), 50);
    }
  }

  private GetDocList() {
    this.docList = this.invoiceDetail
      .filter((v) => v.DOCUMENT_NO?.length > 0)
      .reduce((doc: string[], v) => {
        if (!doc.includes(v.DOCUMENT_NO)) doc.push(v.DOCUMENT_NO);
        return doc;
      }, []);
  }

  private PreparePartial() {
    if (this.IsAccountCompleted) {
      this.partialSelection.parts = this.invoiceDetail
        .filter((v) => this.ActiveCondition(v))
        .reduce((parts: string[], v) => {
          if (!parts.includes(v.PART_NO)) parts.push(v.PART_NO);
          return parts;
        }, []);

      this.partialSelection.cases = this.invoiceDetail
        .filter((v) => this.ActiveCondition(v))
        .reduce((cases: string[], v) => {
          if (!cases.includes(v.CASE_NO)) cases.push(v.CASE_NO);
          return cases;
        }, []);

      this.partialSelection.items = [
        ...this.partialSelection[this.partialSelection.itemType],
      ];

      console.log(this.partialSelection.items);
    }
  }

  public ViewDetail(invoiceHeader) {
    this.IsSearch = false;
    this.documentSelected = "All";
    this.invoiceHeader = invoiceHeader;
    this.GetInvoiceDetail(invoiceHeader);
  }

  public SetVendorCodeSearch(data) {}

  public Save() {
    if (this.selection.selected.length > 0) {
      this.modal.confirm(
        "Do you want to save changed you have made?",
        this.viewRef,
        () => this.ConfirmSave(),
        () => {}
      );
    } else {
      this.modal.warning(
        "Please select at least 1 item to confirm shelve.",
        this.viewRef
      );
    }
  }

  private ConfirmSave() {
    let invoiceData = this.invoiceHeader;
    invoiceData._INVOICE_DATE = this.global.ConvertDateToFormatSTD(
      invoiceData.INVOICE_DATE
    );
    invoiceData.PART_LIST = this.selection.selected;

    this.http.post("WH", "SNPE047_Save", invoiceData).subscribe(
      (res) => {
        if (res.result === "Done") {
          this.documentNo = res.DOCUMENT_NO;
          this.modal.info(
            "The document has been confirmed shelve : " + this.documentNo,
            this.viewRef
          );
          this.ViewDetail(this.invoiceHeader);
        }
      },
      (err) => {
        this.modal.error(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }

  public ReturnAction() {
    this.modal.confirm(
      "Are you sure to return this invoice?",
      this.viewRef,
      () => {
        this.http
          .post("WH", "SNPE047_Cancel_Invoice", this.invoiceHeader)
          .subscribe(
            (res) => {
              if (res.result === "Done") {
                this.modal.info("The invoice has been returned.", this.viewRef);
                this.BackToGrid();
              }
            },
            (err) => {
              this.modal.error(
                "Found an error, please contact support team",
                this.viewRef
              );
            }
          );
      },
      () => {}
    );
  }

  public CancelDocument(documentNo: string) {
    this.modal.confirm(
      "Are you sure to cancel document no." + documentNo + "?",
      this.viewRef,
      () => {
        this.http
          .post("WH", "SNPE047_Cancel", {
            DOCUMENT_NO: documentNo,
          })
          .subscribe(
            (res) => {
              if (res.result === "Done") {
                this.modal.info(
                  "The document has been cancelled.",
                  this.viewRef
                );
                this.ViewDetail(this.invoiceHeader);
                // this.BackToGrid();
              }
            },
            (err) => {
              this.modal.error(
                "Found an error, please contact support team",
                this.viewRef
              );
            }
          );
      },
      () => {}
    );
  }

  public BackToGrid() {
    this.initial(false);
    this.Search(this.lastUrl);
  }

  public IsAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.invoiceDetail.filter((v) =>
      this.ActiveCondition(v)
    ).length;
    return numSelected === numRows;
  }

  public AllToggle() {
    this.IsAllSelected()
      ? this.selection.clear()
      : this.invoiceDetail
          .filter((v) => this.ActiveCondition(v))
          .forEach((row) => this.selection.select(row));
  }

  public DocumentGroupChange(value) {
    this.documentSelected = value;
    this.invoiceDetailTemp = this.invoiceDetail.filter(
      (v) => v.DOCUMENT_NO === value
    );
  }

  public CheckReturnInvoiceButton() {
    return (
      this.invoiceDetail.length > 0 &&
      !this.invoiceDetail.find((v) => !this.ActiveCondition(v))
    ); //Should find checkbox disable not found
  }

  public ActiveCondition(data) {
    return data.ITEM_STATUS === "S";
  }

  private SelectedCase(value: string) {
    this.invoiceDetail
      .filter((v) => this.ActiveCondition(v) && v.CASE_NO === value)
      .forEach((row) => this.selection.select(row));
  }
  private UnSelectedCase(value: string) {
    this.invoiceDetail
      .filter((v) => this.ActiveCondition(v) && v.CASE_NO === value)
      .forEach((row) => this.selection.deselect(row));
  }

  /* Chip section */
  public RemoveSelect(value) {
    const index = this.partialSelection.selected.indexOf(value);

    if (index >= 0) {
      this.partialSelection.selected.splice(index, 1);

      //Unselect case
      this.UnSelectedCase(value);

      this.partialSelection.items.push(value);
    }
  }

  public AddSelect(event: MatChipInputEvent) {
    const value = (event.value || "").trim();
    let itemIndex = this.partialSelection.items.findIndex((v) => v === value);

    if (value && itemIndex >= 0) {
      this.partialSelection.selected.push(value);

      //Select case
      this.SelectedCase(value);

      this.partialSelection.items.splice(itemIndex, 1);
    }

    this.caseInput.nativeElement.value = null;
    // event.input.value = null;
  }

  public PartialSelected(event: MatAutocompleteSelectedEvent): void {
    const value = event.option.viewValue;
    let itemIndex = this.partialSelection.items.findIndex((v) => v === value);

    if (itemIndex >= 0) {
      this.partialSelection.selected.push(value);

      //Select case
      this.SelectedCase(value);

      this.partialSelection.items.splice(itemIndex, 1);

      this.caseInput.nativeElement.value = null;
    }
  }

  public Report2() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_I_DAILY",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }

  public ReportR() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_I_R",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }

  public ReportS() {
    this.global.GoToReport("PMS_RPT_WMS_RECEIVE_I_S",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
        P_START_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_FROM),
        P_TO_DATE: this.global.ConvertDateToFormatSTD(this.paramsSearch.DOCUMENT_DATE_TO),
      }
    );
  }
}
