import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';

@Component({
  selector: 'snpr029',
  templateUrl: './snpr029.component.html',
  styleUrls: ['./snpr029.component.css']
})



export class SNPR029Component implements OnInit {


  constructor(
    private http: HttpService,
    private global: Globals,
    private modal: ModalService,
    private viewRef: ViewContainerRef
  ) { }

  public data: any = [];
  public SelectList: any;

  public VendorConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor Code",
    SourceController: "wPMS",
    SourceMethodName: "GetVendorCode",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public InvoiceConfig: any = {
    ColName: "INVOICE_NO",
    ColValue: "INVOICE_NO",
    PlaceholderText: "Select Invoice No.",
    SourceController: "wPMS",
    SourceMethodName: "GetInvoice",
    NoOfShowRecords: 50,
    MinLengthSearch: 0
  }

  public ConfigPart: any = {
    ColName: "LOOKUP_PART",
    ColValue: "",
    PlaceholderText: "Select Part No.",
    SourceController: "WH",
    SourceMethodName: "SNPR029_GetPart",
    NoOfShowRecords: 30,
    MinLengthSearch: 3,
    UpperCase: true
  }

  public Params: any = {
    VENDOR_CODE: null,
    INVOICE_NO: null,
    VENDOR_NAME: null
  };

  ngOnInit() {
    this.SelectList = [{ ID: 1, SELECT_CODE: "A001" }, { ID: 2, SELECT_CODE: "B002" }, { ID: 3, SELECT_CODE: "C003" }]
    let i = 0;

    while (i < 5) {
      this.data.push({
        PART_NO: '',
        PART_NAME: '',
        LOCATION: '',
        QTY: '',
        PRN: '',
      })
      i++
    }
  }

  public AddRecord() {
    this.data.push({
      PART_NAME: '',
      LOCATION: '',
      QTY: '',
      PRN: '',
    })
    this.data = this.data.slice();
  }

  public SelectedVendor(event) {
    this.Params.VENDOR_CODE = event.VENDOR_CODE
    this.Params.VENDOR_NAME = event.VENDOR_NAME_TH
  }

  public SelectedPart(data, event) {
    data.PART_NO = event.PART_NO
    data.PART_NAME = event.PART_NAME_EN
    data.LOCATION = event.LOCATION_NO
    data.QTY = 1
    data.PRN = 1
  }

  public OpenReport() {
    if (!this.Params.VENDOR_CODE || !this.Params.INVOICE_NO) {
      this.modal.warning("Please fill Vendor Code and Invoice No.!", this.viewRef);
      return;
    }
  

    let data = []
    data = this.data.filter(x => x.PART_NO && x.QTY && x.PRN)

    if (data.length <= 0) {
      this.modal.warning("Data not complete ,Please check part!", this.viewRef);
      return;
    }

    if (data.length > 0) {
      this.http.post('WH', 'SNPR029_INSERT', data)
        .subscribe(res => {
          if (res) {
            this.global.GoToReport("PMS_RPT_WMS_SNPR029_new",
              {
                P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
                P_VENDOR_CODE: this.Params.VENDOR_CODE,
                P_INVOICE_NO: this.Params.INVOICE_NO
              }
            );
          }
        })
    }
  }
}

