import { Component, OnInit, Input, Output, EventEmitter, TemplateRef, ContentChild, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router, ActivatedRoute } from '@angular/router';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from '../../../core/global';


@Component({
  selector: 'snpr037',
  templateUrl: './SNPR037.component.html',
  styleUrls: ['./SNPR037.component.css']
})

export class SNPR037Component implements OnInit {

  constructor(
    private HttpService: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private route: ActivatedRoute,
    private global: Globals
  ) { }

  //@ViewChild('fileUpload') myInputVariable: ElementRef;

  @Input() config: any;
  @Input() data: any;
  @Output() datacallback = new EventEmitter<string>();


  public dataList: any = [];
  public findingWord: string;

  public ConfigInvoiceDate: any = {
    ColName: "INVOICE_DATE",
    ColValue: "INVOICE_DATE",
    PlaceholderText: "Select Invoice Date in format[DD-MM-YYYY]",
    SourceController: "WH",
    SourceMethodName: "SNSE039P_InvoiceDate_Find",
    NoOfShowRecords: 100,
    MinLengthSearch: 1,
    UpperCase: true
  }

  public ConfigDocDate: any = {
    ColName: "DOCUMENT_DATE_F",
    ColValue: "DOCUMENT_DATE_F",
    PlaceholderText: "Select Document Date in format[DD-MM-YYYY]",
    SourceController: "WH",
    SourceMethodName: "SNSE037_DocDate_Find",
    NoOfShowRecords: 100,
    MinLengthSearch: 1,
    UpperCase: true
  }

  public ConfigDocNo: any = {
    ColName: "DOCUMENT_NO",
    ColValue: "DOCUMENT_NO",
    PlaceholderText: "Select Document No",
    SourceController: "WH",
    SourceMethodName: "SNSE037_DocNo_Find",
    NoOfShowRecords: 100,
    MinLengthSearch: 1,
    UpperCase: true
  }

  public ConfigDocType: any = {
    ColName: "LOOKUP_EN",
    ColValue: "DOCUMENT_TYPE",
    PlaceholderText: "Select Document Type",
    SourceController: "WH",
    SourceMethodName: "SNSE037_DocType_Find",
    NoOfShowRecords: 100,
    MinLengthSearch: 1,
    UpperCase: true
  }

  public dataListSeqByInvoiceDate: any = [];

  public ListData: any = {
    DOCUMENT_TYPE_FROM: null,
    DOCUMENT_TYPE_TO: null,
    DOCUMENT_NO_FROM: null,
    DOCUMENT_NO_TO: null,
    DOCUMENT_DATE_FROM: null,
    DOCUMENT_DATE_TO: null

  };

  ngOnInit() {
  }

  OpenReport() {
    if (!this.CheckSearchConditionReq()) {
      //Check Null all condition.
      return;
    }

    if (this.CheckSearchCondition()) {
      //console.log("Sent to Report:",this.ListData);
      //alert('Wait open report.');
      this.GoToReport();//show roport viewer.
    }
  }
  CheckSearchConditionReq(): boolean {
    let boolHadSelectedConditionType: boolean;
    let boolHadSelectedConditionNo: boolean;
    let boolHadSelectedConditionDate: boolean;

    //Check emply string and null value will return false;
    if (!this.ListData.DOCUMENT_TYPE_FROM && !this.ListData.DOCUMENT_TYPE_TO) {
      boolHadSelectedConditionType = false;
    } else {
      boolHadSelectedConditionType = true;
    }

    if (!this.ListData.DOCUMENT_NO_FROM && !this.ListData.DOCUMENT_NO_TO) {
      boolHadSelectedConditionNo = false;
    } else {
      boolHadSelectedConditionNo = true;
    }

    if (!this.ListData.DOCUMENT_DATE_FROM && !this.ListData.DOCUMENT_DATE_TO) {
      boolHadSelectedConditionDate = false;
    }
    else {
      boolHadSelectedConditionDate = true;
    }

    console.table(this.ListData);
    console.log(boolHadSelectedConditionType, boolHadSelectedConditionNo, boolHadSelectedConditionDate);
    return boolHadSelectedConditionType || boolHadSelectedConditionNo || boolHadSelectedConditionDate;
  }//end CheckSearchCondition

  CheckSearchCondition() {
    let boolHadSelectedCondition: boolean;
    boolHadSelectedCondition = false;//maening: User is not selecting condition search report.
    //define Emply string to Null, when call report, emply string will not sent to server report.
    if (!this.ListData.DOCUMENT_TYPE_FROM) { this.ListData.DOCUMENT_TYPE_FROM = null; }
    if (!this.ListData.DOCUMENT_TYPE_TO) { this.ListData.DOCUMENT_TYPE_TO = null; }

    if (!this.ListData.DOCUMENT_NO_FROM) { this.ListData.DOCUMENT_NO_FROM = null; }
    if (!this.ListData.DOCUMENT_NO_TO) { this.ListData.DOCUMENT_NO_TO = null; }

    if (!this.ListData.DOCUMENT_DATE_FROM) { this.ListData.DOCUMENT_DATE_FROM = null; }
    if (!this.ListData.DOCUMENT_DATE_TO) { this.ListData.DOCUMENT_DATE_TO = null; }

    //console.log("When clear emply string:",this.ListData);

    //check From and To condition when user select only from or only to. must put value to it.
    if (this.ListData.DOCUMENT_TYPE_FROM == null) { this.ListData.DOCUMENT_TYPE_FROM = this.ListData.DOCUMENT_TYPE_TO; boolHadSelectedCondition = true; }
    if (this.ListData.DOCUMENT_TYPE_TO == null) { this.ListData.DOCUMENT_TYPE_TO = this.ListData.DOCUMENT_TYPE_FROM; boolHadSelectedCondition = true; }

    if (this.ListData.DOCUMENT_NO_FROM == null) { this.ListData.DOCUMENT_NO_FROM = this.ListData.DOCUMENT_NO_TO; boolHadSelectedCondition = true; }
    if (this.ListData.DOCUMENT_NO_TO == null) { this.ListData.DOCUMENT_NO_TO = this.ListData.DOCUMENT_NO_FROM; boolHadSelectedCondition = true; }

    if (this.ListData.DOCUMENT_DATE_FROM == null) { this.ListData.DOCUMENT_DATE_FROM = this.ListData.DOCUMENT_DATE_TO; boolHadSelectedCondition = true; }
    if (this.ListData.DOCUMENT_DATE_TO == null) { this.ListData.DOCUMENT_DATE_TO = this.ListData.DOCUMENT_DATE_FROM; boolHadSelectedCondition = true; }

    return boolHadSelectedCondition;
  }

  GoToReport() {

    this.ListData.DOCUMENT_TYPE_FROM = this.ListData.DOCUMENT_TYPE_FROM.toUpperCase();
    this.ListData.DOCUMENT_TYPE_TO = this.ListData.DOCUMENT_TYPE_TO.toUpperCase();
    this.ListData.DOCUMENT_NO_FROM = this.ListData.DOCUMENT_NO_FROM.toUpperCase();
    this.ListData.DOCUMENT_NO_TO = this.ListData.DOCUMENT_NO_TO.toUpperCase();

    this.global.GoToReport("PMS_RPT_WH_SNPR037", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_DOC_TYPE_START: this.ListData.DOCUMENT_TYPE_FROM,
      P_DOC_TYPE_END: this.ListData.DOCUMENT_TYPE_TO,
      P_DOC_NO_START: this.ListData.DOCUMENT_NO_FROM,
      P_DOC_NO_END: this.ListData.DOCUMENT_NO_TO,
      P_DOC_DATE_START: this.ListData.DOCUMENT_DATE_FROM,
      P_DOC_DATE_END: this.ListData.DOCUMENT_DATE_TO
    })
  }





}
