
import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { FormControl, Validators } from '@angular/forms';
import { ModalService } from 'src/app/core/modal.service';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'snse054d_z',
  templateUrl: './SNSE054D_Z.component.html',
  styleUrls: ['./SNSE054D_Z.component.css']
})



export class SNSE054D_ZComponent implements OnInit {


  constructor(
    private HttpService: HttpService,
    private router: Router,
    private global: Globals,
    private modal: ModalService,
    private viewref: ViewContainerRef
  ) { }


  public dataListSeqByInvoiceDate: any = [];
  public dataListCustByInvoiceDateBySeq: any = [];

  public PROCESSTYPE: any = {
    E21: "E21",
    E5: "E5",
    R5: "R5",
    I3: "I3"
  };

  public METHOD_E21: string = "SNSE054D_Z_PitkLotDate_Find_E21";
  public METHOD_E5: string = "SNSE054D_Z_PitkLotDate_Find_E5";
  public METHOD_R5: string = "SNSE054D_Z_PitkLotDate_Find_R5";
  public METHOD_I3: string = "SNSE054D_Z_PitkLotDate_Find_I3";

  public selectedProcessType: string;

  public ListDataReportCondition: any = {
    PROCESS_TYPE: null,
    PICK_LOT_DATE: null,
    PICK_LOT_SEQ: null,
    CUSTOMER_CODE: null
  };


  public ConfigPickLotDate: any = {
    ColName: "PICK_LOT_DATE",
    ColValue: "PICK_LOT_DATE",
    SourceController: "WH",
    SourceMethodName: "SNSE054D_Z_PitkLotDate_Find_E21",
    NoOfShowRecords: 100,
    MinLengthSearch: 1,
    UpperCase: true
  }

  ngOnInit() {
    this.ListDataReportCondition.PROCESS_TYPE = this.PROCESSTYPE.E21;
    this.ConfigPickLotDate.SourceMethodName = this.METHOD_E21;//Load Pick lot Date to autocomplete
  }

  FindSeqByInvoiceDate(strDate) {
    var PickLotDate = strDate.format('DD-MMM-YYYY');
    console.log(PickLotDate, 'PickLotDate');
    if (PickLotDate) {
      this.HttpService.get("WH", "SNSE054D_Z_SEQ_BY_DATE", PickLotDate || "null", this.ListDataReportCondition.PROCESS_TYPE)
        .subscribe(result => {

          this.dataListSeqByInvoiceDate = result.Table;
        }, err => {
          this.modal.error(err.message, this.viewref);
        });

    } else {
      this.modal.warning("Please input date for Pick Lot Date!!", this.viewref);
    }
  }



  public ConfigCustCode: any = {
    ColName: "CODE_NAME",
    ColValue: "CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "wPMS",
    SourceMethodName: "GetCustomerCode",
    NoOfShowRecords: 100,
    MinLengthSearch: 0,

  }

  radioChange(value: any) {
    this.clearData();

    //reload autocomplete follow radio change.
    if (this.ListDataReportCondition.PROCESS_TYPE == this.PROCESSTYPE.E21) {
      this.ConfigPickLotDate.SourceMethodName = this.METHOD_E21;
    } else if (this.ListDataReportCondition.PROCESS_TYPE == this.PROCESSTYPE.E5) {
      this.ConfigPickLotDate.SourceMethodName = this.METHOD_E5;
    }
    else if (this.ListDataReportCondition.PROCESS_TYPE == this.PROCESSTYPE.R5) {
      this.ConfigPickLotDate.SourceMethodName = this.METHOD_R5;
    } else if (this.ListDataReportCondition.PROCESS_TYPE == this.PROCESSTYPE.I3) {
      this.ConfigPickLotDate.SourceMethodName = this.METHOD_I3;
    }
  }

  clearData() {
    this.ListDataReportCondition.PICK_LOT_DATE = null;
    this.ListDataReportCondition.PICK_LOT_SEQ = null;
    this.ListDataReportCondition.CUSTOMER_CODE = null;
  }

  OpenReport() {
    if (this.CheckSearchCondition()) {
      console.log("Sent to Report:", this.ListDataReportCondition);

      this.SavePackingCardBeforePrint(this.ListDataReportCondition.PICK_LOT_DATE, this.ListDataReportCondition.PROCESS_TYPE, this.ListDataReportCondition.PICK_LOT_SEQ, this.ListDataReportCondition.CUSTOMER_CODE);

    }
  }

  CheckSearchCondition(): boolean {
    let boolHadSelectedCondition: boolean;
    boolHadSelectedCondition = false;//maening: User is not selecting condition search report.

    if (!this.ListDataReportCondition.PICK_LOT_DATE || !this.ListDataReportCondition.PICK_LOT_SEQ || !this.ListDataReportCondition.PROCESS_TYPE) {
      boolHadSelectedCondition = false;
    } else {
      boolHadSelectedCondition = true;
    }

    //Check Customer_Code if string emply or NULL will set to "null"
    if (!this.ListDataReportCondition.CUSTOMER_CODE) {
      this.ListDataReportCondition.CUSTOMER_CODE = "null";
    }

    return boolHadSelectedCondition;
  }//end CheckSearchCondition

  GoToReport() {
    this.global.GoToReport("PMS_RPT_WH_SNSE054R_ZB",
      {
        P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode")
      });
  }

  SavePackingCardBeforePrint(strDate: any, strProcessType: string, strSeq: string, strCustomerCode: any) {

    this.HttpService.get("WH", "SNSE054D_Z_SAVE_TO_TEMP", this.global.ConvertDateToFormatSTD(strDate), strProcessType, strSeq, strCustomerCode || "null")
      .subscribe(result => {
        console.log("result of save sticker: ", result);

        this.GoToReport();//show roport viewer.

      }, err => {
        this.modal.error(err.message, this.viewref);
      });
    //}
  }

}




