import { Component, OnInit, ViewChild, ViewContainerRef, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { Globals } from 'src/app/core/global';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import * as moment from 'moment';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'SNWR003',
  templateUrl: './SNWR003.component.html',
  styleUrls: ['../warehouse.component.css']
})

export class SNWR003Component implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private global: Globals
  ) { }

  @ViewChild("dialog", { static: true }) dialog: any;
  @ViewChild("autoCompleteVendor", { static: true }) autoCompleteVendor: AutoCompleteQueryComponent;

  public invoiceHeader: any = {};
  public invoiceDetail: any[] = [];
  public invoiceItem: any[] = [];
  public invoiceData: any = {};
  public dataIsSelected: any = [];

  public paramsSearch: any = {};
  public ShowList: boolean = true;
  public docList: Array<any> = [];
  public tempData: any = {};
  public iSCancel: boolean = false;

  public rowSave: any = [];
  public partNumber: string;

  public tblFooter: any = [
    {
      fieldName: "ACCEPT_PART_NO",
      value: "Total",
      show: true
    },
    {
      fieldName: "ORDER_QTY",
      show: true
    },
    {
      fieldName: "QTY",

      show: true
    },  
    {
      fieldName: "AMOUNT",

      show: true
    }]

  public selection = new SelectionModel<any>(true, []);


  public lookupConfig: any = {
    ColName: "LOOKUP_PART_VALUE",
    PlaceholderText: "Select Part",
    SourceController: "IC",
    SourceMethodName: "SNSQ005_GetPart",
    NoOfShowRecords: 50,
    UpperCase: true
  }







  ngOnInit() {
    this.paramsSearch.DOCUMENT_DATE_FROM = moment();
    this.paramsSearch.DOCUMENT_DATE_TO = moment();
  }

  public SetSelectedPart(value: any) {

    this.partNumber = value.PART_NO
    this.Search();
  }


  public SetVendorCodeSearch(value) {
    this.paramsSearch.VENDOR_CODE = value.VENDOR_CODE;
  }


  public Search() {

    if (this.partNumber) {
      this.http.post("WH", "SNWR003_GetDataByPart", {
        PART_NO: this.partNumber
      })
        .subscribe(result => {
          this.docList = result.Table;
        }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contact support team", this.viewRef);
        });
    } else {
      this.modal.warning("Please input Part No.", this.viewRef);
    }
    
  }


  public ViewDetail(invoiceData) {

    this.invoiceHeader = {};
    this.invoiceDetail = [];

    this.invoiceData = invoiceData;
    this.invoiceData.PART_NO = this.partNumber;    

    this.ShowList = false;
    this.http.post("WH", "SNWR003_GetInvoiceDetail", invoiceData).subscribe(
      (res) => {
        this.invoiceHeader = res.Table[0];
        this.invoiceDetail = res.Table1;
        this.invoiceItem = res.Table2;        

      },
      (err) => {
        this.modal.warning(
          "Found an error, please contact support team",
          this.viewRef
        );
      }
    );
  }


  QuantityChange(data) {
    if (data.QTY > data.ACCEPT_QTY) {
      data.QTY = data.ACCEPT_QTY
    }
  }

  SetNewPartNo(part: any, data: any) {
    this.tempData.Part = part;
    data.ACCEPT_PART_NO = part.PART_NO;
  }


  public TotalQty() {
    return this.invoiceDetail.reduce((a, b) => a + (b.QTY || 0), 0);
  }

  public TotalAmount() {
    return this.invoiceDetail.reduce((a, b) => a + ((b.QTY || 0) * (b.UNIT_PRICE_B || 0)), 0);
  }

  public TotalItem() {
    return this.invoiceDetail.reduce(
      (a, b) => a + (((b.QTY || 0) / b.RECEIVE_PACKAGE) || 0),
      0
    );
  }

  public BackToGrid() {
    this.Search();
    this.iSCancel = false;
    this.ShowList = true;
  }


  public isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.invoiceDetail.length;
    return numSelected === numRows;
  }

  public Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
      this.invoiceDetail.map(x => {
        this.dataIsSelected = [];
      })
    } else {
      this.invoiceDetail.forEach(row => this.selection.select(row));
      this.dataIsSelected = [];
      this.invoiceDetail.map(x => {
        this.dataIsSelected.push(x);
      })
    }
   
  }

  public CheckboxChange(event, data) {
    if (event.checked) {
      
      this.dataIsSelected.push(data);
    }
    else {
      this.dataIsSelected.splice(this.dataIsSelected.indexOf(data), 1);
    }

  }

  public RePrint() {
    
    if (this.dataIsSelected.length > 0) {     
    
      this.modal.confirm("Are you sure to re-print the receiving card?", this.viewRef
        , () => {

          var lSeqList = '0';
          this.dataIsSelected.forEach((row) => {
            lSeqList = lSeqList + ',' + row.SEQ;
          });


          this.global.GoToReport("PMS_RPT_WMS_SNPE015",
            {
              P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
              P_DOCUMENT_NO: this.invoiceHeader.DOCUMENT_NO
            }

          //this.global.GoToReport("PMS_RPT_WH_SNPR028",
          //  {
          //    P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
          //    P_DOCUMENT_NO: this.invoiceHeader.RECEIVE_DOC_NO,
          //    P_DOCUMENT_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.CREATE_DATE),
          //    P_VENDOR_CODE: this.invoiceHeader.VENDOR,
          //    P_INVOICE_NO: this.invoiceHeader.INVOICE_NO,
          //    P_INVOICE_DATE: this.global.ConvertDateToFormatSTD(this.invoiceHeader.INVOICE_DATE),
          //    P_PART_NO: this.invoiceHeader.PART_NO,
          //    P_SEQ_LIST: lSeqList,
          //  }
          );
        }, () => { })
    } else {
      this.modal.error("Please select the Seq. to re-print", this.viewRef);
    }
  }



}

