import {
  Component,
  ElementRef,
  HostListener,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { Observable, Subject } from "rxjs";
import {
  PickToken,
  PickTokenExtractor,
} from "src/app/core/WHOutboundController/PickTokenExtractor";
import {
  InvoiceControl,
  InvoiceControlSetValue,
  CaseNo,
} from "src/app/core/WHOutboundController/InvoiceControl";
import { map } from "rxjs/operators";
import * as moment from "moment";
import { DialogTemplateComponent } from "src/app/components/shared/dialog-template/dialog-template.component";
import { Globals } from "src/app/core/global";

type ActionLog = {
  timestamp: moment.Moment;
  actionType: string;
  actionMessage: string;
};
@Component({
  selector: "SNWR008",
  templateUrl: "./SNWR008.component.html",
  styleUrls: ["../warehouse.component.css", "./SNWR008.component.css"],
  providers: [PickTokenExtractor],
})
export class SNWR008Component implements OnInit {
  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private extractor: PickTokenExtractor,
    private global: Globals
  ) {}

  public qrString: string;
  public params: any = {};
  private tableData: any = [];
  public tableDataFiltered: any = [];
  public totalItems: number = 0;
  private prevPicking: PickToken;
  public pickingList: string[] = [];

  public showHistory: boolean = false;
  public actionLogs: ActionLog[] = [];
  private actionLogsSubject: Subject<ActionLog> = new Subject<ActionLog>();

  public unlockPartDetail: any = {};
  public lockQty: boolean = true;
  @ViewChild("inputQtyDialog") inputQtyDialog: DialogTemplateComponent;
  @ViewChild("inputCaseDialog") inputCaseDialog: DialogTemplateComponent;
  @ViewChild("saveButton", { read: ElementRef }) saveButton: ElementRef;
  @ViewChild("saveCaseTypeButton", { read: ElementRef })
  saveCaseTypeButton: ElementRef;
  @ViewChild("scannerInput", { read: ElementRef }) scannerInput: ElementRef;

  public initialControl: InvoiceControl = new InvoiceControl();
  private initialControlSubject: Subject<InvoiceControlSetValue> =
    new Subject<InvoiceControlSetValue>();

  public selectedParts: any[];
  public selectedPartsFiltered: any[];
  public scannedCase: any;
  public selectedCases: CaseNo[];
  public popupCase: CaseNo;
  public isInvoiceIssued: boolean = false;
  public scannedQR: PickToken;

  public lookupPickListNoConfig: any = {
    ColName: "PICKING_LIST_NO",
    PlaceholderText: "Select Picking List No.",
    SourceController: "WH",
    SourceMethodName: "SNWR008_getPickListNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
  };

  public lookupOrderNoConfig: any = {
    ColName: "LOOKUP_ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "WH",
    SourceMethodName: "SNWR008_GetOrderNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: null,
      BRANCH_CODE: null,
    },
    UpperCase: true,
  };

  public lookupPickLotSeqConfig: any = {
    ColName: "LOOKUP_PICK_LOT_SEQ",
    PlaceholderText: "Select Pick Lot Seq",
    SourceController: "WH",
    SourceMethodName: "SNWR008_GetPickLotSeq",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params,
    },
    UpperCase: true,
  };

  ngOnInit() {
    this.InitialProcess();

    this.initialControlSubject
      .pipe(map((control) => control))
      .subscribe((control) => {
        if (control) {
          this.initialControl.set(control);

          if (control.propName === "QR")
            this.CheckFinishedForChangeInstruction() &&
              this.SetScannedQR(this.initialControl.QR);
          if (control.propName === "partNo")
            this.SetScannedPart(this.initialControl.partNo);
          if (control.propName === "caseNo")
            this.SetScannedCase(this.initialControl.caseNo);

          this.qrString = undefined;
        }
      });

    this.actionLogsSubject.subscribe((action: ActionLog) => {
      this.actionLogs.push(action);
      this.actionLogs = this.actionLogs.slice();
    });
  }

  private InitialProcess() {
    this.selectedCases = [];
    this.selectedParts = [];
    this.selectedPartsFiltered = [];
    this.totalItems = 0;
    this.scannedCase = {};
    this.lockQty = true;
    this.params = {};
    this.qrString = null;
    this.tableData = [];
    this.tableDataFiltered = [];
    this.pickingList = [];
    this.unlockPartDetail = {};
    this.actionLogs = [];
    this.initialControl.reset();
    this.isInvoiceIssued = false;
    this.prevPicking = null;
  }

  public ResetData() {
    if (this.scannedCase) {
      this.modal.confirm("Are you sure to reset data?", this.viewref, () => {
        this.InitialProcess();
      });
    } else {
      this.InitialProcess();
    }
    this.scannerInput.nativeElement.focus();
  }

  public ToggleLockQty() {
    this.lockQty = !this.lockQty;
  }

  public ConfirmIssueInvoice() {
    if (this.CheckFinishedForSaveInstruction()) {
      this.modal.confirm(
        "Do you want to issue invoice? " +
          "[Selected : " +
          this.totalItems +
          " Items]",
        this.viewref,
        this.IssueInvoice
      );
    }
  }

  private IssueInvoice = () => {
    
    this.selectedCases.forEach((caseNo) => {
      this.selectedParts
        .filter((item) => item.CASE_NO === caseNo.caseNo)
        .forEach((item) => {
          item.CASE_DATE = caseNo.caseDate;
          item.CASE_TYPE = caseNo.caseType;
        });
    });

    this.Http.post("WH", "SNWR008_IssueInvoice", this.selectedParts).subscribe(
      (res) => {
        if (res.status === "Done") {
          const invoiceNoResult =
            res.invoiceNoFrom === res.invoiceNoTo
              ? res.invoiceNoFrom
              : `${res.invoiceNoFrom} - ${res.invoiceNoTo}`;
          this.modal.info(
            `Issue invoice completed.<br/><br/>Invoice numbers: ${invoiceNoResult}`,
            this.viewref
          );
          this.params.INVOICE_NO_FROM = res.invoiceNoFrom;
          this.params.INVOICE_NO_TO = res.invoiceNoTo;
          this.isInvoiceIssued = true;
          // this.Search();
        } else if (res.status === "Fail") {
          this.modal.error(res.massage, this.viewref);
        } else {
          this.modal.error(
            "Found error, please contact support team",
            this.viewref
          );
        }
      },
      (err) => {
        this.modal.error(
          "Found error, please contact support team",
          this.viewref
        );
      }
    );
  }

  public async Search() {
    let tableDataBK = JSON.parse(JSON.stringify(this.tableData));
    this.tableData = [];

    if (this.params.CUSTOMER_CODE && this.params.BRANCH_CODE) {
      await this.Http.post("WH", "SNWR008_GetData", this.initialControl.QR)
        .toPromise()
        .then((res) => {
          this.tableData = res.Table;

          const {
            CUSTOMER_CODE,
            BRANCH_CODE,
            CUSTOMER_NAME,
            BRANCH_NAME,
            CUSTOMER_ZONE,
          } = this.tableData[0];
          this.params = {
            CUSTOMER_CODE,
            BRANCH_CODE,
            CUSTOMER_NAME,
            BRANCH_NAME,
            CUSTOMER_ZONE,
          };

          this.pickingList = res.Table.reduce((list: string[], item: any) => {
            if (!list.includes(item.PICKING_LIST_NO))
              list.push(item.PICKING_LIST_NO);

            return list;
          }, []);

          this.params.ORDER_TYPE = this.tableData[0]?.ORDER_TYPE || undefined;

          this.tableData = this.tableData.map((item) => {
            const foundBK = tableDataBK.find(
              (itemBK) => itemBK.INSTRUCTION_NO === item.INSTRUCTION_NO
            );
            if (foundBK) {
              item.AVAILABLE_QTY = foundBK.AVAILABLE_QTY;
              item.INVOICE_QTY = foundBK.INVOICE_QTY;
            }

            return item;
          });

          tableDataBK = undefined;
        });
    } else {
      this.modal.warning(
        "Please input customer and branch code.",
        this.viewref
      );
    }
  }

  public PrintInvoices() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR023_INVOICE", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_INVOICE_NO_FROM: this.params.INVOICE_NO_FROM,
      P_INVOICE_NO_TO: this.params.INVOICE_NO_TO,
    });
  }

  public PrintDelivery() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR023_DELIVERY", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_INVOICE_NO_FROM: this.params.INVOICE_NO_FROM,
      P_INVOICE_NO_TO: this.params.INVOICE_NO_TO,
    });
  }

  public OrderClassChange(event) {
    this.params.ORDER_CLASS = event.value;
  }

  private async GetPartConfigure(partNo: string) {
    return await this.Http.get("WH", "SNWR008_GetPartConf", partNo).toPromise(); //awaitable
  }

  private async SetScannedQR(pickToken: PickToken) {
    //Find picking list no from the list
    const foundPickingList = this.tableData.find(
      (item) => item.INSTRUCTION_NO === pickToken.INSTRUCTION_NO
    );
    if (foundPickingList) {
      this.prevPicking = pickToken;
      this.FilterTableData();
      this.actionLogsSubject.next({
        timestamp: moment(),
        actionType: "Scan",
        actionMessage: `Picking List No ${this.initialControl.QR.PICKING_LIST_NO}, Instruction No ${this.initialControl.QR.INSTRUCTION_NO}`,
      });
    } else {
      await this.Search();
      const foundPickingList = this.tableData.find(
        (item) => item.INSTRUCTION_NO === pickToken.INSTRUCTION_NO
      );
      if (foundPickingList) {
        this.prevPicking = pickToken;
        this.FilterTableData();
        this.actionLogsSubject.next({
          timestamp: moment(),
          actionType: "Scan",
          actionMessage: `Picking List No ${this.initialControl.QR.PICKING_LIST_NO}, Instruction No ${this.initialControl.QR.INSTRUCTION_NO}`,
        });
      } else {
        this.initialControl.QR = this.prevPicking;
        this.modal.warning("Invalid picking card.", this.viewref);
      }
    }
    this.scannerInput.nativeElement.focus();
  }

  private async SetScannedPart(partNo: string) {
    if (!this.scannedCase.caseNo) {
      this.modal.warning("Please scan case no.", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
      return;
    }
    let foundAllPart = this.tableData.filter((item) =>
      this.ConditionScannedPart(partNo, item)
    ); //Find all matched part number

    let foundAvailablePart = foundAllPart.find(
      (item) => item.AVAILABLE_QTY > 0
    ); //Find only available

    if (foundAllPart && foundAvailablePart) {
      // //Find unconfigured across cases
      // const foundAnotherCases = this.selectedParts.find(part => part.CASE_NO !== this.scannedCase.caseNo);

      // //Get part configure
      // let exceptionConf;
      // await this.GetPartConfigure(partNo)
      //   .then(result => {
      //     exceptionConf = result[0].EXCEPTION_CONFIG || 'N';
      //   })

      // if(foundAnotherCases && exceptionConf === 'N') {
      //   setTimeout(() => this.modal.warning("Please config exception case.", this.viewref),500)
      // }

      //Case found available
      if (this.lockQty && foundAvailablePart.ACCEPT_QTY <= 10) {
        //Auto move available qty by sales package
        const packageQty = foundAvailablePart.PACKAGE_QTY;

        foundAvailablePart.AVAILABLE_QTY =
          (foundAvailablePart.AVAILABLE_QTY || 0) - packageQty;
        foundAvailablePart.INVOICE_QTY =
          (foundAvailablePart.INVOICE_QTY || 0) + packageQty;

        this.SetSelectedPart(foundAvailablePart, packageQty);
      } else {
        //Manually move available qty by input
        const availableQty = foundAllPart.reduce((qty, item) => {
          qty = qty + item.AVAILABLE_QTY;
          return qty;
        }, 0);
        this.unlockPartDetail = {
          PART_NO: partNo,
          AVAILABLE_QTY: availableQty,
          QTY: 0,
        };

        this.inputQtyDialog.OpenDialog();
      }
    } else if (foundAllPart && !foundAvailablePart) {
      //Case not found available
      this.modal.warning("Parts is unavailable.", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
    } else {
      //Case not found matched part number
      this.modal.error("Parts not found.", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
    }
  }

  public SetInputQty() {
    if (this.unlockPartDetail.QTY <= this.unlockPartDetail.AVAILABLE_QTY) {
      //Case input QTY <= available
      let filterAvailableParts = this.tableData.filter(
        (item) =>
          this.ConditionScannedPart(this.unlockPartDetail.PART_NO, item) &&
          item.AVAILABLE_QTY > 0
      ); //Find available
      filterAvailableParts.forEach((part) => {
        const qty = Math.min(this.unlockPartDetail.QTY, part.AVAILABLE_QTY);
        part.AVAILABLE_QTY = (part.AVAILABLE_QTY || 0) - qty;
        part.INVOICE_QTY = (part.INVOICE_QTY || 0) + qty;
        this.unlockPartDetail.QTY = this.unlockPartDetail.QTY - qty;

        this.SetSelectedPart(part, qty); //Set data to invoice list
      });

      this.unlockPartDetail = {};
      this.inputQtyDialog.Close();
      this.lockQty = true;
      this.scannerInput.nativeElement.focus();
    } else {
      this.modal.warning("Invalid QTY.", this.viewref);
    }
  }

  private SetSelectedPart(data: any, qty: number) {
    let foundCasePart = this.selectedParts.find(
      (item) =>
        item.PART_NO === data.PART_NO &&
        item.ORDER_NO === data.ORDER_NO &&
        item.ITEM_NO === data.ITEM_NO &&
        item.SUB_ITEM_NO === data.SUB_ITEM_NO &&
        item.UNIT_PRICE === data.UNIT_PRICE &&
        item.CASE_NO === this.scannedCase.caseNo
    ); //Find existing part in current case number

    if (!foundCasePart) {
      //Insert when no data found
      let scannedPart = Object.assign({}, data);
      scannedPart.CASE_NO = this.scannedCase.caseNo;
      scannedPart.CASE_DATE = this.scannedCase.caseDate;
      scannedPart.CASE_TYPE = this.scannedCase.caseType;
      scannedPart.QTY = qty;
      scannedPart.TOTAL_AMOUNT = scannedPart.QTY * scannedPart.UNIT_PRICE;
      this.selectedParts.push(scannedPart);
    } else {
      //Update QTY and total amount
      foundCasePart.QTY = (foundCasePart.QTY || 0) + qty;
      foundCasePart.TOTAL_AMOUNT = foundCasePart.QTY * foundCasePart.UNIT_PRICE;
    }
    this.selectedParts = this.selectedParts.slice();
    this.FilterTableData();
    this.actionLogsSubject.next({
      timestamp: moment(),
      actionType: "Scan",
      actionMessage: `Add Part No. ${data.PART_NO} ${qty} Qty into ${this.scannedCase.caseNo}`,
    });
    this.scannerInput.nativeElement.focus();
  }

  private SetScannedCase(CaseNo?: CaseNo) {
    if (this.params.CUSTOMER_ZONE === "UPC") {
      if (CaseNo) {
        let foundIndex = this.selectedCases.findIndex(
          (item) => item.caseNo === CaseNo.caseNo
        );
        CaseNo.caseType = "BOX";
        this.scannedCase = CaseNo;

        if (foundIndex < 0) {
          this.selectedCases.push(CaseNo);
          this.scannedCase.index = this.selectedCases.length + 1;
        } else {
          this.scannedCase.index = foundIndex + 1;
        }

        this.actionLogsSubject.next({
          timestamp: moment(),
          actionType: "Scan",
          actionMessage: `Case No. ${this.scannedCase.caseNo}`,
        });
      } else {
        this.scannedCase.index = 0;
      }
    } else {
      if (CaseNo) {
        this.popupCase = CaseNo;
        this.inputCaseDialog.OpenDialog();
      } else {
        this.scannedCase.index = 0;
      }
    }
    this.scannerInput.nativeElement.focus();
  }

  public SaveCaseType(CaseNo: CaseNo) {
    if (CaseNo.caseType) {
      let foundIndex = this.selectedCases.findIndex(
        (item) => item.caseNo === CaseNo.caseNo
      );
      this.scannedCase = CaseNo;

      if (foundIndex < 0) {
        this.selectedCases.push(CaseNo);
        this.scannedCase.index = this.selectedCases.length + 1;
      } else {
        this.scannedCase.index = foundIndex + 1;
      }

      this.actionLogsSubject.next({
        timestamp: moment(),
        actionType: "Scan",
        actionMessage: `Case No. ${this.scannedCase.caseNo}`,
      });
      this.inputCaseDialog.Close();
      this.scannerInput.nativeElement.focus();
    } else {
      this.modal.warning("Please input Case Type.", this.viewref);
    }
  }

  public CancelSaveCase(CaseNo: CaseNo) {
    this.inputCaseDialog.Close();
    this.scannerInput.nativeElement.focus();
  }

  public DeleteCaseNo(CaseNo: CaseNo, caseIndex: number) {
    const foundParts = this.selectedParts.find(
      (item) => item.CASE_NO === CaseNo.caseNo
    );
    if (foundParts) {
      this.modal.warning("Unable to remove case.", this.viewref);
    } else {
      this.modal.confirm(
        "Are you sure you want to delete this case?",
        this.viewref,
        () => {
          this.selectedCases.splice(caseIndex, 1);
          this.FilterSelectedParts(this.selectedCases[caseIndex - 1]?.caseNo);
        },
        () => {}
      );
    }
  }

  private async GetCaseNo(caseNo: string) {
    return await this.Http.get(
      "WH",
      "SNWR008_GetAvailableCase",
      "DOMESTIC",
      caseNo
    )
      .toPromise()
      .catch((err) => {
        this.modal.error(
          "Found error, please contact support team",
          this.viewref,
          () => {
            this.scannerInput.nativeElement.focus();
          }
        );
      });
  }

  public RemoveSelectedParts(partItem: any, caseNo?: string) {
    if (!this.isInvoiceIssued) {
      this.modal.confirm(
        "Are you sure to remove selected part?",
        this.viewref,
        () => {
          let foundIndex = 0;
          let qty = 0;
          while (foundIndex >= 0) {
            foundIndex = this.selectedParts.findIndex(
              (item) =>
                item.PART_NO === partItem.PART_NO &&
                item.UNIT_PRICE === partItem.UNIT_PRICE &&
                item.CASE_NO === (caseNo || item.CASE_NO)
            );
            qty = this.selectedParts[foundIndex]?.QTY;

            if (foundIndex >= 0) {
              let removedItem = this.selectedParts.splice(foundIndex, 1);
              this.RevertRemainingData(partItem, qty);

              this.actionLogsSubject.next({
                timestamp: moment(),
                actionType: "Manual",
                actionMessage: `Remove Part No. ${partItem.PART_NO} ${qty} Qty from ${removedItem[0].CASE_NO}`,
              });
            }
          }
          this.selectedParts = this.selectedParts.slice();
          this.scannerInput.nativeElement.focus();
        },
        () => {}
      );
    } else {
      this.modal.warning("Cannot remove issued invoice!!", this.viewref);
    }
  }

  private RevertRemainingData(partItem: any, qty: number) {
    //Remove from current picking list
    let foundAllPart = this.tableData.filter(
      (item) =>
        item.PART_NO === partItem.PART_NO &&
        item.UNIT_PRICE === partItem.UNIT_PRICE &&
        item.INVOICE_QTY > 0 &&
        item.PICKING_LIST_NO === this.initialControl.QR.PICKING_LIST_NO
    );
    foundAllPart.forEach((item) => {
      const revertableQty = Math.min(item.INVOICE_QTY, qty);
      item.INVOICE_QTY = item.INVOICE_QTY - revertableQty;
      item.AVAILABLE_QTY = item.AVAILABLE_QTY + revertableQty;
      qty = qty - revertableQty;
    });

    //Remove from all picking list
    if (qty > 0) {
      foundAllPart = this.tableData.filter(
        (item) =>
          item.PART_NO === partItem.PART_NO &&
          item.UNIT_PRICE === partItem.UNIT_PRICE &&
          item.INVOICE_QTY > 0
      );
      foundAllPart.forEach((item) => {
        const revertableQty = Math.min(item.INVOICE_QTY, qty);
        item.INVOICE_QTY = item.INVOICE_QTY - revertableQty;
        item.AVAILABLE_QTY = item.AVAILABLE_QTY + revertableQty;
        qty = qty - revertableQty;
      });
    }

    this.FilterTableData();
  }

  @HostListener("window:keydown", ["$event"])
  async onKeyDown(event: KeyboardEvent) {
    if (
      event.key.toUpperCase() === "ENTER" &&
      (<HTMLElement>event.target).id.toUpperCase() === "QRINPUT"
    ) {
      try {
        const tokenDetail: PickToken = this.extractor.getToken(this.qrString);

        //Initial scan
        if (tokenDetail instanceof PickToken) {
          //Check if same customer
          if (
            this.params.CUSTOMER_CODE &&
            (tokenDetail.CUSTOMER_CODE !== this.params.CUSTOMER_CODE ||
              tokenDetail.BRANCH_CODE !== this.params.BRANCH_CODE)
          ) {
            this.modal.warning("Wrong customer code!!.", this.viewref, () => {
              this.scannerInput.nativeElement.focus();
            });
          } else {
            if (!this.params.CUSTOMER_CODE) {
              this.params = tokenDetail;
            }
            //Set QR code
            this.initialControlSubject.next({
              propName: "QR",
              value: tokenDetail,
            });
          }
        }
      } catch (ex) {
        if (ex === "Invalid QR") {
          if (this.qrString.length <= 5) {
            const checkCase: CaseNo = await this.GetCaseNo(this.qrString).then(
              (response) => {
                if (
                  response.Result === "Done" &&
                  response.CaseNo === this.qrString
                ) {
                  const caseNo: CaseNo = new CaseNo(
                    response.CaseNo,
                    response.PrintDate
                  );

                  return caseNo;
                } else if (response.Result === "Failed") {
                  this.modal.warning(response.Message, this.viewref, () => {
                    this.scannerInput.nativeElement.focus();
                  });
                } else {
                  this.modal.warning(
                    "Case number not found.",
                    this.viewref,
                    () => {
                      this.scannerInput.nativeElement.focus();
                    }
                  );
                  return null;
                }
              }
            );
            if (checkCase) {
              this.initialControlSubject.next({
                propName: "caseNo",
                value: checkCase,
              });
            }
          } else {
            const checkPart = this.tableData.find(
              (item) =>
                item.PART_NO === this.qrString &&
                item.INSTRUCTION_NO === this.initialControl.QR.INSTRUCTION_NO
            );
            if (checkPart) {
              this.initialControlSubject.next({
                propName: "partNo",
                value: this.qrString,
              });
            } else {
              this.modal.warning("Invalid scan data.", this.viewref, () => {
                this.scannerInput.nativeElement.focus();
              });
            }
          }
        } else {
          this.modal.warning(
            "Found error, please contact support team",
            this.viewref,
            () => {
              this.scannerInput.nativeElement.focus();
            }
          );
        }
      }
      this.qrString = null;
    }
  }

  public CalculateInvoiceAmount() {
    const invoiceAmount = this.selectedParts.reduce(
      (amount, item) => amount + item.QTY * item.UNIT_PRICE,
      0
    );
    return invoiceAmount;
  }

  private FilterTableData() {
    this.tableDataFiltered = this.tableData.filter(
      (row) =>
        row.PICKING_LIST_NO === this.initialControl.QR.PICKING_LIST_NO &&
        row.AVAILABLE_QTY > 0
    );

    //Filter show data
    this.FilterSelectedParts(this.scannedCase?.caseNo);

    //Get total items
    this.totalItems = this.GetFilteredSelectedParts(null).length;
  }

  public FilterSelectedParts(caseNo: string, isReturn: boolean = false) {
    caseNo = caseNo === "All" ? null : caseNo;

    //Set case number tab and filter show data
    let caseIndex = this.selectedCases.findIndex(
      (item) => item.caseNo === caseNo
    );
    this.scannedCase.index = caseIndex + 1;

    //Get filtered part
    this.selectedPartsFiltered = this.GetFilteredSelectedParts(caseNo);
  }

  private GetFilteredSelectedParts(caseNo: string) {
    //Group part number
    let count = 0;
    const selectedParts = JSON.parse(JSON.stringify(this.selectedParts));
    const filtered = selectedParts
      .filter((item) => item.CASE_NO === (caseNo || item.CASE_NO))
      .reduce((list: any[], item: any) => {
        let foundMatched = list.find(
          (v) => v.PART_NO === item.PART_NO && v.UNIT_PRICE === item.UNIT_PRICE
        );

        if (!foundMatched) {
          count++;
          item.SEQ = count;
          list.push(item);
        } else {
          foundMatched.QTY = foundMatched.QTY + item.QTY;
          foundMatched.TOTAL_AMOUNT =
            foundMatched.QTY * foundMatched.UNIT_PRICE;
        }

        return list;
      }, []);

    return filtered;
  }

  private ConditionScannedPart(partNo: string, item: any) {
    return (
      item.PART_NO === partNo &&
      item.PICKING_LIST_NO === this.initialControl.QR.PICKING_LIST_NO &&
      item.INSTRUCTION_NO === this.initialControl.QR.INSTRUCTION_NO
    );
  }

  private CheckFinishedForSaveInstruction() {
    const foundUnfinishedInstruction = this.tableData.find(
      (item) => item.AVAILABLE_QTY > 0 && item.INVOICE_QTY > 0
    );
    
    if (foundUnfinishedInstruction) {
      this.modal.warning("Please finished instruction.", this.viewref);
    }
    return !foundUnfinishedInstruction;
  }

  private CheckFinishedForChangeInstruction() {
    if (!this.prevPicking) return true;

    const instructionItem = this.tableData.find(
      (item) => item.INSTRUCTION_NO === this.prevPicking.INSTRUCTION_NO
    );

    const isFinishedInstruction =
      this.initialControl.QR.INSTRUCTION_NO ===
        this.prevPicking.INSTRUCTION_NO ||
      instructionItem.AVAILABLE_QTY === 0 ||
      !instructionItem.INVOICE_QTY;
    if (!isFinishedInstruction) {
      this.initialControl.QR = this.prevPicking;
      this.modal.warning("Please finished instruction.", this.viewref);
    }

    return isFinishedInstruction;
  }
}
