import { SelectionModel } from "@angular/cdk/collections";
import { Component, OnInit, ViewChild, ViewContainerRef } from "@angular/core";
import { FormControl } from "@angular/forms";
import { MatDialog } from "@angular/material/dialog";
import { Observable } from "rxjs";
import { map, startWith } from "rxjs/operators";
import { DialogTemplateComponent } from "src/app/components/shared/dialog-template/dialog-template.component";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "snwr010_bkk",
  templateUrl: "SNWR010_BKK.component.html",
  styleUrls: ["SNWR010.component.css"],
})
export class SNWR010_BKKComponent implements OnInit {
  constructor(
    private global: Globals,
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private dialog: MatDialog
  ) {}

  ngOnInit() {
    this.GetBKKDeliveryList();
  }

  public sentList: SelectionModel<any> = new SelectionModel<any>(true, []);
  public unsentList: SelectionModel<any> = new SelectionModel<any>(true, []);
  public deliveryList: any[] = [];
  public unDeliveryList: any[] = [];

  public deliveryRouteList: any[] = [];
  public bkkCustomers: any[] = [];
  public selectCustomerControl = new FormControl();
  public filteredCustomerOptions: Observable<any[]>;
  @ViewChild("driverDialog", { read: DialogTemplateComponent })
  public driverDialog: DialogTemplateComponent;
  public carList: any[] = [];

  public GetBKKDeliveryList() {
    this.deliveryList = [];
    this.sentList.clear();
    this.unsentList.clear();
    this.http.get("WH", "SNWR010_GetBKKDeliveryList").subscribe(
      (res) => {
        this.deliveryList = res.Table.filter(
          (item) => (item) =>
            item.DELIVERY_ROUTE >= 1 && item.DELIVERY_ROUTE <= 10
        );
        this.unDeliveryList = res.Table1;
      },
      (err) => {
        console.log(err);
        this.modal.error(
          "Found eror, please contact support team.",
          this.viewRef
        );
      }
    );
  }

  public async SaveDeliveryList() {
    await this.http
      .post("WH", "SNWR010_SaveSentList", this.sentList.selected)
      .toPromise()
      .then()
      .catch((err) => {
        this.modal.error(
          "Found eror, please contact support team.",
          this.viewRef
        );
      });

    await this.http
      .post("WH", "SNWR010_SaveUnSentList", this.unsentList.selected)
      .toPromise()
      .then()
      .catch((err) => {
        this.modal.error(
          "Found eror, please contact support team.",
          this.viewRef
        );
      });

    await this.http
      .post("WH", "SNWR010_ConfirmBKK", {})
      .toPromise()
      .then()
      .catch((err) => {
        this.modal.error(
          "Found eror, please contact support team.",
          this.viewRef
        );
      });

    this.modal.info("Save delivery completed.", this.viewRef);
    this.GetBKKDeliveryList();
  }

  public PrintDeliveryList() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR010_DELIVERY_SHEET_DLR", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_TYPE: "BKK",
    });
  }

  public ToggleDelivery(event: any, data: any) {
    if (data.STATUS === "Y") {
      this.unsentList.toggle({
        INVOICE_NO: data.INVOICE_NO,
        CASE_NO: data.CASE_NO,
      });
    }
    if (data.STATUS === "N") {
      this.sentList.toggle({
        INVOICE_NO: data.INVOICE_NO,
        CASE_NO: data.CASE_NO,
      });
    }
  }

  public async OpenDriverListDialog() {
    await this.GetDeliveryRoute();
    await this.GetBKKCustomers();

    this.filteredCustomerOptions = this.selectCustomerControl.valueChanges.pipe(
      startWith(""),
      map((value) =>
        typeof value === "string" ? value : this.displayFn(value)
      ),
      map((custValue) => this._filter(custValue))
    );

    this.driverDialog.OpenDialog();
  }

  private async GetDeliveryRoute() {
    this.deliveryRouteList = [];
    this.carList = [];
    await this.http
      .get("WH", "SNWR010_GetRoute", "BKK")
      .toPromise()
      .then(
        (res) => {
          this.deliveryRouteList = res.Table;
          this.carList = res.Table1;

          this.deliveryRouteList.map((route) => {
            this.ChangeRouteCar(route.CAR_NO, route);
          });
        },
        (err) => {
          this.modal.error(
            "Found eror, please contact support team.",
            this.viewRef
          );
        }
      );
  }

  private async GetBKKCustomers() {
    this.bkkCustomers = [];
    await this.http
      .get("WH", "SNWR010_GetBKKCustomers")
      .toPromise()
      .then(
        (res) => {
          this.bkkCustomers = res;

          this.deliveryRouteList.forEach((route) => {
            route.CUSTOMER_LIST = this.bkkCustomers.filter(
              (customer) => customer.ROUTE_NO === route.ROUTE_NO
            );
          });
        },
        (err) => {
          this.modal.error(
            "Found eror, please contact support team.",
            this.viewRef
          );
        }
      );
  }

  public SaveRouteData() {
    const changedRoutes = this.deliveryRouteList.filter(
      (route) => route._IS_CHANGE
    );
    const changedCustomers = this.bkkCustomers.filter(
      (customer) => customer._IS_CHANGE
    );

    this.http
      .post("WH", "SNWR010_SaveRoute", {
        routes: changedRoutes,
        customers: changedCustomers,
      })
      .subscribe(
        (res) => {
          if (res === "Done") {
            this.driverDialog.Close();
            this.GetBKKDeliveryList();
          }
        },
        (err) => {
          this.modal.error(
            "Found eror, please contact support team.",
            this.viewRef
          );
        }
      );
  }

  private _filter(inputValue: string) {
    const filterValue = (inputValue || "").toLowerCase();

    const filteredOption = this.bkkCustomers.filter(
      (customer) =>
        !customer.ROUTE_NO &&
        this.displayFn(customer).toLowerCase().includes(filterValue)
    );

    return filteredOption;
  }

  public displayFn(customer: any): string {
    return customer
      ? `${customer.LABEL_NAME} (${customer.CUSTOMER_CODE}${customer.BRANCH_CODE})`
      : "";
  }

  public AddCustomer(route: any) {
    let selectedCustomer = this.selectCustomerControl.value;

    selectedCustomer.ROUTE_NO = route.ROUTE_NO;
    this.MarkCustomerChange(selectedCustomer, route);

    route.CUSTOMER_LIST.push(selectedCustomer);
    this.SortDeliveryTime(route);
    this.deliveryRouteList = this.deliveryRouteList.slice();
    this.selectCustomerControl.reset();
  }

  public RemoveCustomer(selectedCustomer: any, route: any) {
    const customerIndex = route.CUSTOMER_LIST.findIndex(
      (customer) =>
        customer.CUSTOMER_CODE === selectedCustomer.CUSTOMER_CODE &&
        customer.BRANCH_CODE === selectedCustomer.BRANCH_CODE
    );

    selectedCustomer.ROUTE_NO = null;
    this.MarkCustomerChange(selectedCustomer, route);

    route.CUSTOMER_LIST.splice(customerIndex, 1);
    this.deliveryRouteList = this.deliveryRouteList.slice();
  }

  public SortDeliveryTime(route: any) {
    route.CUSTOMER_LIST = route.CUSTOMER_LIST.sort(
      (a, b) => a.DELIVERY_TIME - b.DELIVERY_TIME
    );
  }

  public MarkCustomerChange(customer: any, route: any) {
    route._IS_CHANGE = true;
    customer._IS_CHANGE = true;
  }

  public MarkRouteChange(route: any) {
    route._IS_CHANGE = true;
  }

  public ChangeRouteCar(selected: any, route: any, changed?: boolean) {
    if (selected) {
      const carInfo = this.carList.find((car) => car.CAR_NO === selected);
      route = Object.assign(route, carInfo);
      changed && this.MarkRouteChange(route);

      this.deliveryRouteList = this.deliveryRouteList.slice();
    }
  }
}
