import {
  Component,
  ElementRef,
  HostListener,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { Subject } from "rxjs";
import { map } from "rxjs/operators";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "SNWR014",
  templateUrl: "SNWR014.component.html",
})
export class SNWR014Component implements OnInit {
  constructor(private http: HttpService, private modal: ModalService, private viewRef: ViewContainerRef) {}
  public fileUrl;
  public showPreview = false;
  private downloadFileSubject: Subject<any> = new Subject<any>();

  @ViewChild("iframe") iframe: ElementRef;

  ngOnInit() {
    this.downloadFileSubject.pipe(
      map(content => content?.size > 0)
    ).subscribe(content => {
      if(content) this.AutoPrint()
      else this.modal.info("No data found.", this.viewRef)
    })
  }

  public GetNewReport() {
    if(this.showPreview) {
      this.modal.confirm("Are you sure to refresh?<br/>Current print data will be discarded.", this.viewRef,
        () => {
            this.GetReport();
          }, () => {})
      } else {
        this.GetReport();
      }
  }

  private GetReport() {
    this.fileUrl = null;
    this.showPreview = false;
    this.http
      .Download("Batch", "PrintPickingBR", {
        ReportPrefix: "NPMS_PK_",
        ReportSuffix: ".PDF",
      })
      .subscribe((res) => {
          if(res)  {
              this.SetIFrameContent(res);
          }
        }, error => {
          this.modal.warning(
            "Found an error, please contact support team",
            this.viewRef
          );
      });
  }

  private SetIFrameContent(content: any) {
    const file = new Blob([content], { type: content.type });
    const fileUrl = URL.createObjectURL(file);

    this.fileUrl = fileUrl;
    this.downloadFileSubject.next(content);
  }

  public AutoPrint() {
    this.showPreview = true;
    setTimeout(() => {
        this.PrintPDF();
    }, 2000)
  }

  public PrintPDF() {
    this.iframe.nativeElement.contentWindow.print();
  }

}
