import { Component, OnInit, ViewContainerRef, ElementRef, ViewChild } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import { tap } from 'rxjs/operators';
import { Globals } from 'src/app/core/global';
import { AutoCompleteQueryComponent } from 'src/app/components/shared/auto-complete-query/auto-complete-query.component';
import * as moment from 'moment';
import { numberFormat } from 'src/app/core/helpers';

@Component({
  selector: 'app-snpe034',
  templateUrl: './snpe034.component.html',
  styleUrls: ['./snpe034.component.css']
})
export class SNPE034Component implements OnInit {
  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private el: ElementRef,
    private global: Globals
  ) { }

  @ViewChild("autoCompleteOrderClass", { static: false }) autoCompleteOrderClass: any;
  @ViewChild("autoCompleteVendor", { static: true }) autoCompleteVendor: AutoCompleteQueryComponent;

  private currentDate: moment.Moment = moment();


  public tableHeader: any = {};

  public DocNo: string;
  public DocDate: moment.Moment;
  public Vendor: string;
  public VendorCode: string;
  public OrderClass: string;
  public OrderClassDesc: string;
  public OrderType: string;
  public InvoiceNo: string;
  public InvoiceDate: moment.Moment;
  public DeliveryDate: moment.Moment;
  public InvoiceStatus: string;
  public InvoiceProcess: string;
  public Invoice: any = {};
  public SentBy: string;
  public UpdateBy: string;
  public UpdateDate: moment.Moment;

  public TableData: Array<any> = [];
  //public ParamsSearch: any = {
  //  DOCUMENT_DATE: moment()
  //};
  public ParamsSearch: any = {};
  public ShowList: boolean = true;
  public DocList: Array<any> = [];
  public tempData: any = {};
  public TEMP_DUTY : any = 0;
  public ConfirmDate :any;
  public Iscancel : boolean = false;
  public vendorLookupConfig: any = {
    ColName: "LOOKUP_VENDOR_NAME",
    PlaceholderText: "Select Vendor",
    SourceController: "AC",
    SourceMethodName: "SNPE036_GetVendorCode",
    NoOfShowRecords: 50,
    UpperCase: true
  }

  public orderClassLookupConfig: any = {
    ColName: "LOOKUP_NAME",
    PlaceholderText: "Select Order Class",
    SourceController: "wPMS",
    SourceMethodName: "GetOrderClass",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      JOB_TYPE: '2'
    }
  }

  public PartNoLookupConfig: any = {
    ColName: "LOOKUP_PART_NAME",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 20
  }


  public ConfigDocNo: any = {
    ColName: "DOCUMENT_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE034_GetDocList",
    NoOfShowRecords: 50,
    MinLengthSearch: 2,
    UpperCase: true
  }

  public ConfigInvoiceNo: any = {
    ColName: "REF_INV_NO",
    PlaceholderText: "Select Document No.",
    SourceController: "WH",
    SourceMethodName: "SNPE034_GetInvList",
    NoOfShowRecords: 50,
    MinLengthSearch: 2
  }

  ngOnInit() {
  }

  GetDate() {
    return this.http.GetSystemDate()
      .pipe(tap((result: any) => {
        if (result) this.currentDate = result;
      }))
  }

  Search() {
    this.http.post("WH", "SNPE034_GetDocumentList",
    {
        DocumentNo: this.ParamsSearch.DOCUMENT_NO,
        DocumentDate: this.global.ConvertDateToFormatSTD(this.ParamsSearch.DOCUMENT_DATE),
        InvoiceNo: this.ParamsSearch.INVOICE_NO,
        InvoiceDate: this.global.ConvertDateToFormatSTD(this.ParamsSearch.INVOICE_DATE)
    })
      .subscribe(result => {
        this.DocList = result.Table;
      })
  }

  ViewClick(data) {
    this.DocNo = data.DOCUMENT_NO;
    this.DocDate = data.DOCUMENT_DATE;
    this.InvoiceNo = data.REF_INV_NO;
    this.InvoiceStatus = data.INVOICE_STATUS;
    this.InvoiceProcess = '';
    this.InvoiceDate = data.REF_INV_DATE;
    this.DeliveryDate = data.BL_AWB_DATE
    this.Vendor = data.VENDOR;
    this.VendorCode = data.VENDOR_CODE;
    this.SentBy = 'N';
    this.OrderClass = null;
    this.OrderClassDesc = null;
    this.OrderType = null;
    this.UpdateBy = null;
    this.UpdateDate = null;
    this.TEMP_DUTY = 0;

    this.TableData = [];
    this.ShowList = false;

    this.GetDocumentDetail();
  }

  AddClick() {
    this.DocNo = null;
    this.DocDate = this.currentDate;
    this.InvoiceNo = null;
    this.InvoiceStatus = "New";
    this.InvoiceProcess = 'N';
    this.InvoiceDate = this.currentDate;
    this.DeliveryDate = this.currentDate;
    this.Vendor = null;
    this.VendorCode = null;
    this.SentBy = 'N';
    this.OrderClass = null;
    this.OrderClassDesc = null;
    this.OrderType = null;
    this.UpdateBy = null;
    this.UpdateDate = null;
    this.TEMP_DUTY = 0;

    this.TableData = [];
    this.ShowList = false;
    this.tempData = {};


    this.tableHeader = {};
    this.tableHeader.EXP_CLEANRANCE_AMT_B = 0;
    this.tableHeader.EXP_OTHER_AMT_B = 0;

    this.CreateDetailRow(0);
  }

  // NewDocument() {
  //     this.modal.confirm("Are you sure to new document?<br>All changes will be cleared.", this.viewref
  //         , () => {
  //             this.ngOnInit();
  //         }, () => { })
  // }

  SaveDocument() {
    let saveRows = this.TableData.filter((d, i) => {
      d.DOCUMENT_NO = this.DocNo;
      d.VENDOR_CODE = this.VendorCode;
      d.SENT_BY = this.SentBy;
      d.ORDER_CLASS = this.OrderClass;
      d.ORDER_TYPE = this.OrderType;
      d.INVOICE_NO = this.InvoiceNo;
      d.INVOICE_DATE = this.global.ConvertDateToFormatSTD(this.InvoiceDate);
      d.BL_AWB_DATE = this.global.ConvertDateToFormatSTD(this.DeliveryDate);
      d.TOTAL_EXP_CLEANRANCE_AMT_B = this.tableHeader.EXP_CLEANRANCE_AMT_B ? this.tableHeader.EXP_CLEANRANCE_AMT_B : 0 ;
      d.TOTAL_EXP_OTHER_AMT_B = this.tableHeader.EXP_OTHER_AMT_B ? this.tableHeader.EXP_OTHER_AMT_B : 0;
      d.TOTAL_INCOME_TAX = this.TotalDuty();

      return this.CheckRequiredFields(i, false) && d.INVOICE_PROCESS === 'N';
    })

    console.log('saverow', saveRows)

    if (saveRows.length > 0) {
      this.modal.confirm("Are you sure to confirm?", this.viewref
        , () => {
          this.http.post("WH", "SNPE034_SaveDocument", saveRows)
            .subscribe(result => {
              this.modal.info("Transaction complete: applied and saved.", this.viewref);
              this.CheckDocument();
            }, err => {
              this.modal.error(err.message, this.viewref);
            })
        }, () => { })
    } else {
      this.modal.error("Unable to process.", this.viewref);
    }
  }

  DeleteDocument() {
    this.modal.confirm("Are you sure to cancel this document?", this.viewref
      , () => {
        this.http.post("WH", "SNPE034_DeleteDocument", {
            VendorCode: this.VendorCode, 
            InvoiceNo: this.InvoiceNo, 
            DocumentNo: this.DocNo
        })
          .subscribe(result => {
            if (result === "Done") {
              this.modal.info("Transaction complete: applied and saved.", this.viewref);
              this.ngOnInit();
              this.Iscancel = true;
            }
            else if(result === "INVOICE_ERROR"){
              this.modal.warning("Unable to process this invoice",this.viewref);
            }
            else if(result === "CONFIRMED"){
                this.modal.warning("Can not cancel document because this document confirmed",this.viewref);
            }
            else{
                this.modal.error('Found an error, please contact support team', this.viewref);
            }
          }, err => {
            this.modal.error('Found an error, please contact support team', this.viewref);
          })
      }, () => {

      });

  }

  GetDocumentDetail() {
    if (this.VendorCode && this.InvoiceNo && this.DocNo) {
      this.http.get("WH", "SNPE034_CheckDocument", this.DocNo)
        .subscribe(result => {
          if (result.Table.length > 0) {
            this.tableHeader = result.Table[0];

            this.VendorCode = this.tableHeader.VENDOR_CODE;
            this.SentBy = this.tableHeader.SENT_BY;
            this.DocNo = this.tableHeader.DOCUMENT_NO;
            this.DocDate = this.tableHeader.DOCUMENT_DATE;
            this.InvoiceNo = this.tableHeader.INVOICE_NO;
            this.InvoiceDate = this.tableHeader.INVOICE_DATE;
            this.DeliveryDate = this.tableHeader.BL_AWB_DATE;
            this.InvoiceStatus = this.tableHeader.INVOICE_STATUS;
            this.InvoiceProcess = this.tableHeader.INVOICE_PROCESS;
            this.UpdateBy = this.tableHeader.LAST_UPD_BY;
            this.UpdateDate = this.tableHeader.LAST_UPD_DATE;
            this.ConfirmDate = this.tableHeader.CONFIRM_DATE;

            this.TableData = result.Table1;

            if (this.TableData.length > 0) {
              this.OrderClass = this.TableData[0].ORDER_CLASS;
              this.OrderType = this.TableData[0].ORDER_TYPE;
              this.OrderClassDesc = this.TableData[0].ORDER_CLASS_DESC;
            }

            // for (let i = this.TableData.length; i < 10; i++) {
            //   this.TableData.push({
            //     ITEM_NO: i + 1,
            //     INVOICE_PROCESS: this.InvoiceProcess
            //   });
            // }

          }
        })
    }
  }


  CheckDocument() {
    if (this.VendorCode && this.InvoiceNo) {
      this.http.post("WH", "SNPE034_CheckDocument",{
          VendorCode: this.VendorCode, 
          InvoiceNo: this.InvoiceNo
      })
        .subscribe(result => {
          if (result.Table.length > 0) {
            this.tableHeader = result.Table[0];

            this.VendorCode = this.tableHeader.VENDOR_CODE;
            this.SentBy = this.tableHeader.SENT_BY;
            this.DocNo = this.tableHeader.DOCUMENT_NO;
            this.DocDate = this.tableHeader.DOCUMENT_DATE;
            this.InvoiceNo = this.tableHeader.INVOICE_NO;
            this.InvoiceDate = this.tableHeader.INVOICE_DATE;
            this.DeliveryDate = this.tableHeader.BL_AWB_DATE;
            this.InvoiceStatus = this.tableHeader.INVOICE_STATUS;
            this.InvoiceProcess = this.tableHeader.INVOICE_PROCESS;
            this.UpdateBy = this.tableHeader.LAST_UPD_BY;
            this.UpdateDate = this.tableHeader.LAST_UPD_DATE;
            this.ConfirmDate = this.tableHeader.CONFIRM_DATE;

            this.TableData = result.Table1;

            if (this.TableData.length > 0) {
              this.OrderClass = this.TableData[0].ORDER_CLASS;
              this.OrderType = this.TableData[0].ORDER_TYPE;

              this.autoCompleteOrderClass.textInput = this.OrderClass;
              this.autoCompleteOrderClass.query();
            }

            for (let i = this.TableData.length; i < 10; i++) {
              this.TableData.push({
                ITEM_NO: i + 1,
                INVOICE_PROCESS: this.InvoiceProcess
              });
            }

            //this.TableData = this.TableData.slice();
            //console.log('ttttt',this.TableData)
            //console.log('cal',this.TotalDuty())

          }
        })
    }
  }

  SetVendorCodeLookup(value) {
    if (value.VENDOR_CODE) {
      this.Vendor = value.LOOKUP_VENDOR_NAME;
      this.VendorCode = value.VENDOR_CODE;

      // this.PartNoLookupConfig.SourceParams.VENDOR_CODE = this.VendorCode;
    } else {
      this.Vendor = null;
      this.VendorCode = null;
      this.el.nativeElement.querySelector("#vendorcode").focus();
      this.modal.error("Unable to find vendor.", this.viewref);
    }
  }

  SetOrderClassLookup(value) {
    if (value.ORDER_CLASS) {
      this.OrderClass = value.ORDER_CLASS;
      this.OrderClassDesc = value.ORDER_DESCRIPT;
    } else {
      this.OrderClass = null;
      this.OrderClassDesc = null;
      this.el.nativeElement.querySelector("#orderclass").focus();
      this.modal.error("Unable to find order class.", this.viewref);
    }
  }

  SetOrderType() {
    switch (this.OrderClass) {
      case "1":
        this.OrderType = "1"
        break;
      case "2":
        this.OrderType = "1"
        break;
      case "3":
        this.OrderType = "3"
        break;
      case "4":
        this.OrderType = "3"
        break;
      case "5":
        this.OrderType = "4"
        break;
      case "7":
        this.OrderType = "6"
        break;
      default:
        this.OrderType = undefined;
        break;
    }
  }

  SetNewPartNo(part: any, data: any, index: number) {
    let findPartIndex = this.TableData.findIndex(v => v.ACCEPT_PART_NO == part.PART_NO);

    if (part?.PART_NO && findPartIndex >= 0 && findPartIndex !== index) {
      this.modal.warning("Duplicate part number.", this.viewref);
      data.ACCEPT_PART_NO = null;
      console.log(data);
      return;
    }

    this.tempData.Part = part;
    data.ACCEPT_PART_NO = part.PART_NO;
    data.PART_NAME = part.PART_NAME;
    data.UNIT_PRICE = parseFloat(part.BASE_PRICE);

    data.EXP_CLEANRANCE_AMT_B = 0;
    data.EXP_OTHER_AMT_B = 0;
    data.INCOME_TAX = 0;
    data.EXCISE = 0
    data.INTERIOR = 0


    this.GetBasePrice(part.PART_NO, data);
  }

  CancelPartNo(data: any) {
    data.ACCEPT_PART_NO = this.tempData.Part.PART_NO;
  }

  GetBasePrice(part: string, data: any) {
    this.http.get("WH", "SNPE034_GetBasePrice", part)
      .subscribe(result => {
        data.UNIT_PRICE = 0;
        if (result.Table.length > 0) {
          data.UNIT_PRICE = parseFloat(result.Table1[0].BASE_PRICE);
          data.CALCULATE_TYPE = result.Table1[0].CALCULATE_TYPE
          data.CAPACITY = result.Table1[0].CAPACITY
          data.EXCISE_RATE = result.Table1[0].EXCISE_RATE
          data.RETAIL_PRICE = result.Table1[0].RETAIL_PRICE
          data.CAPACITY_RATE = result.Table1[0].CAPACITY_RATE
          data.DUTY_RATE = result.Table1[0].DUTY_RATE
          data.INCOME_TAX = this.global.Round((data.BASE_PRICE * data.ITEM_QTY * data.DUTY_RATE) / 100, 2)
        }
      })
  }

  CheckValidSave() {
    return this.TableData.find((d, i) => this.CheckRequiredFields(i, false) && d.INVOICE_PROCESS === 'N');
  }

  CheckRequiredFields(tblIndex, allowUnChange: boolean = true) {
    let valid: boolean = (this.VendorCode && this.SentBy && this.OrderClass && this.OrderType && this.InvoiceNo && this.InvoiceDate) ? true : false;

    if (valid) {
      let thisRow = this.TableData[tblIndex];
      let countChangeItems: number = 0;

      if (thisRow.ACCEPT_PART_NO) countChangeItems++;
      if (thisRow.ITEM_QTY) countChangeItems++;
      if (thisRow.ACCEPT_PART_NO) countChangeItems++;

      valid = (countChangeItems > 0 && thisRow.ACCEPT_PART_NO && thisRow.ITEM_QTY && thisRow.UNIT_PRICE) || (allowUnChange && countChangeItems === 0) ? true : false;
    }

    return valid;
  }

  CheckActiveRow(tblIndex) {
    let valid: boolean = (this.VendorCode && this.SentBy && this.OrderClass && this.OrderType && this.InvoiceNo && this.InvoiceDate) ? true : false;

    if (valid) {
      valid = (!this.TableData[tblIndex].INVOICE_PROCESS || this.TableData[tblIndex].INVOICE_PROCESS === 'N') && !this.DocNo;
    }

    if (valid && tblIndex > 0) {
      let prevRow = this.TableData[tblIndex - 1];

      valid = (prevRow.ACCEPT_PART_NO && prevRow.ITEM_QTY && prevRow.UNIT_PRICE) ? true : false;
    }

    if (valid) {
      this.CreateDetailRow(tblIndex);
    }

    return valid;
  }

  CreateDetailRow(tblIndex) {
    let currentMaxRow: number = this.TableData.length || 0;
    let nextMaxRow: number = (Math.floor(currentMaxRow / 10) + 1) * 10;

    if (currentMaxRow - tblIndex <= 2) {
      for (let i = currentMaxRow; i < nextMaxRow; i++) {
        this.TableData.push({
          ITEM_NO: i + 1,
          INVOICE_PROCESS: 'N'
        });
      }

      this.TableData = this.TableData.slice();
    }

  }

  DeleteRow(index) {
    this.modal.confirm("Are you sure to remove this row?", this.viewref
      , () => {
        this.TableData.splice(index, 1);
        this.TableData = [...this.TableData];
        this.ExpenseChanged()
      }, () => { })
  }

  public TotalQty() {
    return this.TableData.reduce((a, b) => a + (b.ITEM_QTY || 0), 0);
  }

  TotalDuty() {
    return this.TableData.reduce((a, b) => a + (b.INCOME_TAX || 0), 0);
  }

  TotalPrice() {
    return this.TableData.reduce((a, b) => a + ((b.ITEM_QTY || 0) * (b.UNIT_PRICE || 0))
      , 0);
  }

  TotalExcise() {
    return this.TableData.reduce((a, b) => a + (b.EXCISE || 0)
      , 0);
  }

  TotalInterior() {
    return this.TableData.reduce((a, b) => a + (b.INTERIOR || 0)
      , 0);
  }

  TotalAmount() {
    return this.TableData.reduce((a, b) => a + ((b.ITEM_QTY || 0) * (b.UNIT_PRICE || 0)
      + (b.EXP_CLEANRANCE_AMT_B || 0)
      + (b.EXP_OTHER_AMT_B || 0)
      + (b.INCOME_TAX || 0)
      + (b.EXCISE || 0)
      + (b.INTERIOR || 0)
    )
      , 0);
  }

  public ExpenseChanged() {
    let SumEXP_CLEANRANCE_AMT_B = 0;
    let sumEXP_OTHER_AMT_B = 0;
    //this.TableData
    this.TableData.forEach((value) => {

      if (value.ACCEPT_PART_NO != '' && value.ACCEPT_PART_NO != undefined) {

        value.EXP_CLEANRANCE_AMT_B = (this.tableHeader.EXP_CLEANRANCE_AMT_B * (value.ITEM_QTY * value.UNIT_PRICE) / this.TotalPrice());

        value.EXP_OTHER_AMT_B = (this.tableHeader.EXP_OTHER_AMT_B * (value.ITEM_QTY * value.UNIT_PRICE) / this.TotalPrice());

        SumEXP_CLEANRANCE_AMT_B += value.EXP_CLEANRANCE_AMT_B;
        sumEXP_OTHER_AMT_B += value.EXP_OTHER_AMT_B;
      }

    });

    let diffCLEANRANCE = SumEXP_CLEANRANCE_AMT_B - this.tableHeader.EXP_CLEANRANCE_AMT_B;
    let diffOTHER = sumEXP_OTHER_AMT_B - this.tableHeader.EXP_OTHER_AMT_B;

    if (diffCLEANRANCE !== 0) {
      this.TableData[0].EXP_CLEANRANCE_AMT_B + diffCLEANRANCE
    }

    if (diffOTHER !== 0) {
      this.TableData[0].EXP_OTHER_AMT_B + diffCLEANRANCE
    }


  }

  QtyChange(data) {
    if (data.CALCULATE_TYPE == 'Capacity') {
      data.EXCISE = this.global.Round(data.CAPACITY_RATE * Math.ceil((data.CAPACITY * data.ITEM_QTY)),2);
    } else {
      data.EXCISE = this.global.Round((data.EXCISE_RATE / 100) * data.RETAIL_PRICE * data.ITEM_QTY,2);
    }

    this.CalIncomeTax(data);
    this.CalInterior(data);
  }

  CalIncomeTax(data) {
    data.INCOME_TAX = this.global.Round((data.UNIT_PRICE * data.ITEM_QTY * data.DUTY_RATE) / 100, 2);
  }

  CalInterior(data) {
    data.INTERIOR = this.global.Round(data.EXCISE / 10,2);
  }

  RecalDuty(data) {
    let d = this.global.Round((data.UNIT_PRICE * data.ITEM_QTY * data.DUTY_RATE) / 100, 2)
    data.INCOME_TAX = d
  }

  TotalDutyChange(event){

    let filterTable = this.TableData.filter(x => x.ACCEPT_PART_NO && x.INCOME_TAX)
    event = parseFloat(event.replace(/,/g, ''))
    let diffduty = this.global.Round(event - this.TotalDuty(),2)
    let maxDuty = Math.max.apply(Math, filterTable.map((x) => { return x.INCOME_TAX; }))
    let index = this.TableData.findIndex(x => x. INCOME_TAX == maxDuty);
    
    if(diffduty !== 0){
      this.TableData[index].INCOME_TAX =  this.global.Round(this.TableData[index].INCOME_TAX + diffduty,2)
      this.TableData.slice();
    }
  }




  BackToGrid() {
    this.ShowList = true;
  }
}
